@@ -33,7 +33,7 @@
 #include "SFinternal.h"
 
 #if defined(SVR4) || defined(SYSV) || defined(USG) || defined(__alpha__)      \
- || defined(__CYGWIN32__)
+ || defined(__CYGWIN32__) || defined(__FreeBSD__)
 #include <dirent.h>
 #else /* defined(SVR4) || defined(SYSV) || defined(USG) ... */
 #include <sys/dir.h>
