@@ -1,6 +1,5 @@
 /* The main program of the curses interface to Xconq.
-   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,15 +7,9 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern void low_send PARAMS ((int id, char *buf));
-extern int low_receive PARAMS ((int *id, char *buf, int maxchars,
-				int timeout));
-extern void update_debugging PARAMS ((void));
-extern void toggle_debugging PARAMS ((int *flagp));
-extern char *get_scores PARAMS ((Side *side));
+#include "kpublic.h"
 #include "cmdline.h"
 #include "cconq.h"
-extern void make_generic_image_data PARAMS ((ImageFamily *imf));
 
 #ifdef THINK_C
 /* This is to get the command line reader in Think C on the Mac. */
@@ -30,7 +23,15 @@ extern void make_generic_image_data PARAMS ((ImageFamily *imf));
 #include <sioux.h>
 #endif /* __MWERKS__ */
 
-static void describe_help PARAMS ((int arg, char *key, char *buf));
+extern int hosting;
+
+static void describe_help PARAMS ((int arg, char *key, TextBuffer *buf));
+
+#ifdef MAC /* temporary */
+char **current_cursor;
+char **sendcursor;
+char **receivecursor;
+#endif
 
 int announced = FALSE;
 
@@ -192,6 +193,28 @@ int autofinish_start;
 
 int autofinish_count;
 
+enum movie_type {
+  movie_null,
+  movie_miss,
+  movie_hit,
+  movie_death,
+  movie_nuke
+};
+
+struct a_movie {
+  char *type;
+  enum movie_type itype;
+  int args[5];
+};
+
+/* Number of movies waiting to be played. */
+
+int numscheduled;
+
+/* Movies to be played. */
+
+struct a_movie *movies;
+
 /* The main program just invokes everything else. */
 
 int
@@ -215,7 +238,7 @@ char *argv[];
     SIOUXSettings.showstatusline = TRUE;
 #endif
 #ifdef MAC
-	_maccur_pgm_name = "Cconq";
+    _maccur_pgm_name = "Cconq";
 #endif
     printf("\n              Welcome to Curses Xconq version %s\n\n",
 	   version_string());
@@ -228,36 +251,79 @@ char *argv[];
 #ifdef DEBUGGING
     update_debugging();
 #endif /* DEBUGGING */
-    /* (should volunteer to restore saved game if one found) */
-    load_all_modules();
-    check_game_validity();
-    parse_command_line(argc, argv, variant_options);
-    set_variants_from_options();
-    parse_command_line(argc, argv, player_options);
-    /* Go through once more and complain about anything not used. */
-    parse_command_line(argc, argv, leftover_options);
-    set_players_from_options();
-    make_trial_assignments();
-    calculate_globals();
-    run_synth_methods();
-    final_init();
-    assign_players_to_sides();
-    /* Run through game calcs, but don't let anything move yet; this
-       gives display code better values to work with. */
-    run_game(0);
-    init_display();
+    if (option_game_to_join != NULL) {
+	int rslt;
+ 	extern int hosting;
+	extern char *connection_method_name;
+	extern int connection_method;
+
+	/* (should parse variant specs, then comment that they must be ignored) */
+	/* (should accept player specs) */
+	/* Go through once more and complain about anything not used. */
+	parse_command_line(argc, argv, leftover_options);
+	hosting = FALSE;
+	open_remote_connection(option_game_to_join);
+	if (connection_method > 0) {
+	    rslt = send_join("xxx");
+	    if (rslt) {
+		master_rid = 1;
+		while (TRUE) {
+		    receive_data(0);
+		    if (!beforestart) {
+			break;
+		    }
+		}
+	    } else {
+		close_remote_connection();
+		/* Use of connection_method this way is weak; means
+		   having to reselect everything up reconnection. */
+		connection_method = 0;
+	    }
+	}
+    } else {
+	/* (should volunteer to restore saved game if one found?) */
+	load_all_modules();
+	check_game_validity();
+	parse_command_line(argc, argv, variant_options);
+	set_variants_from_options();
+	parse_command_line(argc, argv, player_options);
+	set_players_from_options();
+	/* Go through once more and complain about anything not used. */
+	parse_command_line(argc, argv, leftover_options);
+	/* (should call launch_game() here) */
+	make_trial_assignments();
+	if (option_game_to_host != NULL) {
+	    hosting = TRUE;
+	    open_remote_connection(option_game_to_host);
+	}
+	calculate_globals();
+	run_synth_methods();
+	final_init();
+	assign_players_to_sides();
+#ifndef HAVE_SELECT
+	/* If no way to check for input periodically, then we must force
+	   sequentiality.  This must happen before the first run_game. */
+	set_g_use_side_priority(TRUE);
+#endif
+	/* Run through game calcs, but don't let anything move yet; this
+	   gives display code better values to work with. */
+	run_game(0);
+	init_display();
+    }
     init_interaction();
     redraw();
     init_signal_handlers();
     while (TRUE) {
 #ifdef HAVE_SELECT
 please somebody write this code
-	run_game(1);
+	net_run_game(1);
 #else
-	set_g_use_side_priority(TRUE);
+	run_local_ai(1, 20);
 	/* assumes no real-timeness */
-	while (run_game(9) > 0)
-	  ;
+	while (probability(99)) {  /* a hack */
+	    net_run_game(9);
+	    run_local_ai(2, 20);
+	}
 	maybe_handle_input(0);
 #endif
     }
@@ -300,8 +366,9 @@ Side *side;
     }
 }
 
-/* Open display, create all the windows we'll need, do misc setup things,
-   and initialize some globals to out-of-range values for recognition later. */
+/* Open display, create all the windows we'll need, do misc setup
+   things, and initialize some globals to out-of-range values for
+   recognition later. */
 
 void
 init_display()
@@ -318,6 +385,9 @@ init_display()
     nonl();
     noecho();
     cbreak();
+#if 0 /* def MAC */ /* can't do this yet */
+    nodelay(stdscr, 1);
+#endif
     clear();
     /* Set up random globals. */
     nw = min(BUFSIZE, 60);
@@ -335,9 +405,9 @@ init_display()
     mh = LINES - 2 - infoh - 1;
 
     mvp = new_vp();
-    /* Each cell is actually 2 chars or "pixels" wide.  This is the
-       difference from the standard power==0 parameters, so we do
-       a cheesy thing and tweak the array directly. */
+    /* Each cell is actually 2 chars or "pixels" wide.  This is a
+       difference from the standard power==0 parameters, so we do a
+       cheesy thing and tweak the array directly. */
     hws[0] = 2;
     set_view_power(mvp, 0);
     set_view_size(mvp, mw, mh);
@@ -378,9 +448,6 @@ init_display()
 	dashbuffer[COLS] = '\0';
     }
     init_ui_chars();
-    /* Give the interface's state-saving mechanism a chance to preallocate
-       anything it might want to use; allocation is not allowed during saving. */
-    ui_save_state(dside);
     active = TRUE;
     Dprintf("Successfully opened display!\n");
 }
@@ -388,6 +455,8 @@ init_display()
 void
 init_interaction()
 {
+    HelpNode *long_commands_help_node;
+
     curunit = NULL;
     find_next_and_look();
     if (curunit) {
@@ -401,6 +470,9 @@ init_interaction()
     if (cur_help_node == NULL) {
 	help_help_node =
 	  add_help_node("help", describe_help, 0, first_help_node);
+	long_commands_help_node =
+	  add_help_node("long commands", describe_long_commands, 0,
+			first_help_node);
 	commands_help_node =
 	  add_help_node("commands", describe_commands, 0, first_help_node);
 	topics_help_node =
@@ -408,7 +480,7 @@ init_interaction()
 	cur_help_node = topics_help_node;
     }
     /* By default, require manual `return' to finish each turn. */
-    set_autofinish(dside, FALSE);
+    net_set_autofinish(dside, FALSE);
 }
 
 /* Windows in curses are just simple bounding boxes. */
@@ -542,7 +614,8 @@ int dir, n;
     if (mode == SURVEY) {
 	if (point_in_dir_n(curx, cury, dir, n, &nx, &ny)) {
 	    move_survey(nx, ny);
-	} else if (n >= 2 && point_in_dir_n(curx, cury, dir, n / 2, &nx, &ny)) {
+	} else if (n >= 2
+		   && point_in_dir_n(curx, cury, dir, n / 2, &nx, &ny)) {
 	    move_survey(nx, ny);
 	} else {
 	    xbeep();
@@ -550,7 +623,7 @@ int dir, n;
     } else if (curunit == NULL) {
 	return;  /* no beep? */
     } else if (n > 1) {
-	set_move_dir_task(curunit, dir, n);
+	net_set_move_dir_task(curunit, dir, n);
     } else {
 	if (point_in_dir(curx, cury, dir, &nx, &ny)) {
 	    rslt = advance_into_cell(dside, curunit, nx, ny, unit_at(nx, ny));
@@ -618,10 +691,10 @@ int dflt;
     return dflt;
 }
 
-/* Prompt for a type of a unit from player, maybe only allowing some types
-   to be accepted.  Also allow specification of no unit type.  We do this
-   by scanning the vector, building a string of chars and a vector of
-   unit types, so as to be able to map back when done. */
+/* Prompt for a type of a unit from player, maybe only allowing some
+   types to be accepted.  Also allow specification of no unit type.
+   We do this by scanning the vector, building a string of chars and a
+   vector of unit types, so as to be able to map back when done. */
 
 int
 ask_unit_type(prompt, possibles)
@@ -665,10 +738,11 @@ short *possibles;
     }
 }
 
-/* Prompt for a type of a terrain from player, maybe only allowing some types
-   to be accepted.  Also allow specification of no terrain type.  We do this
-   by scanning the vector, building a string of chars and a vector of
-   terrain types, so as to be able to map back when done. */
+/* Prompt for a type of a terrain from player, maybe only allowing
+   some types to be accepted.  Also allow specification of no terrain
+   type.  We do this by scanning the vector, building a string of
+   chars and a vector of terrain types, so as to be able to map back
+   when done. */
 
 int
 ask_terrain_type(prompt, possibles)
@@ -840,7 +914,8 @@ char *prompt, *dflt, **strp;
 	switch (ch) {
 	  case '\r':
 	  case '\n':
-	    /* Return a copy of the part of the buffer that was typed into. */
+	    /* Return a copy of the part of the buffer that was typed
+               into. */
 	    if (strp != NULL)
 	      *strp = copy_string(text1 + reqstrbeg);
 	    done = TRUE;
@@ -917,7 +992,7 @@ char *prompt;
 Unit **unitp;
 {
     char ch;
-    int ndirs, dir, nx, ny;
+    int ndirs, dir, nx, ny, rslt = FALSE;
     Unit *nextocc;
 
     *unitp = NULL;
@@ -934,14 +1009,15 @@ Unit **unitp;
 	    mode = prevmode;
 	    clear_toplines();
 	    show_cursor();
-	    return TRUE;
+	    rslt = TRUE;
+	    break;
 	}
 	if (ch == ESCAPE) {
 	    restore_cur();
 	    mode = prevmode;
 	    clear_toplines();
 	    show_cursor();
-	    return FALSE;
+	    break;
 	}
 	/* Also recognize the "in" ("occupant") command. */
 	if (ch == 'i') {
@@ -964,7 +1040,7 @@ Unit **unitp;
 	    xbeep();
 	}
     }
-    return FALSE;
+    return rslt;
 }
 
 /* Given a unit, make it be the one that we are "looking at". */
@@ -976,13 +1052,15 @@ Unit *unit;
     curunit = unit;
     if (in_play(unit)) {
 	curx = unit->x;  cury = unit->y;
+	/* Make sure the screen shows this unit's char. */
+	draw_row(curx, cury, 1);
     }
     show_closeup();
     show_cursor();
 }
 
-/* Give a location, make it be current, and pick a unit there
-   to be the current unit. */
+/* Give a location, make it be current, and pick a unit there to be
+   the current unit. */
 
 void
 make_current_at(x, y)
@@ -1002,8 +1080,8 @@ int x, y;
     put_on_screen(curx, cury);
 }
 
-/* Add 'P' for printing/saving to file (which?). */
-/* (Need individual screen scrolling also) */
+/* (should add 'P' for printing/saving to file (which?)) */
+/* (should add individual screen scrolling also) */
 
 void
 interpret_help()
@@ -1049,15 +1127,16 @@ interpret_help()
 static void
 describe_help(arg, key, buf)
 int arg;
-char *key, *buf;
+char *key;
+TextBuffer *buf;
 {
-    tprintf(buf, "' ' to page down through a node\n");
-    tprintf(buf, "'n' to go to next help node\n");
-    tprintf(buf, "'p' to go to previous node\n");
-    tprintf(buf, "'b' to go back to last node\n");
-    tprintf(buf, "'t' to go to topics list\n");
-    tprintf(buf, "'?' to go to this help info\n");
-    tprintf(buf, "'q' or ESC to go back to game\n");
+    tbcat(buf, "' ' to page down through a node\n");
+    tbcat(buf, "'n' to go to next help node\n");
+    tbcat(buf, "'p' to go to previous node\n");
+    tbcat(buf, "'b' to go back to last node\n");
+    tbcat(buf, "'t' to go to topics list\n");
+    tbcat(buf, "'?' to go to this help info\n");
+    tbcat(buf, "'q' or ESC to go back to game\n");
 }
 
 /* Reading is usually pretty fast, so don't do anything here. */
@@ -1103,8 +1182,7 @@ finish_lengthy_process()
     }
 }
 
-/* This is the basic routine to get out of the program - we assume any
-   game hacking has been taken care of already. */
+/* This should be called before any sort of normal exit. */
 
 void
 exit_cconq()
@@ -1193,6 +1271,7 @@ Unit *unit;
 int rightnow;
 {
     if (active_display(side)) {
+	/* (should impl?) */
     }
 }
 
@@ -1206,7 +1285,7 @@ int rightnow;
 	if (rightnow)
 	  refresh();
 	if (g_turn() >= autofinish_start + autofinish_count) {
-	    set_autofinish(dside, FALSE);
+	    net_set_autofinish(dside, FALSE);
 	}
     }
 }
@@ -1392,7 +1471,7 @@ void
 print_form(form)
 Obj *form;
 {
-	print_form_and_value(stdout, form);
+    print_form_and_value(stdout, form);
 }
 
 void
@@ -1404,44 +1483,121 @@ end_printing_forms()
 
 int
 #ifdef __STDC__
-schedule_movie(Side *side, enum movie_type movie, ...)
+schedule_movie(Side *side, char *movie, ...)
 #else
-schedule_movie(side, movie)
+schedule_movie(side, movie, a1, a2, a3, a4, a5)
 Side *side;
-enum movie_type movie;
+char *movie;
+int a1, a2, a3, a4, a5;
 #endif
 {
-    return 0;
+    int i, itype;
+
+    if (numscheduled >= 10)
+      return FALSE;
+    if (side != dside)
+      return FALSE;
+    if (empty_string(movie))
+      return FALSE;
+    if (movies == NULL)
+      movies = (struct a_movie *) xmalloc(10 * sizeof(struct a_movie));
+    memset(&(movies[numscheduled]), 0, sizeof(struct a_movie));
+    movies[numscheduled].type = movie;
+    itype = movie_null;
+    if (strcmp(movie, "miss") == 0)
+      itype = movie_miss;
+    else if (strcmp(movie, "hit") == 0)
+      itype = movie_hit;
+    else if (strcmp(movie, "death") == 0)
+	  itype = movie_death;
+    else if (strcmp(movie, "nuke") == 0)
+      itype = movie_nuke;
+    movies[numscheduled].itype = itype;
+#ifdef ANSI_PROTOTYPES
+    {
+	va_list ap;
+
+	va_start(ap, movie);
+	for (i = 0; i < 5; ++i)
+	  movies[numscheduled].args[i] = va_arg(ap, int);
+	va_end(ap);
+    }
+#else
+    movies[numscheduled].args[0] = a1;
+    movies[numscheduled].args[1] = a2;
+    movies[numscheduled].args[2] = a3;
+    movies[numscheduled].args[3] = a4;
+    movies[numscheduled].args[4] = a5;
+#endif
+    ++numscheduled;
+    return TRUE;
 }
 
 void
 play_movies(sidemask)
 SideMask sidemask;
 {
+    int j, unitid, blasttype;
+    Unit *unit;
+
+    if (!side_in_set(dside, sidemask))
+      return;
+    for (j = 0; j < numscheduled; ++j) {
+	switch (movies[j].itype) {
+	  case movie_null:
+	    break;
+	  case movie_miss:
+	  case movie_hit:
+	  case movie_death:
+	    unitid = movies[j].args[0];
+	    unit = find_unit(unitid);
+	    if (unit == NULL || !in_area(unit->x, unit->y))
+	      continue;
+	    blasttype = (movies[j].itype == movie_miss ? 0 :
+			 (movies[j].itype == movie_hit ? 1 : 2));
+	    draw_blast(unit->x, unit->y, 0);
+	    break;
+	  case movie_nuke:
+	    draw_blast(unit->x, unit->y, 1);
+	    break;
+	  default:
+	    break;
+	}
+    }
+    numscheduled = 0;
 }
 
 void
-ui_save_state(side)
-Side *side;
-{
-  Obj *state = lispnil;
-
-  side->uidata = replace_at_key(side->uidata, "curses", state);
-}
-
-void
-low_send(id, buf)
+set_you_player(id)
 int id;
-char *buf;
+{
+}
+
+void
+add_remote_player(name)
+char *name;
 {
 }
 
 int
-low_receive(id, buf, maxchars, timeout)
-int *id, maxchars, timeout;
-char *buf;
+launch_game()
 {
-  return 0;
+    /* Trial side/player assignment, will create the default player if
+       nececessary. */
+    make_trial_assignments();
+    /* A last-minute patch to ensure reasonable area dimensions. */
+    if (area.width == 0 || area.height == 0)
+      set_area_shape(DEFAULTWIDTH, DEFAULTHEIGHT, TRUE);
+    /* Synthesize the remainder of the initial game setup. */
+    calculate_globals();
+    run_synth_methods();
+    final_init();
+    assign_players_to_sides();
+    /* Init the display now. */
+    init_display();
+    /* Do first set of turn calcs. */
+    run_game(0);
+    return TRUE;
 }
 
 void
@@ -1449,3 +1605,14 @@ make_generic_image_data(imf)
 ImageFamily *imf;
 {
 }
+
+#ifdef MAC
+
+int
+serial_port_dialog()
+{
+    /* (should provide choice of modem vs printer port) */
+    return 0;
+}
+
+#endif /* MAC */
