@@ -153,9 +153,7 @@ extern void show_help PARAMS ((void));
 
 extern void clear_window PARAMS ((struct ccwin *win));
 
-#if 0
-extern void draw_mushroom PARAMS ((int x, int y, int i));
-#endif
+extern void draw_blast PARAMS ((int x, int y, int r));
 extern int draw_text PARAMS ((struct ccwin *win, int x, int y, char *str));
 extern int cur_at PARAMS ((struct ccwin *win, int x, int y));
 
@@ -168,6 +166,7 @@ extern int auto_attack_on_move PARAMS ((Unit *, Unit *));
 extern Unit *find_next_and_look PARAMS ((void));
 
 extern void execute_command PARAMS ((void));
-extern void describe_commands PARAMS ((int arg, char *key, char *buf));
+extern void describe_commands PARAMS ((int arg, char *key, TextBuffer *buf));
+extern void describe_long_commands PARAMS ((int arg, char *key, TextBuffer *buf));
 
 extern void xbeep PARAMS ((void));
