@@ -8,6 +8,8 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+extern void others_here_desc PARAMS ((char *buf, Unit *unit));
+extern void occupants_desc PARAMS ((char *buf, Unit *unit));
 #include "cconq.h"
 
 static int xform PARAMS ((int x, int y, int *sxp, int *syp));
@@ -196,19 +198,26 @@ show_map()
     if (use_standout)
       standout();
     tmpbuf[0] = '\0';
-    if (drawunits) {
-	strcat(tmpbuf, "units");
-    }
-    if (drawnames) {
+    if (!drawterrain) {
 	if (strlen(tmpbuf) > 0)
 	  strcat(tmpbuf, ",");
-	strcat(tmpbuf, "names");
+	strcat(tmpbuf, "noterrain");
+    }
+    if (drawunits) {
+	if (strlen(tmpbuf) > 0)
+	  strcat(tmpbuf, ",");
+	strcat(tmpbuf, "units");
     }
     if (drawpeople) {
 	if (strlen(tmpbuf) > 0)
 	  strcat(tmpbuf, ",");
 	strcat(tmpbuf, "people");
     }
+    if (drawnames) {
+	if (strlen(tmpbuf) > 0)
+	  strcat(tmpbuf, ",");
+	strcat(tmpbuf, "names");
+    }
     memcpy(spbuf, dashbuffer, mw);
     memcpy(spbuf+2, "Map", 3);
     memcpy(spbuf+7, (mode == SURVEY ? "Survey" : "-Move-"), 6);
@@ -232,22 +241,28 @@ int x0, y0, len;
 	draw_terrain_row(x0, y0, len);
     }
     /* Draw sparse things on top of the basic row. */
-    for (x = x0; x < x0 + len; ++x) {
-	if (!inside_area(x, y0))
-	  continue;
-	/* The relative ordering of these is quite important.  Note that
-	   each should be prepared to run independently also, since the
-	   other displays might have been turned off. */
-	if (people_sides_defined() && drawpeople) {
+    if (people_sides_defined() && drawpeople) {
+	for (x = x0; x < x0 + len; ++x) {
+	    if (!inside_area(x, y0))
+	      continue;
 	    draw_people(x, y0);
 	}
-	if (drawunits) {
+    }
+    /* Draw units on top of people in cell. */
+    if (drawunits) {
+	for (x = x0; x < x0 + len; ++x) {
+	    if (!inside_area(x, y0))
+	      continue;
 	    draw_units(x, y0);
 	}
     }
-    /* Need a second loop so names are superimposed properly. */
-    for (x = x0; x < x0 + len; ++x) {
-	if (drawnames) {
+    /* Draw names last.  A long name may obscure a number of units, but
+       it obscures the terrain anyway, so you can't expect to play
+       effectively when names are displayed. */
+    if (drawnames) {
+	for (x = x0; x < x0 + len; ++x) {
+	    if (!inside_area(x, y0))
+	      continue;
 	    draw_legend(x, y0);
 	}
     }
@@ -451,12 +466,10 @@ clear_toplines()
 
 /* Display all the details of the currently-selected unit/cell. */
 
-/* (much of this should be made into kernel routines shared by interfaces) */
-
 void
 show_closeup()
 {
-    int u, tview, uview;
+    int u, t, uview, tview;
     char *filler = "Empty ";
     Unit *unit = NULL;
     Side *side2;
@@ -490,8 +503,14 @@ show_closeup()
 		draw_text(closeupwin, 0, 0, tmpbuf);
 	    }
 	}
-	/* Describe the cell here. */
-	describe_cell(curx, cury, tview, filler, tmpbuf);
+	/* Describe the cell/transport here. */
+	if (unit != NULL && unit->transport != NULL) {
+	    t = terrain_at(unit->x, unit->y);
+	    sprintf(tmpbuf, "In %s (%s at %d,%d)",
+		    short_unit_handle(unit->transport), t_type_name(t), unit->x, unit->y);
+	} else {
+	    describe_cell(curx, cury, tview, filler, tmpbuf);
+	}
 	draw_text(closeupwin, 0, 1, tmpbuf);
     } else {
 	sprintf(tmpbuf, "??? Off-area %d,%d ???", curx, cury);
@@ -516,7 +535,10 @@ char *filler, *buf;
 	    strcat(buf, " ");
 	    strcat(buf, featname);
 	}
-	elevation_desc(buf+strlen(buf), x, y);
+	if (elevations_defined()) {
+	    strcat(buf, " ");
+	    elevation_desc(buf+strlen(buf), x, y);
+	}
 	/* (should put all weather on own line) */
 	temperature_desc(buf+strlen(buf), x, y);
 	/* add clouds and winds desc here */
@@ -533,9 +555,7 @@ static void
 draw_unit_details(unit)
 Unit *unit;
 {
-    int u = unit->type, u2, m, nums[MAXUTYPES];
-    int terr = terrain_at(unit->x, unit->y);
-    Unit *occ, *top;
+    int t;
 
     /* Say which unit this is. */
     sprintf(spbuf, "%s", unit_handle(dside, unit));
@@ -552,22 +572,12 @@ Unit *unit;
     if (unit->transport != NULL) {
 	sprintf(spbuf, "In %s", short_unit_handle(unit->transport));
     } else {
-	describe_cell(unit->x, unit->y, buildtview(terr), "In ", spbuf);
-	/* make this a "stacked_at" macro? */
-	top = unit_at(unit->x, unit->y);
-	if (top != NULL && top->nexthere != NULL) {
+	t = terrain_at(unit->x, unit->y);
+	describe_cell(unit->x, unit->y, buildtview(t), "In ", spbuf);
+	others_here_desc(tmpbuf, unit_at(unit->x, unit->y));
+	if (strlen(tmpbuf) > 0) {
 	    strcat(spbuf, ", ");
-	    for_all_unit_types(u2)
-	      nums[u2] = 0;
-	    for_all_stack(unit->x, unit->y, occ)
-	      ++nums[occ->type];
-	    --nums[u];  /* don't count ourselves */
-	    for_all_unit_types(u2) {
-		if (nums[u2] > 0) {
-		    tprintf(spbuf, "%d %1s  ", nums[u2], utype_name_n(u2, 1));
-		}
-	    }
-	    strcat(spbuf, " here also");
+	    strcat(spbuf, tmpbuf);
 	}
     }
     draw_text(closeupwin, 0, 1, spbuf);
@@ -575,28 +585,12 @@ Unit *unit;
     spbuf[0] = '\0';
     if (unit->occupant != NULL) {
 	strcpy(spbuf, "Occ ");
-	/* (should be occ_desc in nlang.c?) */
-	for_all_unit_types(u2)
-	  nums[u2] = 0;
-	for_all_occupants(unit, occ)
-	  ++nums[occ->type];
-	for_all_unit_types(u2) {
-	    if (nums[u2] > 0) {
-		tprintf(spbuf, "%d %1s  ", nums[u2], utype_name_n(u2, 1));
-	    }
-	}
+	occupants_desc(tmpbuf, unit);
+	strcat(spbuf, tmpbuf);
     }
     draw_text(closeupwin, 0, 2, spbuf);
     /* Describe the state of all the supplies. */
-    /* (should be supply_desc in kernel?) */
-    spbuf[0] = '\0';
-    for_all_material_types(m) {
-	if (um_storage_x(u, m) > 0) {
-	    sprintf(tmpbuf, "%s %d/%d  ",
-		    m_type_name(m), unit->supply[m], um_storage_x(u, m));
-	    strcat(spbuf, tmpbuf);
-	}
-    }
+    supply_desc(spbuf, unit, 0);
     draw_text(closeupwin, 0, 3, spbuf);
     /* Describe the current plans, tasks, etc. */
     /* (needs much improvement) */
@@ -607,7 +601,7 @@ Unit *unit;
 	plan_desc(spbuf, unit);
 	draw_text(closeupwin, 0, 4, spbuf);
 	for (task = unit->plan->tasks; task != NULL; task = task->next) {
-	    task_desc(spbuf, task);
+	    task_desc(spbuf, unit->side, task);
 	    draw_text(closeupwin, 0, row++, spbuf);
 	}
     }
@@ -771,35 +765,35 @@ show_list()
 	    filter = "??????";
 	    break;
 	}
-	    for (i = 0; i < MAXSORTKEYS; ++i) {
-		if (i == 0) {
-		    strcat(tmpbuf, "by ");
-		} else if (tmpsortkeys[i] != bynothing) {
-		    strcat(tmpbuf, ",");
-		}
-		switch (tmpsortkeys[i]) {
-		  case byside:
-		    strcat(tmpbuf, "side");
-		    break;
-		  case bytype:
-		    strcat(tmpbuf, "type");
-		    break;
-		  case byname:
-		    strcat(tmpbuf, "name");
-		    break;
-		  case byactorder:
-		    strcat(tmpbuf, "act");
-		    break;
-		  case bylocation:
-		    strcat(tmpbuf, "loc");
-		    break;
-		  case bynothing:
-		    break;
-		  default:
-		    strcat(tmpbuf, "???");
-		    break;
-		}
+	for (i = 0; i < MAXSORTKEYS; ++i) {
+	    if (i == 0) {
+		strcat(tmpbuf, "by ");
+	    } else if (tmpsortkeys[i] != bynothing) {
+		strcat(tmpbuf, ",");
 	    }
+	    switch (tmpsortkeys[i]) {
+	      case byside:
+		strcat(tmpbuf, "side");
+		break;
+	      case bytype:
+		strcat(tmpbuf, "type");
+		break;
+	      case byname:
+		strcat(tmpbuf, "name");
+		break;
+	      case byactorder:
+		strcat(tmpbuf, "act");
+		break;
+	      case bylocation:
+		strcat(tmpbuf, "loc");
+		break;
+	      case bynothing:
+		break;
+	      default:
+		strcat(tmpbuf, "???");
+		break;
+	    }
+	}
 	break;
       case 1:
 	for_all_unit_types(line) {
@@ -872,31 +866,30 @@ int line;
     /* Our units under construction (left-justified) */
     num = num_units_incomplete(dside, u);
     if (num > 0) {
-	sprintf(tmpbuf, "(%d)", num);
+	sprintf(tmpbuf, "(%2d)", num);
+	if (num > 9)
+	  strcat(tmpbuf, " ");
     } else {
 	sprintf(tmpbuf, "    ");
     }
     strcat(spbuf, tmpbuf);
+    /* Our total gains over the game. */
+    if (total_gain(dside, u) > 0) {
+	sprintf(tmpbuf, " %4d ", total_gain(dside, u));
+    } else {
+	sprintf(tmpbuf, "      ");
+    }
+    strcat(spbuf, tmpbuf);
+    /* Our total losses over the game. */
+    if (total_loss(dside, u) > 0) {
+	sprintf(tmpbuf, "- %-4d", total_loss(dside, u));
+    } else {
+	sprintf(tmpbuf, "      ");
+    }
+    strcat(spbuf, tmpbuf);
     draw_text(listwin, 1, line, spbuf);
 }
 
-/* (should add these back in somewhere?) */
-#if 0
-	/* Our total possessions over the game */
-	/* Make sure that subsequent write goes into valid string area. */
-	strcat(spbuf,  "            ");
-	if (total_gain(side, u) > 0) {
-	    sprintf(tmpbuf, "%4d", total_gain(side, u));
-	    sprintf(spbuf+10, "%s", tmpbuf);
-	}
-	/* Our total losses over the game */
-	strcat(spbuf,  "            ");
-	if (total_loss(side, u) > 0) {
-	    sprintf(tmpbuf, "- %d", total_loss(side, u));
-	    sprintf(spbuf+15, "%s", tmpbuf);
-	}
-#endif
-
 static void
 draw_unit_list_entry(line)
 int line;
@@ -973,12 +966,11 @@ struct ccwin *win;
     }
 }
 
-#if 0
 /* Draw a large blot over the area. */
 
 void
-draw_mushroom(x, y, i)
-int x, y, i;
+draw_blast(x, y, r)
+int x, y, r;
 {
     int sx, sy;
 
@@ -986,7 +978,7 @@ int x, y, i;
     if (cur_at(mapwin, sx, sy)) {
 	addstr("##");
 	refresh();
-	if (i > 0) {
+	if (r > 0) {
 	    if (cur_at(mapwin, sx-1, sy+1))
 	      addstr("####");
 	    if (cur_at(mapwin, sx-2, sy))
@@ -997,7 +989,6 @@ int x, y, i;
 	}
     }
 }
-#endif
 
 /* Drawing text is easy, but we do need to do clipping manually. */
 
