@@ -1,5 +1,5 @@
 /* Functions common to all AIs.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,6 +7,11 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+#include "kernel.h"
+/* (should do a better separation of kernel in set_side_ai) */
+#include "kpublic.h"
+extern void try_to_draw PARAMS ((Side *side, int flag, char *ainame));
+extern void give_up PARAMS ((Side *side, char *ainame));
 
 /* The number of AI types defined.  Increment for each new AI type
    that is compiled in. */
@@ -38,6 +43,7 @@ Side *side;
     void (*fn) PARAMS ((Side *side));
 
     if (side_wants_ai(side)) {
+	/* (should merge with same bit in set_side_ai) */
 	if (strcmp(side->player->aitypename, "ai") == 0) {
 	    /* (should use the "best" default for this game and side) */
 	    side->player->aitypename = "mplayer";
@@ -78,30 +84,28 @@ char *typename;
 	/* (should use the "best" default for this game and side) */
 	typename = "mplayer";
     }
+    /* (only copy if not an already-defined name, otherwise assign from
+       static list) */
+    if (typename != NULL)
+      typename = copy_string(typename);
     side->player->aitypename = typename;
-    for_all_side_units(side, unit) {
-	unit->aihook = NULL;
-	if (unit->plan && unit->plan->aicontrol) {
-	    unit->plan->type = PLAN_NONE;
-	    unit->plan->maingoal = NULL;
-	    unit->plan->formation = NULL;
-	    unit->plan->funit = NULL;
-	    clear_task_agenda(unit->plan);
-	    if (typename == NULL) {
-		unit->plan->asleep = FALSE;
-		unit->plan->reserve = FALSE;
-		/* Don't touch delay, let action looping clear it eventually. */
-		unit->plan->aicontrol = FALSE;
-	    } else {
-		unit->plan->aicontrol = TRUE;
+    /* If remote, do nothing beyond the recording the name of the AI. */
+    if (numremotes > 0 && side->player->rid != my_rid)
+      return;
+    /* If switching to human, clear out everything that was set up by an AI. */
+    if (empty_string(typename)) {
+	for_all_side_units(side, unit) {
+	    unit->aihook = NULL;
+	    if (unit->plan && unit->plan->aicontrol) {
+		net_force_replan(side, unit, FALSE);
 	    }
+	    /* Still let units finish their currently buffered action. */
 	}
-	/* Still let units finish their currently buffered action. */
     }
-    if (typename == NULL) {
-	/* (should just deactivate) */
+    if (empty_string(typename)) {
+	/* (should just deactivate instead of erasing and maybe
+           reallocating later) */
 	side->ai = NULL;
-	/* Clear out everything that was set up by the AI. */
     } else {
 	init_ai(side);
     }
@@ -130,18 +134,6 @@ Unit *unit;
       (*fn)(side, unit);
 }
 
-void
-ai_react_to_unit_loss(side, unit)
-Side *side;
-Unit *unit;
-{
-    void (*fn) PARAMS ((Side *side, Unit *unit));
-
-    fn = ((all_ai_ops[side->aitype])->to_react_to_unit_loss);
-    if (fn)
-      (*fn)(side, unit);
-}
-
 /* Forward an action result to the appropriate AI routine. */
 
 void
@@ -185,60 +177,19 @@ Side *side, *side2;
 }
 
 int
-ai_planning_to_capture(side, u, x, y)
-Side *side;
-int u, x, y;
-{
-    int (*fn) PARAMS ((Side *side, int u, int x, int y));
-
-    fn = ((all_ai_ops[side->aitype])->planning_to_capture);
-    if (fn)
-      return (*fn)(side, u, x, y);
-    else
-      return FALSE;
-}
-
-int
-ai_guide_explorer(side, unit)
+ai_adjust_plan(side, unit)
 Side *side;
 Unit *unit;
 {
     int (*fn) PARAMS ((Side *side, Unit *unit));
 
-    fn = ((all_ai_ops[side->aitype])->to_guide_explorer);
+    fn = ((all_ai_ops[side->aitype])->to_adjust_plan);
     if (fn)
       return (*fn)(side, unit);
     else
       return FALSE;
 }
 
-int
-ai_preferred_build_type(side, unit, plantype)
-Side *side;
-Unit *unit;
-int plantype;
-{
-    int (*fn) PARAMS ((Side *side, Unit *unit, int plantype));
-
-    fn = ((all_ai_ops[side->aitype])->preferred_build_type);
-    if (fn)
-      return (*fn)(side, unit, plantype);
-    else
-      return NONUTYPE;
-}
-
-void
-ai_analyze_after_moves(side, numacted)
-Side *side;
-int numacted;
-{
-    void (*fn) PARAMS ((Side *side, int numacted));
-
-    fn = ((all_ai_ops[side->aitype])->to_analyze_after_moves);
-    if (fn)
-      (*fn)(side, numacted);
-}
-
 void
 ai_finish_movement(side)
 Side *side;
@@ -307,6 +258,61 @@ int x, y;
       return NULL;
 }
 
+void
+try_to_draw(side, flag, ainame)
+Side *side;
+int flag;
+char *ainame;
+{
+    /* If no change, nothing to say or do. */
+    if (flag == side->willingtodraw)
+      return;
+    /* If there is a human player, allow the human to make the final
+       decision. */
+    /* (should have a way to delegate this also, at least for testing purposes) */
+    if (side_has_display(side)) {
+	if (flag)
+	  notify(side, "Your AI %s recommends being willing to draw",
+		 ainame);
+	else
+	  notify(side, "Your AI %s recommends not being willing to draw",
+		 ainame);
+	return;
+    }
+    net_set_willing_to_draw(side, flag);
+}
+
+/* If an AI resigns, it tries to help its friends. */
+
+void
+give_up(side, ainame)
+Side *side;
+char *ainame;
+{
+    Side *side1;
+
+    /* If there is a human player, allow the human to make the final
+       decision. */
+    /* (should have a way to delegate this also, at least for testing purposes) */
+    if (side_has_display(side)) {
+	notify(side, "Your AI %s recommends resignation", ainame);
+	return;
+    }
+    /* (should have to give units to controlling side if there is one) */
+    /* (should controlling side be able to disallow resignation?) */
+    /* Try to give away all of our units to an ally. */
+    for_all_sides(side1) {
+	if (side != side1 && allied_side(side, side1) && side1->ingame) {
+	    net_resign_game(side, side1);
+	    return;
+	}
+    }
+    /* (should give to any positively-regarded side?) */
+    /* No allies left in game, let everything become independent. */
+    /* (should disband all units that can be disbanded?) */
+    net_resign_game(side, NULL);
+}
+
 /* (should go elsewhere eventually?) */
 
 /* Goal handling. */
@@ -338,6 +344,18 @@ int tf;
     return goal;
 }
 
+int
+lookup_goal_type(name)
+char *name;
+{
+    int i;
+
+    for (i = 0; goaldefns[i].name != NULL; ++i)
+      if (strcmp(name, goaldefns[i].name) == 0)
+	return i; /* should get real enum? */
+    return GOAL_NO;
+}
+
 int
 cell_unknown(x, y)
 int x, y;
@@ -480,7 +498,9 @@ Goal *goal;
 }
 
 
-/* General collections of numbers used by all machine players. */
+/* General collections of numbers used by all machine players and unit plans. */
+
+int bhw_max;
 
 /* Init used by all machine players.  Precompute useful information
    relating to unit types in general, and that usually gets referenced
@@ -569,6 +589,8 @@ ai_init_shared()
 	    set_uu_bfw(u, u2, basic_fire_worth(u, u2));
 	    set_uu_bcw(u, u2, basic_capture_worth(u, u2));
 	    set_uu_btw(u, u2, basic_transport_worth(u, u2));
+	    if (uu_zz_bhw(u, u2) > bhw_max)
+	      bhw_max = uu_zz_bhw(u, u2);
 	}
     }
     /* Tell how things rated. */
@@ -681,16 +703,18 @@ int
 basic_capture_worth(u, e)
 int u, e;
 {
-    int worth1 = 0, worth2 = 0;
+    int acp, worth1 = 0, worth2 = 0;
 
-    if (uu_acp_to_capture(u, e) < 1 && uu_acp_to_attack(u, e) < 1)
+    acp = max(uu_acp_to_capture(u, e), uu_acp_to_attack(u, e));
+    if (acp < 1)
       return -9999;
     if (uu_capture(u, e) > 0) {
-	worth1 = uu_capture(u, e) * u_acp(u) /* divided by acp/attack */;
+	worth1 = (uu_capture(u, e) * u_acp(u)) / acp;
     }
     if (uu_indep_capture(u, e) > 0) {
-	worth2 = uu_indep_capture(u, e) * u_acp(u) /* divided by acp/attack */;
+	worth2 = (uu_indep_capture(u, e) * u_acp(u)) / acp;
     }
+    /* (should account for chance of dying in attempt) */
     return max(worth1, worth2);
 }
 
@@ -720,7 +744,7 @@ int u;
 void
 display_assessment()
 {
-    int t, u, u2;
+    int u, u2;
 
     DMprintf("\nUnit Attributes:\n");
     for_all_unit_types(u) {
