@@ -1,6 +1,5 @@
 /* Command line parsing for Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -10,11 +9,11 @@ any later version.  See the file COPYING.  */
 /* This is a command-line parser that may be used in implementations that
    get a command line from somewhere. */
 
-/* Command lines get parsed in several stages, since for instance
-   the choice of game module will decide which variants are available. */
+/* Command lines get parsed in several stages, since for instance the
+   choice of game module will decide which variants are available. */
 
 #include "conq.h"
-extern void add_library_path PARAMS ((char *path));
+extern short initially_no_ai;
 #include "cmdline.h"
 
 static void version_info PARAMS ((void));
@@ -29,12 +28,17 @@ static void add_a_raw_player_spec PARAMS ((char *specstr));
 
 /* The startup-settable options. */
 
-static int option_alltimeout;
 static int option_totalgametime;
 static int option_perturntime;
 static int option_persidetime;
 static int option_add_default_player;
 
+char *option_game_to_host = NULL;
+
+char *option_game_to_join = NULL;
+
+int option_num_to_wait_for = 0;
+
 static char *default_ai_type = ",mplayer";
 
 /* This says whether the command line arguments actually chose a
@@ -42,6 +46,8 @@ static char *default_ai_type = ",mplayer";
 
 static int game_chosen = FALSE;
 
+char *args_used;
+
 /* The list of asked-for players. */
 
 struct raw_spec {
@@ -51,6 +57,8 @@ struct raw_spec {
 
 static char *raw_default_player_spec;
 
+char *default_player_spec;
+
 /* The list of modules to loaded in addition to the main one. */
 
 struct module_spec {
@@ -58,6 +66,16 @@ struct module_spec {
   struct module_spec *next;
 } *extra_modules, *last_extra_module;
 
+/* The lists of GDL forms to interpret before and after loading. */
+
+struct gdl_spec {
+  char *spec;
+  struct gdl_spec *next;
+} *pre_gdl_specs, *last_pre_gdl_spec, *post_gdl_specs, *last_post_gdl_spec;
+
+static void add_a_gdl_spec PARAMS ((char *specstr, struct gdl_spec **first,
+				    struct gdl_spec **last));
+
 /* The list of accumulated variant choices. */
 
 static Obj *variant_settings;
@@ -96,7 +114,7 @@ int argc, spec;
 char *argv[];
 {
     char *arg, *aispec, tmpspec[100];
-    int i, n, numused;
+    int i, n, numused, total_arg_space;
 
 /* This macro just checks that a required next argument is actually there. */
 
@@ -121,9 +139,16 @@ char *argv[];
     if (spec == general_options)
       init_options();
 
+    total_arg_space = 0;
+    for (i = 0; i < argc; ++i)
+      if (!empty_string(argv[i]))
+	total_arg_space += strlen(argv[i]) + 2;
+    if (args_used == NULL)
+      args_used = xmalloc (total_arg_space);
+
     for (i = 1; i < argc; ++i) {
 	if (argv[i] == NULL || (argv[i])[0] == '\0') {
-	    /* Already munched, nothing to do. */
+	    /* Empty or already munched, nothing to do. */
 	} else if ((argv[i])[0] == '-') {
 	    arg = argv[i];
 	    Dprintf("%s\n", arg);
@@ -131,7 +156,7 @@ char *argv[];
 	    if (strcmp(arg, "-c") == 0) {
 		REQUIRE_ONE_ARG;
 		GENERAL_OPTION;
-		checkpointinterval = atoi(argv[i+1]);
+		checkpoint_interval = atoi(argv[i+1]);
 	    } else if (strcmp(arg, "-design") == 0) {
 		GENERAL_OPTION;
 #ifdef DESIGNERS
@@ -182,17 +207,19 @@ char *argv[];
 	    } else if (strcmp(arg, "-h") == 0) {
 		REQUIRE_ONE_ARG;
 		PLAYER_OPTION;
-		fprintf(stderr, "  %s not implemented yet, sorry\n", arg);
-		had_error = TRUE;
+		option_num_to_wait_for = atoi(argv[i+1]);
 	    } else if (strcmp(arg, "-help") == 0) {
 		GENERAL_OPTION;
 		help_wanted = TRUE;
 		/* Will display help info later. */
+	    } else if (strcmp(arg, "-host") == 0) {
+		REQUIRE_ONE_ARG;
+		PLAYER_OPTION;
+		option_game_to_host = copy_string(argv[i+1]);
 	    } else if (strcmp(arg, "-join") == 0) {
 		REQUIRE_ONE_ARG;
 		GENERAL_OPTION;
-		fprintf(stderr, "  %s not implemented yet, sorry\n", arg);
-		had_error = TRUE;
+		option_game_to_join = copy_string(argv[i+1]);
 	    } else if (strcmp(arg, "-L") == 0) {
 		REQUIRE_ONE_ARG;
 		GENERAL_OPTION;
@@ -204,10 +231,17 @@ char *argv[];
 		REQUIRE_ONE_ARG;
 		VARIANT_OPTION;
 		parse_world_option(argv[i+1]);
-	    } else if (strcmp(arg, "-mail") == 0) {
+	    } else if (strcmp(arg, "-noai") == 0) {
+		PLAYER_OPTION;
+		initially_no_ai = TRUE;
+	    } else if (strcmp(arg, "-pre") == 0) {
+		REQUIRE_ONE_ARG;
 		GENERAL_OPTION;
-		fprintf(stderr, "  %s not implemented yet, sorry\n", arg);
-		had_error = TRUE;
+		add_a_gdl_spec(argv[i+1], &pre_gdl_specs, &last_pre_gdl_spec);
+	    } else if (strcmp(arg, "-post") == 0) {
+		REQUIRE_ONE_ARG;
+		GENERAL_OPTION;
+		add_a_gdl_spec(argv[i+1], &post_gdl_specs, &last_post_gdl_spec);
 	    } else if (strcmp(arg, "-r") == 0) {
 		PLAYER_OPTION;
 		option_add_default_player = FALSE;
@@ -220,16 +254,16 @@ char *argv[];
 		fprintf(stderr,
 			"No debugging available, ignoring option `%s'\n", arg);
 #endif /* DEBUGGING */
-	    } else if (strcmp(arg, "-sm") == 0) {
-		VARIANT_OPTION;
-		push_key_int_binding(&variant_settings, K_SEQUENTIAL, 0);
-	    } else if (strcmp(arg, "-sq") == 0) {
+	    } else if (strcmp(arg, "-seq") == 0) {
 		VARIANT_OPTION;
 		push_key_int_binding(&variant_settings, K_SEQUENTIAL, 1);
+	    } else if (strcmp(arg, "-sim") == 0) {
+		VARIANT_OPTION;
+		push_key_int_binding(&variant_settings, K_SEQUENTIAL, 0);
 	    } else if (strncmp(arg, "-t", 2) == 0) {
 		REQUIRE_ONE_ARG;
 		VARIANT_OPTION;
-		parse_realtime_option(arg + 2, argv[i+1]);
+		parse_realtime_option(arg, argv[i+1]);
 	    } else if (strncmp(arg, "-v", 2) == 0) {
 		VARIANT_OPTION;
 		parse_variant(arg + 2);
@@ -239,11 +273,6 @@ char *argv[];
 	    } else if (strcmp(arg, "-w") == 0) {
 		GENERAL_OPTION;
 		warnings_suppressed = TRUE;
-	    } else if (strcmp(arg, "-wait") == 0) {
-		REQUIRE_ONE_ARG;
-		PLAYER_OPTION;
-		fprintf(stderr, "  %s not implemented yet, sorry\n", arg);
-		had_error = TRUE;
 	    } else if (strcmp(arg, "-x") == 0) {
 		GENERAL_OPTION;
 		option_popup_new_game_dialog = TRUE;
@@ -262,10 +291,16 @@ char *argv[];
 		    had_error = TRUE;
 		}
 	    }
-	    if (numused >= 1)
-	      argv[i] = "";
-	    if (numused >= 2)
-	      argv[i+1] = "";
+	    if (numused >= 1) {
+		strcat(args_used, " ");
+		strcat(args_used, argv[i]);
+		argv[i] = "";
+	    }
+	    if (numused >= 2) {
+		strcat(args_used, " ");
+		strcat(args_used, argv[i+1]);
+		argv[i+1] = "";
+	    }
 	    if (numused >= 1)
 	      i += (numused - 1);
 	} else {
@@ -275,6 +310,8 @@ char *argv[];
 		} else {
 		    add_a_raw_player_spec(argv[i]);
 		}
+		strcat(args_used, " ");
+		strcat(args_used, argv[i]);
 		argv[i] = NULL;
 	    }
 	}
@@ -283,7 +320,8 @@ char *argv[];
 	version_info();
     }
     if (had_error || help_wanted || variant_help_wanted) {
-	/* If we want to get help about a particular game, have to load it first. */
+	/* If we want to get help about a particular game, have to
+           load it first. */
 	if (help_wanted || variant_help_wanted)
 	  load_all_modules();
 	if (had_error || help_wanted)
@@ -309,6 +347,7 @@ add_a_module(name, filename)
 char *name, *filename;
 {
     Module *module;
+    struct module_spec *extra;
 
     module = get_game_module(name);
     if (module == NULL)
@@ -318,9 +357,9 @@ char *name, *filename;
     if (mainmodule == NULL) {
 	mainmodule = module;
     } else {
-	struct module_spec *extra = (struct module_spec *) xmalloc(sizeof(struct module_spec));
-
+	extra = (struct module_spec *) xmalloc(sizeof(struct module_spec));
 	extra->module = module;
+	/* Add the module at the end of the list. */
 	if (extra_modules == NULL)
 	  extra_modules = extra;
 	else
@@ -335,7 +374,7 @@ add_a_raw_player_spec(specstr)
 char *specstr;
 {
     struct raw_spec *spec =
-	(struct raw_spec *) xmalloc (sizeof(struct raw_spec));
+      (struct raw_spec *) xmalloc (sizeof(struct raw_spec));
 
     spec->spec = copy_string(specstr);
     if (raw_player_specs == NULL)
@@ -345,6 +384,22 @@ char *specstr;
     last_raw_player_spec = spec;
 }
 
+static void
+add_a_gdl_spec(specstr, firstp, lastp)
+char *specstr;
+struct gdl_spec **firstp, **lastp;
+{
+    struct gdl_spec *spec =
+      (struct gdl_spec *) xmalloc (sizeof(struct gdl_spec));
+
+    spec->spec = copy_string(specstr);
+    if (*firstp == NULL)
+      *firstp = spec;
+    else
+      (*lastp)->next = spec;
+    *lastp = spec;
+}
+
 static void
 version_info()
 {
@@ -362,14 +417,16 @@ general_usage_info()
     printf("    -f filename\t\trun <filename> as a game\n");
     printf("    -g gamename\t\tfind <gamename> in library and run it\n");
     printf("    -help\t\tdisplay this help info\n");
+    printf("    -host <game@host>\thost the given game\n");
     printf("    -join <game@host>\tconnect to the given game\n");
     printf("    -L pathname\t\tset <pathname> to be library location\n");
     printf("    -mail\t\tset up game as play-by-email\n");
-    printf("    -t mins\t\tlimit each player to <mins> of play time total\n");
-    printf("    -tside mins\t\tlimit each player to <mins> of time each turn\n");
-    printf("    -tturn mins\t\tlimit each turn to <mins> minutes\n");
     printf("    -w\t\t\tsuppress warnings\n");
-    printf("    -design\t\tmake every player a designer\n");
+    printf("    -design\t\tmake every player a designer");
+#ifndef DESIGNERS
+    printf(" (not available)");
+#endif
+    printf("\n");
     printf("Long options:\n");
     printf("    --help\t\tdisplay this help info\n");
     printf("    --version\t\tdisplay version info\n");
@@ -380,7 +437,7 @@ general_usage_info()
 static void
 game_usage_info()
 {
-    int i, wid, hgt, circumf, lat, lon;
+    int i, wid, hgt, circumf, lat, lon, pergame, perside, perturn;
     char *varname = "?", *vartypename = NULL;
     char buf[BUFSIZE];
     Variant *var;
@@ -408,12 +465,12 @@ game_usage_info()
 		    (c_number(eval(vardflt)) ? "true" : "false")));
 	    break;
 	  case K_SEQUENTIAL:
-	    printf("    -sm\t\t\tmove simultaneously (default %s)\n",
-		   (vardflt == lispnil ? "true" :
-		    (c_number(eval(vardflt)) ? "true" : "false")));
-	    printf("    -sq\t\t\tmove sequentially (default %s)\n",
+	    printf("    -seq\t\t\tmove sequentially (default %s)\n",
 		   (vardflt == lispnil ? "false" :
 		    (c_number(eval(vardflt)) ? "false" : "true")));
+	    printf("    -sim\t\t\tmove simultaneously (default %s)\n",
+		   (vardflt == lispnil ? "true" :
+		    (c_number(eval(vardflt)) ? "true" : "false")));
 	    break;
 	  case K_WORLD_SEEN:
 	    printf("    -v\t\t\tmake the world be seen already (default %s)\n",
@@ -421,11 +478,12 @@ game_usage_info()
 		    (c_number(eval(vardflt)) ? "true" : "false")));
 	    break;
 	  case K_WORLD_SIZE:
-	    printf("    -M wid[xhgt][Wcircumf][+lat][+lon]\tset world size (default ");
-	    /* Note that if the game definition sets these values directly using world
-	       or area forms, this is misleading; but that's the fault of the game
-	       designer for including both preset values and a variant whose defaults
-	       don't match those presets. */
+	    printf("    -M wid[xhgt][Wcircumf][+lat][+long]\tset world size (default ");
+	    /* Note that if the game definition sets these values
+	       directly using world or area forms, this is misleading;
+	       but that's the fault of the game designer for including
+	       both preset values and a variant whose defaults don't
+	       match those presets. */
 	    circumf = DEFAULTCIRCUMFERENCE;
 	    wid = DEFAULTWIDTH;  hgt = DEFAULTHEIGHT;
 	    lat = lon = 0;
@@ -457,6 +515,26 @@ game_usage_info()
 	      printf("+%d+%d", lat, lon);
 	    printf(")\n");
 	    break;
+	  case K_REAL_TIME:
+	    pergame = perside = perturn = 0;
+	    if (vardflt != lispnil) {
+		pergame = c_number(eval(car(vardflt)));
+		vardflt = cdr(vardflt);
+	    }
+	    if (vardflt != lispnil) {
+		perside = c_number(eval(car(vardflt)));
+		vardflt = cdr(vardflt);
+	    }
+	    if (vardflt != lispnil) {
+		perturn = c_number(eval(car(vardflt)));
+	    }
+	    printf("    -tgame mins\t\tlimit game time to <mins> minutes (default %d)\n",
+		   pergame);
+	    printf("    -tside mins\t\tlimit each player <mins> minutes in all (default %d)\n",
+		   perside);
+	    printf("    -tturn mins\t\tlimit each turn to <mins> minutes (default %d)\n",
+		   perturn);
+	    break;
 	  default:
 	    unixify_variant_name(buf, varname);
 	    printf("    -v%s[=value] (default ", buf);
@@ -466,7 +544,7 @@ game_usage_info()
 	    } else if (numberp(vardflt) && c_number(vardflt) == 1) {
 		printf("true");
 	    } else {
-		sprintlisp(buf, vardflt);
+		sprintlisp(buf, vardflt, BUFSIZE);
 		printf("%s", buf);
 	    }
 	    printf(")\n");
@@ -505,6 +583,7 @@ player_usage_info()
     printf("        advantage\t= numerical initial advantage (default 1)\n");
     printf("    -e number[,ai]\tadd <number> computer players\n");
     printf("    -h number[,ai]\tadd <number> human players\n");
+    printf("    -noai\tsuppress all AIs\n");
     printf("    -r\t\t\tno default player on local display\n");
     printf("    -wait minutes\t\twait time for players to join\n");
 }
@@ -546,16 +625,14 @@ static void
 parse_realtime_option(subopt, arg)
 char *subopt, *arg;
 {
-    if (strcmp(subopt, "-timeout") == 0) {
-	option_alltimeout = 60 * atoi(arg);
-    } else if (strcmp(subopt, "-tgame") == 0) {
+    if (strcmp(subopt, "-tgame") == 0) {
 	option_totalgametime = 60 * atoi(arg);
     } else if (strcmp(subopt, "-tside") == 0) {
 	option_persidetime = 60 * atoi(arg);
     } else if (strcmp(subopt, "-tturn") == 0) {
 	option_perturntime = 60 * atoi(arg);
     } else {
-    	/* usage? */
+	fprintf(stderr, "Realtime option not one of -tgame, -tside, -tturn, ignoring\n");
     }
 }
 
@@ -595,7 +672,13 @@ void
 load_all_modules()
 {
     struct module_spec *extra;
+    struct gdl_spec *gdlspec;
+    Obj *gdlform;
 
+    for (gdlspec = pre_gdl_specs; gdlspec != NULL; gdlspec = gdlspec->next) {
+	gdlform = read_form_from_string(gdlspec->spec, NULL, NULL, NULL);
+	interp_form(NULL, gdlform);
+    }
     if (mainmodule != NULL) {
 	load_game_module(mainmodule, TRUE);
 	for (extra = extra_modules; extra != NULL; extra = extra->next) {
@@ -606,6 +689,10 @@ load_all_modules()
 	   the one to go for.  It will direct the remainder of random gen. */
 	load_default_game();
     }
+    for (gdlspec = post_gdl_specs; gdlspec != NULL; gdlspec = gdlspec->next) {
+	gdlform = read_form_from_string(gdlspec->spec, NULL, NULL, NULL);
+	interp_form(NULL, gdlform);
+    }
 }
 
 /* Set module variants from command line options. */
@@ -613,6 +700,24 @@ load_all_modules()
 void
 set_variants_from_options()
 {
+    Obj *varval;
+
+    /* Only the host of a networked game can set variants. */
+    if (option_game_to_join != NULL) {
+	if (variant_settings != lispnil)
+	  fprintf(stderr, "Not the host, ignoring variant settings\n");
+	return;
+    }
+    /* Build and add a real-time variant if any times were supplied. */
+    if (option_totalgametime != 0
+	|| option_persidetime != 0
+	|| option_perturntime != 0) {
+	varval = cons(new_number(option_totalgametime),
+		      cons(new_number(option_persidetime),
+			   cons(new_number(option_perturntime), lispnil)));
+	/* Glue onto the list of variant_settings. */
+	push_key_cdr_binding(&variant_settings, K_REAL_TIME, varval);
+    }
     do_module_variants(mainmodule, variant_settings);
 }
 
@@ -625,6 +730,23 @@ set_players_from_options()
     Player *player;
     struct raw_spec *spec;
 
+    /* Only the host of a networked game can set up players. */
+    if (option_game_to_join != NULL) {
+	if (default_player_spec == NULL)
+	  default_player_spec = xmalloc(BUFSIZE);
+	default_player_spec[0] = '\0';
+	if (!empty_string(getenv("USER"))) {
+	    strcpy(default_player_spec, getenv("USER"));
+	    strcat(default_player_spec, "@");
+	}
+	if (!empty_string(raw_default_player_spec)
+	    && raw_default_player_spec[0] == '@') {
+	    strcat(default_player_spec, raw_default_player_spec);
+	} else if (!empty_string(getenv("DISPLAY"))) {
+	    strcat(default_player_spec, getenv("DISPLAY"));
+	}
+	return;
+    }
     player = NULL;
     /* Assume that if any players exist already, then this is a restored
        game of some sort, and merge instead of adding new players. */
@@ -654,10 +776,18 @@ set_players_from_options()
 	canonicalize_player(player);
 	player = player->next;
     }
+    /* If we made a special request to suppress AIs, go through all
+       players existing at this time. */
+    if (initially_no_ai) {
+	for_all_players(player) {
+	    player->aitypename = NULL;
+	}
+    }
 }
 
 /* This is not, strictly speaking, part of command line processing,
-   but command-line programs are also the ones for which stdio is useful. */
+   but command-line programs are also the ones for which stdio is
+   useful. */
 
 void
 print_instructions()
@@ -669,11 +799,11 @@ print_instructions()
 	if (stringp(instructions)) {
 	    printf("%s\n", c_string(instructions));
 	} else if (consp(instructions)) {
-	    for (rest = instructions; rest != lispnil; rest = cdr(rest)) {
+	    for_all_list(instructions, rest) {
 		if (stringp(car(rest))) {
 		    printf("%s\n", c_string(car(rest)));
 		} else {
-		    /* (should probably warn about this case too) */
+		    run_warning("Instructions are of wrong type");
 		}
 	    }
 	} else {
