@@ -20,6 +20,9 @@ extern int checkpointinterval;
 extern int allbedesigners;
 extern int option_popup_new_game_dialog;
 extern int warnings_suppressed;
+extern char *option_game_to_host;
+extern char *option_game_to_join;
+extern int option_num_to_wait_for;
 
 extern void init_options PARAMS ((void));
 extern void parse_command_line PARAMS ((int argc, char **argv, int spec));
