@@ -1,41 +1,43 @@
 /* The combat-related actions of Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
-/* Rules of combat: the attacker hits the defender ("other") unit and its
-   occupants, but the damage does not take effect right away.  If counter
-   attacks are possible in this period, the defender always does so, with
-   the same odds.  If the defender dies, then the attacker moves into the
-   cell.  If the attacker dies, nothing happens.  If both survive, then the
-   attacker may attempt to capture the defender. */
-
 #include "conq.h"
-extern int damaged_terrain_type PARAMS ((int t));
+#include "kernel.h"
+#include "kpublic.h"
 
 static int one_attack PARAMS ((Unit *atker, Unit *defender));
 static void fire_on_unit PARAMS ((Unit *atker, Unit *other));
 static void attack_unit PARAMS ((Unit *atker, Unit *other));
-static void maybe_hit_unit PARAMS ((Unit *atker, Unit *other, int fire, int fallsoff));
+static void maybe_hit_unit PARAMS ((Unit *atker, Unit *other, int fire,
+				    int fallsoff));
 static void hit_unit PARAMS ((Unit *unit, int hit, Unit *atker));
 
+static void reckon_damage PARAMS ((int fire));
 static void reckon_damage_here PARAMS ((int x, int y));
 static void report_damage PARAMS ((Unit *unit, Unit *atker, Unit *mainunit));
+static int will_report_damage PARAMS ((Unit *unit));
+static void rescue_one_occupant PARAMS ((Unit *occ));
 static int retreat_unit PARAMS ((Unit *unit, Unit *atker));
 static int retreat_in_dir PARAMS ((Unit *unit, int dir));
 static void attempt_to_capture_unit PARAMS ((Unit *atker, Unit *other));
 static void capture_unit_2 PARAMS ((Unit *unit, Unit *pris,
 				    Side *prevside));
-static void capture_occupant PARAMS ((Unit *unit, Unit *pris, Unit *occ, Side *newside));
+static void capture_occupant PARAMS ((Unit *unit, Unit *pris, Unit *occ,
+				      Side *newside));
 static void detonate_on_cell PARAMS ((int x, int y));
-static void notify_combat PARAMS ((Unit *unit, Unit *atker, char *str));
+static void hit_unit_with_detonation PARAMS ((Unit *unit, int hit,
+					      Unit *atker));
+
+/* We can't declare all the action functions as static because some of them
+   are in other files, but don't let them be visible to all files. */
 
 #undef  DEF_ACTION
-#define DEF_ACTION(name,code,args,prepfn,DOFN,checkfn,ARGDECL,doc)  \
+#define DEF_ACTION(name,code,args,prepfn,netprepfn,DOFN,checkfn,ARGDECL,doc)  \
   extern int DOFN PARAMS (ARGDECL);
 
 #include "action.def"
@@ -46,12 +48,11 @@ int max_t_detonate_effect_range = -1;
 
 int max_detonate_on_approach_range = -1;
 
-/* Remember what the main units involved are, so display is handled relative
-   to them and not to any occupants. */
+/* Remember what the main units involved are, so display is handled
+   relative to them and not to any occupants. */
 
 static Unit *amain, *omain;
 
-
 int numsoundplays;
 
 /* Attack action. */
@@ -63,9 +64,7 @@ prep_attack_action(unit, unit2, defender, n)
 Unit *unit, *unit2, *defender;
 int n;
 {
-    if (unit == NULL || unit->act == NULL)
-      return FALSE;
-    if (unit2 == NULL)
+    if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
     unit->act->nextaction.type = ACTION_ATTACK;
     unit->act->nextaction.args[0] = defender->id;
@@ -74,6 +73,18 @@ int n;
     return TRUE;
 }
 
+int
+net_prep_attack_action(unit, unit2, defender, n)
+Unit *unit, *unit2, *defender;
+int n;
+{
+    int rslt;
+
+    rslt = prep_attack_action(unit, unit2, defender, n);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
 int
 do_attack_action(unit, unit2, defender, n)
 Unit *unit, *unit2, *defender;
@@ -97,9 +108,11 @@ int n;
     }
     /* Defender might instead choose to surrender right off. */
     surrenderchance = uu_surrender_per_attack(u2, u3);
-    if (surrenderchance > 0) {
+    if (surrenderchance > 0
+	&& unit2->side
+	&& unit2->side->tech[u3] >= u_tech_to_own(u3)) {
 	if (probability(surrenderchance)) {
-	    capture_unit(unit2, defender);
+	    capture_unit(unit2, defender, H_UNIT_CAPTURED);
 	    if (alive(unit))
 	      use_up_acp(unit, uu_acp_to_attack(u2, u3));
 	    return A_ANY_DONE;
@@ -155,7 +168,7 @@ int n;
       return A_ANY_ERROR;
     /* We have to have a minimum level of supply to be able to attack. */
     for_all_material_types(m) {
-	if (unit2->supply[m] < um_to_fight(u2, m))
+	if (unit2->supply[m] < um_to_attack(u2, m))
 	  return A_ANY_NO_MATERIAL;
     }
     /* (should prorate ammo needs by intensity of attack) */
@@ -175,9 +188,7 @@ prep_overrun_action(unit, unit2, x, y, z, n)
 Unit *unit, *unit2;
 int x, y, z, n;
 {
-    if (unit == NULL || unit->act == NULL)
-      return FALSE;
-    if (unit2 == NULL)
+    if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
     unit->act->nextaction.type = ACTION_OVERRUN;
     unit->act->nextaction.args[0] = x;
@@ -188,6 +199,18 @@ int x, y, z, n;
     return TRUE;
 }
 
+int
+net_prep_overrun_action(unit, unit2, x, y, z, n)
+Unit *unit, *unit2;
+int x, y, z, n;
+{
+    int rslt;
+
+    rslt = prep_overrun_action(unit, unit2, x, y, z, n);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
 int
 do_overrun_action(unit, unit2, x, y, z, n)
 Unit *unit, *unit2;
@@ -290,7 +313,7 @@ int x, y, z, n;
       return A_MOVE_NO_MP;
     /* We have to have a minimum level of supply to be able to attack. */
     for_all_material_types(m) {
-	if (unit2->supply[m] < um_to_fight(u2, m))
+	if (unit2->supply[m] < um_to_attack(u2, m))
 	  return A_ANY_NO_MATERIAL;
     }
     for_all_stack(x, y, defender) {
@@ -309,19 +332,30 @@ static int
 one_attack(atker, defender)
 Unit *atker, *defender;
 {
+    int ua = atker->type, ud = defender->type;
     int ax = atker->x, ay = atker->y, ox = defender->x, oy = defender->y;
     int counter;
     Side *as = atker->side, *os = defender->side;
+    SideMask observers;
 
     amain = atker;  omain = defender;
     attack_unit(atker, defender);
-    /* Do a counterattack if appropriate. */
-    counter = uu_counterattack(atker->type, defender->type);
-    if (counter > 0) {
+    /* Do a counterattack if appropriate.  We must be able to counterattack,
+       and have enough acp to do so. */
+    counter = uu_counterattack(ua, ud);
+    if (counter > 0
+	&& has_enough_acp(defender, uu_acp_to_defend(ua, ud))) {
 	/* (should use value to set strength/commitment of counterattack) */
 	attack_unit(defender, atker);
     }
-    reckon_damage();
+    if (0 /* recording attacks */) {
+	observers = add_side_to_set(atker->side, NOSIDES);
+	observers = add_side_to_set(defender->side, observers);
+	/* (should let other watching sides see event also) */
+	record_event(H_UNIT_ASSAULTED, observers, atker->id, defender->id,
+		     ox, oy);
+    }
+    reckon_damage(FALSE);
     see_exact(as, ax, ay);
     see_exact(as, ox, oy);
     see_exact(os, ax, ay);
@@ -355,9 +389,7 @@ prep_fire_at_action(unit, unit2, defender, m)
 Unit *unit, *unit2, *defender;
 int m;
 {
-    if (unit == NULL || unit->act == NULL)
-      return FALSE;
-    if (unit2 == NULL)
+    if (unit == NULL || unit->act == NULL || unit2 == NULL || defender == NULL)
       return FALSE;
     unit->act->nextaction.type = ACTION_FIRE_AT;
     unit->act->nextaction.args[0] = defender->id;
@@ -366,6 +398,18 @@ int m;
     return TRUE;
 }
 
+int
+net_prep_fire_at_action(unit, unit2, defender, m)
+Unit *unit, *unit2, *defender;
+int m;
+{
+    int rslt;
+
+    rslt = prep_fire_at_action(unit, unit2, defender, m);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
 int
 do_fire_at_action(unit, unit2, defender, m)
 Unit *unit, *unit2, *defender;
@@ -380,7 +424,7 @@ int m;
     ox = defender->x;  oy = defender->y;  oz = defender->z;
     amain = unit;  omain = defender;
     fire_on_unit(unit, defender);
-    reckon_damage();
+    reckon_damage(TRUE);
     if (alive(unit))
       use_up_acp(unit, u_acp_to_fire(unit2->type));
     /*	if (alive(defender)) use_up_acp(defender, 1); */
@@ -442,7 +486,7 @@ int m;
       return A_ANY_ERROR;
     /* We have to have a minimum level of supply to be able to attack. */
     for_all_material_types(m2) {
-	if (unit2->supply[m2] < um_to_fight(u2, m2))
+	if (unit2->supply[m2] < um_to_fire(u2, m2))
 	  return A_ANY_NO_MATERIAL;
     }
     /* Check for enough of right kind of ammo. */
@@ -465,9 +509,7 @@ prep_fire_into_action(unit, unit2, x, y, z, m)
 Unit *unit, *unit2;
 int x, y, z, m;
 {
-    if (unit == NULL || unit->act == NULL)
-      return FALSE;
-    if (unit2 == NULL)
+    if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
     unit->act->nextaction.type = ACTION_FIRE_INTO;
     unit->act->nextaction.args[0] = x;
@@ -478,6 +520,18 @@ int x, y, z, m;
     return TRUE;
 }
 
+int
+net_prep_fire_into_action(unit, unit2, x, y, z, m)
+Unit *unit, *unit2;
+int x, y, z, m;
+{
+    int rslt;
+
+    rslt = prep_fire_into_action(unit, unit2, x, y, z, m);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
 /* One can always shoot, if the cell is visible, but there might not
    not be anything to hit!  No counterattacks when shooting, and the
    results might not be visible to the shooter. */
@@ -509,7 +563,7 @@ int x, y, z, m;
 	ox = other->x;  oy = other->y;  oz = other->z;
 	amain = unit;  omain = other;
 	fire_on_unit(unit2, other);
-	reckon_damage();
+	reckon_damage(TRUE);
 	/* Each side sees what happened to its unit that is being hit. */
 	update_unit_display(other->side, other, TRUE);
 	/* The attacking side also sees the remote cell. */
@@ -572,7 +626,7 @@ int x, y, z, m;
       return A_ANY_ERROR;
     /* We have to have a minimum level of supply to be able to attack. */
     for_all_material_types(m2) {
-	if (unit2->supply[m2] < um_to_fight(u2, m2))
+	if (unit2->supply[m2] < um_to_fire(u2, m2))
 	  return A_ANY_NO_MATERIAL;
     }
     /* Check for enough of right kind of ammo. */
@@ -585,35 +639,76 @@ int x, y, z, m;
     return A_ANY_OK;
 }
 
+static int tmpx0, tmpy0, tmpe0, tmpe1, tmpdist01, cellwid, fangle;
+
+static int
+elevation_blocks(x, y)
+int x, y;
+{
+    int interpol, celldist0, dist0, el;
+
+    celldist0 = distance(tmpx0, tmpy0, x, y);
+    dist0 = celldist0 * cellwid;
+    interpol = (tmpe1 - tmpe0) - (fangle * tmpdist01) / 100;
+    /* The following is * dist0^2 / tmpdist01^2, but arranged to avoid overflow. */
+    interpol *= dist0;
+    interpol /= tmpdist01;
+    interpol *= dist0;
+    interpol /= tmpdist01;
+    interpol += (fangle * dist0) / 100;
+    interpol += tmpe0;
+    el = (elevations_defined() ? elev_at(x, y) + t_thickness(terrain_at(x, y)) : 0);
+    Dprintf("Arc at %d: %d, elev %d\n", celldist0, interpol, el);
+    return (el > interpol);
+}
+
 int
 found_blocking_elevation(u, ux, uy, uz, u2, u2x, u2y, u2z)
 int u, ux, uy, uz, u2, u2x, u2y, u2z;
 {
-/*    int maxrise = u_elev_at_max_range(u); */
+    int t, weaponheight, bodyheight, rslt, x1, y1;
 
-    if (world_is_flat())
-      return FALSE;
+    /* Note that a flat world with no elevation defined could
+       still have thick terrain that screens. */
     /* Adjacent cells can't be screened by elevation. */
     /* (should accommodate possibility that target is at top of
        cliff in adj and back away from its edge, thus screened) */
     if (distance(ux, uy, u2x, u2y) <= 1)
       return FALSE;
-    /* (should add scan of path) */
-    return FALSE;
+    tmpx0 = ux;  tmpy0 = uy;
+    t = terrain_at(ux, uy);
+    weaponheight = ut_weapon_height(u, t);
+    tmpe0 = (elevations_defined() ? elev_at(ux, uy) + weaponheight : 0);
+    bodyheight = ut_body_height(u, t);
+    tmpe1 = (elevations_defined() ? elev_at(u2x, u2y) + bodyheight : 0);
+    cellwid = area.cellwidth;
+    if (cellwid <= 0)
+      cellwid = 1;
+    fangle = u_fire_angle_max(u);
+    tmpdist01 = distance(ux, uy, u2x, u2y) * cellwid;
+    Dprintf("Checking arc: %d,%d el %d -> %d,%d el %d\n", ux, uy, tmpe0, u2x, u2y, tmpe1);
+    rslt = search_straight_line(ux, uy, u2x, u2y, elevation_blocks, &x1, &y1);
+    if (rslt)
+      Dprintf("blocked at %d,%d\n", x1, y1);
+    return rslt;
 }
  
 static void
 fire_on_unit(atker, other)
 Unit *atker, *other;
 {
-    int m, dist = distance(atker->x, atker->y, other->x, other->y);
+    int m, dist = distance(atker->x, atker->y, other->x, other->y), consump;
 
     if (alive(atker) && alive(other)) {
 	if (enough_ammo(atker, other)) {
-	    maybe_hit_unit(atker, other, TRUE, (dist > u_hit_falloff_range(atker->type)));
+	    maybe_hit_unit(atker, other, TRUE,
+			   (dist > u_hit_falloff_range(atker->type)));
 	    for_all_material_types(m) {
 		if (um_hit_by(other->type, m) > 0) {
-		    atker->supply[m] -= um_consumption_per_attack(atker->type, m);
+		    consump = um_consumption_per_fire(atker->type, m);
+		    if (consump < 0)
+		      consump = um_consumption_per_attack(atker->type, m);
+		    atker->supply[m] -= consump;
 		}
 	    }
 	    /* The *victim* can lose acp. */
@@ -623,8 +718,8 @@ Unit *atker, *other;
     /* (should ping victim to see if it wants to respond to the attack?) */
 }
 
-/* Test to see if enough ammo is available to make the attack.
-   Need enough of *all* types - semi-bogus but too complicated otherwise? */
+/* Test to see if enough ammo is available to make the attack.  Need
+   enough of *all* types - semi-bogus but too complicated otherwise?  */
 
 int
 enough_ammo(unit, other)
@@ -640,8 +735,9 @@ Unit *unit, *other;
     return TRUE;
 }
 
-/* Single attack, no counterattack.  Check and use ammo - usage independent
-   of outcome, but types used depend on unit types involved. */
+/* Single attack, no counterattack.  Check and use ammo - usage
+   independent of outcome, but types used depend on unit types
+   involved. */
 
 static void
 attack_unit(atker, other)
@@ -654,7 +750,8 @@ Unit *atker, *other;
 	    maybe_hit_unit(atker, other, FALSE, FALSE);
 	    for_all_material_types(m) {
 		if (um_hit_by(other->type, m) > 0) {
-		    atker->supply[m] -= um_consumption_per_attack(atker->type, m);
+		    atker->supply[m] -=
+		      um_consumption_per_attack(atker->type, m);
 		}
 	    }
 	}
@@ -663,8 +760,8 @@ Unit *atker, *other;
 }
 
 /* Make a single hit and maybe hit some passengers also.  Power of hit
-   is constant, but chance is affected by terrain, quality,
-   and occupants' protective abilities.  If a hit is successful, it may
+   is constant, but chance is affected by terrain, quality, and
+   occupants' protective abilities.  If a hit is successful, it may
    have consequences on the defender's occupants, but limited by the
    protection that the transport provides. */
 
@@ -674,10 +771,10 @@ Unit *atker, *other;
 int fire, fallsoff;
 {
     int chance, t, hit = 0, a = atker->type, o = other->type;
-    int cxpeffect, cxpmax, effect, prot;
+    int cxpeffect, cxpmax, effect, prot, morchange;
     int dist, disthit, rangedelta, hitdelta, rangeamt;
     int dmgspec;
-    int hitmovietype;
+    char *hitmovietype;
     Unit *occ, *unit2;
     Side *side3;
     
@@ -758,16 +855,24 @@ int fire, fallsoff;
     	Dprintf(", missed");
     }
     /* (should record a raw statistic?) */
-    /* Ablation is a chance for occupants or stack to take part of a hit themselves. */
+    /* Ablation is a chance for occupants or stack to take part of a
+       hit themselves. */
     if (hit > 0) {
 	/* (should decide how ablation computed) */
     }
+    /* Affect the hitting unit's morale.  Note that morale is still reduced
+       even if the hit becomes 0 because of a successful retreat. */
+    if (hit > 0) {
+	change_morale(atker, 1, uu_morale_hit(a, o));
+    }
     if (hit > 0) {
 	chance = uu_retreat_chance(a, o);
-	/* (should adjust chance by morale etc) */
+	/* Chance of a retreat rises to 100% as morale goes to 0. */
+	if (u_morale_max(o) > 0 && chance < 100)
+	  chance = 100 - ((100 - chance) * other->morale) / u_morale_max(o);
 	if (probability(chance)) {
 	    if (retreat_unit(other, atker)) {
-		notify_combat(other, atker, "%s retreats!");
+		report_combat(atker, other, "retreat");
 		hit = 0; /* should only be reduced hit, may still be > 0 */
 	    }
 	}
@@ -777,8 +882,8 @@ int fire, fallsoff;
     	if (active_display(side3)
 	    && (all_see_all
     	    	|| (side3 == atker->side || side3 == other->side))) {
-	    hitmovietype = ((hit >= other->hp) ? movie_death :
-			    ((hit > 0) ? movie_hit : movie_miss));
+	    hitmovietype = ((hit >= other->hp) ? "death" :
+			    ((hit > 0) ? "hit" : "miss"));
 	    schedule_movie(side3, hitmovietype, other->id);
     	}
     }
@@ -806,7 +911,7 @@ hit_unit(unit, hit, atker)
 Unit *unit, *atker;
 int hit;
 {
-    int u = unit->type, hpmin;
+    int u = unit->type, u2, hpmin, dmgspec, tpdmg, tp;
 
     /* Some units might detonate automatically upon being hit. */
     if (hit > 0
@@ -817,16 +922,31 @@ int hit;
     	/* If the detonating unit still exists, then continue
     	   on to normal damage computation. */
     }
-    /* Record the loss of hp. */
-    unit->hp2 -= hit;
-    /* Attacker might not be able to do any more damage.  Note that the
-       positioning of this code is such that all the usual side effects
-       of combat happen, but the victim doesn't get any more worse off
-       than it is already. */
-    if (atker != NULL) {
-	hpmin = uu_hp_min(atker->type, u);
-	if (hpmin > 0 && hpmin > unit->hp2) {
-	    unit->hp2 = hpmin;
+    if (hit > 0) {
+	/* Record the loss of hp. */
+	unit->hp2 -= hit;
+	if (atker != NULL) {
+	    /* Attacker might not be able to do any more damage.  Note that the
+	       positioning of this code is such that all the usual side effects
+	       of combat happen, but the victim doesn't get any more worse off
+	       than it is already. */
+	    hpmin = uu_hp_min(atker->type, u);
+	    if (hpmin > 0 && hpmin > unit->hp2)
+	      unit->hp2 = hpmin;
+	    /* Affect the morale of the unit being hit. */
+	    change_morale(unit, -1, uu_morale_hit_by(atker->type, u));
+	}
+	/* Collateral damage may include loss of tooling. */
+	if (unit->tooling && 1 /* any_tp_damage */) {
+	    for_all_unit_types(u2) {
+		dmgspec = uu_tp_damage(u, u2);
+		tpdmg = roll_dice(dmgspec);
+		if (tpdmg != 0) {
+		    tp = unit->tooling[u2] - tpdmg;
+		    tp = max(0, min(tp, uu_tp_max(u, u2)));
+		    unit->tooling[u2] = tp;
+		}
+	    }
 	}
     }
     /* Maybe record for statistical analysis. */
@@ -853,8 +973,9 @@ int hit;
 
 /* (What if occupants change type when killed, but transport vanishes?) */
 
-void
-reckon_damage()
+static void
+reckon_damage(fire)
+int fire;
 {
     /* Entertain everybody. */
     play_movies(ALLSIDES);
@@ -865,13 +986,15 @@ reckon_damage()
        first and its death second. */
     if (!(omain->hp2 <= 0 && amain->hp2 > 0)) {
 	report_damage(omain, amain, omain);
-	report_damage(amain, omain, amain);
+	if (!fire && will_report_damage(amain))
+	  report_damage(amain, omain, amain);
     } else {
-	report_damage(amain, omain, amain);
+	if (!fire)
+	  report_damage(amain, omain, amain);
 	report_damage(omain, amain, omain);
     }
-    damage_unit(omain, combat_dmg);
-    damage_unit(amain, combat_dmg);
+    damage_unit(omain, combat_dmg, amain);
+    damage_unit(amain, combat_dmg, omain);
 }
 
 static void
@@ -881,7 +1004,7 @@ int x, y;
     Unit *unit;
 
     for_all_stack(x, y, unit) {
-	damage_unit(unit, combat_dmg);
+	damage_unit(unit, combat_dmg, NULL);
     }
 }
 
@@ -904,29 +1027,48 @@ Unit *unit, *atker, *mainunit;
 
     if (unit->hp2 <= 0) {
 	if (unit == mainunit) {
-	    notify_combat(atker, unit, "%s destroys %s!");
+	    report_combat(atker, unit, "destroy");
 	} else {
-	    notify_combat(unit, atker, "  (and destroys occupant %s!)");
+	    report_combat(atker, unit, "destroy-occupant");
 	}
     } else if (unit->hp2 < unit->hp) {
 	if (unit == mainunit) {
-	    notify_combat(atker, unit, "%s hits %s!");
+	    report_combat(atker, unit, "hit");
 	} else {
-	    notify_combat(unit, atker, "  (and hits occupant %s!)");
+	    report_combat(atker, unit, "hit-occupant");
 	}
     } else {
-	/* messages about misses not too useful */
+	if (unit == mainunit) {
+	    report_combat(atker, unit, "miss");
+	} else {
+	    report_combat(atker, unit, "miss-occupant");
+	}
     }
     for_all_occupants(unit, occ) {
 	report_damage(occ, atker, mainunit);
     }
 }
 
-/* Make the intended damage become real, and do any consequences. */
-
-void
-damage_unit(unit, dmgreason)
+static int
+will_report_damage(unit)
 Unit *unit;
+{
+    Unit *occ;
+
+    if (unit->hp2 < unit->hp)
+      return TRUE;
+    for_all_occupants(unit, occ) {
+	if (will_report_damage(occ))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+/* Make the intended damage become real, and do any consequences. */
+/* (should include agent unit with hevts if appropriate) */
+void
+damage_unit(unit, dmgreason, agent)
+Unit *unit, *agent;
 enum damage_reasons dmgreason;
 {
     int newacp;
@@ -936,7 +1078,7 @@ enum damage_reasons dmgreason;
 
     /* Process all the occupants first. */
     for_all_occupants(unit, occ) {
-	damage_unit(occ, dmgreason);
+	damage_unit(occ, dmgreason, agent);
     }
     /* If no damage was recorded, just return. */
     if (unit->hp2 == unit->hp)
@@ -944,15 +1086,18 @@ enum damage_reasons dmgreason;
     /* If unit is to die, do the consequences. */
     if (unit->hp2 <= 0) {
 	if (u_wrecked_type(unit->type) == NONUTYPE) {
-	    /* (should let occupants escape now?) */
+	    rescue_occupants(unit);
 	    hevttype = (dmgreason == combat_dmg ? H_UNIT_KILLED :
 			(dmgreason == accident_dmg ? H_UNIT_DIED_IN_ACCIDENT :
-			 0));
+			 /* (should define died-from-attrition type) */
+			 (dmgreason == attrition_dmg ? H_UNIT_DIED_IN_ACCIDENT :
+			  0)));
 	    kill_unit(unit, hevttype);
 	} else {
 	    hevttype = (dmgreason == combat_dmg ? H_UNIT_WRECKED :
 			(dmgreason == accident_dmg ? H_UNIT_WRECKED_IN_ACCIDENT :
-			 0));
+			 (dmgreason == attrition_dmg ? H_UNIT_WRECKED_IN_ACCIDENT :
+			  0)));
 	    change_unit_type(unit, u_wrecked_type(unit->type), hevttype);
 	    /* Restore to default hp for the new type. */
 	    unit->hp = unit->hp2 = u_hp(unit->type);
@@ -985,6 +1130,45 @@ enum damage_reasons dmgreason;
     update_unit_display(unit->side, unit, TRUE);
 }
 
+void
+rescue_occupants(unit)
+Unit *unit;
+{
+    Unit *occ, *other;
+
+    for_all_occupants(unit, occ) {
+	if (probability(uu_occ_escape(unit->type, occ->type))) {
+	    rescue_one_occupant(occ);
+	}
+    }
+}
+
+static void
+rescue_one_occupant(occ)
+Unit *occ;
+{
+    int rslt, dir, tmp, nx, ny;
+    Unit *other;
+
+    /* Try to escape into another unit in this cell. */
+    for_all_stack(occ->x, occ->y, other) {
+	if (other != occ->transport) {
+	    rslt = check_enter_action(occ, occ, other);
+	    if (valid(rslt)) {
+		do_enter_action(occ, occ, other);
+		return;
+	    }
+	}
+    }
+    /* Try to move into an adjacent cell. */
+    for_all_directions_randomly(dir, tmp) {
+	if (interior_point_in_dir(occ->x, occ->y, dir, &nx, &ny)) {
+	    if (retreat_in_dir(occ, dir))
+	      return;
+	}
+    }
+}
+
 /* Retreat is a special kind of movement that a unit uses to avoid
    damage during combat. It bypasses some of the normal move rules. */
 
@@ -995,6 +1179,9 @@ Unit *unit, *atker;
     int dir;
     extern int retreating_from;
 
+    /* First check that this type is able to retreat. */
+    if (unit->act == NULL || !mobile(unit->type))
+      return FALSE;
     retreating_from = atker->type;
     if (unit->x == atker->x && unit->y == atker->y) {
     	dir = random_dir();
@@ -1024,6 +1211,7 @@ Unit *unit;
 int dir;
 {
     int nx, ny, rslt;
+    Unit *other;
     extern int retreating;
     extern int retreating_from;
 
@@ -1032,12 +1220,23 @@ int dir;
       return FALSE;
     retreating = TRUE;
     rslt = check_move_action(unit, unit, nx, ny, unit->z);
-    if (!valid(rslt))
-      return FALSE;
-    do_move_action(unit, unit, nx, ny, unit->z);
-    retreating = FALSE;
-    retreating_from = NONUTYPE;
-    return TRUE;
+    if (valid(rslt)) {
+	do_move_action(unit, unit, nx, ny, unit->z);
+	retreating = FALSE;
+	retreating_from = NONUTYPE;
+	return TRUE;
+    }
+    /* No luck moving; see if there's a friendly unit to enter. */
+    for_all_stack(nx, ny, other) {
+	rslt = check_enter_action(unit, unit, other);
+	if (valid(rslt)) {
+	    do_enter_action(unit, unit, other);
+	    retreating = FALSE;
+	    retreating_from = NONUTYPE;
+	    return TRUE;
+	}
+    }
+    return FALSE;
 }
 
 /* Capture action. */
@@ -1048,9 +1247,7 @@ int
 prep_capture_action(unit, unit2, unit3)
 Unit *unit, *unit2, *unit3;
 {
-    if (unit == NULL || unit->act == NULL)
-      return FALSE;
-    if (unit2 == NULL)
+    if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
     unit->act->nextaction.type = ACTION_CAPTURE;
     unit->act->nextaction.args[0] = unit3->id;
@@ -1058,6 +1255,17 @@ Unit *unit, *unit2, *unit3;
     return TRUE;
 }
 
+int
+net_prep_capture_action(unit, unit2, unit3)
+Unit *unit, *unit2, *unit3;
+{
+    int rslt;
+
+    rslt = prep_capture_action(unit, unit2, unit3);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
 /* Execute a capture action. */
 
 int
@@ -1108,8 +1316,9 @@ Unit *unit, *unit2, *unit3;
     if (!has_enough_acp(unit, acp))
       return A_ANY_NO_ACP;
     /* We have to have a minimum level of supply to be able to capture. */
+    /* (attack material table not really appropriate) */
     for_all_material_types(m) {
-	if (unit2->supply[m] < um_to_fight(u2, m))
+	if (unit2->supply[m] < um_to_attack(u2, m))
 	  return A_ANY_NO_MATERIAL;
     }
     return A_ANY_OK;
@@ -1135,6 +1344,11 @@ Unit *atker, *other;
     Unit *occ;
     Side *as = atker->side, *os = other->side;
     
+    /* Low-tech sides are not going to succeed at even touching high-tech
+       types, let alone capturing them. */
+    if (atker->side
+	&& atker->side->tech[o] < u_tech_to_own(o))
+      return;
     chance = capture_chance(a, o, other->side);
     if (alive(atker) && alive(other) && chance > 0) {
 	if (impassable(atker, ox, oy) && !uu_bridge(o, a))
@@ -1160,16 +1374,16 @@ Unit *atker, *other;
 	}
 	/* Test whether the capture actually happens. */
 	if (probability(chance)) {
-	    capture_unit(atker, other);
+	    capture_unit(atker, other, H_UNIT_CAPTURED);
 	} else if (atker->transport != NULL && 
 		   (impassable(atker, ox, oy) ||
 		    impassable(atker, atker->x, atker->y))) {
 	    /* was the capture attempt a one-way trip? */
 	    /* (should fix the test above - needs to be more accurate) */
-	    notify_combat(other, atker, "%s resists capture; %s slaughtered!");
+	    report_combat(atker, other, "resist/slaughter");
 	    kill_unit(atker, H_UNIT_KILLED /* should be something appropriate */);
 	} else {
-	    notify_combat(other, atker, "%s throws back %s!");
+	    report_combat(atker, other, "resist");
 	    /* (should record failed attempt to capture?) */
 	}
 	if (chance > 0) {
@@ -1199,8 +1413,9 @@ Side *side2;
 /* There are many consequences of a unit being captured. */
 
 void
-capture_unit(unit, pris)
+capture_unit(unit, pris, captype)
 Unit *unit, *pris;
+int captype;
 {
     int u = unit->type, px = pris->x, py = pris->y;
     Unit *occ;
@@ -1219,17 +1434,18 @@ Unit *unit, *pris;
 	kill_unit(pris, H_UNIT_KILLED);
     }
     if (alive(pris)) {
-	if (newside == pris->origside)
-	  notify_combat(unit, pris, "%s liberates %s!");
-	else
-	  notify_combat(unit, pris, "%s captures %s!");
+	if (newside == pris->origside) {
+	    report_combat(unit, pris, "liberate");
+	} else {
+	    report_combat(unit, pris, "capture");
+	}
 	/* Decide the fate of each occupant of our prisoner. */
 	for_all_occupants(pris, occ) {
 	    capture_occupant(unit, pris, occ, newside);
 	}
 	/* The change of side itself.  This happens recursively to any
 	   remaining occupants as well. */
-	change_unit_side(pris, newside, H_UNIT_CAPTURED, unit);
+	change_unit_side(pris, newside, captype, unit);
 	/* Garrison the newly-captured unit with hp from the capturing unit. */
 	garrison_unit(unit, pris);
 	capture_unit_2(unit, pris, ps);
@@ -1239,6 +1455,11 @@ Unit *unit, *pris;
 	    && probability(people_surrender_chance(pris->type, px, py))) {
 	    change_people_side_around(px, py, pris->type, unit->side);
 	}
+	if (control_sides_defined()) {
+	    if (ut_control_range(pris->type, terrain_at(px, py)) >= 0) {
+		change_control_side_around(px, py, pris->type, unit->side);
+	    }
+	}
     }
     /* Update everybody's view of the situation. */
     see_exact(ps, px, py);
@@ -1259,15 +1480,21 @@ Side *newside;
 
     if (probability(uu_occ_escape(u, occ->type))) {
 	/* The occupant escapes, along with all its suboccupants. */
-	/* (should impl by moving to nearby cells?) */
-    } else if (0 /* (should allow for occ scuttling also) */) {
-    } else if (/* u_change_side(occ->type) || */ capture_chance(u, occ->type, occ->side) > 0) {
+	/* Retreat is not a perfect model, but close enough for now. */
+	retreat_unit(occ, unit);
+	report_combat(unit, occ, "escape");
+    } else if (probability(uu_scuttle(occ->type, u))) {
+	/* (should add terrain effect on success too) */
+	/* (should characterize as a scuttle) */
+	kill_unit(occ, H_UNIT_DISBANDED);
+    } else if (capture_chance(u, occ->type, occ->side) > 0) {
 	/* Side change will actually happen later. */
 	for_all_occupants(occ, subocc) {
 	    capture_occupant(unit, occ, subocc, newside);
 	}
     } else {
-	/* Occupant can't live as a prisoner, but suboccs might. */
+	/* Occupant can't live as a prisoner, but suboccs might, so recurse
+	   through them. */
 	for_all_occupants(occ, subocc) {
 	    capture_occupant(unit, occ, subocc, newside);
 	}
@@ -1277,17 +1504,25 @@ Side *newside;
     }
 }
 
+/* Do additional consequences of capture. */
+
 static void
 capture_unit_2(unit, pris, prevside)
 Unit *unit, *pris;
 Side *prevside;
 {
-    Unit *occ;
+    int chance, x, y, notesee;
+    Unit *occ, *unit2;
 
     /* Our new unit's experience might be higher or lower, depending on what
        capture really means (change of crew perhaps). */
     pris->cxp = (pris->cxp * u_cxp_on_capture(pris->type)) / 100;
     pris->cxp = min(unit->cxp, u_cxp_max(pris->type));
+    /* Getting captured is always bad for morale, but getting liberated is good. */
+    if (pris->side == pris->origside)
+      change_morale(pris, 1, ((pris->morale + 1) * 50) + 50);
+    else
+      pris->morale = pris->morale / 2;
     /* Clear any actions and plans. */
     /* Don't use init_unit_actorstate!  We just want to cancel any pending
        action, but leave all the acp values untouched. */
@@ -1295,24 +1530,56 @@ Side *prevside;
       pris->act->nextaction.type = ACTION_NONE;
     /* (should probably adjust side's acp sums by new unit's amounts) */
     init_unit_plan(pris);
-    /* Assign a number to this unit if it should have one. */
+    /* Assign a new number to this unit if it should have one. */
     assign_unit_number(pris);
+    /* Maybe get a pile of interesting information. */
+    if (!all_see_all
+	&& unit->side
+	&& !unit->side->see_all
+	&& prevside != NULL) {
+	notesee = FALSE;
+	chance = u_see_terrain_captured(pris->type);
+	if (!g_terrain_seen() && probability(chance)) {
+	    for_all_cells(x, y) {
+		if (terrain_view(prevside, x, y) != UNSEEN
+		    && terrain_view(unit->side, x, y) == UNSEEN) {
+		    set_terrain_view(unit->side, x, y, terrain_view(prevside, x, y));
+		    /* (should also view see-always units here) */
+		    update_cell_display(unit->side, x, y, FALSE);
+		    notesee = TRUE;
+		}
+	    }
+	}
+	if (1 /* any see others chance for this type */) {
+	    for_all_side_units(prevside, unit2) {
+		if (in_play(unit2)) {
+		    chance = uu_see_others_captured(pris->type, unit2->type);
+		    if (probability(chance)) {
+			see_exact(unit->side, unit2->x, unit2->y);
+			update_cell_display(unit->side, unit2->x, unit2->y, TRUE);
+			notesee = TRUE;
+		    }
+		}
+	    }
+	}
+	if (notesee) {
+	    notify(unit->side, "Enemy information fell into your hands!");
+	}
+    }
     /* Likewise for occupants. */
     for_all_occupants(pris, occ) {
 	capture_unit_2(unit, occ, prevside);
     }
 }
 
-/* A detonate action just blasts the vicinity. */
+/* A detonate action just blasts the vicinity indiscriminately. */
 
 int
 prep_detonate_action(unit, unit2, x, y, z)
 Unit *unit, *unit2;
 int x, y, z;
 {
-    if (unit == NULL || unit->act == NULL)
-      return FALSE;
-    if (unit2 == NULL)
+    if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
     unit->act->nextaction.type = ACTION_DETONATE;
     unit->act->nextaction.args[0] = x;
@@ -1322,6 +1589,18 @@ int x, y, z;
     return TRUE;
 }
 
+int
+net_prep_detonate_action(unit, unit2, x, y, z)
+Unit *unit, *unit2;
+int x, y, z;
+{
+    int rslt;
+
+    rslt = prep_detonate_action(unit, unit2, x, y, z);
+    broadcast_next_action(unit);
+    return rslt;
+}
+
 int
 do_detonate_action(unit, unit2, x, y, z)
 Unit *unit, *unit2;
@@ -1381,7 +1660,9 @@ Unit *unit;
 int x, y, z;
 {
     int u = unit->type, dir, x1, y1, dmg, t, maxrange;
+    char *hitmovietype;
     Unit *unit2;
+    Side *side;
 
     if (max_u_detonate_effect_range < 0) {
 	int u2, u3, range;
@@ -1405,14 +1686,21 @@ int x, y, z;
 	    }
 	}
     }
-    notify_combat(unit, unit, "%s detonates!");
+    report_combat(unit, NULL, "detonate");
+    for_all_sides(side) {
+    	if (active_display(side)
+	    && (all_see_all
+    	    	|| (side == unit->side || 1))) {
+	    schedule_movie(side, "flash", unit->x, unit->y);
+    	}
+    }
     set_was_detonated(unit, TRUE);
     /* Hit the detonating unit first. */
-    hit_unit(unit, u_hp_per_detonation(u), NULL);
+    hit_unit_with_detonation(unit, u_hp_per_detonation(u), NULL);
     /* Hit units at ground zero. */
     for_all_stack(x, y, unit2) {
     	if (unit2 != unit) {
-	    hit_unit(unit2, uu_detonation_damage_at(u, unit2->type), unit);
+	    hit_unit_with_detonation(unit2, uu_detonation_damage_at(u, unit2->type), unit);
 	}
     }
     damage_terrain(u, x, y);
@@ -1422,7 +1710,7 @@ int x, y, z;
 	    if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
 		for_all_stack(x1, y1, unit2) {
 		    dmg = uu_detonation_damage_adj(u, unit2->type);
-		    hit_unit(unit2, dmg, unit);
+		    hit_unit_with_detonation(unit2, dmg, unit);
 		}
 	    }
 	}
@@ -1443,6 +1731,8 @@ int x, y, z;
     }
     /* (should test compatibility of any new terrain types with each other;
         only after changes over with) */
+    /* Entertain everybody. */
+    play_movies(ALLSIDES);
     return TRUE;
 }
 
@@ -1461,7 +1751,7 @@ int x, y;
 		/* Reduce by inverse square of the distance. */
 		sdmg = (dmg * 100) / (dist * dist);
 		dmg = prob_fraction(sdmg);
-		hit_unit(unit2, dmg, tmpunit);
+		hit_unit_with_detonation(unit2, dmg, tmpunit);
 	    }
 	}
     }
@@ -1470,6 +1760,27 @@ int x, y;
     }
 }
 
+static void
+hit_unit_with_detonation(unit, hit, atker)
+Unit *unit, *atker;
+int hit;
+{
+    char *hitmovietype;
+    Side *side;
+
+    hit_unit(unit, hit, atker);
+    for_all_sides(side) {
+    	if (active_display(side)
+	    && (all_see_all
+	        || side == unit->side
+	        || 0 /* visible to other sides */)) {
+	    hitmovietype = ((hit >= unit->hp) ? "death" :
+			    ((hit > 0) ? "hit" : "miss"));
+	    schedule_movie(side, hitmovietype, unit->id);
+    	}
+    }
+}
+
 void
 damage_terrain(u, x, y)
 int u, x, y;
@@ -1579,24 +1890,6 @@ int t;
     return rslt;
 }
 
-static void
-notify_combat(unit1, unit2, str)
-Unit *unit1, *unit2;
-char *str;
-{
-    char buf1[BUFSIZE], buf2[BUFSIZE];
-    Side *side3;
-
-    for_all_sides(side3) {
-	if (active_display(side3)
-	    && (all_see_all || side3 == unit1->side || side3 == unit2->side)) {
-	    strcpy(buf1, unit_handle(side3, unit1));
-	    strcpy(buf2, unit_handle(side3, unit2));
-	    notify(side3, str, buf1, buf2);
-	}
-    }
-}
-
 /* Tests for whether given units or types can do particular types of actions. */
 
 int
