@@ -1,17 +1,18 @@
 /* Configuration-related definitions for Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
-/* This file has many things that can be tweaked for local preferences.
-   In general, there will not be any need to touch these. */
+/* This file has definitions that can be tweaked for local
+   preferences.  In general, there will not be any need to touch
+   these. */
    
-/* System dependencies can also go here, although it is much preferable
-   to write things so that they are not needed in the first place. */
+/* System dependencies can also go here, although it is much
+   preferable to write things so that they are not needed in the first
+   place. */
 
 /* Universally required include files. */
 
@@ -46,12 +47,30 @@ any later version.  See the file COPYING.  */
 #define STATSFILE "stats.xconq"
 #endif
 
+#ifndef SAVEFILE
+#define SAVEFILE "save.xconq"
+#endif
+
+#ifndef CHECKPOINTFILE
+#define CHECKPOINTFILE "check.xconq"
+#endif
+
+#ifndef ERRORFILE
+#define ERRORFILE "ack!.xconq"
+#endif
+
 /* The name of the default game. */
 
 #ifndef STANDARD_GAME
 #define STANDARD_GAME "standard"
 #endif
 
+/* The name of the introductory game. */
+
+#ifndef INTRO_GAME
+#define INTRO_GAME "intro"
+#endif
+
 /* Default random world size.  Adjust these to taste - for the standard
    game, 60x60 is a moderate length game, 30x30 is short, 360x120 is
    L-O-N-G !  (Remember to multiply the two numbers to get an idea of the
@@ -122,8 +141,8 @@ any later version.  See the file COPYING.  */
 /* When defined, save files will be removed after the game starts up
    successfully.  This keeps junk from lying around, but some folks
    prefer that they be kept just in case. */
-/* (should make this work or remove it) */
-#define RMSAVE
+
+/* #define RMSAVE */
 
 /* When defined, messages will be somewhat nastier.  Set this to be
    appropriate for your intended audience, heh-heh :-) */
@@ -143,7 +162,3 @@ any later version.  See the file COPYING.  */
 /* If defined, some (actually a lot) debugging code will be included. */
 
 #define DEBUGGING
-
-/* If defined, the compiler will be included. */
-
-#define COMPILER
