@@ -1,5 +1,5 @@
 /* Interface between game parameters and the rest of Xconq.
-   Copyright (C) 1992, 1993, 1994 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -43,22 +43,14 @@ enum terrain_subtype {
 #define TABLO -32768
 #define TABHI 32767
 
+#define TABINT 0
+#define TABDICE 1
+
 /* Ultimate limits on values of globals. */
 
 #define VARLO -2000000000
 #define VARHI 2000000000
 
-/* If a specialized Xconq is being compiled, use the specialized .h
-   instead of the general one.  All of the general game parameter
-   machinery should go away and be replaced by either constant
-   defaults or precalculated tables/formulas. */
-
-#ifdef SPECIAL
-
-#include "special.h"
-
-#else
-
 #include "lisp.h"
 
 /* This is the structure representing info about a property
@@ -73,7 +65,6 @@ typedef struct propertydefn {
     char *doc;
     short dflt;
     char *dfltstr;
-    void (*dlftfn) PARAMS ((void));
     short lo, hi;
 } PropertyDefn;
 
@@ -84,10 +75,10 @@ typedef struct tabledefn {
     int (*getter) PARAMS ((int, int));  /* accessor function */
     char *doc;			/* documentation string */
     short **table;		/* pointer to table itself */
-    short *cnst;		/* pointer to constant value, if table is constant */
     short dflt;			/* default value of entries */
     short lo, hi;		/* bounds of table value */
     char index1, index2;	/* type of row and column indices */
+    char valtype;		/* type of data in table */
 } TableDefn;
 
 /* This is the structure with info about a global variable. */
@@ -229,27 +220,27 @@ typedef struct a_globals {
    for constant and filled-in tables. */
 
 #undef  DEF_UU_TABLE
-#define DEF_UU_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi)  \
+#define DEF_UU_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
   int FNAME PARAMS ((int u1, int u2));  \
   extern short *TABLE, CNST;
 #undef  DEF_UM_TABLE
-#define DEF_UM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi)  \
+#define DEF_UM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
   int FNAME PARAMS ((int u, int m));  \
   extern short *TABLE, CNST;
 #undef  DEF_UT_TABLE
-#define DEF_UT_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi)  \
+#define DEF_UT_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
   int FNAME PARAMS ((int u, int t));  \
   extern short *TABLE, CNST;
 #undef  DEF_TM_TABLE
-#define DEF_TM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi)  \
+#define DEF_TM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
   int FNAME PARAMS ((int t, int m));  \
   extern short *TABLE, CNST;
 #undef  DEF_TT_TABLE
-#define DEF_TT_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi)  \
+#define DEF_TT_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
   int FNAME PARAMS ((int t1, int t2));  \
   extern short *TABLE, CNST;
 #undef  DEF_MM_TABLE
-#define DEF_MM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi)  \
+#define DEF_MM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
   int FNAME PARAMS ((int m1, int m2));  \
   extern short *TABLE, CNST;
 
@@ -275,11 +266,6 @@ extern TableDefn tabledefns[];
 
 extern VarDefn vardefns[];
 
-#endif /* SPECIAL */
-
-/* The following definitions are valid for both general and specialized
-   games. */
-
 /* Macros for iterating over types. */
 
 #define for_all_unit_types(v)      for (v = 0; v < numutypes; ++v)
