@@ -1,5 +1,5 @@
 /* Low-level support for variables and properties in Xconq GDL.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -49,23 +49,6 @@ int t;
     run_warning("Bad ttype %d", t);
 }
 
-#ifdef SPECIAL
-
-/* When specially compiled, all the types and globals are statically
-   initialized. */
-
-void
-init_types()
-{
-}
-
-void
-init_globals()
-{
-}
-
-#else /* SPECIAL */
-
 /* Declarations of the type definitions themselves. */
 
 Utype *utypes;
@@ -236,8 +219,6 @@ int t;
     }
 }
 
-#endif /* SPECIAL */
-
 char *
 index_type_name(x)
 int x;
