@@ -231,7 +231,8 @@ void
 check_game_validity()
 {
     int failed = FALSE, movers = FALSE, actors = FALSE;
-    int u1, u2, m1, t1, t2;
+    int u1, u2, m1, t1, t2, i, methkey, found;
+    Obj *synthlist = g_synth_methods(), *methods, *method;
 
     /* We must have at least one kind of unit. */
     if (numutypes < 1) {
@@ -251,14 +252,10 @@ check_game_validity()
 	init_warning("country separations %d to %d screwed up",
 		     g_separation_min(), g_separation_max());
     }
-
     /* Check all method names in the synthesis list. */
-    {
-      int i, methkey, found = FALSE;
-      Obj *synthlist = g_synth_methods(), *methods, *method;
-
-      for_all_list(synthlist, methods) {
+    for_all_list(synthlist, methods) {
 	method = car(methods);
+	found = FALSE;
 	if (symbolp(method)) {
 	    methkey = keyword_code(c_string(method));
 	    for (i = 0; synthmethods[i].key >= 0; ++i) {
@@ -267,20 +264,13 @@ check_game_validity()
 		    break;
 		}
 	    }
-	} else if (consp(method)) {
-	    if (stringp(car(method))) {
-		/* External program. */
-		run_error("No external synth programs yet");
-	    }
 	}
 	if (!found) {
 	    sprintlisp(spbuf, method, BUFSIZE);
-	    init_warning("bad synthesis method %s, ignoring", spbuf);
+	    init_warning("bad synthesis method `%s', ignoring", spbuf);
 	}
-      }
     }
     /* (Need more general game checks.) */
-
     max_zoc_range = -1;
     /* Check that all the unit names and chars are distinct. */
     for_all_unit_types(u1) {
