@@ -1,5 +1,5 @@
 /* Support for Lisp-style data.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -12,10 +12,22 @@ any later version.  See the file COPYING.  */
 #include "config.h"
 #include "misc.h"
 #include "lisp.h"
+extern char *escaped_symbol PARAMS ((char *str));
+extern char *escaped_string PARAMS ((char *str));
+extern void interp_short_array PARAMS ((short *arr, Obj *lis, int n));
+extern void interp_long_array PARAMS ((long *arr, Obj *lis, int n));
+extern int eval_number PARAMS ((Obj *val, int *isnumber));
 
 static Symentry *lookup_string PARAMS ((char *str));
 static int hash_name PARAMS ((char *str));
 
+static int strmgetc PARAMS ((Strm *strm));
+static void strmungetc PARAMS ((int ch, Strm *strm));
+static void sprintf_context PARAMS ((char *buf, int n, int *start, int *end, Strm *strm));
+static Obj *read_form_aux PARAMS ((Strm *strm));
+static Obj *read_list PARAMS ((Strm *strm));
+static int read_delimited_text PARAMS ((Strm *strm, char *delim, int spacedelimits, int eofdelimits));
+
 /* Pointer to "nil", the empty list. */
 
 Obj *lispnil;
@@ -56,7 +68,9 @@ static char *lispstrbuf = NULL;
 
 static int *startlineno;
 static int *endlineno;
-static char linenobuf[50];
+static char *linenobuf;
+
+static char *escapedthingbuf;
 
 /* Allocate a new Lisp object, count it as such. */
 
@@ -89,24 +103,36 @@ init_lisp()
     symboltablebase = (Symentry **) xmalloc(256 * sizeof(Symentry *));
     numsymbols = 0;
     init_predefined_symbols();
+    escapedthingbuf = xmalloc(BUFSIZE);
 }
 
 /* Ultra-simple "streams" that can be stdio FILEs or strings. */
 
-int
+static int
 strmgetc(strm)
 Strm *strm;
 {
+    int ch;
+
     if (strm->type == stringstrm) {
 	if (*(strm->ptr.sp) == '\0')
-	  return EOF;
-	return *((strm->ptr.sp)++);
+	  ch = EOF;
+	else
+	  ch = *((strm->ptr.sp)++);
     } else {
-	return getc(strm->ptr.fp);
+	ch = getc(strm->ptr.fp);
     }
+    if (ch != EOF) {
+	++(strm->numread);
+	strm->lastread[(strm->numread % (CONTEXTSIZE - 1))] = ch;
+	strm->lastread[((strm->numread + 1) % (CONTEXTSIZE - 1))] = '\0';
+	/* This is redundant unless we're at the end of the buffer. */
+	strm->lastread[(strm->numread % (CONTEXTSIZE - 1)) + 1] = '\0';
+    }
+    return ch;
 }
 
-void
+static void
 strmungetc(ch, strm)
 int ch;
 Strm *strm;
@@ -116,6 +142,7 @@ Strm *strm;
     } else {
 	ungetc(ch, strm->ptr.fp);
     }
+    --(strm->numread);
 }
 
 /* El cheapo Lisp reader.  Lisp objects are generally advertised by their
@@ -135,9 +162,12 @@ int *p1, *p2;
     endlineno = p2;
     tmpstrm.type = filestrm;
     tmpstrm.ptr.fp = fp;
+    tmpstrm.numread = 0;
     rslt = read_form_aux(&tmpstrm);
     if (rslt == lispclosingparen) {
-	sprintf_context(linenobuf, 50, startlineno, endlineno, NULL);
+	if (linenobuf == NULL)
+	  linenobuf = xmalloc(BUFSIZE);
+	sprintf_context(linenobuf, BUFSIZE, startlineno, endlineno, &tmpstrm);
 	init_warning("extra close paren, substituting nil%s", linenobuf);
 	rslt = lispnil;
     }
@@ -145,8 +175,8 @@ int *p1, *p2;
 }
 
 Obj *
-read_form_from_string(str, p1, p2)
-char *str;
+read_form_from_string(str, p1, p2, endstr)
+char *str, **endstr;
 int *p1, *p2;
 {
     Obj *rslt;
@@ -157,39 +187,58 @@ int *p1, *p2;
     endlineno = p2;
     tmpstrm.type = stringstrm;
     tmpstrm.ptr.sp = str;
+    tmpstrm.numread = 0;
     rslt = read_form_aux(&tmpstrm);
     if (rslt == lispclosingparen) {
-	sprintf_context(linenobuf, 50, startlineno, endlineno, NULL);
+	if (linenobuf == NULL)
+	  linenobuf = xmalloc(BUFSIZE);
+	sprintf_context(linenobuf, BUFSIZE, startlineno, endlineno, &tmpstrm);
 	init_warning("extra close paren, substituting nil%s", linenobuf);
 	rslt = lispnil;
     }
+    /* Record the next character to read from the string if possible. */
+    if (endstr != NULL)
+      *endstr = tmpstrm.ptr.sp;
     return rslt;
 }
 
-void
-sprintf_context(buf, n, start, end, context)
-char *buf, *context;
+static void
+sprintf_context(buf, n, start, end, strm)
+char *buf;
 int n, *start, *end;
+Strm *strm;
 {
-    buf[0] = '\0';
+    int printedlineno = FALSE;
+
+    strcpy(buf, "(");
     if (start != NULL && end != NULL) {
 	if (*start == *end) {
-	    sprintf(buf, " (at line %d)", *start);
+	    sprintf(buf + strlen(buf), "at line %d", *start);
 	} else {
-	    sprintf(buf, " (lines %d to %d)", *start, *end);
+	    sprintf(buf + strlen(buf), "lines %d to %d", *start, *end);
 	}
+	printedlineno = TRUE;
     }
-    if (!empty_string(context)) {
-	strncpy(buf+strlen(buf), context, n - 1);
+    if (!empty_string(strm->lastread)) {
+	if (printedlineno)
+	  strcat(buf, ", ");
+	strcat(buf, "context \"");
+	if (strm->numread > (CONTEXTSIZE - 1) && (strm->numread % (CONTEXTSIZE - 1)) > 0) {
+	    strncpy(buf + strlen(buf), strm->lastread + (strm->numread % (CONTEXTSIZE - 1)), n - strlen(buf) - 1);
+	}
+	strncpy(buf + strlen(buf), strm->lastread, n - strlen(buf) - 1);
 	buf[n - 1] = '\0';
+	strcat(buf, "\"");
     }
+    strcat(buf, ")");
 }
 
-Obj *
+static Obj *
 read_form_aux(strm)
 Strm *strm;
 {
     int minus, factor, commentclosed, ch, ch2, ch3, ch4, num;
+    int numdice, dice, indice;
 
     while ((ch = strmgetc(strm)) != EOF) {
 	/* Recognize nested comments specially. */
@@ -269,11 +318,13 @@ Strm *strm;
 	    return cons(intern_symbol("quote"),
 			cons(read_form_aux(strm), lispnil));
 	} else if (isdigit(ch) || ch == '-' || ch == '+' || ch == '.') {
-	    int numdice = 0, dice = 0, indice = FALSE;
-
+	    numdice = dice = 0;
+	    indice = FALSE;
 	    minus = (ch == '-');
 	    factor = (ch == '.' ? 100 : 1);
-	    num = (minus ? 0 : ch - '0');
+	    num = 0;
+	    if (isdigit(ch))
+	      num = ch - '0';
 	    while ((ch = strmgetc(strm)) != EOF) {
 	    	if (isdigit(ch)) {
 	    	    /* should ignore decimal digits past second one */
@@ -296,7 +347,7 @@ Strm *strm;
 	    }
 	    /* If number was followed by a % char, discard the char, otherwise
 	       put it back on the stream. */
-	    if (ch != '%')
+	    if (ch != '%' && ch != EOF)
 	      strmungetc(ch, strm);
 	    if (indice) {
 		dice = num;
@@ -325,6 +376,7 @@ Strm *strm;
 	    if (strcmp("nil", lispstrbuf) == 0) {
 		return lispnil;
 	    } else if (!actually_read_lisp) {
+		/* Recognize boundaries of non-reading specially. */
 	    	if (strcmp("else", lispstrbuf) == 0)
 		  return intern_symbol(lispstrbuf);
 	        if (strcmp("end-if", lispstrbuf) == 0)
@@ -343,7 +395,7 @@ Strm *strm;
    turn tail-recursion into loops, long lists can blow the stack.  (This has
    happened with real saved games.) */
 
-Obj *
+static Obj *
 read_list(strm)
 Strm *strm;
 {
@@ -371,7 +423,9 @@ Strm *strm;
 	return lis;
     }
   at_eof:
-    sprintf_context(linenobuf, 50, startlineno, endlineno, NULL);
+    if (linenobuf == NULL)
+      linenobuf = xmalloc(BUFSIZE);
+    sprintf_context(linenobuf, BUFSIZE, startlineno, endlineno, strm);
     init_warning("missing a close paren, returning EOF%s", linenobuf);
     return lispeof;
 }
@@ -379,7 +433,7 @@ Strm *strm;
 /* Read a quantity of text delimited by a char from the given string,
    possibly also by whitespace or EOF. */
 
-int
+static int
 read_delimited_text(strm, delim, spacedelimits, eofdelimits)
 Strm *strm;
 char *delim;
@@ -532,8 +586,8 @@ Obj *x, *subst;
 {
     char buf1[BUFSIZE], buf2[BUFSIZE];
 
-    sprintlisp(buf1, x);
-    sprintlisp(buf2, subst);
+    sprintlisp(buf1, x, BUFSIZE);
+    sprintlisp(buf2, subst, BUFSIZE);
     run_warning("%s of non-%s `%s' being taken, returning `%s' instead",
                 funname, typename, buf1, buf2);
 }
@@ -585,6 +639,13 @@ Obj *x;
     return car(cdr(cdr(x)));
 }
 
+Obj *
+cdddr(x)
+Obj *x;
+{
+    return cdr(cdr(cdr(x)));
+}
+
 void
 set_cdr(x, v)
 Obj *x, *v;
@@ -1015,32 +1076,43 @@ Obj *obj;
 }
 
 void
-sprintlisp(buf, obj)
+sprintlisp(buf, obj, maxlen)
 char *buf;
 Obj *obj;
+int maxlen;
 {
-    if (strlen(buf) + 20 > BUFSIZE)
-      return;
+    if (maxlen < 10) {
+	strcpy(buf, " ... ");
+	return;
+    }
     switch (obj->type) {
       case NIL:
-	sprintf(buf, "nil");
+	strcpy(buf, "nil");
 	break;
       case NUMBER:
 	sprintf(buf, "%d", obj->v.num);
 	break;
       case STRING:
+	if (maxlen < (strlen(obj->v.str) + 2)) {
+	    strcpy(buf, " ... ");
+	    return;
+	}
 	/* (should print escape chars if needed) */
 	sprintf(buf, "\"%s\"", obj->v.str);
 	break;
       case SYMBOL:
+	if (maxlen < strlen(c_string(obj))) {
+	    strcpy(buf, " ... ");
+	    return;
+	}
 	/* (should print escape chars if needed) */
 	sprintf(buf, "%s", c_string(obj));
 	break;
       case CONS:
 	strcpy(buf, "(");
-	sprintlisp(buf+strlen(buf), car(obj));
+	sprintlisp(buf + 1, car(obj), maxlen - 1);
 	/* No dotted pairs allowed in our version of Lisp. */
-	sprint_list(buf+strlen(buf), cdr(obj));
+	sprint_list(buf+strlen(buf), cdr(obj), maxlen - strlen(buf));
 	break;
       case UTYPE:
 	sprintf(buf, "u#%d", obj->v.num);
@@ -1052,7 +1124,7 @@ Obj *obj;
 	sprintf(buf, "t#%d", obj->v.num);
 	break;
       case POINTER:
-	sprintlisp(buf, obj->v.ptr.sym);
+	sprintlisp(buf, obj->v.ptr.sym, maxlen);
 	sprintf(buf+strlen(buf), " #|0x%lx|#", (long) obj->v.ptr.data);
 	break;
       default:
@@ -1062,22 +1134,72 @@ Obj *obj;
 }
 
 void
-sprint_list(buf, obj)
+sprint_list(buf, obj, maxlen)
 char *buf;
 Obj *obj;
+int maxlen;
 {
     Obj *tmp;
 
     buf[0] = '\0';
     for (tmp = obj; tmp != lispnil; tmp = cdr(tmp)) {
-	if (strlen(buf) + 10 > BUFSIZE)
-	  return;
+	if ((maxlen - strlen(buf)) < 10) {
+	    strcpy(buf, " ... ");
+	    break;
+	}
 	strcat(buf, " ");
-	sprintlisp(buf+strlen(buf), car(tmp));
+	sprintlisp(buf+strlen(buf), car(tmp), maxlen - strlen(buf));
     }	
     strcat(buf, ")");
 }
 
+/* These two routines make sure that any symbols and strings can
+   be read in again. */
+
+char *
+escaped_symbol(str)
+char *str;
+{
+    char *tmp = str;
+
+    if (str[0] == '|' && str[strlen(str)-1] == '|')
+      return str;
+    if (isdigit(str[0])) {
+	sprintf(escapedthingbuf, "|%s|", str);
+	return escapedthingbuf;
+    }
+    while (*tmp != '\0') {
+	if (((char *) strchr(" ()#\";|", *tmp)) != NULL) {
+	    sprintf(escapedthingbuf, "|%s|", str);
+	    return escapedthingbuf;
+	}
+	++tmp;
+    }
+    return str;
+}
+
+/* Note that this works correctly on NULL strings, turning them into
+   strings of length 0. */
+
+char *
+escaped_string(str)
+char *str;
+{
+    char *tmp = str, *rslt = escapedthingbuf;
+
+    *rslt++ = '"';
+    if (str != NULL) {
+	while (*tmp != 0) {
+	    if (*tmp == '"')
+	      *rslt++ = '\\';
+	    *rslt++ = *tmp++;
+	}
+    }
+    *rslt++ = '"';
+    *rslt = '\0';
+    return escapedthingbuf;
+}
+
 #ifdef DEBUGGING
 /* For calling from debuggers, at least that those that support output to stderr. */
 
@@ -1302,6 +1424,28 @@ int (*fn) PARAMS ((Obj *)), dflt;
     }
 }
 
+int
+eval_number(val, isnumber)
+Obj *val;
+int *isnumber;
+{
+    Obj *rslt;
+
+    /* (should have a non-complaining eval for this) */
+    if (numberp(val)) {
+	*isnumber = TRUE;
+	return c_number(val);
+    } else if (symbolp(val)
+	       && boundp(val)
+	       && numberp(symbol_value(val))) {
+	*isnumber = TRUE;
+	return c_number(symbol_value(val));
+    } else {
+	*isnumber = FALSE;
+	return 0;
+    }
+}
+
 int
 interpolate_in_list(val, lis, rslt)
 int val, *rslt;
@@ -1407,3 +1551,54 @@ Obj *lis;
     return (-1);
 }
 
+void
+interp_short_array(arr, lis, n)
+short *arr;
+Obj *lis;
+int n;
+{
+    int i = 0;
+    Obj *rest, *head;
+
+    /* Assume that if the destination array does not exist, there is
+       probably a reason, and it's not our concern. */
+    if (arr == NULL)
+      return;
+    for_all_list(lis, rest) {
+    	head = car(rest);
+    	if (numberp(head)) {
+	    if (i < n) {
+	    	arr[i++] = c_number(head);
+	    } else {
+		init_warning("too many numbers in list");
+		break;
+	    }
+	}
+    }
+}
+
+void
+interp_long_array(arr, lis, n)
+long *arr;
+Obj *lis;
+int n;
+{
+    int i = 0;
+    Obj *rest, *head;
+
+    /* Assume that if the destination array does not exist, there is
+       probably a reason, and it's not our concern. */
+    if (arr == NULL)
+      return;
+    for_all_list(lis, rest) {
+    	head = car(rest);
+    	if (numberp(head)) {
+	    if (i < n) {
+	    	arr[i++] = c_number(head);
+	    } else {
+		init_warning("too many numbers in list");
+		break;
+	    }
+	}
+    }
+}
