@@ -1,5 +1,5 @@
 /* Road generation for Xconq.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,17 +18,21 @@ static int road_at PARAMS ((int x, int y));
 static int find_adj_inside_area PARAMS ((int x, int y, int *xp, int *yp));
 static void lay_road_between PARAMS ((int x1, int y1, int x2, int y2));
 static int test_road_segment PARAMS ((int x, int y, int dir));
-static int sort_road_segments PARAMS ((int x, int y, int *dirchoices, int numchoices));
+static int sort_road_segments PARAMS ((int x, int y, int *dirchoices,
+				       int numchoices));
 static int compare_road_directions PARAMS ((const void *a0, const void *a1));
-static int lay_road_segment PARAMS ((int x, int y, int dir, int choice, int numchoices));
+static int lay_road_segment PARAMS ((int x, int y, int dir, int choice,
+				     int numchoices));
 
-static int roadtype;
+static short roadtype;
 
 static int *doroads;
 
-static int xs[NUMDIRS];
-static int ys[NUMDIRS];
-static int terrs[NUMDIRS];
+static short terrs[NUMDIRS];
+
+/* Note that these arrays have to be int-sized. */
+
+static int xs[NUMDIRS], ys[NUMDIRS];
 
 int
 make_roads(calls, runs)
@@ -137,7 +141,6 @@ static int
 unit_sees_other_unit(unit1, unit2)
 Unit *unit1, *unit2;
 {
-    int u2 = unit2->type;
     Side *side = unit1->side;
 
     /* Independent units are considered to know about all other units
@@ -150,13 +153,15 @@ Unit *unit1, *unit2;
     if (side->unitview != NULL
 	&& unit_view(side, unit2->x, unit2->y) != EMPTY)
       return TRUE;
+    /* If the unit view hasn't been set up yet, but we anticipate that the
+       unit in question will be visible. */
     if (side->terrview != NULL
-        && terrain_view(side, unit2->x, unit2->y) != UNSEEN
-        && (u_see_always(u2)
-            || (indep(unit2) ? u_already_seen_indep(u2) : u_already_seen(u2))))
+	&& terrain_view(side, unit2->x, unit2->y) != UNSEEN
+	&& (u_see_always(unit2->type) || already_seen_chance(side, unit2) >= 100))
       return TRUE;
+    /* The people will report all units. */
     if (people_sides_defined()
-        && people_side_at(unit2->x, unit2->y) == side->id)
+	&& people_side_at(unit2->x, unit2->y) == side->id)
       return TRUE;
     return FALSE;
 }
@@ -169,8 +174,8 @@ int x, y;
 }
 
 /* Find an interior point adjacent to this one (presumed but not
-   required to be on the edge).  This one just picks the first
-   found, which is OK for the road generation usage. */
+   required to be on the edge).  This one just picks the first found,
+   which is OK for the road generation usage. */
 
 static int
 find_adj_inside_area(x, y, xp, yp)
@@ -189,11 +194,15 @@ int x, y, *xp, *yp;
 
 /* Do the whole path. */
 
+static short ultx, ulty;
+
 static void
 lay_road_between(x1, y1, x2, y2)
 int x1, y1, x2, y2;
 {
-    apply_to_path(x1, y1, x2, y2, test_road_segment, sort_road_segments, lay_road_segment, FALSE);
+    ultx = x2;  ulty = y2;
+    apply_to_path(x1, y1, x2, y2, test_road_segment, sort_road_segments,
+		  lay_road_segment, FALSE);
 }
 
 static int
@@ -208,12 +217,15 @@ int x, y, dir;
     return FALSE;
 }
 
+static short basex, basey;
+
 static int
 sort_road_segments(x, y, dirchoices, numchoices)
 int x, y, *dirchoices, numchoices;
 {
     int i, tmp, i0 = 0, i1 = 1, compar;
 
+    basex = x;  basey = y;
     if (numchoices > 1) {
 	/* Cache direction and terrain data. */
 	tmpttype = terrain_at(x, y);
@@ -242,15 +254,16 @@ int x, y, *dirchoices, numchoices;
     return numchoices;
 }
 
-/* This compares the desirability of two different directions.  This is
-   somewhat tricky, because it should return < 0 if i0 designates a BETTER
-   direction than i1. */
+/* This compares the desirability of two different directions.  This
+   is somewhat tricky, because it should return < 0 if i0 designates a
+   BETTER direction than i1. */
 
 static int
 compare_road_directions(a0, a1)
 CONST void *a0, *a1;
 {
-    int i0, i1, t0, t1, tmp0, tmp1;
+    int i0, i1, t0, t1, tmp0, tmp1, chance0, chance1, sum0, sum1, dir, x1, y1;
+    int dist0, dist1, dist, chance;
     Unit *unit;
 
     i0 = *((int *) a0);  i1 = *((int *) a1);
@@ -277,7 +290,48 @@ CONST void *a0, *a1;
     if (tmp0 != tmp1)
       return (tmp1 - tmp0);
     /* Prefer the direction whose terrain is more favorable for a road. */
-    return (tt_road_into_chance(tmpttype, t1) - tt_road_into_chance(tmpttype, t0));
+    dist0 = distance(xs[i0], ys[i0], ultx, ulty);
+    chance0 = tt_road_into_chance(tmpttype, t0);
+    sum0 = chance0;
+    for_all_directions(dir) {
+	if (interior_point_in_dir(xs[i0], ys[i0], dir, &x1, &y1)) {
+	    dist = distance(x1, y1, ultx, ulty);
+	    if (dist <= dist0) {
+		sum0 += chance0;
+		chance = tt_road_into_chance(t0, terrain_at(x1, y1));
+		sum0 += (dist < dist0 ? 3 : 1) * chance;
+	    }
+	}
+    }
+    dist1 = distance(xs[i1], ys[i1], ultx, ulty);
+    chance1 = tt_road_into_chance(tmpttype, t1);
+    sum1 = chance1;
+    for_all_directions(dir) {
+	if (interior_point_in_dir(xs[i1], ys[i1], dir, &x1, &y1)) {
+	    dist = distance(x1, y1, ultx, ulty);
+	    if (dist <= dist1) {
+		sum1 += chance1;
+		chance = tt_road_into_chance(t1, terrain_at(x1, y1));
+		sum1 += (dist < dist1 ? 3 : 1) * chance;
+	    }
+	}
+    }
+#if 0 /* use for debugging lookahead */
+    if (Debug) {
+	int weighta, weightb, signa, signb;
+
+	weighta = chance1 - chance0;
+	signa = (weighta > 0 ? 1 : (weighta < 0 ? -1 : 0));
+	weightb = sum1 - sum0;
+	signb = (weightb > 0 ? 1 : (weightb < 0 ? -1 : 0));
+	if (signa != signb) {
+	    Dprintf("Adj terrain affects road choice at %d,%d;\n", basex, basey);
+	    Dprintf("  to %d,%d base weight is %d, adj terrain makes %d\n", xs[i0], ys[i0], chance0, sum0);
+	    Dprintf("  to %d,%d base weight is %d, adj terrain makes %d\n", xs[i1], ys[i1], chance1, sum1);
+	}
+    }
+#endif
+    return (sum1 - sum0);
 }
 
 /* Lay down a single bit of road, tending to choose directions with more
@@ -305,11 +359,13 @@ int x, y, d, choice, numchoices;
         if (roadchance > 0) {
             if (probability(roadchance) || foundroad || foundunit) {
 		set_connection_at(x, y, d, roadtype, TRUE);
-		/* Return whether to keep going or stop because of shared road. */
-		/* Note that stopping at a road-connected unit is not desirable,
-		   because we know (foundroad being false), that it does not already
-		   have a road running to it, and so we need to finish getting this
-		   one to where it is supposed to go. */
+		/* Return whether to keep going or stop because of
+                   shared road. */
+		/* Note that stopping at a road-connected unit is not
+		   desirable, because we know (foundroad being false),
+		   that it does not already have a road running to it,
+		   and so we need to finish getting this one to where
+		   it is supposed to go. */
 		return (foundroad ? -1 : 1);
 	    }
 	}
