@@ -1,6 +1,5 @@
 /* Terrain generation for Xconq.
-   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -9,13 +8,46 @@ any later version.  See the file COPYING.  */
 
 /* This is the collection of terrain generation methods. */
 
+#include "conq.h"
+#include "kernel.h"
+
+static void make_blobs PARAMS ((short *layer, int numblobs, int blobradius, int blobalt));
+static void limit_layer PARAMS ((short *layer, int hi, int lo));
+static void smooth_layer PARAMS ((short *layer, int times));
+static void percentile PARAMS ((short *layer, int *percentiles));
+static void compose_area PARAMS ((void));
+static int terrain_from_percentiles PARAMS ((int x, int y));
+static void make_earthlike_fractal_terrain PARAMS ((int calls, int runs));
+static void compose_earthlike_area PARAMS ((void));
+static int earthlike_terrain_from_percentiles PARAMS ((int x, int y));
+static void dig_maze_path PARAMS ((int x1, int y1, int dir1));
+static void dig_maze_path_between PARAMS ((int x1, int y1, int x2, int y2));
+static int test_maze_segment PARAMS ((int x, int y, int dir));
+static int sort_maze_segments PARAMS ((int x, int y, int *dirchoices,
+				       int numchoices));
+static int lay_maze_segment PARAMS ((int x, int y, int dir, int choice,
+				     int numchoices));
+static int num_open_adj PARAMS ((int x, int y));
+static int random_solid_terrain PARAMS ((void));
+static int random_room_terrain PARAMS ((void));
+static int random_passage_terrain PARAMS ((void));
+static int high_point PARAMS ((int x, int y));
+static int water_point PARAMS ((int x, int y));
+static int bay_point PARAMS ((int x, int y));
+static void set_room_interior PARAMS ((int x, int y));
+static void name_highest_peaks PARAMS ((Obj *parms));
+static void name_lakes PARAMS ((Obj *parms));
+static char *name_feature_at PARAMS ((int x, int y, char *typename));
+static char *name_feature_at_using PARAMS ((Obj *namerlist, int x, int y, char *typename));
+static void fix_adjacent_terrain PARAMS ((void));
+static void flatten_liquid_terrain PARAMS ((void));
+static void set_edge_values PARAMS ((int x, int y, int t));
+
 /* Fractal terrain generation. */
 
 /* The process is actually done for elevation and water separately, then
    the terrain type is derived from looking at both together. */
 
-#include "conq.h"
-
 /* This bounds the range of high and low spots. */
 
 #define MAXALT 4000
@@ -39,29 +71,6 @@ int partdone;
 
 static int numholes;
 
-static void make_blobs PARAMS ((short *layer, int numblobs, int blobradius, int blobalt));
-static void limit_layer PARAMS ((short *layer, int hi, int lo));
-static void smooth_layer PARAMS ((short *layer, int times));
-static void percentile PARAMS ((short *layer, int *percentiles));
-static void compose_area PARAMS ((void));
-static int terrain_from_percentiles PARAMS ((int x, int y));
-static int dig_maze_path PARAMS ((int x1, int y1, int dir1));
-static int num_open_adj PARAMS ((int x, int y));
-static int random_solid_terrain PARAMS ((void));
-static int random_room_terrain PARAMS ((void));
-static int random_passage_terrain PARAMS ((void));
-static int high_point PARAMS ((int x, int y));
-static int water_point PARAMS ((int x, int y));
-static int bay_point PARAMS ((int x, int y));
-static void set_room_interior PARAMS ((int x, int y));
-static void name_highest_peaks PARAMS ((Obj *parms));
-static void name_lakes PARAMS ((Obj *parms));
-static char *name_feature_at PARAMS ((int x, int y, char *typename));
-static char *name_feature_at_using PARAMS ((Obj *namerlist, int x, int y, char *typename));
-static void fix_adjacent_terrain PARAMS ((void));
-static void flatten_liquid_terrain PARAMS ((void));
-static void set_edge_values PARAMS ((int x, int y, int t));
-
 /* The main function goes through a heuristically-determined process */
 /* (read: I dinked until I liked the results) to make a area. */
 
@@ -270,9 +279,6 @@ int times;
    to do is to calculate the percentile for each raw elevation and moisture
    value, and save them all away.  */
 
-/* Percentile computation is inefficient, should be done incrementally
-   somehow instead of with * and / */
-
 static void
 percentile(layer, percentiles)
 short *layer;
@@ -290,7 +296,7 @@ int *percentiles;
     for_all_interior_cells(x, y) {
 	++histo[aref(layer, x, y)];
     }
-    /* Integrate over the histogram */
+    /* Integrate over the histogram. */
     for (i = 1; i < MAXALT; ++i)
 	histo[i] += histo[i-1];
     /* Total here should actually be same as number of cells in the area */
@@ -302,7 +308,7 @@ int *percentiles;
 }
 
 /* Final creation and output of the area. */
-/* (should reindent) */
+
 static void
 compose_area()
 {
@@ -321,23 +327,17 @@ compose_area()
     if (!world_is_flat()) {
     	/* Compute elevation variations of terrain.  This works on interior
     	   cells only; the edge gets handled later, by edge-specific code. */
-    	for_all_terrain_types(t) {
-    	    elevrange[t] = t_elev_max(t) - t_elev_min(t);
-	    rawlo[t] = rawhi[t] = -1;
-    	}
     	if (!elevations_defined()) {
 	    allocate_area_elevations();
    	}
+    	for_all_terrain_types(t) {
+	    rawlo[t] = MAXALT + 1;
+	    rawhi[t] = -1;
+    	}
 	for_all_interior_cells(x, y) {
 	    t = terrain_at(x, y);
-	    if (rawlo[t] < 0)
-	      rawlo[t] = tmp1_at(x, y);
-	    else
-	      rawlo[t] = min(tmp1_at(x, y), rawlo[t]);
-	    if (rawhi[t] < 0)
-	      rawhi[t] = tmp1_at(x, y);
-	    else
-	      rawhi[t] = max(tmp1_at(x, y), rawhi[t]);
+	    rawlo[t] = min(tmp1_at(x, y), rawlo[t]);
+	    rawhi[t] = max(tmp1_at(x, y), rawhi[t]);
 	}
 	/* If the final elevation ranges for several terrain types
 	   overlap, then they should all be considered as a group;
@@ -352,22 +352,25 @@ compose_area()
 	for_all_terrain_types(t) {
 	    elevlo[t] = t_elev_min(t);
 	    elevhi[t] = t_elev_max(t);
+    	    Dprintf("%10.10s: raw1 %6d - %6d, elev1 %6d - %6d\n", t_type_name(t), rawlo[t], rawhi[t], elevlo[t], elevhi[t]);
 	}
 	for_all_terrain_types(t) {
-	  for_all_terrain_types(t3) {
-	    for_all_terrain_types(t2) {
-		if (between(elevlo[t], elevlo[t2], elevhi[t])
-		    || between(elevlo[t], elevhi[t2], elevhi[t])) {
-		    elevlo[t] = min(elevlo[t], elevlo[t2]);
-		    elevhi[t] = max(elevhi[t], elevhi[t2]);
-		    rawlo[t] = min(rawlo[t], rawlo[t2]);
-		    rawhi[t] = max(rawhi[t], rawhi[t2]);
+	    for_all_terrain_types(t3) {
+		for_all_terrain_types(t2) {
+		    if (between(elevlo[t], elevlo[t2], elevhi[t])
+			|| between(elevlo[t], elevhi[t2], elevhi[t])) {
+			elevlo[t] = min(elevlo[t], elevlo[t2]);
+			elevhi[t] = max(elevhi[t], elevhi[t2]);
+			rawlo[t] = min(rawlo[t], rawlo[t2]);
+			rawhi[t] = max(rawhi[t], rawhi[t2]);
+		    }
 		}
 	    }
-	  }
 	}
     	for_all_terrain_types(t) {
     	    rawrange[t] = rawhi[t] - rawlo[t];
+    	    elevrange[t] = t_elev_max(t) - t_elev_min(t);
+    	    Dprintf("%10.10s: raw2 %6d - %6d, elev2 %6d - %6d\n", t_type_name(t), rawlo[t], rawhi[t], elevlo[t], elevhi[t]);
     	}
  	for_all_interior_cells(x, y) {
 	    t = terrain_at(x, y);
@@ -499,53 +502,276 @@ int calls, runs;
 
 /* (need to compute scale, get avg elevation and rainfall) */
 
-int seatype = -1;
-int landtype = -1;
+enum el_ttypes {
+    el_sea,
+    el_desert,
+    el_plains,
+    el_forest,
+    el_ice,
+    numeltypes
+};
+
+int seatype = NONTTYPE;
+int landtype = NONTTYPE;
+int deserttype = NONTTYPE;
+int foresttype = NONTTYPE;
+int icetype = NONTTYPE;
+
+int coast_distance = 0;
+
+int el_alt_blob_size = 400;
+int el_alt_blob_density = 10000;
+int el_alt_blob_height = 500;
+int el_alt_smoothing = 2;
+int el_wet_blob_size = 1000;
+int el_wet_blob_density = 200;
+int el_wet_blob_height = 100;
+int el_wet_smoothing = 10;
+
+int *el_alt_min;
+int *el_alt_max;
+int *el_wet_min;
+int *el_wet_max;
 
 int
 make_earthlike_terrain(calls, runs)
 int calls, runs;
 {
     int t, x, y, elevrange;
+    int seafrac, desertfrac, forestfrac;
 
     if (terrain_defined())
       return FALSE;
+    if (world.circumference < (area.maxdim * 100)) {
+	init_warning("Scale wrong for random earthlike terrain, use a real map");
+	return FALSE;
+    }
     Dprintf("Categorizing terrain types...\n");
     for_all_terrain_types(t) {
     	if (strcmp("sea", t_type_name(t)) == 0)
     	  seatype = t;
     	if (strcmp("plains", t_type_name(t)) == 0)
     	  landtype = t;
+    	if (strcmp("desert", t_type_name(t)) == 0)
+    	  deserttype = t;
+    	if (strcmp("forest", t_type_name(t)) == 0)
+    	  foresttype = t;
+    	if (strcmp("ice", t_type_name(t)) == 0)
+    	  icetype = t;
     	/* etc */
     }
-    if (seatype <= 0 || landtype <= 0) {
-    	Dprintf("can't find earthlike terrain types");
+    if (seatype == NONTTYPE
+	|| deserttype == NONTTYPE
+	|| landtype == NONTTYPE
+	|| foresttype == NONTTYPE
+	|| icetype == NONTTYPE) {
+    	init_warning("can't find earthlike terrain types");
     	return FALSE;
     }
     announce_lengthy_process("Making Earth-like terrain");
+    el_alt_min = (int *) xmalloc(numeltypes * sizeof(int));
+    el_alt_max = (int *) xmalloc(numeltypes * sizeof(int));
+    el_wet_min = (int *) xmalloc(numeltypes * sizeof(int));
+    el_wet_max = (int *) xmalloc(numeltypes * sizeof(int));
+    if (coast_distance <= - area.maxdim)
+      seafrac = 98;
+    else if (coast_distance >= area.maxdim)
+      seafrac = 2;
+    else
+      seafrac = 98 - ((coast_distance + (area.maxdim / 2)) * 100) / area.maxdim;
+    seafrac = min(seafrac, 98);
+    seafrac = max(seafrac, 2);
+    desertfrac = 5;
+    forestfrac = 95;
+    if (probability(30)) {
+	desertfrac = 80;
+	forestfrac = 98;
+    } else if (probability(30)) {
+	desertfrac = 2;
+	forestfrac = 60;
+    }
+    el_alt_min[el_sea] = 0;
+    el_alt_max[el_sea] = seafrac;
+    el_alt_min[el_desert] = seafrac;
+    el_alt_max[el_desert] = 100;
+    el_alt_min[el_plains] = seafrac;
+    el_alt_max[el_plains] = 100;
+    el_alt_min[el_forest] = seafrac;
+    el_alt_max[el_forest] = 100;
+    el_alt_min[el_ice] = 0;
+    el_alt_max[el_ice] = 100;
+    el_wet_min[el_sea] = 0;
+    el_wet_max[el_sea] = 100;
+    el_wet_min[el_desert] = 0;
+    el_wet_max[el_desert] = desertfrac;
+    el_wet_min[el_plains] = desertfrac;
+    el_wet_max[el_plains] = forestfrac;
+    el_wet_min[el_forest] = forestfrac;
+    el_wet_max[el_forest] = 100;
+    el_wet_min[el_ice] = 0;
+    el_wet_max[el_ice] = 100;
+    if (area.latitude > ((world.circumference / 4) * 80) / 100) {
+	el_alt_min[el_ice] = el_alt_min[el_desert] + 1;
+	el_alt_max[el_desert] = el_alt_min[el_desert] + 1;
+	el_alt_max[el_plains] = el_alt_min[el_plains] + 1;
+	el_alt_max[el_forest] = el_alt_min[el_forest] + 1;
+    }
     allocate_area_terrain();
     /* Overwrite already-defined elevs? */
     if (!elevations_defined() && !world_is_flat())
       allocate_area_elevations();
     elevrange = maxelev - minelev;
-    for_all_cells(x, y) {
-	if (inside_area(x, y)) {
-	    set_terrain_at(x, y, (flip_coin() ? seatype : landtype));
-	    if (elevations_defined() && !world_is_flat()) {
-	    	if (terrain_at(x, y) == seatype) {
-		    set_elev_at(x, y, 0);
-	    	} else {
-		    set_elev_at(x, y, xrandom(elevrange - 1) + 1);
-	    	}
-	    }
-	}
-    }
+    make_earthlike_fractal_terrain(0, 0);
     /* Make sure the border of the area has something in it. */
     add_edge_terrain();
     finish_lengthy_process();
     return TRUE;
 }
 
+static void
+make_earthlike_fractal_terrain(calls, runs)
+int calls, runs;
+{
+    int actualcells, altnumblobs, altblobradius, altblobalt;
+    int wetnumblobs, wetblobradius, wetblobalt;
+
+    /* Heuristic limit - this algorithm would get weird on small areas */
+    if (area.width < 9 || area.height < 9) {
+	init_warning("cannot generate fractal terrain for a %d x %d area, must be at least 9x9",
+		     area.width, area.height);
+	return;
+    }
+    Dprintf("Going to make earthlike fractal terrain ...\n");
+    allocate_area_scratch(3);
+    histo  = (int *) xmalloc(MAXALT * sizeof(int));
+    alts   = (int *) xmalloc(MAXALT * sizeof(int));
+    wets   = (int *) xmalloc(MAXALT * sizeof(int));
+    announce_lengthy_process("Making earthlike fractal terrain");
+    /* Need a rough estimate of how much work involved, so can do progress. */
+    if (el_alt_blob_density > 0) {
+	actualcells = (el_alt_blob_size * area.numcells) / 10000;
+	altblobradius = isqrt((actualcells * 4) / 3) / 2;
+	altnumblobs = (el_alt_blob_density * area.numcells) / 10000;
+	altblobalt = el_alt_blob_height;
+    }
+    if (el_wet_blob_density > 0) {
+	actualcells = (el_wet_blob_size * area.numcells) / 10000;
+	wetblobradius = isqrt((actualcells * 4) / 3) / 2;
+	wetnumblobs = (el_wet_blob_density * area.numcells) / 10000;
+	wetblobalt = el_wet_blob_height;
+    }
+    if (el_alt_blob_density > 0) {
+	/* Build a full relief area. */
+	partdone = 0;
+	make_blobs(area.tmp1, altnumblobs, altblobradius, altblobalt);
+	/* Run the requested number of smoothing steps. */
+	partdone += stepsize;
+	smooth_layer(area.tmp1, el_alt_smoothing);
+	percentile(area.tmp1, alts);
+    }
+    if (el_wet_blob_density > 0) {
+	/* Build a "moisture relief" area. */
+	partdone += stepsize;
+	make_blobs(area.tmp2, wetnumblobs, wetblobradius, wetblobalt);
+	partdone += stepsize;
+	smooth_layer(area.tmp2, el_wet_smoothing);
+	percentile(area.tmp2, wets);
+    }
+    /* Put it all together. */
+    partdone += stepsize;
+    compose_earthlike_area();
+    fix_adjacent_terrain();
+    add_edge_terrain();
+    flatten_liquid_terrain();
+    /* Free up what we don't need anymore. */
+    free(histo);
+    free(alts);
+    free(wets);
+    finish_lengthy_process();
+    /* Report on the substitutions made. */
+    if (numholes > 0) {
+	init_warning("no possible terrain for %d cells, made them into %s",
+		     numholes, t_type_name(0));
+    }
+    return;
+}
+
+/* Final creation and output of the area. */
+
+static void
+compose_earthlike_area()
+{
+    int x, y, t, t2, t3;
+    int elev, elevrange[MAXTTYPES], elevlo[MAXTTYPES], elevhi[MAXTTYPES];
+
+    Dprintf("Assigning terrain types to cells...\n");
+    /* Make the terrain layer itself. */
+    allocate_area_terrain();
+    numholes = 0;
+    for_all_interior_cells(x, y) {
+	t = earthlike_terrain_from_percentiles(x, y);
+	set_terrain_at(x, y, t);
+    }
+    if (!world_is_flat()) {
+    	/* Compute elevation variations of terrain.  This works on interior
+    	   cells only; the edge gets handled later, by edge-specific code. */
+    	for_all_terrain_types(t) {
+    	    elevrange[t] = t_elev_max(t) - t_elev_min(t);
+    	}
+    	if (!elevations_defined()) {
+	    allocate_area_elevations();
+   	}
+ 	for_all_interior_cells(x, y) {
+	    t = terrain_at(x, y);
+	    elev = 0;
+	    if (elevrange[t] > 0) {
+		elev = elevrange[t] / 2;
+	    }
+	    elev += t_elev_min(t);
+	    /* Clip elevation to required bounds. */
+	    if (elev < t_elev_min(t))
+	      elev = t_elev_min(t);
+	    if (elev > t_elev_max(t))
+	      elev = t_elev_max(t);
+	    set_elev_at(x, y, elev);
+	}
+    }
+}
+
+/* Compute the actual terrain types.  This is basically a process of
+   checking the percentile limits on each type against what is actually
+   there. */
+
+static int
+earthlike_terrain_from_percentiles(x, y)
+int x, y;
+{
+    int j, rawalt = tmp1_at(x, y), rawwet = tmp2_at(x, y);
+
+    if (numttypes == 1)
+      return 0;
+    for (j = 0; j < numeltypes; ++j) {
+	if (between(el_alt_min[j], alts[rawalt], el_alt_max[j])
+	    && between(el_wet_min[j], wets[rawwet], el_wet_max[j])) {
+	    switch (j) {
+	    case el_sea:
+	      return seatype;
+	    case el_desert:
+	      return deserttype;
+	    case el_forest:
+	      return foresttype;
+	    case el_plains:
+	      return landtype;
+	    case el_ice:
+	      return icetype;
+	    }
+	}
+    }
+    /* No terrain maybe not an error, so just count and summarize later. */
+    ++numholes;
+    return 0;
+}
+
 /* Maze terrain generation.  */
 
 int numsolidtypes = 0;
@@ -573,11 +799,9 @@ int
 make_maze_terrain(calls, runs)
 int calls, runs;
 {
-    int t, x, y, x1, y1, i;
-    int dir, n;
-    int numcells, tries = 0;
-    int roomcells, roomradius, numrooms;
-    int numsolidcells = 0, numpassagecellsneeded;
+    int t, x, y, x1, y1, i, n;
+    int numcells, tries, numpassagecellsneeded;
+    int roomcells, roomradius, numrooms, *roomx, *roomy;
 
     if (terrain_defined())
       return FALSE;
@@ -615,25 +839,46 @@ int calls, runs;
 	roomcells = 7;
 	roomradius = 1;
 	numrooms = ((numcells * g_maze_room()) / 10000) / roomcells;
+	roomx = (int *) xmalloc(numrooms * sizeof(int));
+	roomy = (int *) xmalloc(numrooms * sizeof(int));
 	for (i = 0; i < numrooms; ++i) {
 	    random_point(&x1, &y1);
 	    apply_to_area(x1, y1, roomradius, set_room_interior);
+	    /* Record room position so passage-making can use. */
+	    roomx[i] = x1;  roomy[i] = y1;
 	}
     }
-    for_all_interior_cells(x, y) {
-    	if (t_occurrence(terrain_at(x, y)) > 0) ++numsolidcells;
-    }
     if (g_maze_passage() > 0) {
-      numpassagecellsneeded = (numcells * g_maze_passage()) / 10000;
-      while (numpassagecells < numpassagecellsneeded && tries++ < 500) {
-    	random_point(&x1, &y1);
-    	if (t_occurrence(terrain_at(x1, y1)) > 0) {
-	    set_terrain_at(x1, y1, random_passage_terrain());
-	    dir = random_dir();
-	    dig_maze_path(x1, y1, dir);
-	    dig_maze_path(x1, y1, opposite_dir(dir));
-    	}
-      }
+	numpassagecellsneeded = (numcells * g_maze_passage()) / 10000;
+	/* Connect all the rooms together, sometimes directly, sometimes
+	   via a randomly-chosen spot. */
+	for (i = 0; i < numrooms - 1; ++i) {
+	    if (flip_coin()) {
+		dig_maze_path_between(roomx[i], roomy[i], roomx[i+1], roomy[i+1]);
+	    } else {
+		random_point(&x1, &y1);
+		dig_maze_path_between(roomx[i], roomy[i], x1, y1);
+		dig_maze_path_between(x1, y1, roomx[i+1], roomy[i+1]);
+	    }
+	}
+	/* If we need more passageway, start from random open cells and dig out
+	   into the surrounding rock. */
+	for (tries = 0; tries < numcells / 2; ++tries) {
+	    if (numpassagecells >= numpassagecellsneeded)
+	      break;
+	    random_point(&x1, &y1);
+	    if (t_maze_room_occurrence(terrain_at(x1, y1)) > 0
+		|| t_maze_passage_occurrence(terrain_at(x1, y1)) > 0) {
+		dig_maze_path(x1, y1, random_dir());
+	    }
+	}
+	/* Find areas of all-passage terrain and reduce them down a little. */
+	for_all_interior_cells(x, y) {
+	    if (t_maze_passage_occurrence(terrain_at(x, y)) > 0
+		&& num_open_adj(x, y) >= NUMDIRS - 1) {
+		set_terrain_at(x, y, random_solid_terrain());
+	    }
+	}
     }
     /* Make sure the border of the area is fixed up. */
     add_edge_terrain();
@@ -641,57 +886,131 @@ int calls, runs;
     return TRUE;
 }
 
-static int
+/* Given a starting position and direction, dig a passageway that
+   generally heads in that direction, but avoids other passages
+   and occasionally turns or branches. */
+
+static void
 dig_maze_path(x1, y1, dir1)
 int x1, y1, dir1;
 {
     int found;
-    int x, y, iter = 0, dir, dir2, nx, ny, nx2, ny2;
-    int dug = 0;
+    int x, y, iter = 0, dir, dir2, nx, ny;
     
-    while (!inside_area(x1+dirx[dir1], y1+diry[dir1])) {
-	dir1 = random_dir();
-    }
-    dir = dir1;
+    while (!interior_point_in_dir(x1, y1, dir1, &nx, &ny))
+      dir1 = random_dir();
     x = x1;  y = y1;
-    while (++iter < 500) {
-	point_in_dir(x, y, dir, &nx, &ny);
-	if (!inside_area(nx, ny))
+    dir = dir1;
+    for (iter = 0; iter < area.maxdim; ++iter) {
+	if (!interior_point_in_dir(x, y, dir, &nx, &ny))
 	  break;
+	found = FALSE;
 	if (t_occurrence(terrain_at(nx, ny)) > 0
 	    && num_open_adj(nx, ny) == 1) {
-	    set_terrain_at(nx, ny, random_passage_terrain());
-	    ++numpassagecells;
-	    ++dug;
+	    found = TRUE;
 	} else {
-	    found = FALSE;
+	    /* Try different directions. */
 	    for_all_directions(dir2) {
-		point_in_dir(x, y, dir2, &nx2, &ny2);
-		if (inside_area(nx2, ny2)
-		    && t_occurrence(terrain_at(nx2, ny2)) > 0
-		    && num_open_adj(nx2, ny2) == 1) {
-		    set_terrain_at(nx2, ny2, random_passage_terrain());
-		    ++numpassagecells;
-		    ++dug;
+		point_in_dir(x, y, dir2, &nx, &ny);
+		if (inside_area(nx, ny)
+		    && t_occurrence(terrain_at(nx, ny)) > 0
+		    && num_open_adj(nx, ny) == 1) {
 		    found = TRUE;
 		    dir = dir2;
 		    break;
 		}
 	    }
 	    if (!found) {
-		return dug;
+		/* Try again, allowing two open cells to be adjacent. */
+		for_all_directions(dir2) {
+		    point_in_dir(x, y, dir2, &nx, &ny);
+		    if (inside_area(nx, ny)
+			&& t_occurrence(terrain_at(nx, ny)) > 0
+			&& num_open_adj(nx, ny) == 2) {
+			found = TRUE;
+			dir = dir2;
+			break;
+		    }
+		}
 	    }
 	}
+	if (found) {
+	    set_terrain_at(nx, ny, random_passage_terrain());
+	    ++numpassagecells;
+	} else {
+	    return;
+	}
 	if (probability(20)) {
 	    dig_maze_path(nx, ny, left_dir(dir));
 	    dig_maze_path(nx, ny, right_dir(dir));
-	    return dug;
 	} else {
 	    x = nx;  y = ny;
 	    dir = (probability(50) ? dir : random_dir());
 	}
     }
-    return dug;
+}
+
+static void
+dig_maze_path_between(x1, y1, x2, y2)
+int x1, y1, x2, y2;
+{
+    apply_to_path(x1, y1, x2, y2, test_maze_segment, sort_maze_segments,
+		  lay_maze_segment, FALSE);
+}
+
+static int
+test_maze_segment(x, y, dir)
+int x, y, dir;
+{
+    int x1, y1;
+
+    return (interior_point_in_dir(x, y, dir, &x1, &y1));
+}
+
+static int
+sort_maze_segments(x, y, dirchoices, numchoices)
+int x, y, *dirchoices, numchoices;
+{
+    int nx, ny, w0, w1, tmp;
+
+    if (numchoices == 2) {
+    	point_in_dir(x, y, dirchoices[0], &nx, &ny);
+	if (t_maze_room_occurrence(terrain_at(nx, ny)) > 0)
+	  w0 = 2;
+	else if (t_maze_passage_occurrence(terrain_at(nx, ny)) > 0)
+	  w0 = 1;
+	else
+	  w0 = 0;
+    	point_in_dir(x, y, dirchoices[1], &nx, &ny);
+	if (t_maze_room_occurrence(terrain_at(nx, ny)) > 0)
+	  w1 = 2;
+	else if (t_maze_passage_occurrence(terrain_at(nx, ny)) > 0)
+	  w1 = 1;
+	else
+	  w1 = 0;
+	if (w1 > w0 || (w1 == w0 && flip_coin())) {
+	    tmp = dirchoices[0];
+	    dirchoices[0] = dirchoices[1];
+	    dirchoices[1] = tmp;
+	}
+    }
+    return numchoices;
+}
+
+static int
+lay_maze_segment(x, y, d, choice, numchoices)
+int x, y, d, choice, numchoices;
+{
+    int x1, y1;
+
+    if (interior_point_in_dir(x, y, d, &x1, &y1)) {
+	if (t_occurrence(terrain_at(x, y)) > 0) {
+	    set_terrain_at(x, y, random_passage_terrain());
+	    ++numpassagecells;
+	}
+	return 1;
+    }
+    return 0;
 }
 
 static int
@@ -703,7 +1022,8 @@ int x, y;
     for_all_directions(dir) {
 	point_in_dir(x, y, dir, &nx, &ny);
 	if (t_maze_room_occurrence(terrain_at(nx, ny)) > 0
-	    || t_maze_passage_occurrence(terrain_at(nx, ny)) > 0) ++rslt;
+	    || t_maze_passage_occurrence(terrain_at(nx, ny)) > 0)
+	  ++rslt;
     }
     return rslt;
 }
@@ -761,7 +1081,7 @@ int calls, runs;
 	if (consp(car(rest)) && stringp(car(car(rest)))) {
 	    classname = c_string(car(car(rest)));
 	    if (strcmp(classname, "peak") == 0) {
-	    	name_highest_peaks(cdr(car(rest)));
+		name_highest_peaks(cdr(car(rest)));
 	    } else if (strcmp(classname, "lake") == 0) {
 		name_lakes(cdr(car(rest)));
 	    } else {
@@ -781,16 +1101,18 @@ static void
 name_highest_peaks(parms)
 Obj *parms;
 {
-    int x, y;
-    int maxpeaks = (area.width * area.height) / 200;
-    int numpeaks, *peakx, *peaky, i, lo;
+    int x, y, maxpeaks, numpeaks, *peakx, *peaky, i, lo;
     char *name;
     Feature *mountain;
 
-    if (!elevations_defined() || world_is_flat()) {
-	run_warning("Can't identify peaks, world is flat");
-	return;
-    }
+    /* If no possibility of peaks, return quietly.  (A warning here seemed like
+       a good idea, but many games happen to be on flat worlds, and there
+       wasn't actually anything wrong with those.) */
+    if (!elevations_defined() || world_is_flat())
+      return;
+    /* This gives a peak for about every 20x10 group of cells, or about 10 for a
+       default-sized game, which seems right. */
+    maxpeaks = max(1, area.numcells / 200);
     peakx = (int *) xmalloc(maxpeaks * sizeof(int));
     peaky = (int *) xmalloc(maxpeaks * sizeof(int));
     numpeaks = 0;
@@ -972,8 +1294,7 @@ char *typename;
     return NULL;
 }
 
-/* Resolve any incompatibilities of terrain types in adjacent cells. */
-/* (should add code for borders and connections) */
+/* Resolve any incompatibilities of synthesized terrain in adjacent cells. */
 
 static void
 fix_adjacent_terrain()
@@ -985,7 +1306,14 @@ fix_adjacent_terrain()
     for_all_terrain_types(t1) {
 	affected[t1] = FALSE;
 	for_all_terrain_types(t2) {
-	    if (tt_adj_terr_effect(t1, t2) >= 0) {
+	    t3 = tt_adj_terr_effect(t1, t2);
+	    if (t3 >= 0) {
+		/* Table definitions have no provision for dynamically-determined
+		   bounds, so we have to check it here. */
+		if (!is_terrain_type(t3)) {
+		    run_warning("Bad value %d in adjacent-terrain-effect table, ignoring", t3);
+		    break;
+		}
 		anyadjeffects = TRUE;
 		affected[t1] = TRUE;
 		break;
@@ -994,6 +1322,7 @@ fix_adjacent_terrain()
     }
     if (!anyadjeffects)
       return;
+    announce_lengthy_process("Fixing adjacent terrain");
     maxpasses = area.height;
     while (maxpasses-- > 0) {
 	anychanges = FALSE;
@@ -1004,7 +1333,7 @@ fix_adjacent_terrain()
 		for_all_directions(dir) {
 		    if (point_in_dir(x, y, dir, &x1, &y1)) {
 			t3 = tt_adj_terr_effect(t, terrain_at(x1, y1));
-			if (t3 >= 0 && t3 != t) {
+			if (is_terrain_type(t3) && t3 != t) {
 			    set_tmp1_at(x, y, t3);
 			    anychanges = TRUE;
 			    break;
@@ -1023,6 +1352,7 @@ fix_adjacent_terrain()
 	    return;
 	}
     }
+    finish_lengthy_process();
 }
 
 /* For efficiency and semantics reasons, the methods might not assign values
