@@ -331,7 +331,7 @@ Side *side;
 int runs;
 {
     int tries, total;
-    Obj *possunits, *chosen, *lis, *rest, *head;
+    Obj *possunits, *origpossunits, *chosen, *lis, *rest, *head;
     Unit *unit;
 
     if (any_side_possible_units < 0) {
@@ -347,7 +347,7 @@ int runs;
 	if ((side->possible_units != lispnil && consp(side->possible_units))
 	    || any_side_possible_units) {
 	    /* Collect a list of possible units for this side. */
-	    possunits = side->possible_units;
+	    possunits = origpossunits = side->possible_units;
 	    if (any_side_possible_units) {
 		for_all_units(unit) {
 		    lis = get_x_property(unit, K_SIDES);
@@ -361,8 +361,9 @@ int runs;
 				&& side == find_side_by_name(c_string(head))) {
 				possunits = cons(new_number(unit->id), possunits);
 			    }
-			    /* Note that we ignore bogus-looking data in the
-			       list, it may be side info for some other game. */
+			    /* Note that we ignore bogus-looking data
+			       in the list, it may be side info for
+			       some other game. */
 			}
 		    } else {
 			init_warning("Unit %s sides prop invalid, ignoring",
@@ -383,7 +384,7 @@ int runs;
 		    Dprintf("For %s, chose ", side_desig(side));
 		    Dprintlisp(chosen);
 		    Dprintf(" from ");
-		    Dprintlisp(possunits);
+		    Dprintlisp(origpossunits);
 		    Dprintf(" possible units\n");
 		}
 		if (stringp(chosen)) {
