@@ -1,5 +1,5 @@
 /* Game modules for Xconq.
-   Copyright (C) 1991, 1992, 1993, 1994 Stanley T. Shebs.
+   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -113,87 +113,6 @@ Module *includer;
     return module;
 }
 
-/* Display game module info to a side. */
-
-void
-describe_game_modules(arg, key, buf)
-int arg;
-char *key, *buf;
-{
-    if (mainmodule != NULL) {
-	/* First put out basic module info. */
-	describe_game_module_aux(buf, mainmodule, 0);
-	/* Now do the lengthy module notes (with no indentation). */
-	describe_module_notes(buf, mainmodule);
-    } else {
-	sprintf(buf, "(No game module information is available.)");
-    }
-}
-
-/* Recurse down through included modules to display docs on each.
-   Indents each file by inclusion level.  Note that modules cannot
-   be loaded more than once, so each will be described only once here. */
-
-void   
-describe_game_module_aux(buf, module, level)
-char *buf;
-Module *module;
-int level;
-{
-    int i;
-    char indentbuf[100];
-    Module *submodule;
-
-    indentbuf[0] = '\0';
-    for (i = 0; i < level; ++i) {
-	strcat(indentbuf, "  ");
-    }
-    tprintf(buf, "%s\"%s\"", indentbuf,
-	    (module->title ? module->title : module->name));
-    if (module->title == NULL) {
-	tprintf(buf, " (\"%s\")", module->name);
-    }
-    if (module->version != NULL) {
-	tprintf(buf, " (version \"%s\")", module->version);
-    }
-    tprintf(buf, "\n");
-    tprintf(buf, "%s          %s\n",
-	    indentbuf,
-	    (module->blurb ? module->blurb : "(no description)"));
-    if (module->notes != lispnil) {
-	tprintf(buf, "%s          (See notes below)\n", indentbuf);
-    }
-    /* Now describe any included modules. */
-    for_all_includes(module, submodule) {
-	describe_game_module_aux(buf, submodule, level + 1);
-    }
-}
-
-/* Dump the module player's and designer's notes into the given buffer. */
-
-void
-describe_module_notes(buf, module)
-char *buf;
-Module *module;
-{
-    Module *submodule;
-
-    if (module->notes != lispnil) {
-	tprintf(buf, "\nNotes to \"%s\":\n", module->name);
-	append_notes(buf, module->notes);
-    }
-#ifdef DESIGNERS
-    /* Only show design notes if any designers around. */
-    if (numdesigners > 0 && module->designnotes != lispnil) {
-	tprintf(buf, "\nDesign Notes to \"%s\":\n", module->name);
-	append_notes(buf, module->designnotes);
-    }
-#endif /* DESIGNERS */
-    for_all_includes(module, submodule) {
-	describe_module_notes(buf, submodule);
-    }
-}
-
 /* Sometimes we find ourselves lacking a game to provide meaning and
    context for interpretation; this routine loads the standard game
    (or a specified alternative default) immediately, but only makes
@@ -428,11 +347,21 @@ Module *module;
     Obj *form;
 
     Dprintf("Trying to read a new format file \"%s\"...\n", module->name);
-    while ((form = read_form(module->fp,
+    if (module->fp != NULL) {
+	while ((form = read_form(module->fp,
 			     &(module->startlineno),
 			     &(module->endlineno)))
-	   != lispeof) {
-	interp_form(module, form);
+		!= lispeof) {
+	    interp_form(module, form);
+	}
+    } else {
+	while ((form = read_form_from_string(module->sp,
+			     &(module->startlineno),
+			     &(module->endlineno),
+			     &(module->sp)))
+		!= lispeof) {
+	    interp_form(module, form);
+	}
     }
     /* Clean up after any print forms that might have been done. */
     end_printing_forms();
@@ -446,12 +375,13 @@ do_module_variants(module, lis)
 Module *module;
 Obj *lis;
 {
-    int i, found;
+    int i, found, didsome;
     Obj *restset, *varset;
     Variant *var;
 
     if (module->variants == NULL)
       return; /* error? */
+    didsome = FALSE;
     for_all_list(lis, restset) {
 	varset = car(restset);
 	found = FALSE;
@@ -460,6 +390,7 @@ Obj *lis;
 	    if (equal(car(varset), var->id)) {
 		do_one_variant(module, var, cdr(varset));
 		found = TRUE;
+		didsome = TRUE;
 	    }
 	}
 	if (!found) {
@@ -469,8 +400,16 @@ Obj *lis;
     /* Now implement all the defaults. */
     for (i = 0; module->variants[i].id != lispnil; ++i) {
 	var = &(module->variants[i]);
-	if (!var->used)
-	  do_one_variant(module, var, lispnil);
+	if (!var->used) {
+	    do_one_variant(module, var, lispnil);
+	    didsome = TRUE;
+	}
+    }
+    if (didsome) {
+	/* Make all the cross-references right. */
+	patch_object_references();
+	/* Recheck everything, the variants might have broken something. */
+	check_game_validity();
     }
 }
 
@@ -489,7 +428,7 @@ Obj *varsetdata;
     if (Debug) {
 	if (readerrbuf == NULL)
 	  readerrbuf = (char *) xmalloc(BUFSIZE);
-    	sprintlisp(readerrbuf, varsetdata);
+    	sprintlisp(readerrbuf, varsetdata, BUFSIZE);
     	Dprintf("Module %s variant %s being set to `%s'\n",
 	    	module_desig(module), vartypename, readerrbuf);
     }
@@ -612,6 +551,8 @@ Obj *varsetdata;
 	    var->intvalue = val;
 	} else {
 	    val = FALSE;
+	    var->hasintvalue = TRUE;
+	    var->intvalue = val;
 	}
 	for_all_list(var->cases, restcases) {
 	    headcase = car(restcases);
