@@ -506,10 +506,7 @@ int u;
     } else if (side == side2) {
 	sprintf(buf, "your ");
     } else {
-	/* (this could be more elaborate) */
-	sprintf(buf, "%s ",
-		(side2->adjective ? side2->adjective :
-		 (side2->noun ? side2->noun : "?")));
+	sprintf(buf, "%s ", side_adjective(side2));
     }
     /* Glue the pieces together and return it. */
     strcat(buf, u_type_name(u));
@@ -523,7 +520,7 @@ unit_handle(side, unit)
 Side *side;
 Unit *unit;
 {
-    char *utypename, *sideadj, *fmtstr;
+    char *utypename, *fmtstr;
     Side *side2, *side3;
     Obj *frest, *fmt1;
 
@@ -539,8 +536,8 @@ Unit *unit;
     /* If this unit represents "yourself", say so. */
     if (side != NULL && unit == side->self_unit)
       return "you";
+    unitbuf[0] = '\0';
     if (u_desc_format(unit->type) != lispnil) {
-	unitbuf[0] = '\0';
 	for_all_list(u_desc_format(unit->type), frest) {
 	    fmt1 = car(frest);
 	    if (stringp(fmt1)) {
@@ -567,29 +564,29 @@ Unit *unit;
     side2 = unit->side;
     side3 = NULL;
     if (unit->origside != NULL && side2 != unit->origside) {
-	side2 = unit->origside;
-	side3 = unit->side;
+	side3 = unit->origside;
     }
     if (side2 == NULL) {
-	sprintf(unitbuf, "the ");
+	strcat(unitbuf, "the");
     } else if (side2 == side) {
-	sprintf(unitbuf, "your ");
+	strcat(unitbuf, "your");
     } else {
-	sideadj = side_adjective(side2);
-	sprintf(unitbuf, "the %s ",
-		(!empty_string(sideadj) ? sideadj : side_desig(side2)));
+	strcat(unitbuf, "the ");
+	strcat(unitbuf, side_adjective(side2));
+	if (side3 != NULL)
+	  strcat(unitbuf, "-held");
     }
     if (side3 != NULL) {
 	if (side3 == side) {
-	    tprintf(unitbuf, "(your) ");
+	    strcat(unitbuf, " (your)");
 	} else {
-	    sideadj = side_adjective(side3);
-	    tprintf(unitbuf, "(%s-held) ",
-		    (!empty_string(sideadj) ? sideadj : side_desig(side3)));
+	    strcat(unitbuf, " (");
+	    strcat(unitbuf, side_adjective(side3));
+	    strcat(unitbuf, ")");
 	}
     }
+    strcat(unitbuf, " ");
     /* Now add the unit's unique description. */
-    /* (Should this ever use short name?) */
     utypename = u_type_name(unit->type);
     if (unit->name) {
 	tprintf(unitbuf, "%s %s", utypename, unit->name);
@@ -618,6 +615,9 @@ Unit *unit;
     	sprintf(unitbuf, "dead #%d", unit->id);
         return unitbuf;
     }
+    /* If this unit represents "yourself", say so. */
+    if (unit->side != NULL && unit == unit->side->self_unit)
+      return "you";
     u = unit->type;
     if (!empty_string(unit->name)) {
 	sprintf(unitbuf, "%s", unit->name);
@@ -668,18 +668,16 @@ PastUnit *past_unit;
     } else if (side2 == side) {
 	sprintf(past_unitbuf, "your ");
     } else {
-	sprintf(past_unitbuf, "the %s ",
-		(side2->adjective ? side2->adjective :
-		 (side2->noun ? side2->noun : side_desig(side2))));
+	sprintf(past_unitbuf, "the %s ", side_adjective(side2));
     }
     /* Now add the past_unit's unique description. */
-    /* (Should this ever use short name?) */
     utypename = u_type_name(past_unit->type);
     if (past_unit->name) {
 	tprintf(past_unitbuf, "%s %s", utypename, past_unit->name);
     } else if (past_unit->number > 0) {
 	tprintf(past_unitbuf, "%d%s %s",
-		past_unit->number, ordinal_suffix(past_unit->number), utypename);
+		past_unit->number, ordinal_suffix(past_unit->number),
+		utypename);
     } else {
 	tprintf(past_unitbuf, "%s", utypename);
     }
@@ -703,28 +701,16 @@ PastUnit *past_unit;
 	sprintf(past_unitbuf, "%s", past_unit->name);
     } else if (!empty_string(u_short_name(u))) {
 	sprintf(past_unitbuf, "%d%s %s",
-		past_unit->number, ordinal_suffix(past_unit->number), u_short_name(u));
+		past_unit->number, ordinal_suffix(past_unit->number),
+		u_short_name(u));
     } else {
 	sprintf(past_unitbuf, "%d%s %s",
-		past_unit->number, ordinal_suffix(past_unit->number), u_type_name(u));
+		past_unit->number, ordinal_suffix(past_unit->number),
+		u_type_name(u));
     }
     return past_unitbuf;
 }
 
-void
-past_name_or_number(past_unit, buf)
-PastUnit *past_unit;
-char *buf;
-{
-    if (past_unit->name) {
-	sprintf(buf, "%s", past_unit->name);
-    } else if (past_unit->number > 0) {
-	sprintf(buf, "%d%s", past_unit->number, ordinal_suffix(past_unit->number));
-    } else {
-	buf[0] = '\0';
-    }
-}
-
 /* Given a unit and optional type u, summarize construction status
    and timing. */
 
@@ -804,7 +790,8 @@ Unit *unit;
 	estbuf[0] = '\0';
     }
     if (u_tech_max(u) > 0) {
-    	sprintf(techbuf, "[T %d/%d/%d] ", side->tech[u], u_tech_to_build(u), u_tech_max(u));
+    	sprintf(techbuf, "[Tech %d/%d/%d] ",
+		side->tech[u], u_tech_to_build(u), u_tech_max(u));
     } else {
 	techbuf[0] = '\0';
     }
@@ -840,7 +827,9 @@ int u2;
     tooluptime = 0;
     tp = (unit->tooling ? unit->tooling[u2] : 0);
     if (tp < uu_tp_to_build(u, u2)) {
-	if (uu_acp_to_toolup(u, u2) < 1 || uu_tp_per_toolup(u, u2) <= 0 || u_acp(u) <= 0)
+	if (uu_acp_to_toolup(u, u2) < 1
+	    || uu_tp_per_toolup(u, u2) <= 0
+	    || u_acp(u) <= 0)
 	  return (-1);
 	tooluptime = ((uu_tp_to_build(u, u2) - tp) * uu_acp_to_toolup(u, u2))
 	 / (uu_tp_per_toolup(u, u2) * u_acp(u));
@@ -2104,7 +2093,10 @@ char *spec;
 
     if (side == NULL)
       return;
-    rest = get_next_arg(spec, substr, &arg);
+    if (!empty_string(spec))
+      rest = get_next_arg(spec, substr, &arg);
+    else
+      arg = "";
     if ((doctrine = find_doctrine_by_name(arg)) != NULL) {
 	/* Found a specific named doctrine. */
 	/* (should say which unit types use it) */
@@ -2114,7 +2106,8 @@ char *spec;
 	doctrine = side->default_doctrine;
 	/* (should say which unit types use it) */
     } else {
-	notify(side, "No doctrine name or unit type recognized.");
+	if (!empty_string(arg))
+	  notify(side, "\"%s\" not recognized as doctrine name or unit type.");
 	/* Note that although we list all doctrines, including those belonging
 	   to other sides, we don't know if the other side is actually *using*
 	   any particular doctrine. */
