@@ -2070,7 +2070,8 @@ Plan *plan;
     free(plan);
 }
 
-/* Describe a plan succinctly. */
+/* Describe a plan succinctly.  This is primarily for debugging, not
+   for normal user display. */
 
 char *planbuf = NULL;
 
@@ -2158,6 +2159,8 @@ int passive_only;
 {
     extern int need_ai_planning;
 
+    if (unit->plan == NULL)
+      return;
     if ((passive_only ? unit->plan->type == PLAN_PASSIVE : TRUE)) {
 	unit->plan->type = PLAN_PASSIVE;
 	clear_task_agenda(unit->plan);
