@@ -1,5 +1,5 @@
 /* Definitions for printing in Xconq.
-   Copyright (C) 1994 Stanley T. Shebs and Massimo Campostrini.
+   Copyright (C) 1994, 1996 Stanley T. Shebs and Massimo Campostrini.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -36,12 +36,5 @@ typedef struct a_print_parameters {
 
 /* Prototypes. */
 
-extern void init_ascii_print PARAMS ((PrintParameters *pp));
-extern void dump_ascii_view PARAMS ((Side *side, PrintParameters *pp, char *filename));
-
 extern void init_ps_print PARAMS ((PrintParameters *pp));
 extern void dump_ps_view PARAMS ((Side *side, PrintParameters *pp, char *filename));
-
-extern int ps_load_bitmap PARAMS ((Side *side, int u));
-extern int store_bitmap PARAMS ((int u));
-extern int make_ps_images PARAMS ((void));
