@@ -881,8 +881,8 @@ int x, y;
 
     if (area.xwrap)
       return x;
-    xlo = max(0, area.height / 2 - y);
-    xhi = min(area.width-1, area.width + area.height / 2 - 1 - y);
+    xlo = max(0, area.halfheight - y);
+    xhi = min(area.width-1, area.width + area.halfheight - 1 - y);
     x = max(x, xlo);
     x = min(x, xhi);
     return x;
@@ -1113,7 +1113,7 @@ char *filename;
 			    for_all_directions(d) {
 				if (border_at(x, y, d, t) &&
 				    (!side
-				     || g_see_all()
+				     || all_see_all
 				     || seen_border(side, x, y, d))) {
 				    fprintf(fp, "%d %g %d %g li\n",
 					    cx + hexc_x[d],
@@ -1148,7 +1148,7 @@ char *filename;
 			if (t_is_connection(t) && aux_terrain_defined(t)) {
 			    for_all_directions(d) {
 				if (connection_at(x, y, d, t) &&
-				    (!side || g_see_all() ||
+				    (!side || all_see_all ||
 				     terrain_view(side, x, y) != UNSEEN)) {
 				    fprintf(fp, "%d %g %d %g li\n",
 					    cx,
