@@ -2777,7 +2777,7 @@ run_unit_fates()
 	    }
 	    if (in_play(unit)) {
 		origside = unit->side;
-		if (any_lost_surrenders[u]) {
+		if (any_lost_surrender && any_lost_surrenders[u]) {
 		    lost_unit_surrender(unit);
 		}
 		if (any_lost_revolt && unit->side == origside) {
