@@ -734,11 +734,11 @@ int why;
 	    s2 = side_number(side2);
 	    for_all_cells(x, y) {
 		changed = FALSE;
-		if (people_side_at(x, y) == s) {
+		if (people_sides_defined() && people_side_at(x, y) == s) {
 		    set_people_side_at(x, y, s2);
 		    changed = TRUE;
 		}
-		if (control_side_at(x, y) == s) {
+		if (control_sides_defined() && control_side_at(x, y) == s) {
 		    set_control_side_at(x, y, s2);
 		    changed = TRUE;
 		}
@@ -865,9 +865,13 @@ record_into_scorefile()
 		} else if (keyword_code(varname) == K_SEQUENTIAL) {
 		    fprintf(fp, "3 %d", var->intvalue);
 		} else if (keyword_code(varname) == K_WORLD_SIZE) {
-		    fprintf(fp, "4 %d %d %d", area.width, area.height, world.circumference);
+		    fprintf(fp, "4 %d %d %d",
+			    area.width, area.height, world.circumference);
 		} else if (keyword_code(varname) == K_REAL_TIME) {
-		    fprintf(fp, "5 %d %d %d", g_rt_for_game(), g_rt_per_side(), g_rt_per_turn());
+		    fprintf(fp, "5 %d %d %d",
+			    g_rt_for_game(),
+			    g_rt_per_side(),
+			    g_rt_per_turn());
 		} else {
 		    fprintf(fp, "%s", escaped_symbol(varname));
 		    if (var->hasintvalue) {
@@ -1043,7 +1047,8 @@ Side *side;
 			if (numberp(car(sd)))
 			  sd = cdr(sd);
 			sdname = car(sd);
-			sdnamestr = ((symbolp(sdname) || stringp(sdname)) ? c_string(sdname) : "");
+			sdnamestr = ((symbolp(sdname) || stringp(sdname))
+				     ? c_string(sdname) : "");
 			sdfate = cadr(sd);
 			sdfatestr = (symbolp(sdfate) ? c_string(sdfate) : "");
 			if (sdnamestr != NULL
@@ -1091,7 +1096,8 @@ Side *side;
 			if (numberp(car(sd)))
 			  sd = cdr(sd);
 			sdname = car(sd);
-			sdnamestr = ((symbolp(sdname) || stringp(sdname)) ? c_string(sdname) : "?");
+			sdnamestr = ((symbolp(sdname) || stringp(sdname))
+				     ? c_string(sdname) : "?");
 			sdfate = cadr(sd);
 			sdfatestr = (symbolp(sdfate) ? c_string(sdfate) : "?");
 			strcpy(sdadvbuf, "");
@@ -1112,11 +1118,13 @@ Side *side;
 			if (sds != sr->sides)
 			  tprintf(buf, ", ");
 			tprintf(buf, "%s%s %s",
-				(!empty_string(sdnamestr) ? sdnamestr : "(no player)"),
+				(!empty_string(sdnamestr)
+				 ? sdnamestr : "(no player)"),
 				sdadvbuf, sdfatestr);
 		    }
 		    if (sr->numturns > 0)
-		      tprintf(buf, " (in %d turn%s)", sr->numturns, (sr->numturns != 1 ? "s" : ""));
+		      tprintf(buf, " (in %d turn%s)",
+			      sr->numturns, (sr->numturns != 1 ? "s" : ""));
 		    score_variant_desc(sr, varbuf);
 		    if (!empty_string(varbuf)) {
 			strcat(buf, " (variants");
@@ -1152,7 +1160,8 @@ char *varbuf;
 	    varset = car(restvarset);
 	    if (consp(varset)) {
 		if (numberp(car(varset))) {
-		    /* Get the names of the standard variants from their numbers. */
+		    /* Get the names of the standard variants from
+                       their numbers. */
 		    switch (c_number(car(varset))) {
 		      case 1:
 			varsetname = intern_symbol(keyword_name(K_WORLD_SEEN));
@@ -1176,11 +1185,13 @@ char *varbuf;
 		    varsetname = car(varset);
 		}
 		var = NULL;
-		for (i = 0; variants[i].id != lispnil; ++i) {
-		    tmpvar = &(variants[i]);
-		    if (tmpvar->id == varsetname) {
-			var = tmpvar;
-			break;
+		if (variants != NULL) {
+		    for (i = 0; variants[i].id != lispnil; ++i) {
+			tmpvar = &(variants[i]);
+			if (tmpvar->id == varsetname) {
+			    var = tmpvar;
+			    break;
+			}
 		    }
 		}
 		hasdflt = dfltval = FALSE;
