@@ -1,6 +1,5 @@
 /* Definitions for sides in Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,6 +14,7 @@ typedef int SideMask;
 #define NOSIDES (0)
 #define ALLSIDES (-1)
 #define add_side_to_set(side,mask) ((mask) | (1 << ((side) ? (side)->id : 0)))
+#define remove_side_from_set(side,mask) ((mask) & ~(1 << ((side) ? (side)->id : 0)))
 #define side_in_set(side,mask) ((mask) & (1 << ((side) ? (side)->id : 0)))
 #else
 not implemented yet
@@ -26,18 +26,30 @@ not implemented yet
 typedef struct a_doctrine {
     short id;			/* a unique id */
     char *name;			/* a distinguishing name for the doctrine */
-    short everaskside;		/* should this unit ever ask for orders? */
-    short *construction_run;
-    short locked;
-    struct a_doctrine *next;
+    short resupply_percent;	/* do resupply task if below this */
+    short rearm_percent;	/* do resupply of ammo if below this */
+    short repair_percent;	/* do repairs if hp at or below this */
+    short *construction_run;	/* number of each type to build usually */
+    short locked;		/* true if values can't be changed */
+    struct a_doctrine *next;	/* pointer to next doctrine defined */
 } Doctrine;
 
+/* A standing order is a conditional order that applies to any matching
+   unit not already doing a task. */
+
+enum sordercond {
+    sorder_always,
+    sorder_at,
+    sorder_in,
+    sorder_near
+};
+
 typedef struct a_standing_order {
     char *types;		/* unit types to which order applies */
-    char etype;			/* type of event to trigger on */
+    enum sordercond condtype;	/* type of condition to trigger on */
     int a1, a2, a3;		/* parameters to test (x, y, etc) */
     struct a_task *task;	/* task to perform */
-    struct a_standing_order *next;
+    struct a_standing_order *next; /* link to next standing order for side */
 } StandingOrder;
 
 /* Each Xconq player is a "side" - more or less one country.  A side may or
@@ -56,8 +68,8 @@ typedef struct a_side {
     char *adjective;		/* adjective for members of this side */
     char *colorscheme;		/* names of the side's characteristic colors */
     char *emblemname;		/* name of the side's emblem */
-    char **unitnamers;
-    Obj *featurenamers;
+    char **unitnamers;		/* array of unit naming method names */
+    Obj *featurenamers;		/* list of feature naming methods */
     short nameslocked;		/* true if names may not be changed by player */
     char *sideclass;		/* general type or class description */
     /* Relationships with other sides, types, etc. */
@@ -72,12 +84,16 @@ typedef struct a_side {
     short *counts;		/* array of numbers for identifying units */
     short *tech;		/* tech level for each unit type */
     short *inittech;		/* tech level at beginning of turn */
+    short *action_priorities;	/* action priority for each unit type */
+    short *already_seen;	/* chance that other units already seen */
+    short *already_seen_indep;	/* chance that independent units already seen */
     Doctrine *default_doctrine;	/* fallback doctrine */
     Doctrine **udoctrine;	/* array of per-unit-type doctrines */
-    short doctrines_locked;
+    short doctrines_locked;	/* true if all doctrines locked */
     StandingOrder *orders;	/* list of standing orders for the side */
     StandingOrder *last_order;
     char *uorders;		/* bit vector of types that have orders */
+    Obj *possible_units;	/* list of units to give to side */
     /* View layers. */
     char *terrview;		/* ptr to view of terrain */
     char **auxterrview;		/* ptr to view of aux terrain */
@@ -109,19 +125,23 @@ typedef struct a_side {
     short totaltimeused;	/* total seconds used */
     short timeouts;		/* total timeouts used */
     short timeoutsused;		/* total timeouts used */
+    short curactor_pos;		/* index in actionvector to current unit */
+    struct a_unit *curactor;
+    short curactor_id;
     /* Setup. */
     short advantage;		/* actual advantage */
-    short minadvantage;		/* max advantage requestable during init */
-    short maxadvantage;		/* min of same */
+    short minadvantage;		/* min advantage requestable during init */
+    short maxadvantage;		/* max of same */
+    short init_center_x, init_center_y;	/* center of preferred initial view */
     Obj *instructions;		/* notes to player about the game */
     /* Other. */
     short last_notice_date;	/* last turn during which notice was posted */
     short realtimeout;		/* how long to wait before just going ahead */
     long startbeeptime;		/* after this time, beep to signal next turn */
-    short *gaincounts;
-    short *losscounts;
-    long **atkstats;
-    long **hitstats;
+    short *gaincounts;		/* array of counts of unit gains by the side */
+    short *losscounts;		/* array of counts of unit losses by the side */
+    long **atkstats;		/* array of counts of attacks by units */
+    long **hitstats;		/* array of damage done by unit attacks */
     /* Pointers to the other major structures of a side. */
     struct a_player *player;	/* pointer to data about the player */
     short playerid;		/* numeric id of the player */
@@ -130,6 +150,8 @@ typedef struct a_side {
     short aitype;		/* type of AI making decisions */		
     struct a_ai *ai;		/* pointer to the AI making decisions */
     Obj *aidata;		/* readable/writable form of AI state */
+    int rid;			/* id of remote program that side is using */
+    struct a_rui *rui;
     /* Misc working slots, never saved. */
     short startx, starty;	/* approx center of side's "country" */
     short busy;			/* true when side state not saveable */
@@ -193,7 +215,7 @@ typedef struct a_side {
 #define UNSEEN (0)
 
 #define terrain_view_date(s,x,y)  \
-  ((s)->terrviewdate ? aref((s)->terrviewdate, x, y) : (-1))
+  ((s)->terrviewdate ? aref((s)->terrviewdate, x, y) : 0)
 
 #define set_terrain_view_date(s,x,y,v)  \
   ((s)->terrviewdate ? aset((s)->terrviewdate, x, y, v) : 0)
@@ -205,7 +227,7 @@ typedef struct a_side {
   (((s)->auxterrview && (s)->auxterrview[t]) ? aset((s)->auxterrview[t], x, y, v) : 0)
 
 #define aux_terrain_view_date(s,x,y,t)  \
-  (((s)->auxterrviewdate && (s)->auxterrviewdate[t]) ? aref((s)->auxterrviewdate[t], x, y) : (-1))
+  (((s)->auxterrviewdate && (s)->auxterrviewdate[t]) ? aref((s)->auxterrviewdate[t], x, y) : 0)
 
 #define set_aux_terrain_view_date(s,x,y,t,v)  \
   (((s)->auxterrviewdate && (s)->auxterrviewdate[t]) ? aset((s)->auxterrviewdate[t], x, y, v) : 0)
@@ -237,7 +259,7 @@ typedef struct a_side {
 #define vside_indep(v) (vside(v) == 0)
 
 #define unit_view_date(s,x,y)  \
-  ((s)->unitviewdate ? aref((s)->unitviewdate, x, y) : (-1))
+  ((s)->unitviewdate ? aref((s)->unitviewdate, x, y) : 0)
 
 #define set_unit_view_date(s,x,y,v)  \
   ((s)->unitviewdate ? aset((s)->unitviewdate, x, y, v) : 0)
@@ -251,7 +273,7 @@ typedef struct a_side {
   (((s)->materialview && (s)->materialview[m]) ? aset((s)->materialview[m], x, y, v) : 0)
 
 #define material_view_date(s,x,y,m)  \
-  (((s)->materialviewdate && (s)->materialviewdate[m]) ? aref((s)->materialviewdate[m], x, y) : (-1))
+  (((s)->materialviewdate && (s)->materialviewdate[m]) ? aref((s)->materialviewdate[m], x, y) : 0)
 
 #define set_material_view_date(s,x,y,m,v)  \
   (((s)->materialviewdate && (s)->materialviewdate[m]) ? aset((s)->materialviewdate[m], x, y, v) : 0)
@@ -265,7 +287,7 @@ typedef struct a_side {
   ((s)->tempview ? aset((s)->tempview, x, y, v) : 0)
 
 #define temperature_view_date(s,x,y)  \
-  ((s)->tempviewdate ? aref((s)->tempviewdate, x, y) : (-1))
+  ((s)->tempviewdate ? aref((s)->tempviewdate, x, y) : 0)
 
 #define set_temperature_view_date(s,x,y,v)  \
   ((s)->tempviewdate ? aset((s)->tempviewdate, x, y, v) : 0)
@@ -277,7 +299,7 @@ typedef struct a_side {
   ((s)->cloudview ? aset((s)->cloudview, x, y, v) : 0)
 
 #define cloud_view_date(s,x,y)  \
-  ((s)->cloudviewdate ? aref((s)->cloudviewdate, x, y) : (-1))
+  ((s)->cloudviewdate ? aref((s)->cloudviewdate, x, y) : 0)
 
 #define set_cloud_view_date(s,x,y,v)  \
   ((s)->cloudviewdate ? aset((s)->cloudviewdate, x, y, v) : 0)
@@ -289,7 +311,7 @@ typedef struct a_side {
   ((s)->cloudbottomview ? aset((s)->cloudbottomview, x, y, v) : 0)
 
 #define cloud_bottom_view_date(s,x,y)  \
-  ((s)->cloudbottomviewdate ? aref((s)->cloudbottomviewdate, x, y) : (-1))
+  ((s)->cloudbottomviewdate ? aref((s)->cloudbottomviewdate, x, y) : 0)
 
 #define set_cloud_bottom_view_date(s,x,y,v)  \
   ((s)->cloudbottomviewdate ? aset((s)->cloudbottomviewdate, x, y, v) : 0)
@@ -301,7 +323,7 @@ typedef struct a_side {
   ((s)->cloudheightview ? aset((s)->cloudheightview, x, y, v) : 0)
 
 #define cloud_height_view_date(s,x,y)  \
-  ((s)->cloudheightviewdate ? aref((s)->cloudheightviewdate, x, y) : (-1))
+  ((s)->cloudheightviewdate ? aref((s)->cloudheightviewdate, x, y) : 0)
 
 #define set_cloud_height_view_date(s,x,y,v)  \
   ((s)->cloudheightviewdate ? aset((s)->cloudheightviewdate, x, y, v) : 0)
@@ -313,7 +335,7 @@ typedef struct a_side {
   ((s)->windview ? aset((s)->windview, x, y, v) : 0)
 
 #define wind_view_date(s,x,y)  \
-  ((s)->windviewdate ? aref((s)->windviewdate, x, y) : (-1))
+  ((s)->windviewdate ? aref((s)->windviewdate, x, y) : 0)
 
 #define set_wind_view_date(s,x,y,v)  \
   ((s)->windviewdate ? aset((s)->windviewdate, x, y, v) : 0)
@@ -341,7 +363,7 @@ typedef struct a_side {
 
 #define side_wants_ai(s) ((s)->player && (s)->player->aitypename)
 
-#define side_has_display(s) (((s)->ui) != NULL)
+#define side_has_display(s) (((s)->ui) != NULL || ((s)->rui) != NULL)
 
 #define side_has_ai(s) (((s)->ai) != NULL)
 
@@ -368,6 +390,12 @@ typedef struct a_side {
 #define units_visible(side, x, y)  \
   (all_see_all || (side)->see_all || (cover((side), wrapx(x), (y)) >= 1))
 
+#ifdef DESIGNERS
+#define is_designer(side) (side->designer)
+#else /* DESIGNERS */
+#define is_designer(side) (FALSE)
+#endif /* DESIGNERS */
+
 /* Side-related variables. */
 
 extern Side *sidelist;
@@ -380,7 +408,7 @@ extern int numsides;
 extern int numplayers;
 #ifdef DESIGNERS
 extern int numdesigners;
-#endif
+#endif /* DESIGNERS */
 
 extern int all_see_all;
 extern int all_see_all_terrain;
@@ -463,25 +491,15 @@ extern int side_acp_reserved PARAMS ((Side *side));
 extern int using_tech_levels PARAMS ((void));
 extern void remove_side_from_game PARAMS ((Side *side));
 extern int num_displayed_sides PARAMS ((void));
-extern void set_side_name PARAMS ((Side *side, Side *side2, char *newname));
-extern void become_designer PARAMS ((Side *side));
-extern void become_nondesigner PARAMS ((Side *side));
 extern int trusted_side PARAMS ((Side *side1, Side *side2));
-extern void set_trust PARAMS ((Side *side, Side *side2, int val));
-extern void set_mutual_trust PARAMS ((Side *side, Side *side2, int val));
-extern void set_autofinish PARAMS ((Side *side, int value));
 extern int enemy_side PARAMS ((Side *s1, Side *s2));
 extern int allied_side PARAMS ((Side *s1, Side *s2));
 extern int neutral_side PARAMS ((Side *s1, Side *s2));
-extern void set_willing_to_save PARAMS ((Side *side, int flag));
-extern void set_willing_to_draw PARAMS ((Side *side, int flag));
 extern void reveal_side PARAMS ((Side *sender, Side *recipient, int *types));
-extern void send_message PARAMS ((Side *side, SideMask sidemask, char *str));
 extern void receive_message PARAMS ((Side *side, Side *sender, char *str));
-extern void calc_vision PARAMS ((Side *side));
 extern void all_see_occupy PARAMS ((struct a_unit *unit, int x, int y, int inopen));
 extern void all_see_leave PARAMS ((struct a_unit *unit, int x, int y, int inopen));
-extern void cover_area PARAMS ((Side *side, struct a_unit *unit, int x0, int y0, int x1, int y1));
+extern void cover_area PARAMS ((Side *side, struct a_unit *unit, struct a_unit *oldtransport, int x0, int y0, int x1, int y1));
 extern void reset_coverage PARAMS ((void));
 extern void calc_coverage PARAMS ((Side *side));
 extern void reset_all_views PARAMS ((void));
@@ -491,6 +509,8 @@ extern void all_see_cell PARAMS ((int x, int y));
 extern int see_cell PARAMS ((Side *side, int x, int y));
 extern void see_exact PARAMS ((Side *side, int x, int y));
 extern int seen_border PARAMS ((Side *side, int x, int y, int dir));
+extern void maybe_track PARAMS ((struct a_unit *unit));
+extern void maybe_lose_track PARAMS ((struct a_unit *unit, int nx, int ny));
 extern char *side_desig PARAMS ((Side *side));
 
 extern Side *parse_side_spec PARAMS ((char *str));
@@ -499,6 +519,7 @@ extern Doctrine *new_doctrine PARAMS ((int id));
 extern Doctrine *find_doctrine PARAMS ((int id));
 extern Doctrine *find_doctrine_by_name PARAMS ((char *name));
 extern Doctrine *clone_doctrine PARAMS ((Doctrine *doctrine));
+extern void set_doctrine PARAMS ((Side *side, char *spec));
 
 extern StandingOrder *new_standing_order PARAMS ((void));
 extern void add_standing_order PARAMS ((Side *side, StandingOrder *sorder, int pos));
@@ -506,15 +527,27 @@ extern int parse_standing_order PARAMS ((Side *side, char *cmdstr));
 extern char *parse_unit_types PARAMS ((Side *side, char *str, char *utypevec));
 extern char *parse_order_cond PARAMS ((Side *side, char *str, StandingOrder *sorder));
 extern char *get_next_arg PARAMS ((char *str, char *buf, char **rsltp));
+extern char *standing_order_desc PARAMS ((StandingOrder *sorder, char *buf));
 
 extern void init_agreements PARAMS ((void));
 extern Agreement *create_agreement PARAMS ((int id));
 extern char *agreement_desig PARAMS ((Agreement *ag));
 
+extern void fn_set_terrain_view PARAMS ((int x, int y, int val));
+extern void fn_set_aux_terrain_view PARAMS ((int x, int y, int val));
+extern void fn_set_terrain_view_date PARAMS ((int x, int y, int val));
+extern void fn_set_aux_terrain_view_date PARAMS ((int x, int y, int val));
+extern void fn_set_unit_view PARAMS ((int x, int y, int val));
+extern void fn_set_unit_view_date PARAMS ((int x, int y, int val));
+extern void fn_set_material_view PARAMS ((int x, int y, int val));
+extern void fn_set_material_view_date PARAMS ((int x, int y, int val));
+extern void fn_set_temp_view PARAMS ((int x, int y, int val));
+extern void fn_set_temp_view_date PARAMS ((int x, int y, int val));
+extern void fn_set_cloud_view PARAMS ((int x, int y, int val));
+extern void fn_set_cloud_bottom_view PARAMS ((int x, int y, int val));
+extern void fn_set_cloud_height_view PARAMS ((int x, int y, int val));
+extern void fn_set_cloud_view_date PARAMS ((int x, int y, int val));
+extern void fn_set_wind_view PARAMS ((int x, int y, int val));
+extern void fn_set_wind_view_date PARAMS ((int x, int y, int val));
+
 extern int load_side_config PARAMS ((Side *side));
-
-extern int set_side_self_unit PARAMS ((Side *side, struct a_unit *unit));
-
-#ifdef DESIGNERS
-extern void paint_view PARAMS ((Side *side, int x, int y, int r, int tview, int uview));
-#endif /* DESIGNERS */
