@@ -1,31 +1,24 @@
 /* A minimal interface to Xconq.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
-/* This interface uses only what is
-   required by ANSI, so it should run just about everywhere without
-   any changes.  This is most useful for testing the kernel and game
-   design libraries.
+/* This interface uses only what is required by ANSI, so it should run
+   just about everywhere without any changes.  This is most useful for
+   testing the kernel and game design libraries.
 
-   This file can also serve as a starting point for writing a new interface,
-   since it has simple or default implementations of the routines that any
-   interface has to support. */
+   This file can also serve as a starting point for writing a new
+   interface, since it has simple or default implementations of the
+   routines that any interface has to support. */
 
 #include "conq.h"
-extern void low_send PARAMS ((int id, char *buf));
-extern int low_receive PARAMS ((int *id, char *buf, int maxchars,
-				int timeout));
+#include "kpublic.h"
 #include "print.h"
 #include "cmdline.h"
 
-#ifdef COMPILER
-extern void compile PARAMS ((void));
-#endif /* COMPILER */
-
 static void init_displays PARAMS ((void));
 static void get_input PARAMS ((void));
 static void interpret_command PARAMS ((Obj *cmd));
@@ -55,18 +48,13 @@ static void do_memory PARAMS ((Side *side, Obj *cmd, Obj *parms));
 static void do_quit PARAMS ((Side *side, Obj *cmd, Obj *parms));
 static int do_action PARAMS ((Side *side, Unit *unit, Obj *cmd, Obj *args));
 static void show_help PARAMS ((Side *side, HelpNode *node));
-static void describe_commands PARAMS ((int, char *, char *));
+static void describe_commands PARAMS ((int arg, char *key, TextBuffer *buf));
 
 #ifdef THINK_C
 /* This is to get the command line reader in Think C on the Mac. */
 #include <console.h>
 #endif /* THINK_C */
 
-#ifdef THINK_C
-/* Think C loses on the compiler. */
-#undef COMPILER
-#endif /* THINK_C */
-
 /* This structure maintains state that is local to a side's display.
    At the very least, it must track when the display is open and closed. */
 
@@ -135,15 +123,15 @@ char *argv[];
     run_synth_methods();
     final_init();
     assign_players_to_sides();
-    init_displays();
     init_signal_handlers();
-    run_game(0);
-    multicmd = lispnil;
     time(&skelturnstart);
+    run_game(0);
+    init_displays();
+    multicmd = lispnil;
     while (1) {
 	if (freerunturns > 0) {
 	    if (probability(10)) {
-	    	Dprintf("No apparent progress, forcing the turn to finish.\n");
+	    	printf("No apparent progress, forcing the turn to finish.\n");
 	    	do_finish_all(NULL, lispnil, lispnil);
 	    }
 	    if (endofgame)
@@ -152,8 +140,10 @@ char *argv[];
 	    get_input();
 	}
 	/* We stay in here until nothing more to do. */
-	while (run_game(99) > 0)
-	  ;
+	run_local_ai(1, 20);
+	while (run_game(99) > 0) {
+	  run_local_ai(2, 20);
+	}
     }
 }
 
@@ -168,6 +158,7 @@ add_default_player()
 }
 
 /* This routine handles all the displays that might need to be opened. */
+/* (Note that skelconq may have multiple "displays" open.) */
 
 static void
 init_displays()
@@ -193,7 +184,8 @@ Side *side;
 	/* Display should not become active yet. */
 	side->ui->active = FALSE;
 	defaultside = side;
-	set_autofinish(side, FALSE);
+	/* Do this so game doesn't run out of control. */
+	net_set_autofinish(side, FALSE);
 	DGprintf("Created a UI for %s\n", side_desig(side));
     } else {
 	side->ui = NULL;
@@ -223,7 +215,7 @@ get_input()
 	    update_clock_display(side, TRUE);
 	}
     }
-    /* should say which sides we're waiting for */
+    /* (should say which sides we're waiting for) */
     printf("> ");
     fflush(stdout);
     if (repetition-- > 0) {
@@ -337,10 +329,15 @@ Obj *cmd, *parms;
     Agreement *ag;
     extern int numagreements;
 
-    printf("Sides:\n");
+    printf("Sides: (%s)\n", (g_use_side_priority() ? "sequential" : "simultaneous"));
     for_all_sides(side2) {
-	printf("%s played by %s\n",
+	printf("%s played by %s",
 	       side_desig(side2), player_desig(side2->player));
+	printf(", autofinish %d", side2->autofinish);
+	if (g_use_side_priority())
+	  printf(", priority %d", side2->priority);
+	printf(", %s", (side2->finishedturn ? "finished" : "moving"));
+	printf("\n");
 	if (using_tech_levels()) {
 	    printf("Tech:");
 	    for_all_unit_types(u) {
@@ -465,9 +462,7 @@ Obj *cmd, *parms;
 			    task->args[j] = 0;
 			}
 		    }
-		    task->next = thisunit->plan->tasks;
-		    thisunit->plan->tasks = task;
-		    thisunit->plan->waitingfortasks = FALSE;
+		    net_add_task(thisunit, 0, task);
 		}
 		return;
 	    }
@@ -481,11 +476,7 @@ do_finish_turn(side, cmd, parms)
 Side *side;
 Obj *cmd, *parms;
 {
-    if (side) {
-	finish_turn(side);
-    } else {
-	fprintf(stderr, "No side?\n");
-    }
+    net_finish_turn(side ? side : defaultside);
 }
 
 static void
@@ -496,7 +487,7 @@ Obj *cmd, *parms;
     Side *side2;
 
     for_all_sides(side2) {
-	finish_turn(side2);
+	net_finish_turn(side2);
     }
 }
 
@@ -505,7 +496,16 @@ do_free_run(side, cmd, parms)
 Side *side;
 Obj *cmd, *parms;
 {
-    freerunturns = c_number(car(parms));
+    Side *side2;
+
+    if (consp(parms) && numberp(car(parms))) {
+	freerunturns = c_number(car(parms));
+    } else {
+	freerunturns = 1;
+    }
+    for_all_sides(side2) {
+	net_set_autofinish(side2, TRUE);
+    }
 }
 
 static void
@@ -593,20 +593,16 @@ do_quit(side, cmd, parms)
 Side *side;
 Obj *cmd, *parms;
 {
+    /* A quit command of the form "(quit if-end)" only actually quits if the
+       game is over. */
+    if (symbolp(car(parms)) && equal(car(parms), intern_symbol("if-end"))) {
+	if (!endofgame)
+	  return;
+    }
     printf("Quitting.\n");
     exit(0);
 }
 
-#ifdef COMPILER
-static void
-do_compile(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
-{
-    compile();
-}
-#endif /* COMPILER */
-
 struct a_cmd {
     char *cmd;
     void (*fn) PARAMS ((Side *side, Obj *cmd, Obj *parms));
@@ -630,9 +626,6 @@ struct a_cmd {
     { "print", do_print },
     { "memory", do_memory },
     { "quit", do_quit },
-#ifdef COMPILER
-    { "compile", do_compile },
-#endif /* COMPILER */
     { NULL, NULL }
 };
 
@@ -646,7 +639,8 @@ Obj *cmd, *parms;
     struct a_cmd *cmdentry = cmdtable;
     char *cmdstr;
 
-    if (!symbolp(cmd)) return FALSE;
+    if (!symbolp(cmd))
+      return FALSE;
     cmdstr = c_string(cmd);
     while (cmdentry->cmd != NULL) {
 	if (strcmp(cmdstr, cmdentry->cmd) == 0) {
@@ -672,7 +666,8 @@ Obj *cmd, *args;
     Obj *rest;
     Action action;
 
-    if (!symbolp(cmd)) return FALSE;
+    if (!symbolp(cmd))
+      return FALSE;
     cmdstr = c_string(cmd);
     if (side == NULL) {
 	side = defaultside;
@@ -774,7 +769,7 @@ int rightnow;
     ++numcellupdatesperturn;
     if (active_display(side)) {
 	++numusefulcellupdatesperturn;
-	DGprintf("To %s: update view of %d,%d%s\n",
+	DGprintf("Update %s: view of %d,%d%s\n",
 		 side_desig(side), x, y, (rightnow ? " (now)" : ""));
     }
 }
@@ -793,7 +788,7 @@ int rightnow;
     if (active_display(side)) {
 	time(&xxx);
 	secs = idifftime(xxx, skelturnstart);
-	printf("To %s: update to turn %d (%ld seconds since last)",
+	printf("Update %s: turn %d (%ld seconds since last)",
 	       side_desig(side), g_turn(), secs);
 	DGprintf("%s", (rightnow ? " (now)" : ""));
 	printf("\n");
@@ -802,7 +797,16 @@ int rightnow;
 	numcellupdatesperturn = numusefulcellupdatesperturn = 0;
 	skelturnstart = xxx;
 	--freerunturns;
-	if (rightnow) fflush(stdout);
+	if (freerunturns == 0) {
+	    Side *side2;
+
+	    for_all_sides(side2) {
+	        if (active_display(side2))
+		  net_set_autofinish(side2, FALSE);
+	    }
+	}
+	if (rightnow)
+	  fflush(stdout);
     }
 }
 
@@ -812,7 +816,7 @@ Side *side;
 int rightnow;
 {
     if (active_display(side) && DebugG) {
-	printf("To %s: ready to act\n", side_desig(side));
+	printf("Update %s: ready to act\n", side_desig(side));
     }
 }
 
@@ -823,7 +827,7 @@ Unit *unit;
 int rslt, rightnow;
 {
     if (active_display(side) && DebugG) {
-	printf("To %s: %s action result is %s\n",
+	printf("Update %s: %s action result is %s\n",
 	       side_desig(side), unit_desig(unit), hevtdefns[rslt].name);
     }
 }
@@ -836,6 +840,10 @@ Side *side;
 Unit *unit, *unit2;
 int m, rightnow;
 {
+    if (active_display(side) && DebugG) {
+	printf("Update %s: %s fire at %s\n",
+	       side_desig(side), unit_desig(unit), unit_desig(unit2));
+    }
 }
 
 /* This is for animation of fire-into actions. */
@@ -846,6 +854,10 @@ Side *side;
 Unit *unit;
 int x, y, z, m, rightnow;
 {
+    if (active_display(side) && DebugG) {
+	printf("Update %s: %s fire into %d,%d\n",
+	       side_desig(side), unit_desig(unit), x, y);
+    }
 }
 
 void
@@ -863,7 +875,7 @@ int rightnow;
 	    printf("%s won!\n",  side_desig(side_n(hevt->data[0])));
 	    break;
 	  default:
-	    DGprintf("To %s: event %s %d\n",
+	    DGprintf("Update %s: event %s %d\n",
 		     side_desig(side), hevtdefns[hevt->type].name,
 		     hevt->data[0]);
 	}
@@ -875,6 +887,9 @@ update_all_progress_displays(str, s)
 char *str;
 int s;
 {
+    if (DebugG) {
+	printf("Update all progress displays\n");
+    }
 }
 
 /* This hook should update the side's view of the given side, no matter
@@ -888,7 +903,7 @@ int rightnow;
     char *side2desc = copy_string(side_desig(side2));
 
     if (active_display(side) && DebugG) {
-	printf("To %s: update side %s%s\n",
+	printf("Update %s: side %s%s\n",
 	       side_desig(side), side2desc, (rightnow ? " (now)" : ""));
     }
 }
@@ -903,7 +918,7 @@ Unit *unit;
 int rightnow;
 {
     if (active_display(side) && DebugG) {
-	printf("To %s: update unit %s%s\n",
+	printf("Update %s: unit %s%s\n",
 	       side_desig(side), unit_desig(unit), (rightnow ? " (now)" : ""));
     }
 }
@@ -915,7 +930,7 @@ Unit *unit;
 int rightnow;
 {
     if (active_display(side) && DebugG) {
-	printf("To %s: update unit %s acp%s\n",
+	printf("Update %s: unit %s acp%s\n",
 	       side_desig(side), unit_desig(unit), (rightnow ? " (now)" : ""));
     }
 }
@@ -929,7 +944,7 @@ Side *side;
 int rightnow;
 {
     if (active_display(side) && DebugG) {
-	printf("To %s: %d secs this turn, %d total\n",
+	printf("Update %s: %d secs this turn, %d total\n",
 	       side_desig(side), side->turntimeused, side->totaltimeused);
 	/* also display total game clock */
     }
@@ -941,6 +956,10 @@ Side *side, *sender;
 char *str;
 int rightnow;
 {
+    if (active_display(side) && DebugG) {
+	printf("Update %s: side %d sends \"%s\"%s\n",
+	       side_desig(side), side_number(sender), str, (rightnow ? " (now)" : ""));
+    }
 }
 
 void
@@ -962,13 +981,14 @@ int x, y;
 static void
 describe_commands(arg, key, buf)
 int arg;
-char *key, *buf;
+char *key;
+TextBuffer *buf;
 {
     struct a_cmd *cmdentry;
 
     for (cmdentry = cmdtable; cmdentry->cmd != NULL; ++cmdentry) {
-	strcat(buf, cmdentry->cmd);
-	strcat(buf, "\n");
+	tbcat(buf, cmdentry->cmd);
+	tbcat(buf, "\n");
     }
 }
 
@@ -979,7 +999,8 @@ char *topic;
 {
     HelpNode *node;
 
-    if (side == NULL) side = defaultside;
+    if (side == NULL)
+      side = defaultside;
     if (side == NULL) {
 	fprintf(stderr, "no side to help?\n");
 	return;
@@ -1023,9 +1044,11 @@ HelpNode *helpnode;
     while (*linebegin != '\0') {
 	skipchar = 0;
 	lineend = (char *) strchr(linebegin, '\n');
-	if (lineend) skipchar = 1;
+	if (lineend)
+	  skipchar = 1;
 	linelen = (lineend ? lineend - linebegin : strlen(linebegin));
-	if (linelen > 75) linelen = 75;
+	if (linelen > 75)
+	  linelen = 75;
 	strncpy(spbuf, linebegin, linelen);
 	spbuf[linelen] = '\0';
 	printf("%s\n", spbuf);
@@ -1075,11 +1098,11 @@ finish_lengthy_process()
 
 int
 #ifdef __STDC__
-schedule_movie(Side *side, enum movie_type movie, ...)
+schedule_movie(Side *side, char *movie, ...)
 #else
 schedule_movie(side, movie)
 Side *side;
-enum movie_type movie;
+char *movie;
 #endif
 {
     return FALSE;
@@ -1091,13 +1114,6 @@ SideMask sidemask;
 {
 }
 
-void
-ui_save_state(side)
-Side *side;
-{
-    /* No state to save. */
-}
-
 void
 flush_display_buffers(side)
 Side *side;
@@ -1113,7 +1129,8 @@ void
 low_init_error(str)
 char *str;
 {
-    if (linemiddle) printf("\n");
+    if (linemiddle)
+      printf("\n");
     fprintf(stderr, "Error: %s.\n", str);
     fflush(stderr);
 }
@@ -1124,7 +1141,8 @@ void
 low_init_warning(str)
 char *str;
 {
-    if (linemiddle) printf("\n");
+    if (linemiddle)
+      printf("\n");
     fprintf(stdout, "Warning: %s.\n", str);
     fflush(stdout);
 }
@@ -1135,7 +1153,8 @@ void
 low_run_error(str)
 char *str;
 {
-    if (linemiddle) fprintf(stderr, "\n");
+    if (linemiddle)
+      fprintf(stderr, "\n");
     fprintf(stderr, "Error: %s.\n", str);
     fflush(stderr);
     exit(1);
@@ -1148,7 +1167,8 @@ void
 low_run_warning(str)
 char *str;
 {
-    if (linemiddle) printf("\n");
+    if (linemiddle)
+      printf("\n");
     fprintf(stdout, "Warning: %s.\n", str);
     fflush(stdout);
 }
@@ -1165,25 +1185,46 @@ end_printing_forms()
 {
 }
 
+/* Note that the fake declaration below will cause problems if imf.h
+   is included. */
+
 void
-low_send(id, buf)
-int id;
-char *buf;
+make_generic_image_data(imf)
+char *imf;
+{
+}
+
+/* Networking functions (dummies for now). */
+
+void
+add_remote_player(name)
+char *name;
 {
 }
 
 int
-low_receive(id, buf, maxchars, timeout)
-int *id, maxchars, timeout;
-char *buf;
+launch_game()
 {
-  return 0;
+    /* (should fill this in) */
+    return FALSE;
 }
 
-#include "imf.h"
-
 void
-make_generic_image_data(imf)
-ImageFamily *imf;
+set_you_player(id)
+int id;
 {
 }
+
+#ifdef MAC
+
+int current_cursor;
+int receivecursor;
+int sendcursor;
+
+int
+serial_port_dialog()
+{
+    return 0;
+}
+
+#endif /* MAC */
