@@ -1,5 +1,5 @@
 /* Definitions of system-specific things for Xconq.
-   Copyright (C) 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1995, 1996, 1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ extern char *error_save_filename PARAMS ((void));
 extern char *statistics_filename PARAMS ((void));
 extern FILE *open_scorefile_for_reading PARAMS ((char *name));
 extern FILE *open_scorefile_for_writing PARAMS ((char *name));
+extern void close_scorefile_for_writing PARAMS ((FILE *fp));
 extern void make_pathname PARAMS ((char *path, char *name, char *extn,
 				  char *pathbuf));
 extern void remove_saved_game PARAMS ((void));
@@ -27,3 +28,14 @@ extern int n_seconds_elapsed PARAMS ((int n));
 extern int n_ms_elapsed PARAMS ((int n));
 extern void record_ms PARAMS ((void));
 
+extern void open_remote_connection PARAMS ((char *methodname));
+extern void close_remote_connection PARAMS ((void));
+extern void low_send PARAMS ((int id, char *buf));
+extern int low_receive PARAMS ((int *id, char *buf, int maxchars,
+				int timeout));
+
+/* defined in init.c but referenced in mac.c ... */
+extern void add_library_path PARAMS ((char *path));
+
+extern int serial_port_dialog PARAMS ((void));
+
