@@ -1,5 +1,5 @@
 /* Unit task execution and general task functions.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,6 +7,12 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+extern int need_ai_task_reaction;
+extern int attack_can_damage_or_capture PARAMS ((Unit *unit, Unit *unit2));
+extern int target_visible PARAMS ((Unit *unit, Task *task));
+#include "kernel.h"
+extern Task *create_repair_task PARAMS ((void));
+extern void set_repair_task PARAMS ((Unit *unit));
 
 /* This is the number of tasks to allocate initially.  More will always be
    allocated as needed, so this should be a "reasonable" value. */
@@ -19,6 +25,8 @@ any later version.  See the file COPYING.  */
 
 static int compare_directions PARAMS ((const void *a0, const void *a1));
 static int test_for_buildable PARAMS ((int x, int y));
+static int repair_test PARAMS ((int x, int y));
+static int resupply_test PARAMS ((int x, int y));
 
 /* Declare all the task functions. */
 
@@ -201,6 +209,8 @@ Task *task;
 	unit2 = find_unit(task->args[1]);
 	if (in_play(unit2) && unit2->type == u2) {
 	    if (fullsized(unit2)) {
+		/* Clear the reference to the unit. */
+		task->args[1] = 0;
 		++(task->args[2]);
 		if (task->args[2] >= run) {
 		    return TASK_IS_COMPLETE;
@@ -267,7 +277,7 @@ Task *task;
 	    && !fullsized(occ)
 	    && occ->type == u2
 	    && occ->side == unit->side) 
-	  return(occ);
+	  return occ;
     }
     /* Or else search for any appropriate incomplete units in this cell. */
     for_all_stack(x, y, occ) {
@@ -275,7 +285,7 @@ Task *task;
 	    && !fullsized(occ)
 	    && occ->type == u2
 	    && occ->side == unit->side) {
-	    return (occ);
+	    return occ;
     	}
     }
     /* Or else search nearby area. */
@@ -285,7 +295,7 @@ Task *task;
 	    tmpbuilder = unit;
 	    tmpbuildutype = u2;
 	    if (search_around(x, y, range, test_for_buildable, &nx, &ny, 1)) {
-	    	return(tmpbuildunit);
+	    	return tmpbuildunit;
 	    }
 	}
     }
@@ -355,7 +365,7 @@ Task *task;
 		    return TASK_PREPPED_ACTION;
 		} else {
 		    /* We get several tries to capture before giving up. */
-		    unit->plan->reserve = TRUE;
+		    set_unit_reserve(unit->side, unit, TRUE, FALSE);
 		    return (task->execnum < 5 ? TASK_IS_INCOMPLETE : TASK_FAILED);
 		}
 	    }
@@ -370,16 +380,10 @@ Task *task;
 	    && flip_coin()) {
 	    return TASK_IS_INCOMPLETE;
 	}
-	/* If out of range and can move, push a task to get closer (maybe). */
-	if (mobile(u) && flip_coin()) {
-	    movedist = max(1 /* attack range */, u_range(u));
-	    if (dist > movedist + u_acp(u) /* or dist that could be covered in 1-2 turns */) {
-		movedist = dist - max(1, (dist - movedist) / 4);
-		/* We're too far away to capture directly, add a move-to
-		   task. */
-		push_move_near_task(unit, tx, ty, movedist);
-		return TASK_IS_INCOMPLETE;
-	    }
+	/* If out of range and can move, push a task to get closer (usually). */
+	if (mobile(u) && probability(90)) {
+	    push_move_near_task(unit, tx, ty, 1);
+	    return TASK_IS_INCOMPLETE;
 	}
 	return TASK_FAILED;
     }
@@ -464,18 +468,35 @@ Task *task;
     tu = task->args[2];  ts = task->args[3];
     dist = distance(tx, ty, unit->x, unit->y);
     if (dist <= 1 /* direct attack range */) {
-	for_all_stack(tx, ty, unit2) {
+	if (can_attack(unit)) {
+	  for_all_stack(tx, ty, unit2) {
 	    if (!trusted_side(us, unit2->side)
 	    	&& (tu == NONUTYPE || unit2->type == tu)
 	    	/* (should check designated side to hit, if defined) */
 	    	) {
-		if (valid(check_attack_action(unit, unit, unit2, 100))) {
+		if (valid(check_attack_action(unit, unit, unit2, 100))
+		    && attack_can_damage_or_capture(unit, unit2)) {
 		    prep_attack_action(unit, unit, unit2, 100);
 		    return TASK_PREPPED_ACTION;
 		}
 	    }
+	  }
+	}
+	/* Might be able to fire at pointblank range. */
+	if (can_fire(unit) && dist >= u_range_min(u)) {
+	  for_all_stack(tx, ty, unit2) {
+	    if (!trusted_side(us, unit2->side)
+	    	&& (tu == NONUTYPE || unit2->type == tu)
+	    	/* (should check designated side to hit, if defined) */
+	    	) {
+		if (valid(check_fire_at_action(unit, unit, unit2, -1))
+		    && fire_can_damage(unit, unit2)) {
+		    prep_fire_at_action(unit, unit, unit2, -1);
+		    return TASK_PREPPED_ACTION;
+		}
+	    }
+	  }
 	}
-	/* but might still be able to fire! */
 	return TASK_FAILED;
     }
     if (dist < u_range_min(u)) {
@@ -499,37 +520,8 @@ Task *task;
 	}
 	return TASK_FAILED;	    
     }
-    if (units_visible(us, tx, ty)) {
-	enemythere = FALSE;
-	for_all_stack(tx, ty, unit2) {
-	    if (!trusted_side(us, unit2->side)
-	    	&& (tu == NONUTYPE || unit2->type == tu)
-	    	/* (should check designated side to hit, if defined) */
-	    	) {
-	    	enemythere = TRUE;
-	    	break;
-	    }
-	}
-	if (!enemythere) {
-	    if (tu != NONUTYPE) {
-		return TASK_FAILED;
-	    } else {
-		return TASK_IS_COMPLETE;
-	    }
-	}
-    } else {
-	/* Have to assess old image or non-image. */
-	uview = unit_view(us, tx, ty);
-	if (uview != EMPTY) {
-	    if (tu == NONUTYPE) {
-	    	/* Just keep going. */
-	    }
-	} else {
-	    /* Not clear if disappearance of target means success or failure,
-	       but go with failure. */
-	    return TASK_FAILED;
-	}
-    }
+    if (!target_visible(unit, task))
+      return TASK_FAILED;
     /* If on mobile transport, let it handle things. */
     if (unit->transport != NULL
         && mobile(unit->transport->type)
@@ -541,7 +533,7 @@ Task *task;
     if (mobile(u) && flip_coin()) {
 	movedist = max(1 /* attack range */, u_range(u));
 	if (dist > movedist + u_acp(u) /* or dist that could be covered in 1-2 turns */) {
-	    movedist = dist - max(1, (dist - movedist) / 4);
+	    movedist = max(movedist, (dist - movedist) / 4);
 	}
 	push_move_near_task(unit, tx, ty, movedist);
 	return TASK_IS_INCOMPLETE;
@@ -551,6 +543,26 @@ Task *task;
 
 /* Return true if the unit can actually damage the other unit. */
 
+int
+attack_can_damage_or_capture(unit, unit2)
+Unit *unit, *unit2;
+{
+    if (!alive(unit) || !alive(unit2))
+      return FALSE;
+    if (capture_chance(unit->type, unit2->type, unit2->side) > 0)
+      return TRUE;
+    /* (should check for right kind of ammo) */
+    if (uu_hit(unit->type, unit2->type) <= 0)
+      return FALSE;
+    /* (this is dubious - a no-damage attack could still consume acp) */
+    if (uu_damage(unit->type, unit2->type) <= 0)
+      return FALSE;
+    /* (should check if victim hp might be under damage low bounds) */
+    return TRUE;
+}
+
+/* Return true if the unit can actually damage the other unit. */
+
 int
 fire_can_damage(unit, unit2)
 Unit *unit, *unit2;
@@ -567,6 +579,40 @@ Unit *unit, *unit2;
     return TRUE;
 }
 
+int
+target_visible(unit, task)
+Unit *unit;
+Task *task;
+{
+    int u = unit->type, tx, ty, uview, tu, ts;
+    Unit *unit2;
+    Side *us = unit->side;
+
+    tx = task->args[0];  ty = task->args[1];
+    tu = task->args[2];  ts = task->args[3];
+
+    if (units_visible(us, tx, ty)) {
+	for_all_stack(tx, ty, unit2) {
+	    if (!trusted_side(us, unit2->side)
+	    	&& (tu == NONUTYPE || unit2->type == tu)
+	    	/* (should check designated side to hit, if defined) */
+	    	) {
+	    	return TRUE;
+	    }
+	}
+    } else {
+	/* Assess old image or non-image. */
+	uview = unit_view(us, tx, ty);
+	if (uview != EMPTY
+	    && (tu == NONUTYPE || tu == vtype(uview))) {
+	    return TRUE;
+	}
+    }
+    return FALSE;
+}
+
+/* Move in a straight line, go through things or stop rather than going around. */
+
 static TaskOutcome
 do_move_dir_task(unit, task)
 Unit *unit;
@@ -578,10 +624,15 @@ Task *task;
     if ((task->args[1])-- > 0) {
 	dir = task->args[0];
 	if (!point_in_dir(unit->x, unit->y, dir, &tx, &ty)) {
-		return TASK_FAILED;
+	    return TASK_FAILED;
 	}
 	if (unit_at(tx, ty)) {
 	    for_all_stack(tx, ty, unit2) {
+		if (unit_trusts_unit(unit, unit2) /* but a spy could enter untrusted unit... */
+		    && valid(check_enter_action(unit, unit, unit2))) {
+		    prep_enter_action(unit, unit, unit2);
+		    return TASK_PREPPED_ACTION;
+		}
 	    }
 	    return TASK_FAILED;
 	} else if (valid(check_move_action(unit, unit, tx, ty, 0))) {
@@ -614,13 +665,13 @@ do_move_to_task(unit, task)
 Unit *unit;
 Task *task;
 {
-    int dist, tx, ty;
+    int dist, tx, ty, check;
     Unit *unit2, *occ;
 
     /* This task is to get to a designated location somehow. */
     tx = task->args[0];  ty = task->args[1];
     dist = distance(tx, ty, unit->x, unit->y);
-    if (dist <= task->args[2]) {
+    if (dist <= task->args[3]) {
 	return TASK_IS_COMPLETE;
     }
     switch (dist) {
@@ -661,17 +712,31 @@ Task *task;
 	    }
 	    return TASK_FAILED;
 	}
-	if (valid(check_move_action(unit, unit, tx, ty, unit->z))) {
+	if (valid(check = check_move_action(unit, unit, tx, ty, unit->z))) {
 	    /* Moving into an empty cell. */
 	    prep_move_action(unit, unit, tx, ty, unit->z);
 	    return TASK_PREPPED_ACTION;
 	} else {
+	    /* If we're just short on mp, wait until the next turn to try to move. */
+	    if (check == A_MOVE_NO_MP) {
+		if (unit->side)
+		  notify(unit->side, "%s has insufficient MP to move this turn; waiting until next",
+				     unit_handle(unit->side, unit));
+		set_unit_reserve(unit->side, unit, TRUE, FALSE);
+		return TASK_IS_INCOMPLETE;
+	    }
 	    return TASK_FAILED;
 	}
 	break;
       default:
-	/* Still some distance away, pick a way to go. */
-	return do_approach_subtask(unit, task, tx, ty, &(task->args[4]));
+	if (dist <= u_move_range(unit->type)
+	    && valid(check_move_action(unit, unit, tx, ty, unit->z))) {
+	    prep_move_action(unit, unit, tx, ty, unit->z);
+	    return TASK_PREPPED_ACTION;
+	} else {
+	    /* Still some distance away, pick a way to go. */
+	    return do_approach_subtask(unit, task, tx, ty, &(task->args[4]));
+	}
     }
     return TASK_FAILED;
 }
@@ -683,7 +748,7 @@ Task *task;
 int tx, ty;
 short *statep;
 {
-    int nx, ny, dirs[NUMDIRS], numdirs, i, numdirs2;
+    int nx, ny, dirs[NUMDIRS], numdirs, i, numdirs2, check;
     Unit *unit2;
 
     /* If on mobile transport, let it handle things. */
@@ -691,7 +756,7 @@ short *statep;
 	&& mobile(unit->transport->type)
 	/* and the transport is not blocked */
 	&& flip_coin()) {
-	unit->plan->reserve = TRUE;
+	set_unit_reserve(unit->side, unit, TRUE, FALSE);
 	return TASK_IS_INCOMPLETE;
     }
     numdirs = choose_move_dirs(unit, tx, ty, TRUE,
@@ -842,10 +907,18 @@ short *statep;
 #endif
 	    }
 	}
-	if (valid(check_move_action(unit, unit, nx, ny, unit->z))) {
+	if (valid(check = check_move_action(unit, unit, nx, ny, unit->z))) {
 	    prep_move_action(unit, unit, nx, ny, unit->z);
 	    return TASK_PREPPED_ACTION;
 	}
+	/* If we're just short on mp, wait until the next turn to try to move. */
+	if (check == A_MOVE_NO_MP) {
+	    if (unit->side)
+	      notify(unit->side, "%s has insufficient MP to move this turn; waiting until next",
+				 unit_handle(unit->side, unit));
+	    set_unit_reserve(unit->side, unit, TRUE, FALSE);
+	    return TASK_IS_INCOMPLETE;
+	}
     }
     return TASK_FAILED;
 }
@@ -879,7 +952,8 @@ do_occupy_task(unit, task)
     }
 }
 
-/* Wait around for a particular unit.  Give up if the unit is not forthcoming. */
+/* Wait around for a particular unit.  Give up if the unit is not
+   forthcoming. */
 
 static TaskOutcome
 do_pickup_task(unit, task)
@@ -890,7 +964,7 @@ Task *task;
 
     if (!in_play(occupant))
       return TASK_FAILED;
-    wake_unit(occupant, FALSE, 0, NULL);
+    wake_unit(occupant->side, occupant, FALSE);
     if (occupant->transport == unit) {
 	return TASK_IS_COMPLETE;
     } else if (task->execnum > 10) {
@@ -919,45 +993,23 @@ Task *task;
     int m, tot, sofar, amt;
 
     m = task->args[0];
+    if (!is_material_type(m)) {
+	/* This may be an indication of code bogosity; warn? */
+	return TASK_FAILED;
+    }
     tot = task->args[1];
     sofar = task->args[2];
     if (sofar >= tot)
       return TASK_IS_COMPLETE;
     amt = um_material_per_production(unit->type, m);
     if (valid(check_produce_action(unit, unit, m, amt))) {
+	task->args[1] += amt;
 	prep_produce_action(unit, unit, m, amt);
 	return TASK_PREPPED_ACTION;
     }
     return TASK_FAILED;
 }
 
-Unit *
-repair_here(x, y)
-int x, y;
-{
-    Unit *unit;
-
-    for_all_stack(x, y, unit) {
-	/* what about allies? */
-	if (unit->side == tmpside && can_carry(unit, tmpunit)) {
-	    /* this should be controlled by doctrine? */
-	    /* shouldn't wake up, should get a new task to "wait up"
-	       or even approach if possible */
-	/*    wake_unit(unit, FALSE, WAKEOWNER, NULL);  */
-	    return unit;
-	}
-	/* should look at occupants in stack too */
-    }
-    return NULL;
-}
-
-int
-repair_test(x, y)
-int x, y;
-{
-    return (repair_here(x, y) != NULL);
-}
-
 static TaskOutcome
 do_repair_task(unit, task)
 Unit *unit;
@@ -972,15 +1024,23 @@ Task *task;
 	    range = min(range, unit->supply[m] / um_consumption_per_move(u, m));
 	}
     }
-    tmpside = unit->side;
     tmpunit = unit;
-    if (unit->hp == u_hp(u)) {  /* what if unit is multi-part? */
+    /* (should use doctrine to decide when repairs sufficient) */
+    if (unit->hp >= (u_hp(u) * 80) / 100) {  /* what if unit is multi-part? */
     	return TASK_IS_COMPLETE;
-    } else if (unit->transport != NULL) {
+    } else if (u_hp_recovery(u) > 0) {
+	if (0 /* too close to enemies */) {
+	    /* (should move away from danger) */
+	} else {
+	    /* Just hang out and wait to get better. */
+	    set_unit_reserve(unit->side, unit, TRUE, FALSE);
+	}
+    	return TASK_IS_INCOMPLETE;
+    } else if (unit->transport != NULL /* and transport repairs */) {
 	set_unit_reserve(unit->side, unit, TRUE, FALSE);
     	return TASK_IS_INCOMPLETE;
     } else if ((unit2 = repair_here(ux, uy)) != NULL
-    	&& unit2 != unit->transport) {	
+	       && unit2 != unit->transport) {	
     	prep_enter_action(unit, unit, unit2);
 	return TASK_PREPPED_ACTION;
     } else if (search_around(ux, uy, range, repair_test, &x, &y, 1)) {
@@ -993,6 +1053,31 @@ Task *task;
     }
 }
 
+Unit *
+repair_here(x, y)
+int x, y;
+{
+    Unit *unit;
+
+    for_all_stack(x, y, unit) {
+	if (unit != tmpunit
+	    && unit_trusts_unit(tmpunit, unit)
+	    && (uu_auto_repair(unit->type, tmpunit->type) > 0
+		|| 0 /* (should allow for explicit repair actions) */)) {
+	    return unit;
+	}
+	/* should look at occupants in stack too */
+    }
+    return NULL;
+}
+
+static int
+repair_test(x, y)
+int x, y;
+{
+    return (repair_here(x, y) != NULL);
+}
+
 static int *lowm = NULL, numlow; 
 
 Unit *
@@ -1012,7 +1097,7 @@ Unit *unit;
 	/* this should be controlled by doctrine? */
 	/* shouldn't wake up, should get a new task to "wait up"
 	   or even approach if possible */
-	/*    wake_unit(unit, FALSE, WAKEOWNER, NULL);  */
+	/*    wake_unit(unit->side, unit, FALSE);  */
 	if (enough)
 	  return unit;
     }
@@ -1038,7 +1123,7 @@ int x, y;
     return NULL;
 }
 
-int
+static int
 resupply_test(x, y)
 int x, y;
 {
@@ -1062,6 +1147,11 @@ Task *task;
     int ux = unit->x, uy = unit->y;
     Unit *unit2;
 
+    /* A unit acting randomly might have gotten this task, even if the game
+       has no materials or the unit no supply.  Pretend that it was OK and
+       get out of here. */
+    if (nummtypes == 0 || unit->supply == NULL)
+      return TASK_IS_COMPLETE;
     tmpside = unit->side;
     tmpunit = unit;
     if (lowm == NULL)
@@ -1139,7 +1229,7 @@ Unit *unit;
 Task *task;
 {
     if (task->args[0] > 0) {
-	unit->plan->reserve = TRUE;
+	set_unit_reserve(unit->side, unit, TRUE, FALSE);
 	--(task->args[0]);
 	return TASK_IS_INCOMPLETE;
     } else {
@@ -1167,13 +1257,23 @@ Unit *unit;
     Plan *plan = unit->plan;
     TaskOutcome rslt;
     Task *task;
+    extern int taskexecs;
 
     /* This should never happen. */
     if (unit->plan == NULL)
       run_error("???");
+    if (need_ai_task_reaction && unit->plan->last_task_outcome != TASK_UNKNOWN)
+      return unit->plan->last_task_outcome;
     task = plan->tasks;
+    ++taskexecs;
     rslt = execute_task_aux(unit, task);
-    DMprintf("%s did task %s: ", unit_desig(unit), task_desig(task));
+    DMprintf("%s did task %s, ", unit_desig(unit), task_desig(task));
+    /* Record it. */
+    memcpy(&(unit->plan->last_task), task, sizeof(Task));
+    unit->plan->last_task_outcome = rslt;
+    if (unit->side != NULL && side_has_ai(unit->side)) {
+	need_ai_task_reaction = TRUE;
+    }
     /* Now look at what happened with task execution. */
     switch (rslt) {
       case TASK_UNKNOWN:
@@ -1210,15 +1310,11 @@ Unit *unit;
 	break;
     }
     DMprintf("\n");
-    /* Give AIs a chance to decide what to do with the result of a task. */
-    if (unit->side != NULL && side_has_ai(unit->side)) {
-    	ai_react_to_task_result(unit->side, unit, task, rslt);
-    }
     return rslt;
 }
 
 /* Perform a single given task. */
-
+/* (should make static) */
 TaskOutcome
 execute_task_aux(unit, task)
 Unit *unit;
@@ -1436,7 +1532,10 @@ CONST void *a0, *a1;
 	return rslt;
     }
     /* Prefer to go over cells that we can change to our side. */
-    if (any_people_surrenders != NULL && any_people_surrenders[u]) {
+    /* (should also do for control-side?) */
+    if (any_people_surrenders != NULL
+	&& any_people_surrenders[u]
+	&& people_sides_defined()) {
     	s = side_number(tmpunit->side);
     	ps0 = people_side_at(xs[i0], ys[i0]);
     	ps1 = people_side_at(xs[i1], ys[i1]);
@@ -1495,18 +1594,21 @@ Task *task;
 {
     int numcleared;
 
+    if (unit->plan == NULL && !completed(unit)) {
+	init_unit_plan(unit);
+    }
     if (!in_play(unit) || unit->plan == NULL) {
 	run_warning("Trying to do %s task with bad %s",
 		    task_desig(task), unit_desig(unit));
 	return;
     }
-    Dprintf("To %s task agenda, add %s",
+    DMprintf("%s add task %s",
 	    unit_desig(unit), task_desig(task));
     if (pos == CLEAR_AGENDA) {
 	numcleared = clear_task_agenda(unit->plan);
-	Dprintf(" (cleared %d existing tasks)", numcleared);
+	DMprintf(" (cleared %d existing tasks)", numcleared);
     }
-    Dprintf("\n");
+    DMprintf("\n");
     switch (pos) {
       case 0:
       case CLEAR_AGENDA:
@@ -1523,9 +1625,7 @@ Task *task;
     /* We're not in reserve. */
     unit->plan->reserve = FALSE;
     /* Presumably we're no longer waiting to be told what to do. */
-    if (unit->plan->waitingfortasks && unit->side)
-      --(unit->side->numwaiting);
-    unit->plan->waitingfortasks = FALSE;
+    set_waiting_for_tasks(unit, FALSE);
     /* Reflect all this on displays. */
     update_unit_display(unit->side, unit, FALSE);
 }
@@ -1543,6 +1643,16 @@ int x, y;
     return task;
 }
 
+/* Give the unit a task to move to a given place, erasing every other task. */
+
+void
+set_move_to_task(unit, x, y)
+Unit *unit;
+int x, y;
+{
+    add_task(unit, CLEAR_AGENDA, create_move_to_task(x, y));
+}
+
 void
 push_move_to_task(unit, x, y)
 Unit *unit;
@@ -1555,16 +1665,6 @@ int x, y;
     add_task(unit, 0, create_move_to_task(x, y));
 }
 
-/* Give the unit a task to move to a given place, erasing every other task. */
-
-void
-set_move_to_task(unit, x, y)
-Unit *unit;
-int x, y;
-{
-    add_task(unit, CLEAR_AGENDA, create_move_to_task(x, y));
-}
-
 Task *
 create_move_near_task(x, y, dist)
 int x, y, dist;
@@ -1630,6 +1730,14 @@ int u2, run;
     return task;
 }
 
+void
+set_build_task(unit, u2, run)
+Unit *unit;
+int u2, run;
+{
+    add_task(unit, CLEAR_AGENDA, create_build_task(u2, run));
+}
+
 void
 push_build_task(unit, u2, run)
 Unit *unit;
@@ -1659,9 +1767,8 @@ int u2, n;
     add_task(unit, 0, create_research_task(u2, n));
 }
 
-void
-set_hit_task(unit, x, y)
-Unit *unit;
+Task *
+create_hit_task(x, y)
 int x, y;
 {
     Task *task = create_task(TASK_HIT_UNIT);
@@ -1669,46 +1776,51 @@ int x, y;
     task->args[0] = x;  task->args[1] = y;
     task->args[2] = NONUTYPE;
     task->args[3] = -1;
-    add_task(unit, CLEAR_AGENDA, task);
+    return task;
 }
 
 void
-push_specific_hit_task(unit, x, y, u, s)
+set_hit_task(unit, x, y)
 Unit *unit;
-int x, y, u, s;
+int x, y;
 {
-    Task *task = create_task(TASK_HIT_UNIT);
-
-    task->args[0] = x;  task->args[1] = y;
-    task->args[2] = u;
-    task->args[3] = s;
-    add_task(unit, 0, task);
-}
-
-void
-set_specific_hit_task(unit, x, y, u, s)
-Unit *unit;
-int x, y, u, s;
-{
-    Task *task = create_task(TASK_HIT_UNIT);
-
-    task->args[0] = x;  task->args[1] = y;
-    task->args[2] = u;
-    task->args[3] = s;
-    add_task(unit, CLEAR_AGENDA, task);
+    add_task(unit, CLEAR_AGENDA, create_hit_task(x, y));
 }
 
 void
 push_hit_task(unit, x, y)
 Unit *unit;
 int x, y;
+{
+    add_task(unit, 0, create_hit_task(x, y));
+}
+
+Task *
+create_specific_hit_task(x, y, u, s)
+int x, y, u, s;
 {
     Task *task = create_task(TASK_HIT_UNIT);
 
     task->args[0] = x;  task->args[1] = y;
-    task->args[2] = NONUTYPE;
-    task->args[3] = -1;
-    add_task(unit, 0, task);
+    task->args[2] = u;
+    task->args[3] = s;
+    return task;
+}
+
+void
+set_specific_hit_task(unit, x, y, u, s)
+Unit *unit;
+int x, y, u, s;
+{
+    add_task(unit, CLEAR_AGENDA, create_specific_hit_task(x, y, u, s));
+}
+
+void
+push_specific_hit_task(unit, x, y, u, s)
+Unit *unit;
+int x, y, u, s;
+{
+    add_task(unit, 0, create_specific_hit_task(x, y, u, s));
 }
 
 Task *
@@ -1746,6 +1858,19 @@ Unit *unit;
     add_task(unit, CLEAR_AGENDA, create_task(TASK_DISBAND));
 }
 
+Task *
+create_repair_task()
+{
+    return create_task(TASK_REPAIR);
+}
+
+void
+set_repair_task(unit)
+Unit *unit;
+{
+    add_task(unit, CLEAR_AGENDA, create_repair_task());
+}
+
 Task *
 create_resupply_task(m)
 int m;
@@ -1894,12 +2019,11 @@ int *xp, *yp;
     *xp = strtol(arg, &arg2, 10);
     if (arg != arg2 && *arg2 == ',') {
 	*yp = strtol(arg2 + 1, &arg, 10);
-	if (arg2 + 1 != arg) {
+	if (arg2 + 1 != arg)
 	    return TRUE;
-	} else if ((unit = parse_unit(side, arg)) != NULL) {
-	    *xp = unit->x;  *yp = unit->y;
-	    return TRUE;
-	}
+    } else if ((unit = parse_unit(side, arg)) != NULL) {
+	*xp = unit->x;  *yp = unit->y;
+	return TRUE;
     }
     notify(side, "location \"%s\" not recognized", arg);
     return FALSE;
@@ -1915,14 +2039,15 @@ Side *side;
 char *str;
 Task **taskp;
 {
-    int tasktype, i, x, y, n, dir, u, taskargs[MAXTASKARGS], numargs, rslt;
+    int tasktype, i, x, y, n, dir, u, taskargs[MAXTASKARGS], numargs;
     char *arg, *arg2, substr[BUFSIZE], *rest, *argtypes;
     Unit *unit;
 
+    *taskp = NULL;
     rest = get_next_arg(str, substr, &arg);
     /* Recognize special cases of task types first. */
     if (strcmp(arg, "nil") == 0 || strcmp(arg, "nothing") == 0) {
-	/* NULL task but non-NULL return indicates order cancellation. */
+	/* NULL task with non-NULL return indicates order cancellation. */
 	*taskp = NULL;
 	return rest;
     } else if (strcmp(arg, "move-near") == 0) {
