@@ -14,6 +14,9 @@ extern Task *create_specific_hit_task PARAMS ((int x, int y, int u, int s));
 #include "kernel.h"
 #include "kpublic.h"
 
+void flush_outgoing_queue PARAMS ((void));
+void flush_incoming_queue PARAMS ((void));
+
 extern long randstate;
 
 /* Iteration over all remote programs, by rid. */
@@ -51,6 +54,8 @@ static void broadcast_layer_change PARAMS ((char *layername, Side *side,
 					    int x, int y,
 					    int a1, int a2, int a3));
 static void broadcast_packet PARAMS ((char *buf));
+static void save_outgoing_packet PARAMS ((int id, char *inbuf));
+static void save_incoming_packet PARAMS ((int id, char *inbuf));
 static void remove_chars PARAMS ((char *buf, int n));
 static void receive_net_message PARAMS ((char *str));
 static void receive_command PARAMS ((char *str));
@@ -85,6 +90,8 @@ int expecting_ack;
 
 int timeout_warnings = TRUE;
 
+int sendnow;
+
 /* Flag indicating that we're in the middle of downloading. */
 
 int downloading;
@@ -683,8 +690,12 @@ Goal *goal;
 	set_unit_main_goal(side, unit, goal);
     }
     if (numremotes > 0) {
-	broadcast_unit_property_5(side, unit, "maingoal", goal->type, goal->args[0],
-				  goal->args[1], goal->args[2], goal->args[3]);
+	broadcast_unit_property_5(side, unit, "maingoal",
+				  (goal ? goal->type : 0),
+				  (goal ? goal->args[0] : 0),
+				  (goal ? goal->args[1] : 0),
+				  (goal ? goal->args[2] : 0),
+				  (goal ? goal->args[3] : 0));
     }
 }
 
@@ -1537,8 +1548,11 @@ char *inbuf;
     int i, j, csum, numtimeouts;
     char buf[BUFSIZE];
 
-    if (my_rid != master_rid) {
-	receive_data(0);
+    if (my_rid != master_rid && !sendnow) {
+	/* Add packet to the outgoing queue. */
+	save_outgoing_packet(id, inbuf);
+	Dprintf("OutQ: %d \"%s\"\n", id, inbuf);
+	return;
     }
     Dprintf("Send: %d \"%s\"\n", id, inbuf);
     j = 0;
@@ -1597,14 +1611,80 @@ char *inbuf;
     return TRUE;
 }
 
+struct q_entry {
+  int id;
+  char *buf;
+  struct q_entry *next;
+} *outgoing, *outgoing_last, *incoming, *incoming_last;
+
+static void
+save_outgoing_packet(id, inbuf)
+int id;
+char *inbuf;
+{
+    struct q_entry *entry;
+
+    entry = (struct q_entry *) xmalloc (sizeof (struct q_entry));
+    entry->id = id;
+    entry->buf = copy_string(inbuf);
+    if (outgoing_last) {
+	outgoing_last->next = entry;
+    } else {
+	outgoing = entry;
+    }
+    outgoing_last = entry;
+}
+
+void
+flush_outgoing_queue()
+{
+    struct q_entry *entry;
+
+    if (my_rid != master_rid) {
+	sendnow = TRUE;
+	for (entry = outgoing; entry != NULL; entry = entry->next) {
+	    send_packet(entry->id, entry->buf);
+	}
+	outgoing = outgoing_last = NULL;
+	sendnow = FALSE;
+    }
+}
+
+static void
+save_incoming_packet(id, inbuf)
+int id;
+char *inbuf;
+{
+    struct q_entry *entry;
+
+    entry = (struct q_entry *) xmalloc (sizeof (struct q_entry));
+    entry->id = id;
+    entry->buf = copy_string(inbuf);
+    if (incoming_last) {
+	incoming_last->next = entry;
+    } else {
+	incoming = entry;
+    }
+    incoming_last = entry;
+}
+
+void
+flush_incoming_queue()
+{
+    struct q_entry *entry;
+
+    for (entry = incoming; entry != NULL; entry = entry->next) {
+	receive_packet(entry->id, entry->buf);
+    }
+    incoming = incoming_last = NULL;
+}
+
 static char *packetbuf;
 
 static char *rsltbuf;
 
 static int rsltid;
 
-static char *crossbuf;
-
 #define PACKETBUFSIZE 1000
 
 static int nothing_count;
@@ -1634,12 +1714,8 @@ int timeout;
       packetbuf = xmalloc(PACKETBUFSIZE);
     if (rsltbuf == NULL)
       rsltbuf = xmalloc(BUFSIZE);
-    if (crossbuf == NULL)
-      crossbuf = xmalloc(BUFSIZE);
-    if (!empty_string(crossbuf) && !expecting_ack) {
-	strcpy(rsltbuf, crossbuf);
-	crossbuf[0] = '\0';
-	receive_packet(rsltid, rsltbuf);
+    if (incoming != NULL && !expecting_ack) {
+	flush_incoming_queue();
 	return;
     }
     while (retry > 0) {
@@ -1686,9 +1762,7 @@ int timeout;
 		}
 		if (expecting_ack) {
 		    Dprintf("Packet received while expecting ack, saving \"%s\"\n", rsltbuf);
-		    if (!empty_string(crossbuf))
-		      run_warning("Overwriting saved packet \"%s\" with \"%s\"", crossbuf, rsltbuf);
-		    strcpy(crossbuf, rsltbuf);
+		    save_incoming_packet(rsltid, rsltbuf);
 		    Dprintf("Sending ack to unexpected packet\n");
 		    low_send(rsltid, "+");
 		    goto foo;
@@ -2216,12 +2290,16 @@ char *str;
 
 	str += 9;
 	val = strtol(str, &nstr, 10);
-	str = nstr;
-	goal = create_goal(val, side, TRUE);
-	for (i = 0; i < 4; ++i) {
-	    val = strtol(str, &nstr, 10);
+	if (val > 0) {
 	    str = nstr;
-	    goal->args[i] = val;
+	    goal = create_goal(val, side, TRUE);
+	    for (i = 0; i < 4; ++i) {
+		val = strtol(str, &nstr, 10);
+		str = nstr;
+		goal->args[i] = val;
+	    }
+	} else {
+	    goal = NULL;
 	}
 	set_unit_main_goal(side, unit, goal);
     } else if (strncmp(str, "name ", 5) == 0) {
