@@ -18,6 +18,7 @@ any later version.  See the file COPYING.  */
 #include "kpublic.h"
 #include "imf.h"
 #include "ui.h"
+extern void generic_do_doctrine PARAMS ((Side *side, char *str));
 
 #ifdef MAC /* temporary */
 #include <Types.h>
@@ -29,10 +30,10 @@ extern int error_is_bug;
 static void compute_q PARAMS ((void));
 static void calc_view_misc PARAMS ((VP *vp));
 static int blocking_utype PARAMS ((int u, int block));
-
-static ImageFamily *get_generic_images PARAMS ((Side *side, char *name, 
-					      void (*interp_hook)(ImageFamily *imf),
-					      void (*load_hook)(ImageFamily *imf)));
+static ImageFamily *get_generic_images PARAMS ((Side *side, char *name));
+static ImageFamily *add_default_terrain_image PARAMS ((ImageFamily *imf, int t));
+static ImageFamily *add_default_unit_image PARAMS ((ImageFamily *imf, int u));
+static ImageFamily *add_default_emblem_image PARAMS ((ImageFamily *imf, int s2));
 
 /* The two games that should be always be available. */
 
@@ -1251,11 +1252,30 @@ void (*text_callback) PARAMS ((int x1, int y1, int x1f, int y1f, char *str));
 		}
 		if (text_callback) {
 		    if (lon == lonmin || lon == lonmax) {
-			latlong_desc(tmpbuf, x1, y1, x1f, y1f, 2);
+			char minbuf[10];
+    			int latdeg, latminu;
+
+			latdeg = abs(lat) / 60;
+			latminu = abs(lat) % 60;
+			minbuf[0] = '\0';
+			if (latminu != 0)
+			  sprintf(minbuf, "%dm", latminu);
+			sprintf(tmpbuf, "%dd%s%c",
+				latdeg, minbuf, (lat >= 0 ? 'N' : 'S'));
+
 			(*text_callback)(x1, y1, x1f, y1f, tmpbuf);
 		    }
 		    if (lat == latmin || lat == latmax) {
-			latlong_desc(tmpbuf, x1, y1, x1f, y1f, 1);
+			char minbuf[10];
+    			int londeg, lonminu;
+
+			londeg = abs(lon) / 60;
+			lonminu = abs(lon) % 60;
+			minbuf[0] = '\0';
+			if (lonminu != 0)
+			  sprintf(minbuf, "%dm", lonminu);
+			sprintf(tmpbuf, "%dm%s%c",
+				londeg, minbuf, (lon >= 0 ? 'E' : 'W'));
 			(*text_callback)(x1, y1, x1f, y1f, tmpbuf);
 		    }
 		}
@@ -2096,11 +2116,9 @@ Side *side;
 static short imf_dir_loaded;
 
 static ImageFamily *
-get_generic_images(side, name, interp_hook, load_hook)
+get_generic_images(side, name)
 Side *side;
 char *name;
-void (*interp_hook) PARAMS ((ImageFamily *imf));
-void (*load_hook) PARAMS ((ImageFamily *imf));
 {
     FILE *fp;
     int cloned = FALSE;
@@ -2109,19 +2127,10 @@ void (*load_hook) PARAMS ((ImageFamily *imf));
     imf = get_imf(name);
     if (imf == NULL)
       return NULL;
-    if (imf->numsizes > 0)
-      if (interp_hook != NULL) {
-	  imf = clone_imf(imf);
-	  cloned = TRUE;
-	  (*interp_hook)(imf);
-      }
-    if (load_hook != NULL) {
-	if (!cloned) {
-	    imf = clone_imf(imf);
-	    cloned = TRUE;
-	}
-	(*load_hook)(imf);
-    }
+    if (imf->numsizes > 0 && imf_interp_hook != NULL)
+      imf = (*imf_interp_hook)(imf, NULL, FALSE);
+    if (imf_load_hook != NULL)
+      imf = (*imf_load_hook)(imf);
     if (imf->numsizes == 0) {
 	/* Maybe collect the names/locations of all image families. */
 	if (!imf_dir_loaded) {
@@ -2148,24 +2157,18 @@ void (*load_hook) PARAMS ((ImageFamily *imf));
 	    } else {
 		/* complain here, or not? */
 	    }
-	    if (interp_hook != NULL) {
-		imf = clone_imf(imf);
-		cloned = TRUE;
-		(*interp_hook)(imf);
-	    }
+	    if (imf_interp_hook != NULL)
+	      imf = (*imf_interp_hook)(imf, NULL, FALSE);
 	}
     }
     return imf;
 }
 
 ImageFamily *
-get_unit_type_images(side, u, stylepref, interp_hook, load_hook, default_hook)
+get_unit_type_images(side, u, stylepref)
 Side *side;
 int u;
 char *stylepref;
-void (*interp_hook) PARAMS ((ImageFamily *imf));
-void (*load_hook) PARAMS ((ImageFamily *imf));
-void (*default_hook) PARAMS ((ImageFamily *imf, int n, char *name));
 {
     char *name, styledname[BUFSIZE];
     ImageFamily *imf;
@@ -2174,32 +2177,64 @@ void (*default_hook) PARAMS ((ImageFamily *imf, int n, char *name));
       name = u_image_name(u);
     else
       name = u_internal_name(u);
+    /* Look for an image in the given style, such as "c" for color
+       or "s" for silhouette, using <name>-<style> as the name, and
+       return that image if found. */
     if (!empty_string(stylepref)) {
 	strcpy(styledname, name);
 	strcat(styledname, "-");
 	strcat(styledname, stylepref);
-	imf = get_generic_images(side, styledname, interp_hook, load_hook);
-	if (imf != NULL && imf->numsizes > 0 && default_hook != NULL) {
+	imf = get_generic_images(side, styledname);
+	if (imf != NULL && imf->numsizes > 0) {
 	    record_imf_get(imf);
 	    return imf;
 	}
     }
-    imf = get_generic_images(side, name, interp_hook, load_hook);
-    if (imf != NULL && imf->numsizes == 0 && default_hook != NULL) {
+    imf = get_generic_images(side, name);
+    if (imf != NULL && imf->numsizes == 0) {
 	imf->ersatz = TRUE;
-	(*default_hook)(imf, u, u_type_name(u));
+	imf = add_default_unit_image(imf, u);
     }
     record_imf_get(imf);
     return imf;
 }
 
+static ImageFamily *
+add_default_unit_image(ImageFamily *imf, int u)
+{
+    int i, hi, lo;
+    Image *img;
+
+    img = get_img(imf, 16, 16);
+    if (img == NULL)
+      return imf;
+    img->rawmonodata = xmalloc(32);
+    img->rawmaskdata = xmalloc(32);
+    hi = u >> 4;
+    lo = (u & 0xf) << 4;
+    for (i = 4; i < 28; i += 2) {
+	(img->rawmonodata)[i] = hi;
+	(img->rawmonodata)[i + 1] = lo;
+	(img->rawmaskdata)[i] = 0x7f;
+	(img->rawmaskdata)[i + 1] = 0xfe;
+    }
+    (img->rawmonodata)[14] = 0x1f;
+    (img->rawmonodata)[15] = 0xf8;
+    (img->rawmonodata)[16] = 0x1f;
+    (img->rawmonodata)[17] = 0xf8;
+    (img->rawmaskdata)[2] = 0x7f;
+    (img->rawmaskdata)[3] = 0xfe;
+    (img->rawmaskdata)[28] = 0x7f;
+    (img->rawmaskdata)[29] = 0xfe;
+    if (imf_interp_hook)
+      imf = (*imf_interp_hook)(imf, NULL, FALSE);
+    return imf;
+}
+
 ImageFamily *
-get_terrain_type_images(side, t, interp_hook, load_hook, default_hook)
+get_terrain_type_images(side, t)
 Side *side;
 int t;
-void (*interp_hook) PARAMS ((ImageFamily *imf));
-void (*load_hook) PARAMS ((ImageFamily *imf));
-void (*default_hook) PARAMS ((ImageFamily *imf, int n, char *name));
 {
     char *name;
     ImageFamily *imf;
@@ -2208,25 +2243,44 @@ void (*default_hook) PARAMS ((ImageFamily *imf, int n, char *name));
       name = t_image_name(t);
     else
       name = t_type_name(t);
-    imf = get_generic_images(side, name, interp_hook, load_hook);
-    if (imf != NULL && imf->numsizes == 0 && default_hook != NULL) {
+    imf = get_generic_images(side, name);
+    if (imf != NULL && imf->numsizes == 0) {
 	imf->ersatz = TRUE;
-	(*default_hook)(imf, t, t_type_name(t));
+	imf = add_default_terrain_image(imf, t);
     }
     record_imf_get(imf);
     return imf;
 }
 
+/* The default terrain image is ugly but functional; basically a binary
+   encoding of the terrain type number. */
+
+static ImageFamily *
+add_default_terrain_image(imf, t)
+ImageFamily *imf;
+int t;
+{
+    Image *img;
+
+    img = get_img(imf, 8, 8);
+    img->istile = TRUE;
+    img->rawmonodata = xmalloc(8);
+    (img->rawmonodata)[1] = (t << 2);
+    (img->rawmonodata)[2] = (t << 2);
+    (img->rawmonodata)[3] = 0x7e;
+    (img->rawmonodata)[4] = (t << 2);
+    (img->rawmonodata)[5] = (t << 2);
+    if (imf_interp_hook)
+      imf = (*imf_interp_hook)(imf, NULL, FALSE);
+    return imf;
+}
+
 ImageFamily *
-get_unseen_images(side, interp_hook, load_hook, default_hook)
+get_unseen_images(side)
 Side *side;
-void (*interp_hook) PARAMS ((ImageFamily *imf));
-void (*load_hook) PARAMS ((ImageFamily *imf));
-void (*default_hook) PARAMS ((ImageFamily *imf, char *name));
 {
     if (!empty_string(g_unseen_image_name())) {
-	unseen_image = get_generic_images(side, g_unseen_image_name(),
-					  interp_hook, load_hook); 
+	unseen_image = get_generic_images(side, g_unseen_image_name());
 	if (unseen_image != NULL && unseen_image->numsizes == 0) {
 	    /* Appears to have failed - clear the unseen image then. */
 	    unseen_image = NULL;
@@ -2239,11 +2293,8 @@ void (*default_hook) PARAMS ((ImageFamily *imf, char *name));
 }
 
 ImageFamily *
-get_emblem_images(side, side2, interp_hook, load_hook, default_hook)
+get_emblem_images(side, side2)
 Side *side, *side2;
-void (*interp_hook) PARAMS ((ImageFamily *imf));
-void (*load_hook) PARAMS ((ImageFamily *imf));
-void (*default_hook) PARAMS ((ImageFamily *imf, int n, char *name));
 {
     char *name, tmpbuf[BUFSIZE];
     int s2 = side_number(side2);
@@ -2256,18 +2307,42 @@ void (*default_hook) PARAMS ((ImageFamily *imf, int n, char *name));
 	sprintf(tmpbuf, "s%d", s2);
 	name = copy_string(tmpbuf);
     }
-    imf = get_generic_images(side, name, interp_hook, load_hook);
-    /* If no images acquired, invoke the default image getter if possible. */
-    if (imf != NULL
-	&& imf->numsizes == 0
-	&& default_hook != NULL
-	&& strcmp(name, "none") != 0) {
-	(*default_hook)(imf, s2, name);
+    imf = get_generic_images(side, name);
+    /* If we must have an image, and none were acquired, invoke the default
+       image getter. */
+    if (imf != NULL && imf->numsizes == 0 && strcmp(name, "none") != 0) {
+	imf->ersatz = TRUE;
+	imf = add_default_emblem_image(imf, s2);
     }
     record_imf_get(imf);
     return imf;
 }
 
+/* Make a solid white square with stripes at top and bottom, with the encoding
+   of the side number in the middle. */
+
+static ImageFamily *
+add_default_emblem_image(ImageFamily *imf, int s2)
+{
+    int i;
+    Image *img;
+
+    img = get_img(imf, 8, 8);
+    if (img == NULL)
+      return imf;
+    img->rawmonodata = xmalloc(8);
+    img->rawmaskdata = xmalloc(8);
+    for (i = 0; i < 8; ++i) {
+	(img->rawmonodata)[i] = s2;
+	(img->rawmaskdata)[i] = 0xff;
+    }
+    (img->rawmonodata)[0] = 0xff;
+    (img->rawmonodata)[7] = 0xff;
+    if (imf_interp_hook)
+      imf = (*imf_interp_hook)(imf, NULL, FALSE);
+    return imf;
+}
+
 void
 record_imf_get(imf)
 ImageFamily *imf;
@@ -2484,3 +2559,24 @@ int x, y;
     return NONUTYPE;
 }
 
+/* Generic versions of commands. */
+
+void
+generic_do_doctrine(side, str)
+Side *side;
+char *str;
+{
+    if (str) {
+	if (strcmp(str, "set") == 0) {
+	    notify(side, "Set what?");
+	} else if (strncmp(str, "set ", 4) == 0) {
+	    net_set_doctrine(side, str + 4);
+	} else if (strcmp(str, "help") == 0
+		   || strcmp(str, "?") == 0) {
+	    notify(side, "doctrine set <doct-name> <property> <value>");
+	    notify(side, "\"doctrine\" alone to see current settings");
+	}
+	return;
+    }
+    notify_doctrine(side, str);
+}
