@@ -185,25 +185,10 @@ extern int favored_type PARAMS ((Unit *unit));
 extern void force_global_replan PARAMS ((Side *side));
 
 extern ImageFamily *get_unit_type_images
-  PARAMS ((Side *side, int u, char *stylepref,
-	   void (*interp_hook)(ImageFamily *imf),
-	   void (*load_hook)(ImageFamily *imf),
-	   void (*default_hook)(ImageFamily *imf, int u, char *name)));
-extern ImageFamily *get_terrain_type_images
-  PARAMS ((Side *side, int t,
-	   void (*interp_hook)(ImageFamily *imf),
-	   void (*load_hook)(ImageFamily *imf),
-	   void (*default_hook)(ImageFamily *imf, int u, char *name)));
-extern ImageFamily *get_unseen_images
-  PARAMS ((Side *side,
-	   void (*interp_hook)(ImageFamily *imf),
-	   void (*load_hook)(ImageFamily *imf),
-	   void (*default_hook)(ImageFamily *imf, char *name)));
-extern ImageFamily *get_emblem_images
-  PARAMS ((Side *side, Side *side2,
-	   void (*interp_hook)(ImageFamily *imf),
-	   void (*load_hook)(ImageFamily *imf),
-	   void (*default_hook)(ImageFamily *imf, int u, char *name)));
+  PARAMS ((Side *side, int u, char *stylepref));
+extern ImageFamily *get_terrain_type_images PARAMS ((Side *side, int t));
+extern ImageFamily *get_unseen_images PARAMS ((Side *side));
+extern ImageFamily *get_emblem_images PARAMS ((Side *side, Side *side2));
 
 extern void record_imf_get PARAMS ((ImageFamily *imf));
 
