@@ -1,6 +1,5 @@
 /* Units in Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,12 +7,11 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int change_cell PARAMS ((Unit *unit, int x, int y));
-
-extern void set_unit_position PARAMS ((Unit *unit, int x, int y, int z));
-extern void add_unit_to_stack PARAMS ((Unit *unit, int x, int y));
-extern void remove_unit_from_stack PARAMS ((Unit *unit));
-extern void glimpse_adjacent_terrain PARAMS ((Unit *unit));
+extern void interp_utype_list PARAMS ((short *arr, Obj *lis));
+#include "kernel.h"
+extern int num_sides_allowed PARAMS ((int u));
+extern void wreck_unit PARAMS ((Unit *unit));
+extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
 
 /* This is not a limit, just sets initial allocation of unit objects
    and how many additional to get if more are needed.  This can be
@@ -28,6 +26,8 @@ static void allocate_unit_block PARAMS ((void));
 static int compare_units PARAMS ((Unit *unit1, Unit *unit2));
 static int compare_units_by_keys PARAMS ((const void *e1, const void *e2));
 
+static void leave_cell_aux PARAMS ((Unit *unit, Unit *oldtransport));
+
 static void change_cell_aux PARAMS ((Unit *unit, int x0, int y0, int x, int y));
 
 /* The list of available units. */
@@ -50,11 +50,15 @@ Unit *tmpunit;
 
 #define NUMSHORTBUFS 3
 
-int curshortbuf;
+static int curshortbuf;
 
-char *shortbufs[NUMSHORTBUFS] = { NULL, NULL, NULL };
+static char *shortbufs[NUMSHORTBUFS] = { NULL, NULL, NULL };
 
-char utypenamen[100];
+static char *utypenamen;
+
+char **shortestnames = NULL;
+
+char **shortestgnames = NULL;
 
 char *actorstatebuf = NULL;
 
@@ -215,6 +219,7 @@ int type, makebrains;
     unitlist = newunit;
     /* Init more random slots. */
     newunit->prevx = newunit->prevy = -1;
+    newunit->transport_id = lispnil;
     newunit->aihook = NULL;
     /* Add to the global unit counts. */
     ++numunits;
@@ -230,6 +235,8 @@ Unit *unit;
     unit->tooling = (short *) xmalloc(numutypes * sizeof(short));
 }
 
+/* Set or resize the array of a unit's opinions about each side in a game. */
+
 void
 init_unit_opinions(unit, nsides)
 Unit *unit;
@@ -239,7 +246,7 @@ int nsides;
     short *temp;
 
     temp = NULL;
-    if (u_has_opinions(unit->type)) {
+    if (u_opinion_min(unit->type) != u_opinion_max(unit->type)) {
 	if (unit->opinions != NULL && nsides > numsides) {
 	    temp = unit->opinions;
 	    unit->opinions = NULL;
@@ -293,11 +300,15 @@ Unit *unit;
     if (unit->extras == NULL) {
 	unit->extras = (UnitExtras *) xmalloc(sizeof(UnitExtras));
     }
+    /* Each slot must get the same value as the accessor would return
+       if the structure had not been allocated. */ 
     unit->extras->point_value = -1;
     unit->extras->appear = -1;
-    unit->extras->appearx = -1;
-    unit->extras->appeary = -1;
+    unit->extras->appear_var_x = -1;
+    unit->extras->appear_var_y = -1;
     unit->extras->disappear = -1;
+    unit->extras->priority = -1;
+    unit->extras->sym = lispnil;
     unit->extras->hook = lispnil;
 }
 
@@ -320,11 +331,8 @@ int newtype, reason;
     if (reason >= 0)
       record_event(reason, ALLSIDES, pastunit->id, unit->id);
     /* Decrement viewing coverage of our old type. */
-    if (completed(unit)
-        && (unit->transport == NULL
-	   || uu_occ_can_see(unit->type, unit->transport->type))) {
-	cover_area(unit->side, unit, unit->x, unit->y, -1, -1);
-    }
+    cover_area(unit->side, unit, unit->transport, unit->x, unit->y, -1, -1);
+    all_see_leave(unit, unit->x, unit->y, (unit->transport == NULL));
     /* Do the actual change. */
     unit->type = newtype;
     /* Set the new hp to the same ratio of max as the unit had before.
@@ -354,17 +362,13 @@ int newtype, reason;
     init_unit_actorstate(unit, FALSE);
     init_unit_plan(unit);
     unit->aihook = NULL;
-    /* Increment viewing coverage, if unit is complete and can see out
-       of its transport. */
-    if (completed(unit)
-    	&& (unit->transport == NULL
-	    || uu_occ_can_see(unit->type, unit->transport->type))) {
-	cover_area(unit->side, unit, -1, -1, unit->x, unit->y);
-	/* If vision range is 0, allow glimpses of adjacent cell terrain.
-	   This applies to terrain only, adjacent units cannot be seen. */
-	if (u_vision_range(unit->type) == 0)
-	  glimpse_adjacent_terrain(unit);
-    }
+    /* Increment viewing coverage. */
+    cover_area(unit->side, unit, unit->transport, -1, -1, unit->x, unit->y);
+    /* If vision range is 0, allow glimpses of adjacent cell terrain.
+       This applies to terrain only, adjacent units cannot be seen. */
+    if (u_vision_range(unit->type) == 0)
+      glimpse_adjacent_terrain(unit);
+    all_see_occupy(unit, unit->x, unit->y, (unit->transport == NULL));
     count_loss(oldside, oldtype, (reason == H_UNIT_WRECKED ? combat_loss : other_loss));
     count_gain(unit->side, newtype, other_gain);
     /* Update global counts. */
@@ -385,19 +389,42 @@ int u;
     return 0;
 }
 
+int
+side_owns_occupant(side, unit)
+Side *side;
+Unit *unit;
+{
+    int suboccs = FALSE;
+    Unit *occ;
+
+    if (unit->occupant == NULL)
+      return FALSE;
+    for_all_occupants(unit, occ) {
+	if (occ->side == side)
+	  return TRUE;
+	if (occ->occupant)
+	  suboccs = TRUE;
+    }
+    if (suboccs) {
+	for_all_occupants(unit, occ) {
+	    if (side_owns_occupant(side, occ))
+	      return TRUE;
+	}
+    }
+    return FALSE;
+}
+
 /* A unit occupies a cell by adding itself to the list of occupants.
    It will not occupy a transport even if one is at this position
-   (other code should have taken care of this case already)
-   If something goes wrong, return false.  This routine is heavily used. */
+   (other code should have taken care of this case already) If
+   something goes wrong, return false.  This routine is heavily
+   used. */
 
 int
 enter_cell(unit, x, y)
 Unit *unit;
 int x, y;
 {
-    register int u = unit->type;
-    Unit *prevunit = NULL, *nextunit = NULL;
-
 #ifdef DEBUGGING
     /* Not necessarily an error, but indicative of bugs elsewhere. */
     if (unit->x >= 0 || unit->y >= 0) {
@@ -412,7 +439,8 @@ int x, y;
 	/* Let the unit remain off-world. */
 	return FALSE;
     }
-    if (!can_occupy_cell(unit, x, y)) {
+    if (!can_occupy_cell(unit, x, y)
+	&& !can_occupy_conn(unit, x, y, unit->z)) {
 	run_warning("Cell at %d,%d is too full for %s",
 		    x, y, unit_desig(unit));
 	/* Let the unit remain off-world. */
@@ -539,8 +567,7 @@ int u, x, y;
 Unit *unit3;
 {
     int t = terrain_at(x, y), u2, u3, numthistype = 0, fullness = 0;
-    int tcap, utcap;
-    int numtypes[MAXUTYPES];
+    int tcap, utcap, numtypes[MAXUTYPES];
     Unit *unit2;
 
     tcap = t_capacity(t);
@@ -570,47 +597,71 @@ Unit *unit3;
 
 /* Test whether the given position has a connection that the unit may use. */
 
+int can_occupy_conn_1 PARAMS ((Unit *unit, int nx, int ny, int c));
+
 int
 can_occupy_conn(unit, nx, ny, nz)
 Unit *unit;
 int nx, ny, nz;
 {
-    int c, dir, numhere;
-    Unit *unit2;
+    int c;
 
     if (numconntypes == 0)
       return FALSE;
-    if (nz & 1 == 1) {
+    if ((nz & 1) == 1) {
 	c = nz / 2;
+	if (!t_is_connection(c) || !aux_terrain_defined(c))
+	  run_warning("%s is on an invalid connection type %d?",
+		      unit_desig(unit), c);
+	return can_occupy_conn_1(unit, nx, ny, c);
     } else {
+	/* Test each connection type to see if it will work. */
 	for_all_terrain_types(c) {
-	    if (t_is_connection(c) && aux_terrain_defined(c))
-	      break;
+	    if (t_is_connection(c)
+		&& aux_terrain_defined(c)
+		&& can_occupy_conn_1(unit, nx, ny, c))
+	      return TRUE;
 	}
+	return FALSE;
     }
-    if (!t_is_connection(c) || !aux_terrain_defined(c))
-    	run_warning("%s is on an invalid connection type %d?",
-		    unit_desig(unit), c);
-    if (unit->x == nx && unit->y == ny) {
-	for_all_directions(dir) {
-	    if (connection_at(unit->x, unit->y, dir, c))
-	      break;
-	}
-    } else {
-	dir = closest_dir(nx - unit->x, ny - unit->y);
-    }
-    if (!connection_at(unit->x, unit->y, dir, c))
+}
+
+int
+can_occupy_conn_1(unit, nx, ny, c)
+Unit *unit;
+int nx, ny, c;
+{
+    int u, u2, tcap, utcap, fullness, numthistype, numtypes[MAXUTYPES];
+    Unit *unit2;
+
+    if (!any_connections_at(nx, ny, c))
       return FALSE;
-    numhere = 0;
+    u = unit->type;
+    tcap = t_capacity(c);
+    utcap = ut_capacity_x(u, c);
+    if (tcap <= 0 && utcap <= 0)
+      return FALSE;
+    for_all_unit_types(u2)
+      numtypes[u2] = 0;
+    fullness = 0;
+    numthistype = 0;
     for_all_stack(nx, ny, unit2) {
-	if (unit->z == nz) {
-	    ++numhere;
+	u2 = unit2->type;
+	if (ut_capacity_neg(u2, c))
+	  return FALSE;
+	/* (should also count units assumed to be on connection) */
+	if ((unit2->z & 1) == 1 && (unit2->z >> 1) == c) {
+	    ++numtypes[u2];
+	    if (u2 == u)
+	      ++numthistype;
+	    /* Only count against fullness if exclusive capacity exceeded. */
+	   if (numtypes[u2] > ut_capacity_x(u2, c))
+	     fullness += ut_size(u2, c);
 	}
     }
-    if (numhere + 1 <= ut_capacity_x(unit->type, c))
-      return FALSE;
-    /* (should also calc general fullness) */
-    return FALSE;
+    if (numthistype + 1 <= utcap)
+      return TRUE;
+    return (fullness + ut_size(u, c) <= tcap);
 }
 
 /* Recursive helper to update everybody's position.  This should be one of
@@ -623,7 +674,6 @@ int x, y;
 {
     int u = unit->type;
     Unit *occ;
-    Side *side = unit->side;
 
 #ifdef DEBUGGING
     /* Not necessarily an error, but indicative of bugs elsewhere. */
@@ -636,17 +686,12 @@ int x, y;
       run_error("trying to enter cell outside world");
     /* Actually set the unit position. */
     set_unit_position(unit, x, y, unit->z);
-    /* Increment viewing coverage, if unit is complete and can see out
-       of its transport. */
-    if (completed(unit)
-    	&& (unit->transport == NULL
-	    || uu_occ_can_see(u, unit->transport->type))) {
-	cover_area(unit->side, unit, -1, -1, x, y);
-	/* If vision range is 0, allow glimpses of adjacent cell terrain.
-	   This applies to terrain only, adjacent units cannot be seen. */
-	if (u_vision_range(u) == 0)
-	  glimpse_adjacent_terrain(unit);
-    }
+    /* Increment viewing coverage. */
+    cover_area(unit->side, unit, unit->transport, -1, -1, x, y);
+    /* If vision range is 0, allow glimpses of adjacent cell terrain.
+       This applies to terrain only, adjacent units cannot be seen. */
+    if (u_vision_range(u) == 0)
+      glimpse_adjacent_terrain(unit);
     /* Do for all the occupants too, recursively. */
     for_all_occupants(unit, occ) {
 	enter_cell_aux(occ, x, y);
@@ -852,14 +897,14 @@ Unit *unit;
 	/* Sometimes called twice */
     } else if (transport != NULL) {
 	leave_transport(unit);
-	leave_cell_aux(unit);
+	leave_cell_aux(unit, transport);
 	all_see_leave(unit, ux, uy, FALSE);
 	/* not all_see_cell here because can't see inside transports */
 	update_unit_display(transport->side, transport, TRUE);
     } else {
 	remove_unit_from_stack(unit);
 	/* Now bash the coords. */
-	leave_cell_aux(unit);
+	leave_cell_aux(unit, NULL);
 	/* Now let everybody observe that the unit is gone. */
 	all_see_leave(unit, ux, uy, TRUE);
     }
@@ -869,29 +914,24 @@ Unit *unit;
    trash the old coordinates just in case.  Catches many bugs.  Do
    this for all the occupants as well. */
 
-void
-leave_cell_aux(unit)
-Unit *unit;
+static void
+leave_cell_aux(unit, oldtransport)
+Unit *unit, *oldtransport;
 {
     Unit *occ;
 
-    if (unit->x < 0 && unit->y < 0) {
-	run_warning("unit has already left the cell");
-    }
-    /* Decrement viewing coverage around our about-to-be-old location. */
-    if (completed(unit)
-        && (unit->transport == NULL
-	   || uu_occ_can_see(unit->type, unit->transport->type))) {
-	cover_area(unit->side, unit, unit->x, unit->y, -1, -1);
-    }
+    if (unit->x < 0 && unit->y < 0)
+      run_warning("unit %s has already left the cell", unit_desig(unit));
     /* Stash the old coords. */
     unit->prevx = unit->x;  unit->prevy = unit->y;
     /* Set to a recognizable value. */
     unit->x = -1;  unit->y = -1;
     /* Make any occupants leave too. */
     for_all_occupants(unit, occ) {
-	leave_cell_aux(occ);
+	leave_cell_aux(occ, unit);
     }
+    /* Decrement viewing coverage around our old location. */
+    cover_area(unit->side, unit, oldtransport, unit->prevx, unit->prevy, -1, -1);
 }
 
 /* Disembarking unlinks from the list of passengers only, leaves the unit
@@ -925,10 +965,8 @@ change_cell(unit, x, y)
 Unit *unit;
 int x, y;
 {
-    register int u = unit->type;
     int ux = unit->x, uy = unit->y;
     Unit *transport = unit->transport;
-    Side *side = unit->side;
 
     /* Always check this one, but not necessarily fatal. */
     if (!inside_area(x, y)) {
@@ -992,17 +1030,12 @@ int x0, y0, x, y;
     /* Stash the old coords. */
     unit->prevx = x0;  unit->prevy = y0;
     set_unit_position(unit, x, y, unit->z);
-    /* Change viewing coverage, if unit is complete and can see out
-       of its transport. */
-    if (completed(unit)
-    	&& (unit->transport == NULL
-	    || uu_occ_can_see(u, unit->transport->type))) {
-	cover_area(unit->side, unit, x0, y0, x, y);
-	/* If vision range is 0, allow glimpses of adjacent cell terrain.
-	   This applies to terrain only, adjacent units cannot be seen. */
-	if (u_vision_range(u) == 0)
-	  glimpse_adjacent_terrain(unit);
-    }
+    /* Change viewing coverage. */
+    cover_area(unit->side, unit, unit->transport, x0, y0, x, y);
+    /* If vision range is 0, allow glimpses of adjacent cell terrain.
+       This applies to terrain only, adjacent units cannot be seen. */
+    if (u_vision_range(u) == 0)
+      glimpse_adjacent_terrain(unit);
     /* Do for all the occupants too, recursively. */
     for_all_occupants(unit, occ) {
 	change_cell_aux(occ, x0, y0, x, y);
@@ -1222,11 +1255,11 @@ int reason;
        viewing coverage gets left on or off inadvertantly. */
     if (alive(unit) && inside_area(ux, uy)) {
 	/* Uncover the current viewed area. */
-	cover_area(unit->side, unit, ux, uy, -1, -1);
+	cover_area(unit->side, unit, unit->transport, ux, uy, -1, -1);
 	/* Actually set the side slot of the unit here. */
 	set_unit_side(unit, newside);
 	/* Cover it for the new side now. */
-	cover_area(unit->side, unit, -1, -1, ux, uy);
+	cover_area(unit->side, unit, unit->transport, -1, -1, ux, uy);
 	/* A freebie for the unit's previous side. */
 	see_exact(oldside, ux, uy);
     }
@@ -1316,6 +1349,22 @@ Side *side;
     return side->uavail[u];
 }
 
+int
+num_sides_allowed(u)
+int u;
+{
+    int rslt;
+    Side *side;
+
+    rslt = 0;
+    for_all_sides_plus_indep(side) {
+	if (type_allowed_on_side(u, side)) {
+	    ++rslt;
+	}
+    }
+    return rslt;
+}
+
 int
 unit_trusts_unit(unit1, unit2)
 Unit *unit1, *unit2;
@@ -1340,6 +1389,7 @@ set_unit_side(unit, side)
 Unit *unit;
 Side *side;
 {
+    int u = unit->type;
     Side *oldside, *newside;
 
     if (side == indepside)
@@ -1348,9 +1398,9 @@ Side *side;
 	/* Subtract from the counts for the ex-side. */
 	oldside = (unit->side ? unit->side : indepside);
 	if (oldside->numunits)
-	  --(oldside->numunits[unit->type]);
+	  --(oldside->numunits[u]);
 	if (oldside->numlive && in_play(unit))
-	  --(oldside->numlive[unit->type]);
+	  --(oldside->numlive[u]);
 	/* Set the unit's slot. */
 	/* Note that indep units have a NULL side, even though there
 	   is an actual side object for independents. */
@@ -1361,12 +1411,18 @@ Side *side;
 	insert_unit(newside->unithead, unit);
 	/* Add to counts for the side. */
 	if (newside->numunits)
-	  ++(newside->numunits[unit->type]);
+	  ++(newside->numunits[u]);
 	if (newside->numlive && in_play(unit))
-	  ++(newside->numlive[unit->type]);
+	  ++(newside->numlive[u]);
 	/* Invalidate both sides' point value caches. */
 	oldside->point_value_valid = FALSE;
 	newside->point_value_valid = FALSE;
+	/* Bump the tech level if owning this type helps. */
+	if (side != NULL
+	    && side->tech[u] < u_tech_from_ownership(u)) {
+	    side->tech[u] = u_tech_from_ownership(u);
+	    /* (should update any displays of tech - how to ensure?) */
+	}
     }
     return TRUE;
 }
@@ -1406,6 +1462,56 @@ char *newname;
     /* (should also send to any other side directly viewing this unit!) */
 }
 
+void
+change_morale(unit, sign, morchange)
+Unit *unit;
+int morchange;
+{
+    int u = unit->type, oldmorale;
+
+    if (morchange != 0) {
+	oldmorale = unit->morale;
+	unit->morale += (sign * prob_fraction(morchange));
+	if (unit->morale < 0)
+	  unit->morale = 0;
+	if (unit->morale > u_morale_max(u))
+	  unit->morale = u_morale_max(u);
+	if (unit->morale != oldmorale) {
+	    update_unit_display(unit->side, unit, TRUE);
+	    /* (should also send to any other side directly viewing this unit?) */
+	}
+    }
+}
+
+int
+disband_unit(side, unit)
+Side *side;
+Unit *unit;
+{
+    int rslt;
+
+#ifdef DESIGNERS
+    if (side->designer) {
+	return designer_disband(unit);
+    }
+#endif /* DESIGNERS */
+    if (side_can_disband(side, unit)) {
+	rslt = disband_unit_directly(side, unit);
+	if (rslt) {
+	    /* Nothing to do */
+	} else if (unit->plan) {
+	    set_disband_task(unit);
+	} else {
+	    /* In order for this to work, we would need a way to direct one
+	       sort of unit to disband another.  Just fail for now. */
+	    return FALSE;
+	}
+	return TRUE;
+    } else {
+	return FALSE;
+    }
+}
+
 int
 disband_unit_directly(side, unit)
 Side *side;
@@ -1424,6 +1530,21 @@ Unit *unit;
     }
 }
 
+void
+wreck_unit(unit)
+Unit *unit;
+{
+    int u = unit->type, nu;
+
+    /* Change the unit's type at its new location. */
+    change_unit_type(unit, u_wrecked_type(u), H_UNIT_WRECKED);
+    nu = unit->type;
+    /* Restore to default hp for the new type. */
+    unit->hp = unit->hp2 = u_hp(nu);
+    /* Get rid of occupants if now overfull. */
+    eject_excess_occupants(unit);
+}
+
 /* Remove a unit from play.  This is different from making it available for
    reallocation - only the unit flusher can do that.  We remove all the
    passengers too, recursively.  Sometimes units are "killed twice", so
@@ -1485,9 +1606,6 @@ int reason;
     if (side != NULL) {
 	/* Invalidate the side's point value cache. */
 	side->point_value_valid = FALSE;
-	if (side_has_ai(side)) {
-	    ai_react_to_unit_loss(side, unit);
-	}
 	update_unit_display(side, unit, TRUE);
     }
     /* Kill all the occupants in turn. */
@@ -1522,6 +1640,8 @@ flush_dead_units()
 	delete_unit(unit);
 	flush_one_unit(unit);
 	unit = nextunit;
+	if (unit == NULL)
+	  break;
     }
     unitlist = unit;
     /* Since the first unit of unitlist is guaranteed live now,
@@ -1567,7 +1687,8 @@ Unit *unit;
    sure that average performance in real games is what's being improved. */
 
 void
-sort_units()
+sort_units(byidonly)
+int byidonly;
 {
     int flips;
     int passes = 0;
@@ -1581,7 +1702,8 @@ sort_units()
 	    flips = FALSE;
 	    for_all_side_units(side, unit) {
 		if (unit->next != side->unithead
-		    && compare_units(unit, unit->next) > 0) {
+		    && (byidonly ? ((unit->id - unit->next->id) > 0)
+				 : (compare_units(unit, unit->next) > 0))) {
 		    flips = TRUE;
 		    /* Reorder the units by fiddling with their links. */
 		    nextunit = unit->next;
@@ -1662,15 +1784,11 @@ Unit *unit;
 	sprintf(shortbuf, "s%d head", side_number(unit->side));
 	return shortbuf;
     } else if (is_unit_type(unit->type)) {
-	sprintf(shortbuf, "s%d %-3.3s %d (%d,%d",
+	sprintf(shortbuf, "s%d %s %d (%d,%d",
 		side_number(unit->side), shortest_unique_name(unit->type),
 		unit->id, unit->x, unit->y);
 	if (unit->z != 0)
 	  tprintf(shortbuf, ",%d", unit->z);
-	/* This has the potential to generate some very misleading
-	   results, if this routine is called on a unit before its
-	   transport field is patched from a Lisp object into a real
-	   unit reference. */
 	if (unit->transport)
 	  tprintf(shortbuf, ",in%d", unit->transport->id);
 	strcat(shortbuf, ")");  /* close out the unit location */
@@ -1734,66 +1852,179 @@ int u, n;
     	rawname = utypename;
     }
     /* Copy what will fit. */
+    if (utypenamen == NULL)
+      utypenamen = xmalloc(BUFSIZE);
+    if (n > BUFSIZE - 1)
+      n = BUFSIZE - 1;
     strncpy(utypenamen, rawname, n);
     utypenamen[n] = '\0';
     return utypenamen;
 }
 
-char **shortestnames = NULL;
-
 char *
 shortest_unique_name(u)
 int u;
 {
     char namebuf[BUFSIZE], *name1;
     int u1, u2, i, allhavechars, firstuniq[MAXUTYPES], firstuniq1;
+    int shortestdone[MAXUTYPES];
 
     if (shortestnames == NULL) {
 	shortestnames = (char **) xmalloc(numutypes * sizeof(char *));
+	/* First use game definition's single chars if possible. */
 	allhavechars = TRUE;
 	for_all_unit_types(u1) {
+	    shortestdone[u1] = FALSE;
 	    if (!empty_string(u_uchar(u1))) {
 		namebuf[0] = (u_uchar(u1))[0];
 		namebuf[1] = '\0';
 		shortestnames[u1] = copy_string(namebuf);
+		shortestdone[u1] = TRUE;
+		firstuniq[u1] = 0;
 	    } else {
 		allhavechars = FALSE;
 	    }
 	}
 	if (!allhavechars) {
+	    /* Start with copies of full name for all types not
+               already named. */
 	    for_all_unit_types(u1) {
-		shortestnames[u1] = copy_string(u_type_name(u1));
-		firstuniq[u1] = 0;
+		if (shortestnames[u1] == NULL) {
+		    shortestnames[u1] = copy_string(u_type_name(u1));
+		    firstuniq[u1] = 0;
+		}
 	    }
 	    for_all_unit_types(u1) {
-		name1 = shortestnames[u1];
-		firstuniq1 = firstuniq[u1];
-		for_all_unit_types(u2) {
-		    if (u1 != u2) {
-			for (i = 0; i < firstuniq1; ++i ) {
-			    if (name1[i] != (shortestnames[u2])[i]) {
-				break;
+		if (!shortestdone[u1]) {
+		    name1 = shortestnames[u1];
+		    firstuniq1 = firstuniq[u1];
+		    for_all_unit_types(u2) {
+			if (u1 != u2) {
+			    /* Look through the supposedly minimal
+			       unique substring and see if it is the
+			       same. */
+			    for (i = 0; i < firstuniq1; ++i ) {
+				if (name1[i] != (shortestnames[u2])[i]) {
+				    break;
+				}
 			    }
-			}
-			if (i == firstuniq1) {
-			    while (name1[firstuniq1] == (shortestnames[u2])[firstuniq1]) {
-				++firstuniq1;
+			    /* If so, must extend the unique substring. */
+			    if (i == firstuniq1) {
+				/* Look for the first nonmatching char. */
+				while (name1[firstuniq1] == (shortestnames[u2])[firstuniq1]) {
+				    ++firstuniq1;
+				}
 			    }
 			}
 		    }
+		    firstuniq[u1] = firstuniq1;
 		}
-		firstuniq[u1] = firstuniq1;
 	    }
+	    /* For any types where the unique short name is shorter
+	       than the seed name, truncate appropriately. */
 	    for_all_unit_types(u1) {
 		if (firstuniq[u1] + 1 < strlen(shortestnames[u1])) {
 		    (shortestnames[u1])[firstuniq[u1] + 1] = '\0';
 		}
 	    }
 	}
+	if (Debug) {
+	    for_all_unit_types(u1) {
+		Dprintf("Shortest type name: %s for %s\n",
+			shortestnames[u1], u_type_name(u1));
+	    }
+	}
     }
     return shortestnames[u];
 }
 
+/* Similar to shortest_unique_name, but returns a generic name/char
+   instead. */
+
+char *
+shortest_generic_name(u)
+int u;
+{
+    char namebuf[BUFSIZE], *name1;
+    int u1, u2, i, allhavechars, firstuniq[MAXUTYPES], firstuniq1;
+    int shortestdone[MAXUTYPES];
+
+    if (shortestgnames == NULL) {
+	shortestgnames = (char **) xmalloc(numutypes * sizeof(char *));
+	/* First use game definition's single chars if possible. */
+	allhavechars = TRUE;
+	for_all_unit_types(u1) {
+	    shortestdone[u1] = FALSE;
+	    if (!empty_string(u_gchar(u1))) {
+		namebuf[0] = (u_gchar(u1))[0];
+		namebuf[1] = '\0';
+		shortestgnames[u1] = copy_string(namebuf);
+		shortestdone[u1] = TRUE;
+		firstuniq[u1] = 0;
+	    } else if (!empty_string(u_uchar(u1))) {
+		namebuf[0] = (u_uchar(u1))[0];
+		namebuf[1] = '\0';
+		shortestgnames[u1] = copy_string(namebuf);
+		shortestdone[u1] = TRUE;
+		firstuniq[u1] = 0;
+	    } else {
+		allhavechars = FALSE;
+	    }
+	}
+	if (!allhavechars) {
+	    /* Start with copies of full name for all types not
+               already named. */
+	    for_all_unit_types(u1) {
+		if (shortestgnames[u1] == NULL) {
+		    name1 = (!empty_string(u_generic_name(u1)) ? u_generic_name(u1) : u_type_name(u1));
+		    shortestgnames[u1] = copy_string(name1);
+		    firstuniq[u1] = 0;
+		}
+	    }
+	    for_all_unit_types(u1) {
+		if (!shortestdone[u1]) {
+		    name1 = shortestgnames[u1];
+		    firstuniq1 = firstuniq[u1];
+		    for_all_unit_types(u2) {
+			if (u1 != u2) {
+			    /* Look through the supposedly minimal
+			       unique substring and see if it is the
+			       same. */
+			    for (i = 0; i < firstuniq1; ++i ) {
+				if (name1[i] != (shortestgnames[u2])[i]) {
+				    break;
+				}
+			    }
+			    /* If so, must extend the unique substring. */
+			    if (i == firstuniq1) {
+				/* Look for the first nonmatching char. */
+				while (name1[firstuniq1] == (shortestgnames[u2])[firstuniq1]) {
+				    ++firstuniq1;
+				}
+			    }
+			}
+		    }
+		    firstuniq[u1] = firstuniq1;
+		}
+	    }
+	    /* For any types where the unique short name is shorter
+	       than the seed name, truncate appropriately. */
+	    for_all_unit_types(u1) {
+		if (firstuniq[u1] + 1 < strlen(shortestgnames[u1])) {
+		    (shortestgnames[u1])[firstuniq[u1] + 1] = '\0';
+		}
+	    }
+	}
+	if (Debug) {
+	    for_all_unit_types(u1) {
+		Dprintf("Shortest generic type name: %s for %s\n",
+			shortestgnames[u1], u_type_name(u1));
+	    }
+	}
+    }
+    return shortestgnames[u];
+}
+
 /* This formats an actorstate readably. */
 
 char *
@@ -1822,7 +2053,22 @@ int n;
     Unit *unit;
 
     for_all_units(unit) {
-	if (alive(unit) && unit->id == n)
+	if (unit->id == n && alive(unit))
+	  return unit;
+    }
+    return NULL;
+}
+
+/* Same, but don't by picky about liveness. */
+
+Unit *
+find_unit_dead_or_alive(n)
+int n;
+{
+    Unit *unit;
+
+    for_all_units(unit) {
+	if (unit->id == n)
 	  return unit;
     }
     return NULL;
@@ -1860,14 +2106,18 @@ int nb;
     return NULL;
 }
 
+/* Find a unit with the given symbol, either alive or dead. */
+
 Unit *
-find_unit_dead_or_alive(n)
-int n;
+find_unit_by_symbol(sym)
+Obj *sym;
 {
     Unit *unit;
 
+    if (sym == lispnil)
+      return NULL;
     for_all_units(unit) {
-	if (unit->id == n)
+	if (equal(unit_symbol(unit), sym))
 	  return unit;
     }
     return NULL;
@@ -2169,6 +2419,41 @@ char *str;
     return lispnil;
 }
 
+/* This isn't the best place, but read.c getting too large, no
+   overflow file. */
+/* Note that the array is not initialized, allows for multiple calls. */
+
+void
+interp_utype_list(arr, lis)
+short *arr;
+Obj *lis;
+{
+    int u = 0;
+    Obj *rest, *head;
+
+    /* Assume that if the destination array does not exist, there is
+       probably a reason, and it's not our concern. */
+    if (arr == NULL)
+      return;
+    lis = eval(lis);
+    if (!consp(lis))
+      lis = cons(lis, lispnil);
+    if (numberp(car(lis)) && length(lis) == numutypes) {
+	u = 0;
+	for_all_list(lis, rest)
+	  arr[u++] = c_number(car(rest));
+    } else {
+	for_all_list(lis, rest) {
+	    head = car(rest);
+	    if (utypep(head)) {
+		arr[c_number(head)] = TRUE;
+	    } else {
+		/* syntax error */
+	    }
+	}
+    }
+}
+
 #ifdef DESIGNERS
 
 /* A designer can call this to create an arbitrary unit during the game. */
@@ -2197,9 +2482,8 @@ int u, s, x, y;
 	if (can_occupy_cell(newunit, x, y)) {
 	    enter_cell(newunit, x, y);
 	} else {
-	    /* what? */
+	    return NULL;
 	}
-	see_exact(side, x, y);
 	update_cell_display(side, x, y, TRUE);
 	update_unit_display(side, newunit, TRUE);
 	return newunit;
@@ -2208,8 +2492,7 @@ int u, s, x, y;
     }
 }
 
-/* Move a unit to a given location instantly, with all sides observing.
-   This is for use by designers only! */
+/* Move a unit to a given location instantly, with all sides observing. */
 
 int
 designer_teleport(unit, x, y, other)
