@@ -1,6 +1,5 @@
 /* Worlds and areas in Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -9,6 +8,9 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 extern int num_features PARAMS ((void));
+#include "kernel.h"
+
+#include <math.h>
 
 /* The main world structure. */
 
@@ -34,8 +36,11 @@ int numconntypes = 0;
 
 int numcoattypes = 0;
 
-/* (Features could also be used to implement the region 
-   labeling scheme now used only in the machine player) */
+/* This is true if any terrain type has nonzero thickness. */
+
+int any_thickness;
+
+/* This is the head and tail of the list of features. */
 
 Feature *featurelist = NULL;
 
@@ -51,8 +56,13 @@ int maxelev;
 int mintemp;
 int maxtemp;
 
+/* This is true if the temperature ever changes during a game. */
+
 int any_temp_variation = FALSE;
 
+/* This is true if the temperatures in a layer can have different values in
+   different cells. */
+
 int any_temp_variation_in_layer = FALSE;
 
 int minwindforce;
@@ -60,13 +70,18 @@ int maxwindforce;
 
 int any_wind_variation = FALSE;
 
+/* This is true if the winds in a layer can have different values in
+   different cells. */
+
 int any_wind_variation_in_layer = FALSE;
 
+/* This is true if clouds can ever exist. */
+
+int any_clouds = FALSE;
+
 int minclouds;
 int maxclouds;
 
-int any_clouds = FALSE;
-
 /* This is true if it is ever possible to have quantities of material
    in cells. */
 
@@ -74,9 +89,7 @@ int any_materials_in_terrain = FALSE;
 
 static Feature *tmpfeature;
 
-static int tmpint;
-
-static int tmpint2, tmpint3;
+static int tmpint, tmpint2, tmpint3;
 
 /* Clean out all the world and area data. */
 
@@ -87,9 +100,13 @@ init_world()
     /* These default values effectively disable day and year effects. */
     world.daylength = 1;
     world.yearlength = 1;
+    world.daylight_fraction = 50;
+    world.twilight_fraction = 60;
     /* Init the current (default) area. */
     memset(&area, 0, sizeof(Area));
     /* Note especially that area width and height dimensions are now zero. */
+    area.sunx = area.suny = -1;
+    area.temp_year = lispnil;
 }
 
 int
@@ -98,8 +115,8 @@ int circum, warn;
 {
     /* All world circumferences are valid, no checks necessary. */
     world.circumference = circum;
-    world.daylight_width = ((50 * world.circumference) / 100) / 2;
-    world.twilight_width = ((60 * world.circumference) / 100) / 2;
+    world.daylight_width = ((world.daylight_fraction * world.circumference) / 100) / 2;
+    world.twilight_width = ((world.twilight_fraction * world.circumference) / 100) / 2;
     area.xwrap = (world.circumference == area.width);
     return TRUE;
 }
@@ -111,7 +128,7 @@ int width, height, warn;
     if (!valid_area_shape(width, height, warn))
       return FALSE;
     area.width = width;  area.height = height;
-    area.halfheight = area.height / 2;
+    area.halfwidth = area.width / 2;  area.halfheight = area.height / 2;
     area.maxdim = max(area.width, area.height);
     area.xwrap = (world.circumference == area.width);
     area.numcells = area.width * area.height;
@@ -159,6 +176,11 @@ calculate_world_globals()
        Will usually be set already, by players or by module. */
     if (area.width <= 0 && area.height <= 0)
       set_area_shape(DEFAULTWIDTH, DEFAULTHEIGHT, 0);
+    /* Set uninitialized sun position to something reasonable. */
+    if (area.sunx < 0)
+      area.sunx = area.width / 2;
+    if (area.suny < 0)
+      area.suny = area.halfheight;
     minelev = t_elev_min(0);
     maxelev = t_elev_max(0);
     mintemp = t_temp_min(0);
@@ -194,19 +216,22 @@ calculate_world_globals()
       any_temp_variation = TRUE;
     if (minwindforce != maxwindforce)
       any_wind_variation = TRUE;
-    /* (should figure out what this is supposed to mean really) */
     if (minwindforce != maxwindforce)
       any_wind_variation_in_layer = TRUE;
     if (minclouds != maxclouds)
       any_clouds = TRUE;
 }
 
+/* Calculate and cache some info about terrain types.  Not, strictly
+   speaking, part of world data, but no better place for this to go... */
+
 void
 count_terrain_subtypes()
 {
     int t;
 
     numcelltypes = numbordtypes = numconntypes = numcoattypes = 0;
+    any_thickness = FALSE;
     for_all_terrain_types(t) {
 	switch (t_subtype(t)) {
 	  case cellsubtype:
@@ -222,9 +247,13 @@ count_terrain_subtypes()
 	    ++numcoattypes;
 	    break;
 	}
+	if (t_thickness(t) > 0)
+	  any_thickness = TRUE;
     }
 }
 
+/* Do the final setup that the world needs. */
+
 void
 final_init_world()
 {
@@ -355,6 +384,19 @@ allocate_area_people_sides()
     }
 }
 
+/* Allocate a layer indicating the side in control of each cell. */
+
+void
+allocate_area_control_sides()
+{
+    if (!control_sides_defined()) {
+	check_area_shape();
+	area.controlside = malloc_area_layer(char);
+	/* NOCONTROL != 0, so need to blast it over the layer. */
+	memset(area.controlside, NOCONTROL, area.width * area.height);
+    }
+}
+
 /* Set up a cell material layer of the area. */
 
 void
@@ -448,6 +490,13 @@ int x, y;
     return people_side_at(x, y);
 }
 
+int
+fn_control_side_at(x, y)
+int x, y;
+{
+    return control_side_at(x, y);
+}
+
 int
 fn_material_at(x, y)
 int x, y;
@@ -525,6 +574,13 @@ int x, y, val;
     set_people_side_at(x, y, val);
 }
 
+void
+fn_set_control_side_at(x, y, val)
+int x, y, val;
+{
+    set_control_side_at(x, y, val);
+}
+
 void
 fn_set_raw_feature_at(x, y, val)
 int x, y, val;
@@ -843,8 +899,8 @@ void (*fn) PARAMS ((int, int));
 {
     int x, y, x1, y1, x2, y2;
 
-    y1 = limit(y0 - h2);
-    y2 = limit(y0 + h2);
+    y1 = limity(y0 - h2);
+    y2 = limity(y0 + h2);
     for (y = y1; y <= y2; ++y) {
 	if (between(0, y, area.height-1)) {  /* always true? */
 	    /* Compute endpoints of row, but don't wrap or loop will confuse */
@@ -1010,23 +1066,33 @@ Waypoint *waypoints;
 }
 #endif
 
-#if 0	    
-/* Find the type of the border on the given side of the given hex. */
-
 int
-border_at(x, y, dir, t)
-int x, y, dir, t;
+search_straight_line(x0, y0, x1, y1, pred, rxp, ryp)
+int x0, y0, x1, y1, (*pred) PARAMS ((int, int)), *rxp, *ryp;
 {
-    int bord;
+    int i, dist, x, y, xw;
 
-    if (!inside_area(x, y)
-	|| !t_is_border(t)
-	|| !aux_terrain_defined(t))
+    dist = distance(x0, y0, x1, y1);
+    if (dist <= 1)
       return FALSE;
-    bord = aux_terrain_at(x, y, t);
-    return (bord & (1 << dir));
+    for (i = 1; i < dist; ++i) {
+	/* Interpolate to get an x,y */
+	if (x0 == x1)
+	  x = x0;
+	else
+	  x = x0 + (i * (x1 - x0)) / dist;
+	if (y0 == y1)
+	  y = y0;
+	else
+	  y = y0 + (i * (y1 - y0)) / dist;
+	xw = wrapx(x);
+	if (inside_area(x, y) && (*pred)(xw, y)) {
+	    *rxp = xw;  *ryp = y;
+	    return TRUE;
+	}
+    }
+    return FALSE;
 }
-#endif
 
 /* For now, set a bit on both sides of a border. */
 
@@ -1168,13 +1234,18 @@ char *typename, *name;
     return newfeature;
 }
 
+/* Given a feature id, find the corresponding object. */
+
 Feature *
 find_feature(fid)
 int fid;
 {
     Feature *feature;
 
-    for (feature = featurelist; feature != NULL; feature = feature->next) {
+    /* Feature id of 0 is always a non-feature. */
+    if (fid == 0)
+      return NULL;
+    for_all_features(feature) {
 	if (feature->id == fid)
 	  return feature;
     }
@@ -1191,11 +1262,7 @@ int x, y;
       return NULL;
 
     fid = raw_feature_at(x, y);
-    if (fid == 0) {
-	return NULL;
-    } else {
-	return find_feature(fid);
-    }
+    return find_feature(fid);
 }
 
 void
@@ -1224,21 +1291,26 @@ void
 destroy_feature(feature)
 Feature *feature;
 {
-    Feature *tmp, *prev = NULL;
+    Feature *tmp, *prev;
 
     if (feature == NULL)
       return; /* (should be error?) */
-    if (feature == featurelist)
-      featurelist = feature->next;
-    else {
-	for (tmp = featurelist; tmp != NULL; tmp = tmp->next) {
+    if (feature == featurelist) {
+	featurelist = feature->next;
+	if (feature == last_feature)
+	  last_feature = NULL;
+    } else {
+	prev = NULL;
+	for_all_features(tmp) {
     	    if (tmp == feature) {
 		if (prev != NULL)
 		  prev->next = tmp->next;
 		break;
 	    }
+	    prev = tmp;
 	}
-	prev = tmp;
+	if (feature == last_feature)
+	  last_feature = prev;
     }
     /* (should dealloc also?) */
 }
@@ -1254,72 +1326,86 @@ renumber_features()
 
     if (!features_defined())
       return;
-    for (feature = featurelist; feature != NULL; feature = feature->next) {
+    for_all_features(feature) {
     	maxoldfid = max(maxoldfid, feature->id);
     	feature->relabel = newfid++;
     }
     if (maxoldfid > 1000)
       return; /* don't risk it */
     newlabels = (short *) xmalloc((maxoldfid + 1) * sizeof(short));
-    for (feature = featurelist; feature != NULL; feature = feature->next) {
+    for_all_features(feature) {
     	newlabels[feature->id] = feature->relabel;
     }
     for_all_cells(x, y) {
     	set_raw_feature_at(x, y, newlabels[raw_feature_at(x, y)]);
     }
-    for (feature = featurelist; feature != NULL; feature = feature->next) {
+    for_all_features(feature) {
     	feature->id = feature->relabel;
     }
 }
 
+static int
+is_feature_cell(x, y)
+int x, y;
+{
+    return (raw_feature_at(x, y) == tmpint);
+}
+
 void
 compute_all_feature_centroids()
 {
-    int x, y;
-    Feature *feature;
-
-    /* Only do this if features and a feature layer to work with. */
-    if (featurelist == NULL)
-      return;
-    if (!features_defined())
-      return;
-    /* Clear out the features. */
-    for (feature = featurelist; feature != NULL; feature = feature->next) {
-	feature->size = 0;
-	feature->x = feature->y = 0;
-    }
-    for_all_cells(x, y) {
-	feature = feature_at(x, y);
-	if (feature != NULL) {
-	    ++(feature->size);
-	    feature->x += x;  feature->y += y;
-	}
-    }
-    for (feature = featurelist; feature != NULL; feature = feature->next) {
-	if (feature->size > 0) {
-	    feature->x = feature->x / feature->size;
-	    feature->y = feature->y / feature->size;
-	}
-    }
+    compute_feature_centroid(NULL);
 }
 
+/* The same, but for a single feature.  This would be very inefficient to call repeatedly
+   for each feature in the world. */
+
 void
 compute_feature_centroid(feature)
 Feature *feature;
 {
-    int x, y, fid;
+    int x, y, fid, maxdistx, maxdisty, maxdist;
+    Feature *tmpfeature;
 
-    feature->size = 0;
-    feature->x = feature->y = 0;
-    for_all_cells(x, y) {
-	fid = raw_feature_at(x, y);
-	if (feature->id == fid) {
-	    feature->x += x;  feature->y += y;
+    /* Only do this if features and a feature layer to work with. */
+    if (featurelist == NULL || !features_defined())
+      return;
+
+    for_all_features(tmpfeature) {
+	if (feature == NULL || tmpfeature == feature) {
+	    tmpfeature->size = 0;
+	    tmpfeature->x = tmpfeature->y = 0;
+	    tmpfeature->xmin = tmpfeature->ymin = area.maxdim + 1;
+	    tmpfeature->xmax = tmpfeature->ymax = -1;
 	}
     }
-    if (feature->size > 0) {
-	feature->x = feature->x / feature->size;
-	feature->y = feature->y / feature->size;
+    tmpfeature = feature;
+    for_all_cells(x, y) {
+	if ((feature && feature->id == raw_feature_at(x, y))
+	    || (tmpfeature = feature_at(x, y))) {
+	    ++(tmpfeature->size);
+	    tmpfeature->x += x;  tmpfeature->y += y;
+	    tmpfeature->xmin = min(x, tmpfeature->xmin); tmpfeature->ymin = min(y, tmpfeature->ymin);
+	    tmpfeature->xmax = max(x, tmpfeature->xmax); tmpfeature->ymax = max(y, tmpfeature->ymax);
+	}
+    }
+    for_all_features(tmpfeature) {
+	if (feature == NULL || tmpfeature == feature) {
+	    if (tmpfeature->size > 0) {
+		tmpfeature->x = tmpfeature->x / tmpfeature->size;
+		tmpfeature->y = tmpfeature->y / tmpfeature->size;
+	    }
+	    /* If tmpfeature centroid not actually inside tmpfeature, find closest point. */
+	    if (in_area(tmpfeature->x, tmpfeature->y)
+		&& raw_feature_at(x, y) != tmpfeature->id) {
+		maxdistx = max(tmpfeature->xmax - tmpfeature->x, tmpfeature->x - tmpfeature->xmin);
+		maxdisty = max(tmpfeature->ymax - tmpfeature->y, tmpfeature->y - tmpfeature->ymin);
+		maxdist = max(maxdistx, maxdisty);
+		tmpint = tmpfeature->id;
+		search_around(tmpfeature->x, tmpfeature->y, maxdist, is_feature_cell, &x, &y, 1);
+		tmpfeature->x = x;  tmpfeature->y = y;
+	    }
+	}
     }
 }
 
@@ -1337,6 +1423,21 @@ num_features()
     return f;
 }
 
+int
+num_people_at(x, y)
+int x, y;
+{
+    int m, num;
+
+    num = 0;
+    for_all_material_types(m) {
+	if (cell_material_defined(m)) {
+	    num += material_at(x, y, m) * m_people(m);
+	}
+    }
+    return num;
+}
+
 /* Compute the coords of a point in the given direction. */
 
 int
@@ -1475,6 +1576,16 @@ int cx, cy, rx, ry, *xp, *yp;
 {
     int tries = 500;
 
+    if (rx < 0)
+      rx = 0;
+    if (ry < 0)
+      ry = 0;
+    /* If no variation allowed, just return the center directly. */
+    if (rx == 0 && ry == 0) {
+	*xp = cx;
+	*yp = cy;
+	return inside_area(*xp, *yp);
+    }
     while (tries-- > 0) {
 	*xp = cx + xrandom(2 * rx + 1) - rx;
 	*yp = cy + xrandom(2 * ry + 1) - ry;
@@ -1617,22 +1728,104 @@ int x, y;
     return -1;
 }
 
-/* should put date-handling code here? */
+/* 360 degrees * 60 minutes */
+
+#define MINUTES (21600)
+
+/* Given an x,y and position within a cell, compute the latitude and longitude
+   in arc minutes. */
+
+void
+xy_to_latlong(x, y, xf, yf, latp, lonp)
+int x, y, xf, yf, *latp, *lonp;
+{
+    int lat, lon, center;
+    float fx1, fy1, ratio;
+
+    /* The world must be considered to be spherical. */
+    if (world.circumference <= 0) {
+	*latp = *lonp = 0;
+	return;
+    }
+    fy1 = y + area.latitude - area.halfheight + ((float) yf) / 1000;
+    /* Convert to arc minutes. */
+    lat = (fy1 * MINUTES) / world.circumference;
+    center = (1000 * (area.width + area.halfheight - y)) / 2 - 500;
+    fx1 = 1000 * x + xf;
+    fx1 -= center;
+    if (area.projection == 1) {
+	/* Compute compression ratio due to latitude. */
+	/* The magic value is 2 pi / MINUTES. */
+	ratio = cos(abs(lat) * 0.000290888);
+	fx1 = fx1 / ratio;
+    }
+    /* Offset by the longitude. */
+    fx1 += 1000 * area.longitude;
+    /* Convert to arc minutes. */
+    lon = (fx1 * MINUTES) / world.circumference;
+    lon /= 1000;
+    /* Push the longitude into the range -180 to 180 degrees. */
+    if (lon > MINUTES / 2)
+      lon -= MINUTES;
+    if (lon <= (- MINUTES / 2))
+      lon += MINUTES;
+    *latp = lat;  *lonp = lon;
+}
+
+/* Given latitude/longitude in arc minutes, compute the cell and position
+   within the cell, in 1/1000ths of the cell. */
+
+int
+latlong_to_xy(lat, lon, xp, yp, xfp, yfp)
+int lat, lon, *xp, *yp, *xfp, *yfp;
+{
+    int x1, y1, center;
+    float fx1, fy1, ratio;
+
+    /* Convert latitude to 1000ths of a cell. */
+    fy1 = (((float) lat) * world.circumference) / MINUTES;
+    y1 = fy1 * 1000;
+    /* Latitude of 0 passes through the center of the area. */
+    *yp = y1 / 1000 - area.latitude + area.halfheight;
+    *yfp = y1 % 1000;
+    /* Convert longitude to cell. */
+    fx1 = (((float) lon) * world.circumference) / MINUTES;
+    fx1 -= area.longitude;
+    if (area.projection == 1) {
+	/* Compute compression ratio due to latitude. */
+	/* The magic value is 2 pi / MINUTES. */
+	ratio = cos(abs(lat) * 0.000290888);
+	/* Compute x by compressing longitude towards center of the area. */
+	fx1 = fx1 * ratio;
+    }
+    x1 = fx1 * 1000;
+    /* Compute the effect of obliqueness on x. */
+    center = (1000 * (area.width + area.halfheight - *yp)) / 2 - 500;
+    x1 += center;
+    /* Convert to cell and cell fraction. */
+    *xp = x1 / 1000;
+    *xfp = x1 % 1000;
+    if (*xp < 0)
+      *xp += world.circumference;
+    return TRUE;
+}
+
+/* (should put internal date-handling code here) */
 
 #ifdef DESIGNERS
 
-/* Cell painting. */
+static void paint_cell_1 PARAMS ((int x, int y));
+static void paint_coating_1 PARAMS ((int x, int y));
+static void paint_people_1 PARAMS ((int x, int y));
+static void paint_control_1 PARAMS ((int x, int y));
+static void paint_feature_1 PARAMS ((int x, int y));
+static void paint_elevation_1 PARAMS ((int x, int y));
+static void paint_temperature_1 PARAMS ((int x, int y));
+static void paint_material_1 PARAMS ((int x, int y));
+static void paint_clouds_1 PARAMS ((int x, int y));
+static void paint_winds_1 PARAMS ((int x, int y));
 
-static void
-paint_cell_1(x, y)
-int x, y;
-{
-    /* Only do anything if we're actually changing to a different type. */
-    if (terrain_at(x, y) != tmpttype) {
-	set_terrain_at(x, y, tmpttype);
-	see_exact(tmpside, x, y);
-    }
-}
+/* Cell painting. */
 
 void
 paint_cell(side, x, y, r, t)
@@ -1644,12 +1837,25 @@ int x, y, r, t;
     apply_to_area_plus_edge(x, y, r, paint_cell_1);
 }
 
+static void
+paint_cell_1(x, y)
+int x, y;
+{
+    /* Only do anything if we're actually changing to a different type. */
+    if (terrain_at(x, y) != tmpttype) {
+	set_terrain_at(x, y, tmpttype);
+	update_cell_display(tmpside, x, y, FALSE);
+    }
+}
+
+/* Border/connection painting. */
+
 void
 paint_border(side, x, y, dir, t, mode)
 Side *side;
 int x, y, dir, t, mode;
 {
-    int oldbord;
+    int oldbord, x1, y1;
 
     if (!inside_area(x, y))
       return;
@@ -1657,12 +1863,9 @@ int x, y, dir, t, mode;
     oldbord = border_at(x, y, dir, t);
     set_border_at(x, y, dir, t, (mode < 0 ? !oldbord : mode));
     if (oldbord != border_at(x, y, dir, t)) {
-        /* We do both a see_exact and update because see_exact doesn't
-           account for linear terrain changes. */
-	see_exact(side, x, y);
 	update_cell_display(side, x, y, TRUE);
-	see_exact(side, x+dirx[dir], y+diry[dir]);
-	update_cell_display(side, x+dirx[dir], y+diry[dir], TRUE);
+	if (point_in_dir(x, y, dir, &x1, &y1))
+	  update_cell_display(side, x1, y1, TRUE);
     }
 }
 
@@ -1671,7 +1874,7 @@ paint_connection(side, x, y, dir, t, mode)
 Side *side;
 int x, y, dir, t, mode;
 {
-    int oldconn;
+    int oldconn, x1, y1;
 
     if (!inside_area(x, y))
       return;
@@ -1679,29 +1882,14 @@ int x, y, dir, t, mode;
     oldconn = connection_at(x, y, dir, t);
     set_connection_at(x, y, dir, t, (mode < 0 ? !oldconn : mode));
     if (oldconn != connection_at(x, y, dir, t)) {
-        /* We do both a see_exact and update because see_exact doesn't
-           account for linear terrain changes. */
-	see_exact(side, x, y);
 	update_cell_display(side, x, y, TRUE);
-	see_exact(side, x+dirx[dir], y+diry[dir]);
-	update_cell_display(side, x+dirx[dir], y+diry[dir], TRUE);
+	if (point_in_dir(x, y, dir, &x1, &y1))
+	  update_cell_display(side, x1, y1, TRUE);
     }
 }
 
 /* Coating painting. */
 
-static void
-paint_coating_1(x, y)
-int x, y;
-{
-    int olddepth = aux_terrain_at(x, y, tmpttype);
-
-    if (olddepth != tmpint) {
-	set_aux_terrain_at(x, y, tmpttype, tmpint);
-	see_exact(tmpside, x, y);
-    }
-}
-
 void
 paint_coating(side, x, y, r, t, depth)
 Side *side;
@@ -1714,20 +1902,20 @@ int x, y, r, t, depth;
     apply_to_area_plus_edge(x, y, r, paint_coating_1);
 }
 
-/* Painting of people sides. */
-
 static void
-paint_people_1(x, y)
+paint_coating_1(x, y)
 int x, y;
 {
-    int oldpeop = people_side_at(x, y);
+    int olddepth = aux_terrain_at(x, y, tmpttype);
 
-    if (oldpeop != tmpint) {
-	set_people_side_at(x, y, tmpint);
-	see_exact(tmpside, x, y);
+    if (olddepth != tmpint) {
+	set_aux_terrain_at(x, y, tmpttype, tmpint);
+	update_cell_display(tmpside, x, y, TRUE);
     }
 }
 
+/* Painting of people sides. */
+
 void
 paint_people(side, x, y, r, s)
 Side *side;
@@ -1739,60 +1927,102 @@ int x, y, r, s;
     apply_to_area(x, y, r, paint_people_1);
 }
 
-/* Painting of geographical features. */
-
 static void
-paint_feature_1(x, y)
+paint_people_1(x, y)
 int x, y;
 {
-    int oldfid = raw_feature_at(x, y);
-    Feature *oldfeature;
+    int oldpeop = people_side_at(x, y), dir, x1, y1;
 
-    if (oldfid != tmpint) {
-	set_raw_feature_at(x, y, tmpint);
-	++(tmpfeature->size);
-	see_exact(tmpside, x, y);
-    	if (oldfid != 0) {
-	    oldfeature = find_feature(oldfid);
-	    if (oldfeature != NULL)
-	      --(oldfeature->size);
+    if (oldpeop != tmpint) {
+	set_people_side_at(x, y, tmpint);
+	/* Appearance of adjacent cells may change also. */
+	for_all_directions(dir) {
+	    if (point_in_dir(x, y, dir, &x1, &y1))
+	      update_cell_display(tmpside, x1, y1, TRUE);
 	}
+	update_cell_display(tmpside, x, y, TRUE);
+
     }
 }
 
+/* Painting of controlling side. */
+
+void
+paint_control(side, x, y, r, s)
+Side *side;
+int x, y, r, s;
+{
+    allocate_area_control_sides();
+    tmpside = side;
+    tmpint = s;
+    apply_to_area(x, y, r, paint_control_1);
+}
+
+static void
+paint_control_1(x, y)
+int x, y;
+{
+    int oldpeop = control_side_at(x, y), dir, x1, y1;
+
+    if (oldpeop != tmpint) {
+	set_control_side_at(x, y, tmpint);
+	/* Appearance of adjacent cells may change also. */
+	for_all_directions(dir) {
+	    if (point_in_dir(x, y, dir, &x1, &y1))
+	      update_cell_display(tmpside, x1, y1, TRUE);
+	}
+	update_cell_display(tmpside, x, y, TRUE);
+    }
+}
+
+/* Painting of geographical features.  A feature id of 0 directs feature
+   removal from the given area. */
+
 void
 paint_feature(side, x, y, r, f)
 Side *side;
 int x, y, r, f;
 {
-    Feature *newfeature;
-
     init_features();
-    newfeature = find_feature(f);
-    if (newfeature != NULL) {
-	tmpside = side;
-	tmpfeature = newfeature;
-	tmpint = f;
-	apply_to_area(x, y, r, paint_feature_1);
+    tmpside = side;
+    tmpint = f;
+    tmpfeature = find_feature(f);
+    apply_to_area(x, y, r, paint_feature_1);
+}
+
+static void
+paint_feature_1(x, y)
+int x, y;
+{
+    int oldfid = raw_feature_at(x, y), dir, x1, y1;
+    Feature *oldfeature;
+
+    if (oldfid != tmpint) {
+	set_raw_feature_at(x, y, tmpint);
+	if (tmpfeature != NULL) {
+	    ++(tmpfeature->size);
+	    /* too expensive while painting */
+	    /* compute_feature_centroid(tmpfeature); */
+	}
+    	if (oldfid != 0) {
+	    oldfeature = find_feature(oldfid);
+	    if (oldfeature != NULL) {
+		--(oldfeature->size);
+		/* compute_feature_centroid(oldfeature); */
+	    }
+	}
+	/* Might need to redraw feature borders in adjacent cells. */
+	/* (should test before redirecting redraw?) */
+	for_all_directions(dir) {
+	    if (point_in_dir(x, y, dir, &x1, &y1))
+	      update_cell_display(tmpside, x1, y1, TRUE);
+	}
+	update_cell_display(tmpside, x, y, TRUE);
     }
 }
 
 /* Painting of terrain elevations. */
 
-static void
-paint_elev_1(x, y)
-int x, y;
-{
-    int n, t = terrain_at(x, y), oldelev = elev_at(x, y);
-
-    /* Clip desired elevation to what's allowed for the terrain here. */
-    n = max(t_elev_min(t), min(tmpint, t_elev_max(t)));
-    if (n != oldelev) {
-	set_elev_at(x, y, n);
-	see_exact(tmpside, x, y);
-    }
-}
-
 void
 paint_elevation(side, x, y, r, elev)
 Side *side;
@@ -1801,19 +2031,30 @@ int x, y, r, elev;
     allocate_area_elevations();
     tmpside = side;
     tmpint = elev;
-    apply_to_area_plus_edge(x, y, r, paint_elev_1);
+    apply_to_area_plus_edge(x, y, r, paint_elevation_1);
 }
 
 static void
-paint_temp_1(x, y)
+paint_elevation_1(x, y)
 int x, y;
 {
-    int n, t = terrain_at(x, y), oldtemp = temperature_at(x, y);
-    
-    n = max(t_temp_min(t), min(tmpint, t_temp_max(t)));
-    if (n != oldtemp) {
-	set_temperature_at(x, y, n);
-	see_exact(tmpside, x, y);
+    int n, t = terrain_at(x, y), oldelev = elev_at(x, y), dir, x1, y1;
+
+    /* Clip desired elevation to what's allowed for the terrain here. */
+    n = max(t_elev_min(t), min(tmpint, t_elev_max(t)));
+    if (n != oldelev) {
+	set_elev_at(x, y, n);
+	if (n < area.minelev)
+	  area.minelev = n;
+	if (n > area.maxelev)
+	  area.maxelev = n;
+	/* (should force recomputation of contour intervals) */
+	for_all_directions(dir) {
+	    if (point_in_dir(x, y, dir, &x1, &y1)) {
+		update_cell_display(tmpside, x1, y1, TRUE);
+	    }
+	}
+	update_cell_display(tmpside, x, y, TRUE);
     }
 }
 
@@ -1825,21 +2066,24 @@ int x, y, r, temp;
     allocate_area_temperatures();
     tmpside = side;
     tmpint = temp;
-    apply_to_area_plus_edge(x, y, r, paint_temp_1);
+    apply_to_area_plus_edge(x, y, r, paint_temperature_1);
 }
 
 static void
-paint_material_1(x, y)
+paint_temperature_1(x, y)
 int x, y;
 {
-    int oldm = material_at(x, y, tmpmtype);
-
-    if (oldm != tmpint) {
-	set_material_at(x, y, tmpmtype, tmpint);
-	see_exact(tmpside, x, y);
+    int n, t = terrain_at(x, y), oldtemp = temperature_at(x, y);
+    
+    n = max(t_temp_min(t), min(tmpint, t_temp_max(t)));
+    if (n != oldtemp) {
+	set_temperature_at(x, y, n);
+	update_cell_display(tmpside, x, y, TRUE);
     }
 }
 
+/* Material layer painting. */
+
 void
 paint_material(side, x, y, r, m, amt)
 Side *side;
@@ -1852,9 +2096,36 @@ int x, y, r, m, amt;
     apply_to_area_plus_edge(x, y, r, paint_material_1);
 }
 
+static void
+paint_material_1(x, y)
+int x, y;
+{
+    int oldm = material_at(x, y, tmpmtype);
+
+    if (oldm != tmpint) {
+	set_material_at(x, y, tmpmtype, tmpint);
+	update_cell_display(tmpside, x, y, TRUE);
+    }
+}
+
 /* Cloud painting is more complicated because up to three separate
    layers are involved. */
 
+void
+paint_clouds(side, x, y, r, cloudtype, bot, hgt)
+Side *side;
+int x, y, r, cloudtype, bot, hgt;
+{
+    allocate_area_clouds();
+    /* (should not always do altitudes) */
+    allocate_area_cloud_altitudes();
+    tmpside = side;
+    tmpint = cloudtype;
+    tmpint2 = bot;
+    tmpint3 = hgt;
+    apply_to_area_plus_edge(x, y, r, paint_clouds_1);
+}
+
 static void
 paint_clouds_1(x, y)
 int x, y;
@@ -1877,35 +2148,10 @@ int x, y;
 	changed = TRUE;
     }
     if (changed)
-      see_exact(tmpside, x, y);
+      update_cell_display(tmpside, x, y, TRUE);
 }
 
-void
-paint_clouds(side, x, y, r, cloudtype, bot, hgt)
-Side *side;
-int x, y, r, cloudtype, bot, hgt;
-{
-    allocate_area_clouds();
-    /* (should not always do altitudes) */
-    allocate_area_cloud_altitudes();
-    tmpside = side;
-    tmpint = cloudtype;
-    tmpint2 = bot;
-    tmpint3 = hgt;
-    apply_to_area_plus_edge(x, y, r, paint_clouds_1);
-}
-
-static void
-paint_winds_1(x, y)
-int x, y;
-{
-    int oldw = raw_wind_at(x, y);
-
-    if (oldw != tmpint) {
-	set_raw_wind_at(x, y, tmpint);
-	see_exact(tmpside, x, y);
-    }
-}
+/* Winds painting. */
 
 void
 paint_winds(side, x, y, r, dir, force)
@@ -1918,4 +2164,16 @@ int x, y, r, dir, force;
     apply_to_area_plus_edge(x, y, r, paint_winds_1);
 }
 
+static void
+paint_winds_1(x, y)
+int x, y;
+{
+    int oldw = raw_wind_at(x, y);
+
+    if (oldw != tmpint) {
+	set_raw_wind_at(x, y, tmpint);
+	update_cell_display(tmpside, x, y, TRUE);
+    }
+}
+
 #endif /* DESIGNERS */
