@@ -1,6 +1,5 @@
 /* Definitions relating to worlds and areas in Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -11,9 +10,11 @@ typedef struct a_world {
     int circumference;		/* number of cells going around the world */
     int daylength;		/* number of turns in a day */
     int yearlength;		/* number of turns in a year */
-    int axialtilt;		/* controls extrema of seasons */
-    int daylight_width;		/* width of area that is lit */
-    int twilight_width;		/* width of area that is at least partly lit */
+    int axial_tilt;		/* controls extrema of seasons */
+    int daylight_fraction;	/* percentage of circumference that is lit */
+    int twilight_fraction;	/* percentage of circumference that is partly lit */
+    int daylight_width;		/* width of world that is lit */
+    int twilight_width;		/* width of world that is at least partly lit */
 } World;
 
 /* Theoretically, there is no maximum size to Xconq areas, but the minimum
@@ -24,39 +25,39 @@ typedef struct a_world {
 
 /* An "area" is always basically a rectangular array of positions.  The
    hex effect can be achieved by interpreting neighborliness and direction
-   differently, but that's all that's needed - storagewise the rect
-   array is still the best choice.  All of the "layers" are dynamically
-   allocated as needed, to save (considerable!) space. */
+   differently, but that's all that's needed.  All of the "layers" are
+   dynamically allocated as needed, to save (considerable!) space. */
 
 typedef struct a_area {
     short width, height;	/* size of the area */
-    short halfheight;		/* height / 2 */
+    short halfwidth, halfheight; /* width / 2, height / 2 */
     short maxdim;		/* max of the two dims */
     short xwrap;		/* true if x coords wrap around */
     int numcells;		/* number of cells in area */
     int latitude, longitude;	/* position within whole world */
+    short projection;
     int cellwidth;		/* distance across one cell */
     short fullwidth, fullheight; /* size of the area being used for data */
     short fullx, fully;		/* offset within full area to get data from */
+    short sunx, suny;		/* location at which sun is overhead */
+    Obj *temp_year;		/* cycle of temperatures within year */
+    char *image_name;		/* name of image to display for map */
     /* Pointers to the various "layers". */
     struct a_unit **units;	/* pointer to units if any */
     char *terrain;		/* terrain type at this spot */
     char **auxterrain;		/* vector of extra types */
     char *peopleside;		/* the overt side alignment of the locals */
+    char *controlside;		/* the side actually in charge */
     short *features;		/* layer of ids of features */
     short *elevations;		/* layer of elevations */
-    short constelev;		/* a constant elevation */
-    short avgelev;
-    short minelev;
-    short maxelev;
+    short avgelev;		/* calculated average of elevations */
+    short minelev, maxelev;	/* calculated min/max of elevations */
     short **materials;		/* layer of materials in each cell */
     short *temperature;		/* layer of cell temperatures */
-    short consttemp;		/* a constant temperature */
     short *clouds;		/* types of clouds in the layer */
     short *cloudbottoms;	/* altitudes of clouds of cloud layer */
     short *cloudheights;	/* heights of clouds in the cloud layer */
     short *winds;		/* layer of force/dir of winds. */
-    short constwinds;		/* constant wind */
     /* These layers are for temporary use in calculations. */
     short *tmp1;
     short *tmp2;
@@ -75,6 +76,8 @@ typedef struct a_feature {
     /* caches */
     int size;
     int x, y;
+    short xmin, ymin;
+    short xmax, ymax;
     int mindiam;
     int maxdiam;
     int relabel;
@@ -190,6 +193,23 @@ typedef struct a_path {
 
 #define populated(x,y) (people_side_at(x,y) != NOBODY)
 
+/* Control layer. */
+
+#define control_sides_defined() (area.controlside != NULL)
+
+#define control_side_at(x,y) aref(area.controlside, x, y)
+
+#define set_control_side_at(x,y,s) aset(area.controlside, x, y, s)
+
+/* A cell might be entirely uncontrolled, so need an extra value to indicate. */
+
+/* This value is chosen to be well above any possible MAXSIDES, and encodes
+   in layers as 'X', which is convenient. */
+
+#define NOCONTROL (60)
+
+#define controlled(x,y) (control_side_at(x,y) != NOCONTROL)
+
 /* Array of material layers. */
 
 #define any_cell_materials_defined() (area.materials != NULL)
@@ -266,7 +286,7 @@ typedef struct a_path {
 
 /* Constrain y to northern and southern edges. */
 
-#define limit(y) (max(0, min((y), (area.height-1))))
+#define limity(y) (max(0, min((y), (area.height-1))))
 
 #define interior(y) (max(1, min((y), (area.height-2))))
 
@@ -317,6 +337,9 @@ typedef struct a_path {
     for (y = 1; y < area.height - 1; ++y)  \
       if (x_inside_area(x, y))
 
+#define for_all_features(feat)  \
+  for ((feat) = featurelist; (feat) != NULL; (feat) = (feat)->next)
+
 /* Returns the lighting state of a given position. */
 /* (should opencode distance call here) */
 
@@ -373,6 +396,7 @@ extern void allocate_area_scratch PARAMS ((int n));
 extern void allocate_area_elevations PARAMS ((void));
 extern void allocate_area_temperatures PARAMS ((void));
 extern void allocate_area_people_sides PARAMS ((void));
+extern void allocate_area_control_sides PARAMS ((void));
 extern void allocate_area_material PARAMS ((int m));
 extern void allocate_area_clouds PARAMS ((void));
 extern void allocate_area_cloud_altitudes PARAMS ((void));
@@ -385,6 +409,7 @@ extern int fn_aux_terrain_at PARAMS ((int x, int y));
 extern int fn_feature_at PARAMS ((int x, int y));
 extern int fn_elevation_at PARAMS ((int x, int y));
 extern int fn_people_side_at PARAMS ((int x, int y));
+extern int fn_control_side_at PARAMS ((int x, int y));
 extern int fn_material_at PARAMS ((int x, int y));
 extern int fn_temperature_at PARAMS ((int x, int y));
 extern int fn_raw_cloud_at PARAMS ((int x, int y));
@@ -395,6 +420,7 @@ extern int fn_raw_wind_at PARAMS ((int x, int y));
 extern void fn_set_terrain_at PARAMS ((int x, int y, int val));
 extern void fn_set_aux_terrain_at PARAMS ((int x, int y, int val));
 extern void fn_set_people_side_at PARAMS ((int x, int y, int val));
+extern void fn_set_control_side_at PARAMS ((int x, int y, int val));
 extern void fn_set_raw_feature_at PARAMS ((int x, int y, int val));
 extern void fn_set_elevation_at PARAMS ((int x, int y, int val));
 extern void fn_set_material_at PARAMS ((int x, int y, int val));
@@ -431,24 +457,17 @@ int find_path PARAMS ((int fx, int fy, int tx, int ty,
 		      int (*chooser)(int, int, int, int, int *),
 		      int maxwps, Waypoint *waypoints, int *numwpsp));
 #endif
+extern int search_straight_line PARAMS ((int x0, int y0, int x1, int y1,
+					 int (*pred)(int, int),
+					 int *rxp, int *ryp));
 
-#if 0
-extern int border_at PARAMS ((int x, int y, int dir, int t));
-#endif
 extern void set_border_at PARAMS ((int x, int y, int dir, int t, int onoff));
-#if 0
-extern int connection_at PARAMS ((int x, int y, int dir, int t));
-#endif
 extern void set_connection_at PARAMS ((int x, int y, int dir, int t, int onoff));
 extern void patch_linear_terrain PARAMS ((int t));
+
 extern void init_features PARAMS ((void));
-extern Feature *create_feature PARAMS ((char *typename, char *name));
 extern Feature *find_feature PARAMS ((int fid));
 extern Feature *feature_at PARAMS ((int x, int y));
-extern void set_feature_type_name PARAMS ((Feature *feature, char *typename));
-extern void set_feature_name PARAMS ((Feature *feature, char *name));
-extern void destroy_feature PARAMS ((Feature *feature));
-extern void renumber_features PARAMS ((void));
 extern void compute_all_feature_centroids PARAMS ((void));
 extern void compute_feature_centroid PARAMS ((Feature *feature));
 
@@ -472,26 +491,5 @@ extern int distance PARAMS ((int x1, int y1, int x2, int y2));
 extern int world_distance PARAMS ((int x1, int y1, int x2, int y2));
 extern int closest_dir PARAMS ((int x, int y));
 
-#ifdef DESIGNERS
-
-extern void paint_cell PARAMS ((Side *side, int x, int y, int r, int t));
-extern void paint_border PARAMS ((Side *side, int x, int y, int dir,
-				  int t, int mode));
-extern void paint_connection PARAMS ((Side *side, int x, int y, int dir,
-				      int t, int mode));
-extern void paint_coating PARAMS ((Side *side, int x, int y, int r,
-				   int t, int depth));
-extern void paint_people PARAMS ((Side *side, int x, int y, int r, int s));
-extern void paint_feature PARAMS ((Side *side, int x, int y, int r, int f));
-extern void paint_elevation PARAMS ((Side *side, int x, int y, int r,
-				     int elev));
-extern void paint_temperature PARAMS ((Side *side, int x, int y, int r,
-				       int temp));
-extern void paint_material PARAMS ((Side *side, int x, int y, int r,
-				    int m, int amt));
-extern void paint_clouds PARAMS ((Side *side, int x, int y, int r,
-				  int cloudtype, int bot, int hgt));
-extern void paint_winds PARAMS ((Side *side, int x, int y, int r,
-				 int dir, int force));
-
-#endif /* DESIGNERS */
+extern void xy_to_latlong PARAMS ((int x, int y, int xf, int yf, int *latp, int *lonp));
+extern int latlong_to_xy PARAMS ((int lat, int lon, int *xp, int *yp, int *xfp, int *yfp));
