@@ -1212,6 +1212,7 @@ static void
 write_globals()
 {
     int i, complete = FALSE;
+    Obj *val, *dfltval;
     time_t now;
 
     /* Snapshot realtime values, but only after they've been set up. */
@@ -1235,7 +1236,9 @@ write_globals()
 		newline_form();
 	    }
 	} else if (vardefns[i].strgetter != NULL) {
-	    if (complete || string_not_default((*(vardefns[i].strgetter))(), vardefns[i].dfltstr)) {
+	    if (complete
+		|| string_not_default((*(vardefns[i].strgetter))(),
+				      vardefns[i].dfltstr)) {
 		start_form(key(K_SET));
 		add_to_form(vardefns[i].name);
 		add_to_form(escaped_string((*(vardefns[i].strgetter))()));
@@ -1243,24 +1246,21 @@ write_globals()
 		newline_form();
 	    }
 	} else if (vardefns[i].objgetter != NULL) {
-	    if (complete
-		|| (vardefns[i].dfltfn == NULL
-		    && (*(vardefns[i].objgetter))() != lispnil)) {
-		if ((*(vardefns[i].objgetter))() != NULL) {
-		    start_form(key(K_SET));
-		    add_to_form(vardefns[i].name);
-		    space_form();
-		    /* Suppress evaluation upon readin. */
-		    add_to_form_no_space("'");
-		    add_form_to_form((*(vardefns[i].objgetter))());
-		    end_form();
-		    newline_form();
-		} else {
-		    add_to_form(";");
-		    add_to_form(vardefns[i].name);
-		    add_to_form("was unbound?");
-		    newline_form();
-		}
+	    val = (*(vardefns[i].objgetter))();
+	    /* (should recognize the function-defaulted Lisp values,
+	       but since the default-generating functions may allocate,
+	       this could only work if the original values were recorded;
+	       only synth list and side library involved, so just write
+	       them always) */
+	    if (complete || val != lispnil) {
+		start_form(key(K_SET));
+		add_to_form(vardefns[i].name);
+		space_form();
+		/* Suppress evaluation upon readin. */
+		add_to_form_no_space("'");
+		add_form_to_form((*(vardefns[i].objgetter))());
+		end_form();
+		newline_form();
 	    }
 	} else {
 	    abort();
@@ -1308,10 +1308,14 @@ write_doctrines()
 	  add_num_to_form(doc->id);
 	newline_form();
 	space_form();
-	write_num_prop(key(K_RESUPPLY_PERCENT), doc->resupply_percent, 50, FALSE, TRUE); 
-	write_num_prop(key(K_REARM_PERCENT), doc->rearm_percent, 20, FALSE, TRUE); 
-	write_num_prop(key(K_REPAIR_PERCENT), doc->repair_percent, 35, FALSE, TRUE); 
-	write_utype_value_list(key(K_CONSTRUCTION_RUN), doc->construction_run, 0, TRUE);
+	write_num_prop(key(K_RESUPPLY_PERCENT), doc->resupply_percent,
+		       50, FALSE, TRUE); 
+	write_num_prop(key(K_REARM_PERCENT), doc->rearm_percent,
+		       20, FALSE, TRUE); 
+	write_num_prop(key(K_REPAIR_PERCENT), doc->repair_percent,
+		       35, FALSE, TRUE); 
+	write_utype_value_list(key(K_CONSTRUCTION_RUN), doc->construction_run,
+			       0, TRUE);
 	write_num_prop(key(K_LOCKED), doc->locked, 0, FALSE, TRUE); 
 	space_form();
 	end_form();
@@ -1375,7 +1379,7 @@ Side *side;
     write_str_prop(key(K_COLOR), side->colorscheme, "", FALSE, TRUE);
     write_str_prop(key(K_EMBLEM_NAME), side->emblemname, "", FALSE, TRUE);
     write_utype_string_list(key(K_UNIT_NAMERS), side->unitnamers, "", TRUE);
-    write_lisp_prop(key(K_UNITS), side->possible_units, lispnil, FALSE, TRUE, TRUE);
+    write_lisp_prop(key(K_UNITS), side->possible_units, lispnil, FALSE, FALSE, TRUE);
     write_str_prop(key(K_CLASS), side->sideclass, "", FALSE, TRUE);
     write_bool_prop(key(K_ACTIVE), side->ingame, 1, FALSE, TRUE);
     write_bool_prop(key(K_EVER_ACTIVE), side->everingame, 1, FALSE, TRUE);
@@ -1443,7 +1447,8 @@ Side *side;
 	newline_form();
 	space_form();
     }
-    write_lisp_prop(key(K_INSTRUCTIONS), side->instructions, lispnil, FALSE, FALSE, TRUE);
+    write_lisp_prop(key(K_INSTRUCTIONS), side->instructions,
+		    lispnil, FALSE, FALSE, TRUE);
     /* Write out statistics. */
     if (side->gaincounts != NULL) {
 	space_form();
