@@ -117,29 +117,28 @@ extern void mac_add_cicn(ImageFamily *imf, Handle handle);
 
 #define mView 131
 
-#define miView4x4 1
-#define miView8x8 2
-#define miView16x16 3
-#define miView32x32 4
-#define miView64x64 5
-/* 6 */
-#define miViewColor 7
-#define miViewNames 8
-#define miViewMask 9
-/* 10 is Background submenu */
-#define miViewBBox 11
-#define miViewSolidColor 12
-/* 13 */
-#define miViewAsUnit 14
-#define miViewAsTerrain 15
-#define miViewAsEmblem 16
-#define miViewWithUnit 17
-#define miViewWithTerrain 18
-#define miViewWithEmblem 19
-/* 20 */
-#define miViewIcons 21
-#define miViewTiles 22
-#define miViewCQD 23
+#define miView8x8 1
+#define miView16x16 2
+#define miView32x32 3
+#define miView64x64 4
+/* 5 */
+#define miViewColor 6
+#define miViewNames 7
+#define miViewMask 8
+/* 9 is Background submenu */
+#define miViewBBox 10
+#define miViewSolidColor 11
+/* 12 */
+#define miViewAsUnit 13
+#define miViewAsTerrain 14
+#define miViewAsEmblem 15
+#define miViewWithUnit 16
+#define miViewWithTerrain 17
+#define miViewWithEmblem 18
+/* 19 */
+#define miViewIcons 20
+#define miViewTiles 21
+#define miViewCQD 22
 
 #define mBackground 132
 
@@ -299,7 +298,7 @@ void draw_one_image(ImageFamily *imf, int col, int row);
 void set_scrollbar(void);
 void invert_selected_imf(void);
 void draw_as_terrain_image(int sx, int sy, int sw, int sh, ImageFamily *imf);
-Image *draw_as_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, ImageFamily *imf);
+Image *draw_as_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, ImageFamily *imf, int *sx2p, int *sy2p);
 void draw_as_emblem_image(WindowPtr win, int ex, int ey, int ew, int eh, ImageFamily *imf);
 
 char *copy_pascal_string(char *str);
@@ -344,6 +343,9 @@ main()
 
 	init_lisp();
 
+	imf_load_hook = mac_load_imf;
+	imf_interp_hook = mac_interp_imf;
+
 	/* Create the main window we're going to play in. */
 	if (hasColorQD) {
 		imagewin = GetNewCWindow(wImages, NULL, (WindowPtr) -1L);
@@ -593,9 +595,6 @@ long which;
 			break;
 		case mView:
 			switch (menuitem) {
-				case miView4x4:
-					iw = ih = 4;
-					break;
 				case miView8x8:
 					iw = ih = 8;
 					break;
@@ -701,7 +700,6 @@ adjust_menus()
 		CheckItem(menu, miFileBWOnly, !write_color);
 	}
 	if ((menu = GetMHandle(mView)) != nil) {
-		CheckItem(menu, miView4x4, (iw == 4));
 		CheckItem(menu, miView8x8, (iw == 8));
 		CheckItem(menu, miView16x16, (iw == 16));
 		CheckItem(menu, miView32x32, (iw == 32));
@@ -880,7 +878,7 @@ ImageFamily *imf;
 int loadnow;
 {
 	if (loadnow && imf != NULL)
-	  mac_interp_imf(imf);
+	  mac_interp_imf(imf, NULL, FALSE);
 	/* Do some visual feedback periodically. */
 	if (numimages % 10 == 0)
 	  draw_topline();
@@ -1597,7 +1595,7 @@ draw_one_image(imf, col, row)
 ImageFamily *imf;
 int col, row;
 {
-	int ex, ey, ew, eh, bx, by, bw, bh, namex, namey;
+	int ex, ey, ew, eh, sx, sy, sw, sh, bx, by, bw, bh, namex, namey;
 	Image *uimg;
 	ImageFamily *uimf, *eimf;
 	Rect tmprect, maskrect, bboxrect;
@@ -1621,11 +1619,13 @@ int col, row;
 	}
 	uimg = NULL;
 	if (uimf) {
-		uimg = draw_as_unit_image(imagewin, tmprect.left, tmprect.top, iw, ih, uimf);
+		uimg = draw_as_unit_image(imagewin, tmprect.left, tmprect.top, iw, ih, uimf, &sx, &sy);
 		if (uimg) {
 			/* Scale bounding box by space given to image. */
-			bx = (uimg->bboxx * iw) / uimg->w;  by = (uimg->bboxy * ih) / uimg->h;
-			bw = (uimg->bboxw * iw) / uimg->w;  bh = (uimg->bboxh * ih) / uimg->h;
+			/* (for now, no images are scaled) */
+			bx = sx + uimg->bboxx;  by = sy + uimg->bboxy;
+			sw = uimg->w;  sh = uimg->h;
+			bw = (uimg->bboxw * sw) / uimg->w;  bh = (uimg->bboxh * sh) / uimg->h;
 		}
 	}
 	/* Maybe draw an emblem. */
@@ -1659,8 +1659,8 @@ int col, row;
 	}
 	if (show_bbox && uimg) {
 		bboxrect = tmprect;
-		bboxrect.left += bx;  bboxrect.top += by;
-		bboxrect.right = bboxrect.left + bw;  bboxrect.bottom = bboxrect.top + bh;
+		bboxrect.left = bx;  bboxrect.top = by;
+		bboxrect.right = bboxrect.left + bw + 1;  bboxrect.bottom = bboxrect.top + bh + 1;
 		InsetRect(&bboxrect, -1, -1);
 		/* This inverts a rectangle showing the bounding box. */
 		InvertRect(&bboxrect);
@@ -1759,7 +1759,7 @@ draw_as_terrain_image(int sx, int sy, int sw, int sh, ImageFamily *imf)
    but it uses an arbitrary image family instead. */
 
 Image *
-draw_as_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, ImageFamily *imf)
+draw_as_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, ImageFamily *imf, int *sx2p, int *sy2p)
 {
 	Rect srcrect, imagerect;
 	BitMap bm, *winbits;
@@ -1771,6 +1771,21 @@ draw_as_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, ImageFamily *i
 		if ((uimg->istile ? !select_tiles : !select_icons))
 		  return uimg;
 		imagerect = win->portRect;
+		if (!uimg->istile) {
+			/* Offset the image to draw in the middle of its area,
+			   whether larger or smaller than the given area. */
+			*sx2p = sx + (sw - uimg->w) / 2;  *sy2p = sy + (sh - uimg->h) / 2;
+			/* Only change the size of the rectangle being drawn if it's
+			   smaller than what was passed in. */
+			if (uimg->w < sw) {
+			    sx = *sx2p;
+			    sw = uimg->w;
+			}
+			if (uimg->h < sh) {
+			    sy = *sy2p;
+			    sh = uimg->h;
+			}
+		}
 		imagerect.top += sy;  imagerect.left += sx;
 		imagerect.bottom = imagerect.top + sh;  imagerect.right = imagerect.left + sw;
 		winbits = &(((GrafPtr) win)->portBits);
