@@ -1,5 +1,5 @@
 /* Commands for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,18 +7,16 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern void update_debugging PARAMS ((void));
-extern void toggle_debugging PARAMS ((int *flagp));
-extern int can_build_or_help PARAMS ((Unit *unit));
-extern int can_produce PARAMS ((Unit *unit));
-extern Task *create_produce_task PARAMS ((int m, int n));
-extern void push_produce_task PARAMS ((Unit *unit, int m, int n));
-/* Only defined if ansi */
-extern void vnotify(Side *side, char *fmt, va_list ap);
+extern void report_take PARAMS ((Side *side, Unit *unit, int needed, short *rslts));
 #include "print.h"
+#include "kpublic.h"
 #include "macconq.h"
 extern void enable_command(void);
 extern void set_construction_run_length(int len);
+extern Map *front_map(void);
+extern void do_idle_command(int n);
+extern int do_down_command(void);
+extern int do_up_command(void);
 
 /* Define in order to get profiling controls. */
 #undef PROFILING
@@ -55,6 +53,10 @@ Map *modal_map;
 
 extern char *cursavename;
 
+extern int givetakeprefixarg;
+
+extern Side *tmpcmdside;
+
 #undef DEF_CMD
 #define DEF_CMD(letter,name,args,FN,help) extern void FN(void);
 
@@ -66,6 +68,7 @@ extern char *cursavename;
 
 static int do_one_add_terrain(Unit *unit);
 static int do_one_ai_control(Unit *unit);
+static int do_one_attack(Unit *unit);
 static void unit_do_build_2(Unit *unit);
 extern int do_one_clear_plan(Unit *unit);
 static int do_one_delay(Unit *unit);
@@ -76,18 +79,21 @@ extern int do_one_dir_multiple_move(Unit *unit);
 extern int do_one_disband(Unit *unit);
 extern int do_one_disembark(Unit *unit);
 extern int do_one_embark(Unit *unit);
+static int do_one_fire(Unit *unit);
+static int do_one_fire_into(Unit *unit);
 extern int do_one_give(Unit *unit);
 extern int do_one_give_unit(Unit *unit);
+extern int do_one_idle(Unit *unit);
+static int do_one_move_to(Unit *unit);
 static int do_one_occupant(Unit *unit);
 static int do_one_remove_terrain(Unit *unit);
+extern int do_one_repair(Unit *unit);
 static int do_one_reserve(Unit *unit);
 extern int do_one_return(Unit *unit);
-extern int do_one_set_name(Unit *unit);
+static int do_one_set_name(Unit *unit);
 static int do_one_asleep(Unit *unit);
 extern int do_one_take(Unit *unit);
 
-static void resign_the_game(int forced);
-
 static void cmd_error(Side *side, char *fmt, ...);
 
 /* Static (local) variables. */
@@ -97,6 +103,8 @@ static int tmprecurse;
 
 static int tmpcmdx, tmpcmdy, tmpcmddir;
 
+static Unit *tmpcmdunit;
+
 /* This is the actual key typed, for use if several keyboard commands
    share a single function. */
 
@@ -104,7 +112,7 @@ static char tmpkey;
 
 /* Prefixed numeric argument to commands. */
 
-static int prefixarg;
+static int prefixarg = -1;
 
 static int tmpdir;
 
@@ -131,8 +139,6 @@ typedef struct cmdtab {
 
 CmdTab commands[] = {
 
-#include "maccmd.def"
-
 #include "cmd.def"
 
   { 0, NULL, NULL, NULL }
@@ -177,6 +183,20 @@ do_keyboard_command(int key)
 				return;
 			}
 		}
+		/* Look through the Mac-specific command table. */
+	    for (cmd = mac_commands; cmd->name != NULL; ++cmd) {
+			if (key == cmd->fchar) {
+		    	if ((fn = cmd->fn) == NULL) {
+					run_warning("no command function for %s (0x%x)?", cmd->name, key);
+					return;
+		    	}
+		    	tmpkey = key;
+				(*fn)();
+				/* Reset the prefix argument. */
+				prefixarg = -1;
+				return;
+			}
+		}
 	}
 }
 
@@ -184,16 +204,35 @@ void
 execute_named_command(char *cmdstr)
 {
 	char *cmdname;
+	int prefix;
 	CmdTab *cmd;
     void (*fn)(void);
 
-    parse_long_name_command(cmdstr, &cmdname, &cmdargstr, copy_string(cmdstr)); 
+    parse_long_name_command(cmdstr, &prefix, &cmdname, &cmdargstr, copy_string(cmdstr)); 
     if (empty_string(cmdname)) {
 		notify(dside, "No command name.");
 		beep();
 		return;
 	}
     for (cmd = commands; cmd->name != NULL; ++cmd) {
+		if (strcmp(cmdname, cmd->name) == 0) {
+			fn = cmd->fn;
+	    	if (fn == NULL) {
+				run_warning("no command function for %s?", cmd->name);
+				return;
+	    	}
+	    	/* Set the prefix arg if one was on the command line. */
+	    	if (prefix >= 0)
+	    	  prefixarg = prefix;
+	    	/* Use the command's char as the apparent key. */
+	    	tmpkey = cmd->fchar;
+			(*fn)();
+			/* Reset the prefix argument. */
+			prefixarg = -1;
+			return;
+		}
+	}
+    for (cmd = mac_commands; cmd->name != NULL; ++cmd) {
 		if (strcmp(cmdname, cmd->name) == 0) {
 			fn = cmd->fn;
 	    	if (fn == NULL) {
@@ -213,23 +252,31 @@ execute_named_command(char *cmdstr)
 }
 
 void
-describe_commands(int arg, char *key, char *buf)
-{
-    describe_command_table(arg, key, buf, commands);
-    /* (should split out Mac-specific commands??) */
-}
-
-void
-describe_command_table(int arg, char *key, char *buf, CmdTab *cmdtab)
+describe_keyboard(int arg, char *key, TextBuffer *buf)
 {
 	CmdTab *cmd;
 
-	strcat(buf, "Single-key commands:\n\n");
-    for (cmd = cmdtab; cmd->name != NULL; ++cmd) {
+	tbcat(buf, "Single-key commands:\n\n");
+    for (cmd = commands; cmd->name != NULL; ++cmd) {
 		describe_command(cmd->fchar, cmd->name, cmd->help, TRUE, buf);
     }
-    strcat(buf, "\nLong name commands:\n\n");
-    for (cmd = cmdtab; cmd->name != NULL; ++cmd) {
+	tbcat(buf, "\n\nSingle-key commands (Mac-only):\n\n");
+    for (cmd = mac_commands; cmd->name != NULL; ++cmd) {
+		describe_command(cmd->fchar, cmd->name, cmd->help, TRUE, buf);
+    }
+}
+
+void
+describe_text_commands(int arg, char *key, TextBuffer *buf)
+{
+	CmdTab *cmd;
+
+    tbcat(buf, "\nText commands:\n\n");
+    for (cmd = commands; cmd->name != NULL; ++cmd) {
+		describe_command (cmd->fchar, cmd->name, cmd->help, FALSE, buf);
+    }
+    tbcat(buf, "\nText commands (Mac-only):\n\n");
+    for (cmd = mac_commands; cmd->name != NULL; ++cmd) {
 		describe_command (cmd->fchar, cmd->name, cmd->help, FALSE, buf);
     }
 }
@@ -266,12 +313,36 @@ get_a_unit(Map **mapp, Unit **unitp)
 	return FALSE;
 }
 
+int get_a_visible_unit(Map **mapp, Unit **unitp);
+
+int
+get_a_visible_unit(Map **mapp, Unit **unitp)
+{
+	int rslt = FALSE;
+	Point at;
+
+	if (position_set_modally) {
+		position_set_modally = FALSE;
+		*mapp = modal_map;
+		rslt = m_nearest_unit(*mapp, modal_point.h, modal_point.v, unitp);
+	} else if ((*mapp = map_from_window(FrontWindow())) != NULL) {
+		GetMouse(&at);
+		rslt = m_nearest_unit(*mapp, at.h, at.v, unitp);
+	}
+	if (rslt
+		&& !(*mapp)->see_all
+		&& !units_visible(dside, (*unitp)->x, (*unitp)->y)
+		)
+	  rslt = FALSE;
+	return rslt;
+}
+
 /* Start of alphabetized commands. */
 
 void
 do_add_player()
 {
-    request_additional_side(cmdargstr);
+    net_request_additional_side(cmdargstr);
 }
 
 void
@@ -317,7 +388,7 @@ do_one_add_terrain(Unit *unit)
 	switch (t_subtype(t1)) {
 		case cellsubtype:
 			if (valid(check_alter_cell_action(unit, unit, tmpcmdx, tmpcmdy, t1))) {
-				if (prep_alter_cell_action(unit, unit, tmpcmdx, tmpcmdy, t1))
+				if (net_prep_alter_cell_action(unit, unit, tmpcmdx, tmpcmdy, t1))
 				  return TRUE;
 			}
 			break;
@@ -331,7 +402,7 @@ do_one_add_terrain(Unit *unit)
 				dir = approx_dir(tmpcmdx - unit->x, tmpcmdy - unit->y);
 			}
 			if (valid(check_add_terrain_action(unit, unit, x, y, dir, t1))) {
-				if (prep_add_terrain_action(unit, unit, x, y, dir, t1))
+				if (net_prep_add_terrain_action(unit, unit, x, y, dir, t1))
 				  return TRUE;
 			}
 			break;
@@ -345,16 +416,44 @@ void
 do_ai_side()
 {
 	if (side_has_ai(dside)) {
-		set_side_ai(dside, NULL);
+		net_set_side_ai(dside, NULL);
 	} else {
-		set_side_ai(dside, "mplayer");
+		net_set_side_ai(dside, "mplayer");
 	}
 }
 
 void
 do_attack()
 {
-	do_attack_command();
+	Map *map;
+	
+	if (get_a_visible_unit(&map, &tmpcmdunit)) {
+		apply_to_all_selected(do_one_attack, TRUE);
+		return;
+	}
+	beep();
+}
+
+static int
+do_one_attack(Unit *atker)
+{
+	int i;
+	Unit *other;
+
+	other = tmpcmdunit;
+	if (other != NULL
+		&& other != atker
+		&& other->side != atker->side) {
+		if (distance(atker->x, atker->y, other->x, other->y) <= 1
+			&& valid(check_attack_action(atker, atker, other, 100))) {
+			net_prep_attack_action(atker, atker, other, 100);
+			return TRUE;
+		} else if (valid(check_fire_at_action(atker, atker, other, -1))) {
+			net_prep_fire_at_action(atker, atker, other, -1);
+			return TRUE;
+		}
+	}
+	return FALSE;
 }
 
 void
@@ -375,9 +474,7 @@ do_one_ai_control(Unit *unit)
 		  newval = 0;
 		else
 		  newval = 1;
-		set_unit_ai_control(dside, unit, newval, FALSE);
-		/* a hack */
-		unit->plan->waitingfortasks = !unit->plan->aicontrol;
+		net_set_unit_ai_control(dside, unit, newval, FALSE);
 	}
 	return TRUE;
 }
@@ -389,17 +486,6 @@ do_build()
 	Map *map;  List *list;  UnitCloseup *closeup;
 	Unit *unit;
 
-	map = map_from_window(FrontWindow());
-	if (map != NULL) {
-		for (i = 0; i < map->numselections; ++i) {
-			unit = map->selections[i];
-			if (unit != NULL && can_build_or_help(unit)) {
-				unit_do_build_2(unit);
-				return;
-			}
-		}
-		return;
-	}
 	list = list_from_window(FrontWindow());
 	if (list != NULL) {
 		unit = selected_unit_in_list(list);
@@ -418,6 +504,17 @@ do_build()
 		}
 		return;
 	}
+	map = front_map();
+	if (map != NULL) {
+		for (i = 0; i < map->numselections; ++i) {
+			unit = map->selections[i];
+			if (unit != NULL && can_build_or_help(unit)) {
+				unit_do_build_2(unit);
+				return;
+			}
+		}
+		return;
+	}
 	/* No way to figure out the unit to be building with, so complain. */
 	beep();
 }
@@ -460,7 +557,8 @@ int
 do_one_clear_plan(Unit *unit)
 {
 	if (unit->plan) {
-		set_unit_plan_type(dside, unit, PLAN_NONE);
+		net_set_unit_plan_type(dside, unit, PLAN_NONE);
+		net_clear_task_agenda(dside, unit);
 		return TRUE;
 	}
 	return FALSE;
@@ -488,7 +586,7 @@ int
 do_one_delay(Unit *unit)
 {
 	if (unit->plan) {
-	    delay_unit(unit, TRUE);
+	    net_delay_unit(unit, TRUE);
 		return TRUE;
 	}
 	return FALSE;
@@ -506,7 +604,7 @@ do_one_detach(Unit *unit)
 	if (!completed(unit)) {
 		return FALSE;
 	} else if (valid(check_transfer_part_action(unit, unit, unit->hp / 2, NULL))) {
-		prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
+		net_prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
 		return TRUE;
 	} else {
 		/* try to find a nearby unit to do it */
@@ -514,10 +612,31 @@ do_one_detach(Unit *unit)
 	return FALSE;
 }
 
+/* This command samples the current mouse to get the desired detonate location. */
+
 void
 do_detonate()
 {
-	do_detonate_command();
+    int x, y, dir;
+	Map *map;
+	
+	if (get_a_position(&map, &x, &y, &dir)) {
+		tmpcmdx = x;  tmpcmdy = y;  tmpcmddir = dir;
+		apply_to_all_selected(do_one_detonate, TRUE);
+		return;
+	}
+	beep();
+}
+
+int
+do_one_detonate(Unit *unit)
+{
+	/* (should use tmpcmdx etc, but fall back to unit pos if x,y not valid?) */
+    if (valid(check_detonate_action(unit, unit, unit->x, unit->y, unit->z))) {
+		net_prep_detonate_action(unit, unit, unit->x, unit->y, unit->z);
+		return TRUE;
+	}
+	return FALSE;
 }
 
 /* Command all selected mobile units to move in a given direction. */
@@ -546,7 +665,7 @@ do_dir()
 	int ndirs, dir1, dir2, modif;
 	Map *map;
 
-	map = map_from_window(FrontWindow());
+	map = front_map();
 	if (map != NULL) {
 		ndirs = char_to_dir(tmpkey, &dir1, &dir2, &modif);
 		if (ndirs >= 1) {
@@ -556,6 +675,8 @@ do_dir()
 			return;
 		}
 		apply_to_all_selected(do_one_dir_move, TRUE);
+		if (map->moveonclick)
+		  map->scrolltocurunit = TRUE;
 	}
 }
 
@@ -567,7 +688,7 @@ int
 do_one_dir_multiple_move(Unit *unit)
 {
 	if (mobile(unit->type)) {
-		set_move_dir_task(unit, tmpdir, (prefixarg <= 0 ? 9999 : prefixarg));
+		net_set_move_dir_task(unit, tmpdir, (prefixarg <= 0 ? 9999 : prefixarg));
 		return TRUE;
 	}
 	return FALSE;
@@ -581,7 +702,7 @@ do_dir_multiple()
 	int ndirs, dir1, dir2, modif;
 	Map *map;
 
-	map = map_from_window(FrontWindow());
+	map = front_map();
 	if (map != NULL) {
 		ndirs = char_to_dir(tmpkey, &dir1, &dir2, &modif);
 		if (ndirs >= 1) {
@@ -591,6 +712,8 @@ do_dir_multiple()
 			return;
 		}
 		apply_to_all_selected(do_one_dir_multiple_move, TRUE);
+		if (map->moveonclick)
+		  map->scrolltocurunit = TRUE;
 	}
 }
 
@@ -603,7 +726,7 @@ do_disband()
 int
 do_one_disband(Unit *unit)
 {
-    return disband_unit(dside, unit);
+    return net_disband_unit(dside, unit);
 }
 
 void
@@ -622,18 +745,18 @@ do_one_disembark(Unit *unit)
 	/* Try moving into the transport's transport, if there is one. */
 	if (transport->transport != NULL
 		&& can_occupy(unit, transport->transport)) {
-		prep_enter_action(unit, unit, transport->transport);
+		net_prep_enter_action(unit, unit, transport->transport);
 		/* (should be able to set up task if can't do action immediately) */
 		return TRUE;
 	}
 	/* Try moving into the open in the cell. */
 	if (!inside_area(unit->x, unit->y))
 	  return FALSE;
-	if (can_occupy_cell(unit, unit->x, unit->y)
-		|| can_occupy_conn(unit, unit->x, unit->y, unit->z)) {
-		prep_move_action(unit, unit, unit->x, unit->y, unit->z);
-		/* (should be able to set up task if can't do action immediately) */
+	if (valid(check_move_action(unit, unit, unit->x, unit->y, unit->z))) {
+		net_prep_move_action(unit, unit, unit->x, unit->y, unit->z);
 		return TRUE;
+	} else {
+		/* (should be able to set up task if can't do action immediately) */
 	}
 	return FALSE;
 }
@@ -653,19 +776,38 @@ do_distrust()
 	if (cmdargstr) {
 		side2 = parse_side_spec(cmdargstr);
 		if (side2 != NULL && side2 != dside) {
-			set_trust(dside, side2, 0);
+			net_set_trust(dside, side2, 0);
 			return;
 		} 
 	}
 	beep();
 }
 
+void
+do_doctrine()
+{
+	if (strcmp(cmdargstr, "set") == 0) {
+		notify(dside, "Set what?");
+		beep();
+	} else if (strncmp(cmdargstr, "set ", 4) == 0) {
+		net_set_doctrine(dside, cmdargstr + 4);
+	} else {
+		notify_doctrine(dside, cmdargstr);
+	}
+}
+
+void
+do_down()
+{
+	do_down_command();
+}
+
 void
 do_draw_willingness()
 {
     if (prefixarg < 0)
 	  prefixarg = 1;
-    set_willing_to_draw(dside, (prefixarg ? 1 : 0));
+    net_set_willing_to_draw(dside, (prefixarg ? 1 : 0));
 }
 
 void
@@ -686,14 +828,14 @@ do_one_embark(Unit *unit)
 		/* Unit is in the open. */
 		for_all_stack(x, y, unit2) {
 			if (unit2 != transport && can_occupy(unit, unit2)) {
-				prep_enter_action(unit, unit, unit2);
+				net_prep_enter_action(unit, unit, unit2);
 				return TRUE;
 			}
 		}
 		for_all_stack(x, y, unit2) {
 			for_all_occupants(unit2, unit3) {
 				if (unit3 != transport && can_occupy(unit, unit3)) {
-					prep_enter_action(unit, unit, unit3);
+					net_prep_enter_action(unit, unit, unit3);
 					return TRUE;
 				}
 			}
@@ -701,21 +843,21 @@ do_one_embark(Unit *unit)
 	} else {
 		for_all_occupants(transport, unit2) {
 			if (unit2 != transport && can_occupy(unit, unit2)) {
-				prep_enter_action(unit, unit, unit2);
+				net_prep_enter_action(unit, unit, unit2);
 				return TRUE;
 			}
 		}
 		if (transport->transport == NULL) {
 			for_all_stack(x, y, unit2) {
 				if (unit2 != transport && can_occupy(unit, unit2)) {
-					prep_enter_action(unit, unit, unit2);
+					net_prep_enter_action(unit, unit, unit2);
 					return TRUE;
 				}
 			}
 			for_all_stack(x, y, unit2) {
 				for_all_occupants(unit2, unit3) {
 					if (unit3 != transport && can_occupy(unit, unit3)) {
-						prep_enter_action(unit, unit, unit3);
+						net_prep_enter_action(unit, unit, unit3);
 						return TRUE;
 					}
 				}
@@ -723,7 +865,7 @@ do_one_embark(Unit *unit)
 		} else {
 			for_all_occupants(transport->transport, unit2) {
 				if (unit2 != transport && can_occupy(unit, unit2)) {
-					prep_enter_action(unit, unit, unit2);
+					net_prep_enter_action(unit, unit, unit2);
 					return TRUE;
 				}
 			}
@@ -745,20 +887,61 @@ do_end_turn()
 		pt.v = (*constructbutton)->contrlRect.top + 8;
 		do_mouse_down_construction(pt, 0);
 	} else {
-		finish_turn(dside);
+		net_finish_turn(dside);
 	}
 }
 
 void
 do_fire()
 {
-	do_fire_command();
+	Map *map;
+	
+	if (!get_a_visible_unit(&map, &tmpcmdunit)) {
+		beep();
+		return;
+	}
+	apply_to_all_selected(do_one_fire, TRUE);
+}
+
+static int
+do_one_fire(Unit *atker)
+{
+	Unit *other;
+
+	other = tmpcmdunit;
+	if (other != NULL
+		&& other != atker
+		&& other->side != atker->side
+		&& valid(check_fire_at_action(atker, atker, other, -1))) {
+		net_prep_fire_at_action(atker, atker, other, -1);
+		return TRUE;
+	}
+	return FALSE;
 }
 
 void
 do_fire_into()
 {
-	do_fire_into_command();
+	Map *map;
+	
+	if (!get_a_position(&map, &tmpcmdx, &tmpcmdy, &tmpcmddir)) {
+		beep();
+		return;
+	}
+	apply_to_all_selected(do_one_fire_into, TRUE);
+}
+
+static int
+do_one_fire_into(Unit *atker)
+{
+	int x, y;
+
+	x = tmpcmdx;  y = tmpcmdy;	
+	if (valid(check_fire_into_action(atker, atker, x, y, 0, -1))) {
+		net_prep_fire_into_action(atker, atker, x, y, 0, -1);
+		return TRUE;
+	}
+	return FALSE;
 }
 
 void
@@ -766,9 +949,15 @@ do_follow_action()
 {
 	Map *map;
 
-	map = map_from_window(FrontWindow());
+	map = front_map();
 	if (map != NULL) {
 		map->follow_action = !map->follow_action;
+		if (map->follow_action)
+		  notify(dside, "Following the action on map %d", map->id);
+		else
+		  notify(dside, "No longer following action on map %d", map->id);
+	} else {
+		beep();
 	}
 }
 
@@ -781,20 +970,53 @@ do_force_global_replan()
 void
 do_give()
 {
-    /* (should use argument) */
+	givetakeprefixarg = prefixarg;
 	apply_to_all_selected(do_one_give, TRUE);
 }
 
+int
+do_one_give(Unit *unit)
+{
+	give_supplies(unit, NULL, NULL);
+	return TRUE;
+}
+
 void
 do_give_unit()
 {
 	int sn = 0;
-	Side *side;
 
-	side = side_n(sn);
+    if (prefixarg >= 0) {
+		if (prefixarg <= numsides) {
+			sn = prefixarg;
+		} else {
+			beep();
+			return;
+		}
+    }
+	tmpcmdside = side_n(sn);
 	apply_to_all_selected(do_one_give_unit, TRUE);
 }
 
+int
+do_one_give_unit(Unit *unit)
+{
+	if (unit->side == tmpcmdside)
+	  return FALSE;
+#ifdef DESIGNERS
+	if (is_designer(dside)) {
+		net_designer_change_side(unit, tmpcmdside);
+		return TRUE;
+	}
+#endif /* DESIGNERS */
+    if (0 /* unit is a type that cannot act */) {
+		/* (should add case for nonacting units) */
+   	} else {
+		net_prep_change_side_action(unit, unit, tmpcmdside);
+	}
+	return TRUE;
+}
+
 void
 do_help()
 {
@@ -803,10 +1025,30 @@ do_help()
 	help_dialog(NULL);
 }
 
+void
+do_idle()
+{
+	do_idle_command(prefixarg);
+}
+
+void
+do_idle_command(int value)
+{
+	tmpcmdarg = value;
+	apply_to_all_selected(do_one_idle, TRUE);	
+}
+
+static int
+do_one_idle(Unit *unit)
+{
+	net_set_sentry_task(unit, tmpcmdarg);
+	return TRUE;
+}
+
 void
 do_message()
 {
-	message_dialog();
+	message_dialog(prefixarg);
 }
 
 /* Dialog for the input of a textual message. */
@@ -814,14 +1056,32 @@ do_message()
 /* (should add way to specify which sides to receive this) */
 
 void
-message_dialog()
+message_dialog(int arg)
 {
 	short done = FALSE, ditem;
-	char *msg = NULL;
+	char *msg = NULL, *sidedesc, *nsidedesc, buf[100], buf2[100];
+	SideMask sidemask;
+	Side *side2;
+	Str255 tmpstr;
 	DialogPtr win;
 	short itemtype;  Handle itemhandle;  Rect itemrect;
 
+	if (arg < 0) {
+		sidemask = ALLSIDES;
+	} else {
+		sidemask = NOSIDES;
+		side2 = side_n(arg);
+		if (side2)
+		  sidemask = add_side_to_set(side2, sidemask);
+		else
+		  beep();
+	}
+	sidedesc = sidemask_desc(buf2, sidemask);
+	strcpy(buf, sidedesc);
+	c2p(buf, tmpstr);
 	win = GetNewDialog(dMessage, NULL, (DialogPtr) -1L);
+	GetDItem(win, diMessageTo, &itemtype, &itemhandle, &itemrect);
+	SetIText(itemhandle, tmpstr);
 	ShowWindow(win);
 	while (!done) {
 		draw_default_button(win, diMessageOK);
@@ -831,6 +1091,8 @@ message_dialog()
 			case diMessageOK:
 				GetDItem(win, diMessageText, &itemtype, &itemhandle, &itemrect);
 				msg = get_string_from_item(itemhandle);
+				GetDItem(win, diMessageTo, &itemtype, &itemhandle, &itemrect);
+				nsidedesc = get_string_from_item(itemhandle);
 				/* Fall into next case. */
 			case diMessageCancel:
 				done = TRUE;
@@ -838,10 +1100,12 @@ message_dialog()
 		}
 	}
 	/* Close down the dialog (*before* executing any command). */
-	DisposDialog(win);
+	DisposeDialog(win);
+	update_all_map_windows();
 	/* Now send the message (if it wasn't cancelled) */
-	if (msg != NULL) {
-		send_message(dside, ALLSIDES, msg);
+	if (msg != NULL && nsidedesc != NULL) {
+		/* (should parse nsidedesc and use) */
+		net_send_message(dside, sidemask, msg);
 	}
 }
 
@@ -852,8 +1116,6 @@ do_move_to()
 	query_position_modally(MOVE_TO_MODAL);
 }
 
-extern int do_one_move_to(Unit *unit);
-
 int
 do_move_to_command()
 {
@@ -868,31 +1130,31 @@ do_move_to_command()
 	return FALSE;
 }
 
-int
+static int
 do_one_move_to(Unit *unit)
 {
 #ifdef DESIGNERS
 	if (dside->designer) {
-		designer_teleport(unit, tmpcmdx, tmpcmdy, NULL);
+		net_designer_teleport(unit, tmpcmdx, tmpcmdy, NULL);
 		return TRUE;
 	}
 #endif /* DESIGNERS */
-	set_move_to_task(unit, tmpcmdx, tmpcmdy);
+	net_set_move_to_task(unit, tmpcmdx, tmpcmdy);
 	return TRUE;
 }
 
-int
-do_one_set_name(Unit *unit)
-{
-	return unit_rename_dialog(unit);
-}
-
 void
 do_name()
 {
 	apply_to_all_selected(do_one_set_name, TRUE);
 }
 
+int
+do_one_set_name(Unit *unit)
+{
+	return unit_rename_dialog(unit);
+}
+
 int
 do_one_occupant(Unit *unit)
 {
@@ -918,13 +1180,17 @@ do_occupant()
 	apply_to_all_selected(do_one_occupant, TRUE);
 }
 
+int other_command_prefixarg;
+
 void
 do_other()
 {
 	/* Don't allow recursion with this command. */
 	if (!doingother) {
 		doingother = TRUE;
+		other_command_prefixarg = prefixarg;
 		enable_command();
+		other_command_prefixarg = -1;
 		doingother = FALSE;
 	} else {
 		beep();
@@ -977,6 +1243,11 @@ create_command_dialog()
 	command_text = TENew(&commandtextrect, &commandtextrect);
 	TESetSelect(0, 32767, command_text);
 	TEDelete(command_text);
+	/* Seed the command buffer with a prefix arg if was supplied. */
+	if (other_command_prefixarg >= 0) {
+		sprintf(spbuf, "%d ", other_command_prefixarg);
+		TEInsert(spbuf, strlen(spbuf), command_text);
+	}
 	ShowWindow(commandwin);
 }
 
@@ -1061,17 +1332,6 @@ do_produce()
 	Unit *unit;
 
 	/* Try different alternatives to find a unit. */
-	map = map_from_window(FrontWindow());
-	if (map != NULL) {
-		for (i = 0; i < map->numselections; ++i) {
-			unit = map->selections[i];
-			if (unit != NULL && can_produce(unit)) {
-				unit_do_produce_2(unit);
-				return;
-			}
-		}
-		return;
-	}
 	list = list_from_window(FrontWindow());
 	if (list != NULL) {
 		unit = selected_unit_in_list(list);
@@ -1090,6 +1350,17 @@ do_produce()
 		}
 		return;
 	}
+	map = front_map();
+	if (map != NULL) {
+		for (i = 0; i < map->numselections; ++i) {
+			unit = map->selections[i];
+			if (unit != NULL && can_produce(unit)) {
+				unit_do_produce_2(unit);
+				return;
+			}
+		}
+		return;
+	}
 	/* No way to figure out the unit to be producing with, so complain. */
 	beep();
 }
@@ -1104,7 +1375,7 @@ unit_do_produce_2(Unit *unit)
 	  n = prefixarg;
 	for_all_material_types(m) {
 		if (um_acp_to_produce(unit->type, m) > 0) {
-			push_produce_task(unit, m, n);
+			net_push_produce_task(unit, m, n);
 			return;
 		}
 	}
@@ -1132,8 +1403,10 @@ do_recenter()
 	int avgx, avgy;
 	Map *map;
 
-	map = map_from_window(FrontWindow());
-	if (map != NULL) {
+	map = front_map();
+	/* Only apply recentering if the front map window is also
+	   the overall front window. */
+	if (map != NULL && map->window == FrontWindow()) {
 		allsumx = allsumy = 0;
 		unitcount = 0;
 		apply_to_all_selected(add_unit_position, FALSE);
@@ -1146,15 +1419,28 @@ do_recenter()
 	}
 }
 
+
+
 void
 set_focus(Map *map, int x, int y)
 {
-	if (!inside_area(x, y))
+	int	oldsx, oldsy, newsx, newsy;
+
+	if (!inside_area(x, y)) 
 	  return;
+	/* Save the old vp. */
+	oldsx = map->vp->sx;
+	oldsy = map->vp->sy;
+	/* Find the new vp. */
 	set_view_focus(map->vp, x, y);
 	m_center_on_focus(map);
-	set_map_scrollbars(map);
-	force_map_update(map);
+	newsx = map->vp->sx;
+	newsy = map->vp->sy;
+	/* Restore the old vp. */
+	map->vp->sx = oldsx;
+	map->vp->sy = oldsy;
+	/* Let scroll_map_window do the rest of the updating work. */	
+	scroll_map_window(map, newsx - oldsx, newsy - oldsy);
 }
 
 /* Recalculate and redraw everything. */
@@ -1186,6 +1472,21 @@ do_refresh()
 	}
 }
 
+void
+do_release()
+{
+	Side *side2;
+
+	if (cmdargstr) {
+		side2 = parse_side_spec(cmdargstr);
+		if (side2 != NULL && side2 != dside) {
+			net_set_controlled_by(side2, dside, FALSE);
+			return;
+		} 
+	}
+	beep();
+}
+
 void
 do_remove_terrain()
 {
@@ -1218,7 +1519,7 @@ do_one_remove_terrain(Unit *unit)
 				dir = approx_dir(tmpcmdx - unit->x, tmpcmdy - unit->y);
 			}
 			if (valid(check_remove_terrain_action(unit, unit, x, y, dir, t))) {
-				if (prep_remove_terrain_action(unit, unit, x, y, dir, t))
+				if (net_prep_remove_terrain_action(unit, unit, x, y, dir, t))
 				  return TRUE;
 			}
 		}
@@ -1226,10 +1527,41 @@ do_one_remove_terrain(Unit *unit)
 	return FALSE;
 }
 
+void
+do_repair()
+{
+	Map *map;
+	
+	if (get_a_unit(&map, &tmpcmdunit)) {
+		apply_to_all_selected(do_one_repair, TRUE);
+		return;
+	}
+	beep();
+}
+
+static int
+do_one_repair(Unit *unit)
+{
+	int i;
+	Unit *other;
+
+	other = tmpcmdunit;
+	/* If no unit designated, assume self-repair desired. */
+	if (tmpcmdunit == NULL)
+	  other = unit;
+	if (other != NULL
+		&& other->side == unit->side
+		&& valid(check_repair_action(unit, unit, other))) {
+		net_prep_repair_action(unit, unit, other);
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static int
 do_one_reserve(Unit *unit)
 {
-	set_unit_reserve(dside, unit, tmpcmdarg, tmprecurse);
+	net_set_unit_reserve(dside, unit, tmpcmdarg, tmprecurse);
 	return TRUE;
 }
 
@@ -1251,7 +1583,7 @@ static int
 do_one_return(Unit *unit)
 {
 	if (1 /* has a place to return to */) {
-		set_resupply_task(unit, NONMTYPE);
+		net_set_resupply_task(unit, NONMTYPE);
 		return TRUE;
 	}
 	return FALSE;
@@ -1273,7 +1605,7 @@ do_resign()
 		notify(dside, "You are already out of the game.");
 		beep();
 	} else if (CautionAlert(aConfirmResign, nil) == aiConfirmResignResign) {
-		resign_game(dside, NULL);
+		net_resign_game(dside, NULL);
 	}
 }
 
@@ -1297,7 +1629,7 @@ do_set_formation()
 					if (leader != follower
 						&& leader->side == follower->side
 						) {
-						set_formation(follower, leader, follower->x - leader->x, follower->y - leader->y, 1, 1);
+						net_set_formation(follower, leader, follower->x - leader->x, follower->y - leader->y, 1, 1);
 						++numcould;
 					} else {
 						++numnot;
@@ -1325,14 +1657,15 @@ do_set_rate()
 		slow = strtol(cmdargstr, &reststr, 10);
 		fast = strtol(reststr, &reststr, 10);
 		set_play_rate(slow, fast);
+		return;
 	}
+	beep();
 }
 
-static int
-do_one_asleep(Unit *unit)
+void
+do_sleep()
 {
-	set_unit_asleep(dside, unit, tmpcmdarg, tmprecurse);
-	return TRUE;
+	do_sleep_command(TRUE, prefixarg, FALSE);
 }
 
 void
@@ -1343,10 +1676,11 @@ do_sleep_command(int value, int radius, int recurse)
 	apply_to_all_selected(do_one_asleep, TRUE);	
 }
 
-void
-do_sleep()
+static int
+do_one_asleep(Unit *unit)
 {
-	do_sleep_command(TRUE, prefixarg, FALSE);
+	net_set_unit_asleep(dside, unit, tmpcmdarg, tmprecurse);
+	return TRUE;
 }
 
 void
@@ -1358,13 +1692,23 @@ do_standing_orders()
 		rslt = parse_standing_order(dside, cmdargstr);
 		if (rslt < 0)
 		  beep();
-	} else
-	  beep();
+		return;
+	}
+	beep();
 }
 
 void
-do_surrender_to()
+do_submit()
 {
+	Side *side2;
+
+	if (cmdargstr) {
+		side2 = parse_side_spec(cmdargstr);
+		if (side2 != NULL && side2 != dside) {
+			net_set_controlled_by(dside, side2, TRUE);
+			return;
+		} 
+	}
 	beep();
 }
 
@@ -1373,23 +1717,36 @@ do_survey()
 {
 	Map *map;
 
-	map = map_from_window(FrontWindow());
+	map = front_map();
 	if (map != NULL) {
 		toggle_survey(map);
+	} else {
+		beep();
 	}
 }
 
 void
 do_take()
 {
-    /* (should use argument) */
+	if (nummtypes == 0) {
+		beep();
+		return;
+	}
+	givetakeprefixarg = prefixarg;
 	apply_to_all_selected(do_one_take, TRUE);
 }
 
-void
-do_take_unit()
+int
+do_one_take(Unit *unit)
 {
-	beep();
+	short m, amts[MAXMTYPES], rslts[MAXMTYPES], needed;
+
+	for_all_material_types(m) {
+		amts[m] = givetakeprefixarg;
+	}
+	needed = take_supplies(unit, amts, rslts);
+	report_take(dside, unit, needed, rslts);
+	return TRUE;
 }
 
 void
@@ -1398,15 +1755,45 @@ do_trust()
 	Side *side2;
 
 	if (cmdargstr) {
+		if (*cmdargstr == '?') {
+			/* (should be an nlang.c function) */
+			char buf[BUFSIZE];
+			int somebody;
+			Side *side3;
+
+			for_all_sides(side2) {
+				sprintf(buf, "%s trust%s", short_side_title(side2),
+						(short_side_title_plural_p(side2) ? "" : "s"));
+				somebody = FALSE;
+				for_all_sides(side3) {
+					if (side3 != side2 && trusted_side(side2, side3)) {
+						strcat(buf, " ");
+						strcat(buf, short_side_title(side3));
+						somebody = TRUE;
+					}
+				}
+				if (!somebody) {
+					strcat(buf, " nobody");
+				}
+				notify(dside, "%s.", buf);
+			}
+			return;
+		}
 		side2 = parse_side_spec(cmdargstr);
 		if (side2 != NULL && side2 != dside) {
-			set_trust(dside, side2, 1);
+			net_set_trust(dside, side2, 1);
 			return;
 		} 
 	}
 	beep();
 }
 
+void
+do_up()
+{
+	do_up_command();
+}
+
 void
 do_version()
 {
@@ -1451,7 +1838,7 @@ void
 do_gdl()
 {
 	if (cmdargstr)
-	  interp_form(NULL, read_form_from_string(cmdargstr, NULL, NULL));
+	  interp_form(NULL, read_form_from_string(cmdargstr, NULL, NULL, NULL));
 	else
 	  beep();
 	end_printing_forms();
@@ -1515,7 +1902,7 @@ do_mac_set_map_angle()
 	int angle;
 	Map *map;
 
-	map = map_from_window(FrontWindow());
+	map = front_map();
 	if (map != NULL) {
 		angle = map->vp->angle;
 		if (angle == 90)
@@ -1525,8 +1912,31 @@ do_mac_set_map_angle()
 		else
 		  angle = 90;
 		map->vp->vertscale = (prefixarg < 1 ? 1 : prefixarg);
+		/* Erase other-map boxes in other windows */
+		draw_related_maps(map);
+
+		/* Change the view angle */
 		set_view_angle(map->vp, angle);
-		force_map_update(map);
+
+		/* Update the gworld, but dont show the redrawn map yet */
+		ValidRect(&map->contentrect);
+		BeginUpdate(map->window);		
+		update_gworld(map);
+
+		/* Redraw other-map boxes in other windows */
+		draw_related_maps(map);
+		
+		/* Focus on active unit */
+		do_recenter();
+		
+		/* Now finally copy the result to the screen */
+		EndUpdate(map->window);
+		copy_from_gworld(map, map->contentrect);
+		
+		/* Fix scrollbars if necessary */
+		adjust_map_decor(map);
+	} else {
+		beep();
 	}
 }
 
@@ -1535,7 +1945,7 @@ do_mac_zoom_in()
 {
 	Map *map;
 
-	map = map_from_window(FrontWindow());
+	map = front_map();
 	if (map != NULL) {
 		magnify_map(map, 1);
 	} else {
@@ -1543,13 +1953,12 @@ do_mac_zoom_in()
 	}
 }
 
-
 void
 do_mac_zoom_out()
 {
 	Map *map;
 
-	map = map_from_window(FrontWindow());
+	map = front_map();
 	if (map != NULL) {
 		magnify_map(map, -1);
 	} else {
