@@ -41,7 +41,7 @@ int default_draw_topline = TRUE;	/* Draw top line boxes in maps */
 int default_draw_topunit = TRUE;	/* Draw unit info boxes in maps */
 int default_drawothermaps = TRUE;	/* Draw other maps as boxes */
 int default_erase_names = FALSE;	/* Draw extra rows to erase name of moving unit */
-int default_max_bufx = 400;			/* Horizontal scrolling buffers in offscreen graphics */
+int default_max_bufx = 600;			/* Horizontal scrolling buffers in offscreen graphics */
 int default_max_bufy = 400;			/* Vertical scrolling buffers in offscreen graphics */
 int default_optimize_fonts = TRUE;	/* Use optimized font sizes in map content */
 int default_power = 5;				/* Default power of new map */
@@ -67,7 +67,16 @@ RGBColor *featColor;				/* Feature colors by RGB value */
 RGBColor *sideColor;				/* sideColor[0-15][1-3] are defined by default color schemes */
 									/* sideColor[][0] is not defined, and available for future use */
 
-char **colorscheme;					/* Color schemes by name (red,pink,brown) etc */
+// --- 68K COLORSCHEME BUG FIX -----------------------------------------------------
+
+extern char **current_colorscheme = 0;		/* Current color schemes by names (red,pink,blue) */
+									/* Unlike side->colorscheme defined also for null sides */	
+									/* Declared extern for use in macwins.c */
+
+//	char **colorscheme; 			/* Color schemes by name (red,pink,brown) etc */
+
+// ----------------------------------------------------------------------------
+
 char **default_featColorName;		/* Default feature colors by name (black etc) */
 char **default_colorscheme;			/* Default color schemes (red,pink,brown) etc */
 
@@ -93,12 +102,22 @@ set_default_colors(void)			/* Default colors used in absence of saved preference
 	default_featColorName = (char **) xmalloc((FEATURES+1) * sizeof(char *));
 	sideColor = (RGBColor *) xmalloc((MAXSIDES+1) * 4 * sizeof(RGBColor));
 	sideColorName = (char **) xmalloc((MAXSIDES+1) * 4 * sizeof(char *));
-	colorscheme = (char **) xmalloc((MAXSIDES+1) * sizeof(char *));
+
+// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
+
+	current_colorscheme = (char **) xmalloc((MAXSIDES+1) * sizeof(char *)); 
+
+//	colorscheme = (char **) xmalloc((MAXSIDES+1) * sizeof(char *)); 
+
 	default_colorscheme = (char **) xmalloc((MAXSIDES+1) * sizeof(char *));
-	for (i = 0; i <= MAXSIDES; i++) {
-		side_n(i)->colorscheme = (char *) xmalloc(96);
-		strcpy(side_n(i)->colorscheme,  "");
-	}
+
+//	for (i = 0; i <= MAXSIDES; i++) {
+//		side_n(i)->colorscheme = (char *) xmalloc(96);
+//		strcpy(side_n(i)->colorscheme,  "");
+//	}
+
+// ----------------------------------------------------------------------------
+
 	main_icon_color = (int *) xmalloc((MAXSIDES+1) * sizeof(int));
 	half_icon_color = (int *) xmalloc((MAXSIDES+1) * sizeof(int));
 	icon_mask_color = (int *) xmalloc((MAXSIDES+1) * sizeof(int));
@@ -127,7 +146,15 @@ set_default_colors(void)			/* Default colors used in absence of saved preference
 	default_featColorName[3] = "midnight-blue";	/* Text color */
 	default_featColorName[4] = "dark-gray";			/* Grid color */
 	default_featColorName[5] = "black";			/* Unseen color */
-	default_featColorName[6] = "dark-gray";			/* Shorelines */
+
+// --- DONT USE SAME DEFAULT COLORS FOR SHORELINES AND GRID ------------------------
+
+	default_featColorName[6] = "blue"; 		/* Shorelines */   // "mountains-3" also looks OK!
+
+//	default_featColorName[6] = "dark-gray";	/* Shorelines */
+
+// ---------------------------------------------------------------------
+
 	default_featColorName[7] = "deep-pink";		/* Feature text and borders */
 	default_featColorName[8] = "red";			/* Frontlines */
 	default_featColorName[9] = "midnight-blue";	/* Meridians */
@@ -149,11 +176,27 @@ void
 set_default_side_colors(int e)
 {
 	/* First reset current colors to default colors */
-	strcpy( side_n(e)->colorscheme, default_colorscheme[e]);
+
+// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
+
+	side_n(e)->colorscheme = default_colorscheme[e];
+
+//	strcpy( side_n(e)->colorscheme, default_colorscheme[e]);
+
+// ----------------------------------------------------------------------------
+
 	/* Change the current colors */
 	set_side_colors(e);
 	/* Copy the result back to default colors */
+
+// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
+
 	default_colorscheme[e] = side_n(e)->colorscheme;
+
+//	strcpy(default_colorscheme[e], side_n(e)->colorscheme);
+
+// ---------------------------------------------------------------------------
+
 	force_overall_update();
 	save_preferences();
 }
@@ -189,15 +232,26 @@ init_side_colors(void)
 			set_sideColorName(i, 2, "undefined");
 			set_sideColorName(i, 3, "undefined");
 
-			/* First load the default colorschemes */
-			colorscheme[i] = default_colorscheme[i];
+// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
 
-			/* Then overwrite them with schemes from game file */
-			if (strcmp(side_n(i)->colorscheme, "") != 0) {
-				colorscheme[i] = side_n(i)->colorscheme;
-			}
-			/* Dissect scheme into colors and color names */
-			dissect_colorscheme(i, colorscheme[i]);
+			/* Use schemes defined in game file if present or else default schemes */
+			if (side_n(i) && side_n(i)->colorscheme)
+				current_colorscheme[i] = copy_string(side_n(i)->colorscheme);
+			else	current_colorscheme[i] = copy_string(default_colorscheme[i]);
+				
+			dissect_colorscheme(i);
+
+//			/* First load the default colorschemes */
+//			colorscheme[i] = default_colorscheme[i];
+
+//			/* Then overwrite them with schemes from game file */
+//			if (strcmp(side_n(i)->colorscheme, "") != 0) {
+//				colorscheme[i] = side_n(i)->colorscheme;
+//			}
+//			/* Dissect scheme into colors and color names */
+//			dissect_colorscheme(i, colorscheme[i]);
+
+// ----------------------------------------------------------------------------
 
 		}
 
@@ -224,14 +278,33 @@ init_side_colors(void)
 		featColorName[i] = default_featColorName[i];
 		featColor[i] = get_color(featColorName[i]);
 	}
+
+// --- MOVED HERE FROM INIT_TERRAIN_IMAGES -----------------------------------------
+
+	if (strcmp(gridcolor_name, unseencolor_name) == 0
+	    && empty_string(g_unseen_image_name()))
+		  grid_matches_unseen = TRUE;
+	blackcolor.red = blackcolor.green = blackcolor.blue = 0;
+
+// ---------------------------------------------------------------------------
+
 }
 
 /* This function is based on init_emblem from the X11 interface. */
 /* It also tests each name for validity and stores it in an array. */
 
+// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
+
 void
-dissect_colorscheme(int sidenum, char* colorscheme)
+dissect_colorscheme(int sidenum)
+
+//	dissect_colorscheme(int sidenum, char* colorscheme)
+
 {
+	char		*colorscheme = current_colorscheme[sidenum];
+	
+// ----------------------------------------------------------------------------
+
 	char	 	*s, *c;
 	char		cbuf[BUFSIZE];
 	int 		i, button;
