@@ -277,6 +277,18 @@ static void
 init_toolbox()
 {
 	SysEnvRec se;
+	
+// --- 68K DEFAULT STACK SIZE INCREASE ---------------------------
+
+#if  __MC68K__
+
+	/* Increase stack size from default 24K (ColorQD) to 64K by adding 40K. */
+
+	SetApplLimit(GetApplLimit() - 40960);
+	
+#endif
+
+// --------------------------------------------------------	
 
 	MaxApplZone();
 
