@@ -743,9 +743,17 @@ extern void draw_other_maps(Map *map);
 extern void draw_related_maps(Map *map);
 extern void draw_other_map(Map *map, Map *map2);
 extern void draw_row(Map *map, int x0, int y0, int len, int clearit);
-extern int cell_update(Map *map, int x, int y);
-extern void draw_terrain_row(Map *map, int x0, int y0, int len);
-extern void draw_legend(Map *map, int x, int y);
+
+// --- MOVED TO MACROW.C -----------------------------
+
+//	extern int cell_update(Map *map, int x, int y);
+//	extern void draw_terrain_row(Map *map, int x0, int y0, int len);
+
+// --- NO LONGER USED --------------------------------
+
+//	extern void draw_legend(Map *map, int x, int y);
+
+// ------------------------------------------------
 
 extern void draw_selections(Map *map);
 extern void draw_selections_at(Map *map, int x, int y);
@@ -1126,8 +1134,15 @@ extern void init_unit_cicns(void);			/* Intitialize cicns by a dummy PlotCIcon c
 extern void init_emblem_cicns(void);			/* Intitialize cicns ny a dummy PlotCIcon call */
 extern void dispose_unit_cicns(void);			/* Trash every unit cicn in use */
 extern void dispose_emblem_cicns(void);		/* Trash every emblem cicn in use */
-extern void dissect_colorscheme(int sidenum, 	/* Dissect a colorscheme into sideColorNames */
-		 char* colorscheme);				
+
+// --- 68K COLORSCHEME BUG FIX -----------------------------------------------
+
+extern void dissect_colorscheme(int e);	 	/* Dissect a colorscheme into sideColorNames */
+
+//  extern void dissect_colorscheme(int sidenum, 	/* Dissect a colorscheme into sideColorNames */
+//		 char* colorscheme);				
+
+// -----------------------------------------------------------------------
 
 /* Functions in maccmd.c now declared here for use elsewhere */
 
