@@ -658,6 +658,109 @@ draw_hex_region(int sx, int sy, int power, int t, int over, int angle, int dogri
 	}
 }
 
+// --- MOVED HERE FROM MACMAP.C ---------------------------------
+
+/* Given a magnification power, compute the clipping regions to be used at
+   that power. */
+
+void
+make_cell_clip(int power)
+{
+	int hw = hws[power], hh = hhs[power], delt = (hhs[power] - hcs[power]);
+	PolyHandle poly;
+	RgnHandle tmprgn;
+
+	/* Make a hexagon region by drawing a polygon and then framing it while
+	   a region is open. */
+	poly = OpenPoly();
+	MoveTo(hw / 2, 0);
+	LineTo(hw, delt);
+	LineTo(hw, hh - delt);
+	LineTo(hw / 2, hh);
+	LineTo(0, hh - delt);
+	LineTo(0, delt);
+	LineTo(hw / 2, 0);
+	ClosePoly();
+	cellrgns[power] = NewRgn();
+	OpenRgn();
+	FramePoly(poly);
+	CloseRgn(cellrgns[power]);
+	/* Make the grid-displaying version of the hexagon. */
+	gridcellrgns[power] = NewRgn();
+	CopyRgn(cellrgns[power], gridcellrgns[power]);
+	/* Cut off a one-pixel line on the side. */
+	tmprgn = NewRgn();
+	SetRectRgn(tmprgn, hw - 1, 0, hw + 1, hh); 
+	DiffRgn(gridcellrgns[power], tmprgn, gridcellrgns[power]);
+	/* Now intersect with a region shifted upwards by one, which makes
+	   the grid line along the bottom of the hex. */
+	tmprgn = NewRgn();
+	CopyRgn(cellrgns[power], tmprgn);
+	OffsetRgn(tmprgn, 0, -1);
+	SectRgn(gridcellrgns[power], tmprgn, gridcellrgns[power]);
+
+	/* Similarly, but for cells at an angle. */
+	/* (should only calc when angle view first requested) */
+	/* First make a region as viewed at a 30-degree angle. */
+	hh /= 2;  delt /= 2;
+	poly = OpenPoly();
+	MoveTo(hw / 2, 0);
+	LineTo(hw, delt);
+	LineTo(hw, hh - delt);
+	LineTo(hw / 2, hh);
+	LineTo(0, hh - delt);
+	LineTo(0, delt);
+	LineTo(hw / 2, 0);
+	ClosePoly();
+	cellrgns30[power] = NewRgn();
+	OpenRgn();
+	FramePoly(poly);
+	CloseRgn(cellrgns30[power]);
+	gridcellrgns30[power] = NewRgn();
+	CopyRgn(cellrgns30[power], gridcellrgns30[power]);
+	/* Cut off a one-pixel line on the side. */
+	tmprgn = NewRgn();
+	SetRectRgn(tmprgn, hw - 1, 0, hw + 1, hh/2); 
+	DiffRgn(gridcellrgns30[power], tmprgn, gridcellrgns30[power]);
+	/* Now intersect with a region shifted upwards by one, which makes
+	   the grid line along the bottom of the hex. */
+	tmprgn = NewRgn();
+	CopyRgn(cellrgns30[power], tmprgn);
+	OffsetRgn(tmprgn, 0, -1);
+	SectRgn(gridcellrgns30[power], tmprgn, gridcellrgns30[power]);
+
+	/* Now make a region as viewed at a 15-degree angle. */
+	hh = hhs[power] / 4;
+	delt = (hhs[power] - hcs[power]) / 4;
+	poly = OpenPoly();
+	MoveTo(hw / 2, 0);
+	LineTo(hw, delt);
+	LineTo(hw, hh - delt);
+	LineTo(hw / 2, hh);
+	LineTo(0, hh - delt);
+	LineTo(0, delt);
+	LineTo(hw / 2, 0);
+	ClosePoly();
+	cellrgns15[power] = NewRgn();
+	OpenRgn();
+	FramePoly(poly);
+	CloseRgn(cellrgns15[power]);
+	gridcellrgns15[power] = NewRgn();
+	CopyRgn(cellrgns15[power], gridcellrgns15[power]);
+	/* Cut off a one-pixel line on the side. */
+	tmprgn = NewRgn();
+	SetRectRgn(tmprgn, hw - 1, 0, hw + 1, hh/2); 
+	DiffRgn(gridcellrgns15[power], tmprgn, gridcellrgns15[power]);
+	/* Now intersect with a region shifted upwards by one, which makes
+	   the grid line along the bottom of the hex. */
+	tmprgn = NewRgn();
+	CopyRgn(cellrgns15[power], tmprgn);
+	OffsetRgn(tmprgn, 0, -1);
+	SectRgn(gridcellrgns15[power], tmprgn, gridcellrgns15[power]);
+}
+
+// --------------------------------------------------------------
+
 /* Draw a set of borders for the given position. */
 
 void
@@ -728,6 +831,14 @@ draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power,
 		}
 	}
 	for_all_directions(dir) {
+
+// --- SPEEDS UP DRAWING & PREVENTS DOUBLE LINES AT CLIFF TOP & BOTTOM BUG ----------
+
+		if (dir == EAST || dir == SOUTHWEST || dir == SOUTHEAST)
+			continue;
+
+// ----------------------------------------------------------------------
+
 		if (bitmask & (1 << dir)) {
 			/* Actually draw the line. */
 			sx1 = bsx[power][dir];  sy1 = bsy[power][dir];
@@ -756,6 +867,14 @@ draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power,
 			PenMode(notPatOr);
 		}
 		for_all_directions(dir) {
+
+// --- SPEEDS UP DRAWING & PREVENTS DOUBLE LINES AT CLIFF TOP & BOTTOM BUG ----------
+
+			if (dir == EAST || dir == SOUTHWEST || dir == SOUTHEAST)
+				continue;
+
+// ----------------------------------------------------------------------
+
 			if (bitmask & (1 << dir)) {
 				/* Actually draw the line. */
 				sx1 = bsx[power][dir];  sy1 = bsy[power][dir];
