@@ -13,11 +13,65 @@ any later version.  See the file COPYING.  */
 /* Needed for changing screen depths and handling the control strip */
 
 #include <Palettes.h>
+
 #ifdef CONTROL_STRIP
+
+// --- NEW STUFF MOVED HERE FROM APPLESTUFF.H ------------------
+
+/*	Apple control strip routine descriptors follow below. */
+/*	Must be included in PPC projects that use the control strip. *
+/*	See http://devworld.apple.com/dev/qa/ops/ops15.html. */
+
+#if GENERATINGCFM
+
+/* If we're not generating CFM, then assume the
+   68K inlines in the headers apply instead. */
+
+#include <MixedMode.h>
+#include <OSUtils.h>
+pascal Boolean SBIsControlStripVisible ( void );
+pascal void SBShowHideControlStrip(Boolean showIt);
+
+/*  SBIsControlStripVisible is a Pascal routine, */
+/*  dispatched from the selector in D0, returning */
+/*  a Boolean result */
+pascal Boolean SBIsControlStripVisible ( void )
+{
+    enum
+        {
+        uppSBIsControlStripVisibleInfo = kD0DispatchedPascalStackBased
+            | RESULT_SIZE (SIZE_CODE (sizeof(Boolean)))
+            | DISPATCHED_STACK_ROUTINE_SELECTOR_SIZE (kFourByteCode)
+            };
+
+            return CallUniversalProc (
+            GetToolTrapAddress (_ControlStripDispatch),
+            uppSBIsControlStripVisibleInfo, 0x00);
+}
+
+pascal void SBShowHideControlStrip(Boolean showIt)
+{
+    enum
+        {
+        uppSBShowHideControlStripInfo =
+            kD0DispatchedPascalStackBased
+            | DISPATCHED_STACK_ROUTINE_SELECTOR_SIZE (kFourByteCode)
+            | DISPATCHED_STACK_ROUTINE_PARAMETER
+                (1, SIZE_CODE (sizeof (showIt)))
+            };
+
+            CallUniversalProc (
+            GetToolTrapAddress (_ControlStripDispatch),
+            uppSBShowHideControlStripInfo, 0x01, showIt);
+}
+
+#else   /*  not GENERATINGCFM */
 #include <ControlStrip.h>
-/* Apple control strip routine descriptors */
-#include "applestuff.h"
-#endif
+#endif /*  GENERATINGCFM */
+
+// --- END NEW STUFF -------------------------------------------------------
+
+#endif /* CONTROL_STRIP */
 
 // --- USED TO DEBUG OFFSCREEN GRAPHICS ----------------------------------------
 
