@@ -1,5 +1,5 @@
 /* Help for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -67,10 +67,12 @@ draw_instructions_text(WindowPtr win, short ditem)
 	GetDItem(instructionswin, diInstructionsText, &itemtype, &itemhandle, &itemrect);
 	GetPort(&oldport);
 	SetPort(instructionswin);
-	TextSize(14);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	TEUpdate(&itemrect, instructions_text);
 	/* This makes the title item draw big. */
-	TextSize(18);
+	TextFont(title_font_id);
+	TextSize(title_font_size);
 	SetPort(oldport);
 }
 
@@ -97,9 +99,29 @@ instructions_dialog()
 	SelectWindow(instructionswin);
 }
 
+void insert_newlines(TEHandle te, int n);
+
+void
+insert_newlines(TEHandle te, int n)
+{
+	int i;
+	char str[20];
+
+	str[0] = '\0';
+	for (i = 0; i < n; ++i) {
+#ifdef THINK_C
+		strcat(str, "\r");
+#else
+		strcat(str, "\n");
+#endif
+	}
+	TEInsert(str, strlen(str), te);
+}
+
 void
 create_instructions_dialog()
 {
+	int any_instructions = FALSE;
 	Obj *instructions, *rest;
 	char *str;
 	Str255 tmpstr;
@@ -119,73 +141,98 @@ create_instructions_dialog()
 	destrect = itemrect;
 	viewrect = itemrect;
 	SetPort(instructionswin);
-	/* All text will be in Times. */
-	/* (should put sizes in a resource so can be edited) */
-	TextFont(times);
-	TextSize(14);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	instructions_text = TENew(&destrect, &viewrect);
 	TESetSelect(0, 32767, instructions_text);
 	TEDelete(instructions_text);
-	if (mainmodule) {
-		instructions = mainmodule->instructions;
-		if (instructions != lispnil) {
-			if (stringp(instructions)) {
-				str = c_string(instructions);
-				TEInsert(str, strlen(str), instructions_text);
-			} else if (consp(instructions)) {
-			    for (rest = instructions; rest != lispnil; rest = cdr(rest)) {
-			    	if (stringp(car(rest))) {
-			    		str = c_string(car(rest));
-			    		/* An empty line is a line break. */
-			    		if (strlen(str) == 0) {
-#ifdef THINK_C
-			    			str = "\r";
-#else
-			    			str = "\n";
-#endif
-			    		}
-			    		TEInsert(str, strlen(str), instructions_text);
-			    		TESetSelect(32767, 32767, instructions_text);
-			    		/* Insert a blank between strings if none present, since they are
-			    		   usually word breaks. */
-			    		if (str[strlen(str)-1] != ' ')
-			    		  TEInsert(" ", 1, instructions_text);
-			    		TESetSelect(32767, 32767, instructions_text);
-			    	}
+	/* (should generify, add parameter for how to line break) */
+	if (mainmodule == NULL) {
+		run_error("no main module?");
+	}
+	instructions = mainmodule->instructions;
+	if (instructions != lispnil) {
+		if (stringp(instructions)) {
+			str = c_string(instructions);
+			TEInsert(str, strlen(str), instructions_text);
+			any_instructions = TRUE;
+		} else if (consp(instructions)) {
+		    for (rest = instructions; rest != lispnil; rest = cdr(rest)) {
+		    	if (stringp(car(rest))) {
+		    		str = c_string(car(rest));
+		    		/* An empty string is a line break. */
+		    		if (strlen(str) == 0) {
+		    			insert_newlines(instructions_text, 1);
+		    		} else {
+		    			TEInsert(str, strlen(str), instructions_text);
+		    		}
+		    		TESetSelect(32767, 32767, instructions_text);
+		    		/* Insert a blank between strings if none present, since they are
+		    		   usually word breaks. */
+		    		if (str[strlen(str)-1] != ' ')
+		    		  TEInsert(" ", 1, instructions_text);
+		    		TESetSelect(32767, 32767, instructions_text);
 		    	}
-		    } else {
-		    	/* error? */
-		    }
-		} else {
-			if (mainmodule->blurb) {
-				str = mainmodule->blurb;
-				TEInsert(str, strlen(str), instructions_text);
-#ifdef THINK_C
-				str = "\r\r";
-#else
-				str = "\n\n";
-#endif
-				TEInsert(str, strlen(str), instructions_text);
-			}
-			str = "(no instructions supplied)";
+	    	}
+			any_instructions = TRUE;
+	    } else {
+	    	/* error? */
+	    }
+	}
+	instructions = dside->instructions;
+	if (instructions != lispnil) {
+		if (stringp(instructions)) {
+			str = "To you only:";
+		    insert_newlines(instructions_text, 1);
+			TEInsert(str, strlen(str), instructions_text);
+			str = c_string(instructions);
+			TEInsert(str, strlen(str), instructions_text);
+			any_instructions = TRUE;
+		} else if (consp(instructions)) {
+			str = "To you only:";
+			TEInsert(str, strlen(str), instructions_text);
+		    insert_newlines(instructions_text, 1);
+		    for (rest = instructions; rest != lispnil; rest = cdr(rest)) {
+		    	if (stringp(car(rest))) {
+		    		str = c_string(car(rest));
+		    		/* An empty string is a line break. */
+		    		if (strlen(str) == 0) {
+		    			insert_newlines(instructions_text, 1);
+		    		} else {
+		    			TEInsert(str, strlen(str), instructions_text);
+		    		}
+		    		TESetSelect(32767, 32767, instructions_text);
+		    		/* Insert a blank between strings if none present, since they are
+		    		   usually word breaks. */
+		    		if (str[strlen(str)-1] != ' ')
+		    		  TEInsert(" ", 1, instructions_text);
+		    		TESetSelect(32767, 32767, instructions_text);
+		    	}
+	    	}
+			any_instructions = TRUE;
+	    } else {
+	    	/* error? */
+	    }
+	}
+	/* Fall back on some general info. */
+	if (!any_instructions) {
+		if (mainmodule->blurb) {
+			str = mainmodule->blurb;
+			TEInsert(str, strlen(str), instructions_text);
+			insert_newlines(instructions_text, 2);
+		}
+		str = "(no instructions supplied)";
+		TEInsert(str, strlen(str), instructions_text);
+		for_all_scorekeepers(sk) {
+			/* (should add better scorekeeper descriptions) */
+			insert_newlines(instructions_text, 1);
+			str = "Scoring: ";
 			TEInsert(str, strlen(str), instructions_text);
-			for_all_scorekeepers(sk) {
-				/* (should add better scorekeeper descriptions) */
-#ifdef THINK_C
-			    str = "\r";
-#else
-			    str = "\n";
-#endif
-				TEInsert(str, strlen(str), instructions_text);
-				str = "a scorekeeper";
-				TEInsert(str, strlen(str), instructions_text);
-			}
 		}
-	} else {
-		/* this should be impossible? */
 	}
 	/* This makes the title item draw big. */
-	TextSize(18);
+	TextFont(title_font_id);
+	TextSize(title_font_size);
 	add_window_menu_item("Instructions", instructionswin);
 }
 
@@ -221,84 +268,78 @@ help_dialog(HelpNode *helpnode)
 }
 
 void
-describe_menus(int arg, char *key, char *buf)
+describe_menus(int arg, char *key, TextBuffer *buf)
 {
-	strcat(buf, "File\n");
-	strcat(buf, "  The usual file operations.\n");
-	strcat(buf, "\n");
-	strcat(buf, "Edit\n");
-	strcat(buf, "  Select All selects all of your units at once.\n");
-	strcat(buf, "\n");
-	strcat(buf, "Find\n");
-	strcat(buf, "  Find Selected goes to the next map back and looks at the unit selected or viewed in the front window.\n");
-	strcat(buf, "\n");
-	strcat(buf, "Play\n");
-	strcat(buf, "  This is the main menu for unit actions.");
-	strcat(buf, "  Each item represents something that you can do to the currently selected units.\n");
-	strcat(buf, "  If an action is grayed out, then none of the selected units can do it.\n");
-	strcat(buf, "  Closeup shows details of the selected units.\n");
-	strcat(buf, "  Build brings up the construction dialog.\n");
-	strcat(buf, "Side\n");
-	strcat(buf, "  This menu is for things that affect your whole side.\n");
-	strcat(buf, "Windows\n");
-	strcat(buf, "  This menu does window control.\n");
-	strcat(buf, "View (when a map is front window)\n");
-	strcat(buf, "  Items toggle various display elements in the front map window.\n");
-	strcat(buf, "\n");
-	strcat(buf, "View (when a list is front window)\n");
-	strcat(buf, "  Items toggle various display elements in the front list window.\n");
-	strcat(buf, "\n");
+	tbcat(buf, "File\n");
+	tbcat(buf, "  The usual file operations.\n");
+	tbcat(buf, "\n");
+	tbcat(buf, "Edit\n");
+	tbcat(buf, "  Select All selects all of your units at once.\n");
+	tbcat(buf, "\n");
+	tbcat(buf, "Find\n");
+	tbcat(buf, "  Find Selected goes to the next map back and looks at the unit selected or viewed in the front window.\n");
+	tbcat(buf, "\n");
+	tbcat(buf, "Play\n");
+	tbcat(buf, "  This is the main menu for unit actions.");
+	tbcat(buf, "  Each item represents something that you can do to the currently selected units.\n");
+	tbcat(buf, "  If an action is grayed out, then none of the selected units can do it.\n");
+	tbcat(buf, "  Closeup shows details of the selected units.\n");
+	tbcat(buf, "  Build brings up the construction dialog.\n");
+	tbcat(buf, "Side\n");
+	tbcat(buf, "  This menu is for things that affect your whole side.\n");
+	tbcat(buf, "Windows\n");
+	tbcat(buf, "  This menu does window control.\n");
+	tbcat(buf, "View (when a map is front window)\n");
+	tbcat(buf, "  Items toggle various display elements in the front map window.\n");
+	tbcat(buf, "\n");
+	tbcat(buf, "View (when a list is front window)\n");
+	tbcat(buf, "  Items toggle various display elements in the front list window.\n");
+	tbcat(buf, "\n");
 }
 
 void
-describe_mouse(int arg, char *key, char *buf)
+describe_mouse(int arg, char *key, TextBuffer *buf)
 {
-	strcat(buf, "In move-on-click mode:\n");
-	strcat(buf, "  The next unit that can do anything will be selected automatically.\n");
-	strcat(buf, "  Click once on a destination to move the selected unit there.\n");
-	strcat(buf, "\n");
-	strcat(buf, "In normal mode:\n");
-	strcat(buf, "  Click once on a unit to select it.\n");
-	strcat(buf, "  Drag to destination to move it.\n");
-	strcat(buf, "  Shift-click to select additional units.\n");
-	strcat(buf, "  Drag out rectangle to select all units inside.\n");
-	strcat(buf, "\n");
-	strcat(buf, "Command-click to moves all selected units to the clicked-on location.");
-	strcat(buf, "\n");
-	strcat(buf, "Combat is automatic if another side's unit is adjacent and clicked on.");
+	tbcat(buf, "In move-on-click mode:\n");
+	tbcat(buf, "  The next unit that can do anything will be selected automatically.\n");
+	tbcat(buf, "  Click once on a destination to move the selected unit there.\n");
+	tbcat(buf, "\n");
+	tbcat(buf, "In normal mode:\n");
+	tbcat(buf, "  Click once on a unit to select it.\n");
+	tbcat(buf, "  Drag to destination to move it.\n");
+	tbcat(buf, "  Shift-click to select additional units.\n");
+	tbcat(buf, "  Drag out rectangle to select all units inside.\n");
+	tbcat(buf, "\n");
+	tbcat(buf, "Command-click to moves all selected units to the clicked-on location.");
+	tbcat(buf, "\n");
+	tbcat(buf, "Combat is automatic if another side's unit is adjacent and clicked on.");
 }
 
 void
-describe_keyboard(int arg, char *key, char *buf)
+describe_help(int arg, char *key, TextBuffer *buf)
 {
-	describe_commands(arg, key, buf);
-}
-
-void
-describe_help(int arg, char *key, char *buf)
-{
-	strcat(buf, "Xconq Help consists of a number of ``topics''.  ");
-	strcat(buf, "Topics include generic information about Xconq as ");
-	strcat(buf, "well as specific details of the game in progress.  ");
-	strcat(buf, "To navigate, click the buttons above.");
-	strcat(buf, "\n");
-	strcat(buf, "Topics: ");
-	strcat(buf, "Clicking this button shows you the list of topics.  ");
-	strcat(buf, "Clicking on a topic in the list takes you to it directly.");
-	strcat(buf, "\n");
-	strcat(buf, "Help:  ");
-	strcat(buf, "Clicking this button shows you the topic you're looking at right now.");
-	strcat(buf, "\n");
-	strcat(buf, "Next:  ");
-	strcat(buf, "Clicking this button goes to the next topic in order.");
-	strcat(buf, "\n");
-	strcat(buf, "Previous:  ");
-	strcat(buf, "Clicking this button goes to the previous topic in order.");
-	strcat(buf, "\n");
-	strcat(buf, "Back:  ");
-	strcat(buf, "Clicking this button goes to the last topic you were looking at.  ");
-	strcat(buf, "Multiple clicks take you farther and farther back.");
-	strcat(buf, "\n");
+	tbcat(buf, "Xconq Help consists of a number of ``topics''.  ");
+	tbcat(buf, "Topics include generic information about Xconq as ");
+	tbcat(buf, "well as specific details of the game in progress.  ");
+	tbcat(buf, "To navigate, click the buttons above.");
+	tbcat(buf, "\n");
+	tbcat(buf, "Topics: ");
+	tbcat(buf, "Clicking this button shows you the list of topics.  ");
+	tbcat(buf, "Clicking on a topic in the list takes you to it directly.");
+	tbcat(buf, "\n");
+	tbcat(buf, "Help:  ");
+	tbcat(buf, "Clicking this button shows you the topic you're looking at right now.");
+	tbcat(buf, "\n");
+	tbcat(buf, "Next:  ");
+	tbcat(buf, "Clicking this button goes to the next topic in order.");
+	tbcat(buf, "\n");
+	tbcat(buf, "Previous:  ");
+	tbcat(buf, "Clicking this button goes to the previous topic in order.");
+	tbcat(buf, "\n");
+	tbcat(buf, "Back:  ");
+	tbcat(buf, "Clicking this button goes to the last topic you were looking at.  ");
+	tbcat(buf, "Multiple clicks take you farther and farther back.");
+	tbcat(buf, "\n");
 }
 
 void
@@ -319,15 +360,12 @@ create_help_window()
 	nextbutton = GetNewControl(cNextButton, helpwin);
 	backbutton = GetNewControl(cBackButton, helpwin);
 	SetPort(helpwin);
-	/* All text will be in Times. */
-	/* (should these be choosable?) */
-	TextFont(times);
-	/* Set up the topic title. */
-	TextSize(18);
+	TextFont(title_font_id);
+	TextSize(title_font_size);
 	SetRect(&helptopicrect, 45, 45, 305, 75); 
 	helptopic = TENew(&helptopicrect, &helptopicrect);
-	/* Set up the help text proper. */
-	TextSize(14);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	h = window_width(helpwin);  v = window_height(helpwin);
 	SetRect(&viewrect, 5, 75, h - sbarwid, v - sbarwid); 
 	destrect = viewrect;
@@ -344,9 +382,10 @@ create_help_window()
 	/* Add the Mac-specific help nodes. */
 	/* (Note that these will appear in *reverse* order from here, because each
 	   is being glued right after the first help node. */
+	add_help_node("textual commands", describe_text_commands, 0, first_help_node);
+	add_help_node("keyboard", describe_keyboard, 0, first_help_node);
 	add_help_node("menus", describe_menus, 0, first_help_node);
 	add_help_node("mouse", describe_mouse, 0, first_help_node);
-	add_help_node("keyboard", describe_keyboard, 0, first_help_node);
 	help_help_node = add_help_node("help", describe_help, 0, first_help_node);
 	topics_help_node = add_help_node("topics", describe_topics, 0, first_help_node);
 	cur_help_node = topics_help_node;
@@ -401,17 +440,29 @@ draw_help()
 
 	GetPort(&oldport);
 	SetPort(helpwin);
+	/* Draw a horizontal line under the navigation buttons. */
+	MoveTo(0, 30);
+	LineTo(helpwin->portRect.right - sbarwid, 30);
 	SetRect(&tmprect, 5, 40, 5 + 32, 40 + 32);
+	/* Grow so as to erase extra junk that may have been drawn. */
+	InsetRect(&tmprect, -2, -2);
 	EraseRect(&tmprect);
+	/* Back to intended size. */
+	InsetRect(&tmprect, 2, 2);
+	/* Draw a bit of graphic for some help nodes. */
 	if (cur_help_node->nclass == utypenode && is_unit_type(cur_help_node->arg)) {
 		draw_unit_image(helpwin, tmprect.left, tmprect.top, 32, 32,
-						cur_help_node->arg, -1, 0);
+						cur_help_node->arg, -1, 0, FALSE);
 	} else if (cur_help_node->nclass == ttypenode && is_terrain_type(cur_help_node->arg)) {
 		draw_terrain_sample(helpwin, tmprect, cur_help_node->arg); 
+	} else if (cur_help_node->nclass == mtypenode && is_material_type(cur_help_node->arg)) {
+		/* (should draw material type usefully) */ 
 	}
-	TextSize(18);
+	TextFont(title_font_id);
+	TextSize(title_font_size);
 	TEUpdate(&(helpwin->portRect), helptopic);
-	TextSize(14);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	TEUpdate(&(helpwin->portRect), helptext);
 	SetPort(oldport);
 	adjust_help_scrollbar();
