@@ -266,10 +266,25 @@ mac_add_cicn(ImageFamily *imf, Handle handle)
 			/* Mask is built in, don't need to load separately. */
 		}
 	}
-	/* We want to unlock the cicn now, and lock it high later on;
-	   but not if it was already processed. */
-	if (newcicn)
-	  HUnlock((Handle) cicnhandle);
+
+// --- HEAP FRAGMENTATION FIX -----------------------------------------
+
+	/* Always unlock this handle or the heap will be messed up beyond all recognition!
+	Moreover, if get_mac_image returns an already loaded cicn, the new cicnhandle is 
+	never used, as macimg->colricon still points to the old cicnhandle at the top of the 
+	heap. The new cicnhandle should therefore not only be unlocked but also trashed. */	
+
+	HUnlock((Handle) cicnhandle);
+	if (!newcicn)
+		DisposeCIcon(cicnhandle);
+
+//	/* We want to unlock the cicn now, and lock it high later on;
+//	   but not if it was already processed. */
+//	if (newcicn)
+//	  HUnlock((Handle) cicnhandle);
+	
+// --------------------------------------------------------------
+
 }
 
 void
