@@ -17,6 +17,7 @@ typedef struct a_mac_image {
 	Handle monoicon;				/* 32x32 mono icon */
 	Handle maskicon;				/* 32x32 mask icon */
 	Handle colricon;				/* color icon (can be any size) */
+	short cicn_inited;				/* true when the color icon has been run thru PlotCIcon */
 	PixPatHandle colrpat;			/* color pattern (can be 8x8 up to 64x64) */
 	short colordefined;
 	RGBColor color;
@@ -61,5 +62,5 @@ typedef struct a_mac_image {
   BUF[PSTR[0]] = '\0';
 
 extern MacImage *get_mac_image(Image *img);
-extern void mac_load_imf(ImageFamily *imf);
-extern void mac_interp_imf(ImageFamily *imf);
+extern ImageFamily *mac_load_imf(ImageFamily *imf);
+extern ImageFamily *mac_interp_imf(ImageFamily *imf, Image *img, int force);
