@@ -2113,13 +2113,29 @@ launch_game()
 	}
 	/* Set up the images we'll be needing. */
 	init_terrain_images();
-	init_unit_images();
-	init_side_colors();
+
+// --- HEAP FRAGMENTATION FIX -------------------------------------
+
+//	init_unit_images();		/* Moved below. */
+
+// -----------------------------------------------------------
+
+	init_side_colors();		
 	/* Trial side/player assignment, will create the default player if nececessary. */
 	make_trial_assignments();
 	/* Get the emblems so they can be displayed in the player dialog. */
 	init_all_emblem_images();
 	init_emblem_cicns();
+
+// --- HEAP FRAGMENTATION FIX -------------------------------------
+
+	/* It is important that unit cicns are loaded after the emblems since functions
+	that reinit cicns assume that emblems always are located above the unit cicns. */
+	
+	init_unit_images();
+
+// -----------------------------------------------------------
+
 	/* Ask for modifications to the trial assignment. */
 	return player_setup_dialog();
 }
@@ -2526,10 +2542,14 @@ init_terrain_images()
 	Image *timg;
 	MacImage *mactimg;
 
-	if (strcmp(gridcolor_name, unseencolor_name) == 0
-	    && empty_string(g_unseen_image_name()))
-	  grid_matches_unseen = TRUE;
-	blackcolor.red = blackcolor.green = blackcolor.blue = 0;
+// --- MOVED TO INIT_SIDE_COLORS -------------------------------------------------
+
+//	if (strcmp(gridcolor_name, unseencolor_name) == 0
+//	    && empty_string(g_unseen_image_name()))
+//	  grid_matches_unseen = TRUE;
+//	blackcolor.red = blackcolor.green = blackcolor.blue = 0;
+
+// --------------------------------------------------------------------------
 
 	timages = (ImageFamily **) xmalloc(numttypes * sizeof(ImageFamily *));
 
@@ -2607,11 +2627,36 @@ void
 reinit_all_emblem_images()
 {
 	dispose_emblem_cicns();
+
+// --- HEAP FRAGMENTATION FIX -----------------------------------
+
+	/* The emblem cicns are locked in place above the unit cicns, which are
+	also locked. To reinit the emblem cicns in their correct position we must 
+	therefore first dispose of the unit cicns and then reinit them after we are
+	done with the emblem cicns. */
+
+	/* Also free the emblem images. */
+	free(eimages);
+
+	/* Dispose of and free unit images. */
+	dispose_unit_cicns();
+	free(uimages);
+
+// ----------------------------------------------------------
+
 	/* This isn't great, because all the collected data goes away, and we have to
 	   reread and reallocate everything.  It would be smarter just to reinit the
 	   Mac-specific parts alone. */
 	init_all_emblem_images();
 	init_emblem_cicns();
+
+// --- HEAP FRAGMENTATION FIX -----------------------------------
+
+	/* Finally load the unit cicns again below the new emblems. */
+	init_unit_images();
+
+// ----------------------------------------------------------
+
 }
 
 /* Set up the emblems for a particular side.  This is used both to do all sides
@@ -2644,6 +2689,22 @@ init_unit_cicns()
 				mac_init_cicn(img);
 			}
 		}
+
+// --- HEAP FRAGMENTATION FIX ---------------------------------
+
+		/* The purpose of these dummy calls is to avoid later calls to
+		mac_init_cicn during the game which will mess up the heap. 
+		In particular, calls to best_image during offscreen drawing 
+		will put locked cicn handles all over the heap, since the gworld 
+		pixmap is locked in place at drawing time. */
+
+		/* Trigger add_shrunken_image once and for all. */
+		img = best_image(uimages[u], 1, 1);
+		/* Trigger add_magnified_image once and for all. */
+		img = best_image(uimages[u], 128, 128);
+
+// -------------------------------------------------------
+
 	}
 }
 
@@ -2688,6 +2749,22 @@ init_emblem_cicns()
 				mac_init_cicn(img);
 			}
 		}
+
+// --- HEAP FRAGMENTATION FIX ---------------------------------
+
+		/* The purpose of these dummy calls is to avoid later calls to
+		mac_init_cicn during the game which will mess up the heap. 
+		In particular, calls to best_image during offscreen drawing 
+		will put locked cicn handles all over the heap, since the gworld 
+		pixmap is locked in place at drawing time. */
+
+		/* Trigger add_shrunken_image once and for all. */
+		img = best_image(eimages[side_number(side)], 1, 1);
+		/* Trigger add_magnified_image once and for all. */
+		img = best_image(eimages[side_number(side)], 128, 128);
+
+// -------------------------------------------------------
+
 	}
 }
 
@@ -2711,6 +2788,13 @@ dispose_emblem_cicns()
 					DisposHandle((Handle) (*cicnhandle)->iconData);
 					DisposHandle((Handle) cicnhandle);
 					macimg->colricon = nil;
+
+// --- HEAP FRAGMENTATION FIX -------------------------------------------
+
+					macimg->cicn_inited = FALSE;
+
+// -----------------------------------------------------------------
+
 				}
 			}
 		}
