@@ -1,5 +1,5 @@
 /* Unit lists for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@ int hpcolw = 40;
 int acpcolw = 60;
 int poscolw = 60;
 int supcolw = 40;
+int taskcolw = 100;
 int notecolw = 50;
 
 /* Sum of non-scrolling field widths. */
@@ -71,17 +72,21 @@ create_list()
 	maxlistwidth += acpcolw;
 	maxlistwidth += poscolw;
 	maxlistwidth += nummtypes * supcolw;
+	maxlistwidth += taskcolw;
 	maxlistwidth += notecolw;
 
 	DGprintf("Creating a list\n");
-	list->sides == -1L;  /* lists every side and indeps too */
-	for (i = 0; i < MAXSORTKEYS; ++i) {
-		list->sortkeys[i] = bynothing;
-	}
-	list->sortkeys[0] = byside;
+	/* Default to listing every side and indeps too. */
+	list->sides == ALLSIDES;
+	list->completed_units = TRUE;
+	list->incomplete_units = TRUE;
+	for (i = 0; i < MAXSORTKEYS; ++i)
+	  list->sortkeys[i] = bynothing;
+	list->sortkeys[0] = byside; /* (should be a preference) */
 	list->mainsortmi = miViewBySide;
-	list->listglimpsed = FALSE;
 	list->largeicons = FALSE; /* (should be a preference) */
+	list->sidecolors = default_sidecolors;	/* same default as for maps */
+	/* (should share with toggle routine?) */
 	list->entryspacing = (list->largeicons ? largeentryspacing : smallentryspacing);
 	list->firstvisible = 0;
 	init_list_contents(list);
@@ -93,6 +98,8 @@ create_list()
 	stagger_window(list->window, &lastlisth, &lastlistv);
 	ShowWindow(list->window);
 	SetPort(list->window);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	/* Create the scrollbars. */
 	hscrollrect = list->window->portRect;
 	hscrollrect.top = hscrollrect.bottom - sbarwid;
@@ -172,7 +179,8 @@ sort_list_contents(List *list)
 void
 add_unit_to_list(List *list, Unit *unit)
 {
-	if (alive(unit)) {
+	if (alive(unit)
+	    && (completed(unit) ? list->completed_units : list->incomplete_units)) {
 		list->contents = add_unit_to_vector(list->contents, unit, FALSE);
 		/* (should apply other inclusion criteria too?) */
 		++list->numunits;
@@ -240,7 +248,6 @@ draw_list(List *list)
 	cliprect.right -= sbarwid;
 	tmprect.bottom -= sbarwid;
 	ClipRect(&cliprect);
-	TextSize(9);
 	draw_list_contents(list);
 	SetClip(tmprgn);
 	DisposeRgn(tmprgn);
@@ -329,6 +336,9 @@ draw_list_headings(List *list)
 		x += supcolw;
 	}
 	MoveTo(x, listtopbaseline);
+	DrawString("\pTask");
+	x += taskcolw;
+	MoveTo(x, listtopbaseline);
 	DrawString("\pNotes");
 	x += notecolw;
 	/* Draw a gray line indicating the end of the data columns. */
@@ -395,7 +405,7 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 	/* Draw an icon with side emblem for this unit. */
 	draw_unit_image(list->window, curactcolw + 2, y + 1,
 					(list->largeicons ? 32 : 16), (list->largeicons ? 32 : 16),
-					u, side_number(unit->side), !completed(unit));
+					u, side_number(unit->side), !completed(unit), TRUE);
 	/* Write the name or ordinal number. */
 	name_or_number(unit, spbuf);
 	spbuf[15] = '\0';  /* (should be clipping by pixels) */
@@ -460,6 +470,17 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 	} else {
 		x += nummtypes * supcolw;
 	}
+	/* Draw the task. */
+	MoveTo(x, texty);
+	if (unit->plan && unit->plan->tasks) {
+		task_desc(spbuf, unit->side, unit->plan->tasks);
+	} else {
+		strcpy(spbuf, "  -  ");
+	}
+	DrawText(spbuf, 0, strlen(spbuf));
+	/* Draw extra notes. */
+	x += taskcolw;
+	MoveTo(x, texty);
 	spbuf[0] = '\0';
 	if (side_sees_unit(dside, unit)
 		&& !inside_area(unit->x, unit->y)
@@ -563,7 +584,7 @@ list_vscroll_fn(ControlHandle control, short code)
 	SetCtlValue(control, curvalue);
 	/* Scroll the already-drawn bits. */
 	tmprgn = NewRgn();
-	SetRect(&tmprect, 0, listtoph, listwin->portRect.right - sbarwid, listwin->portRect.bottom - sbarwid);
+	SetRect(&tmprect, 0, listtoph, listwin->portRect.right - sbarwid, listtoph + pagesize * curlist->entryspacing);
 	ScrollRect(&tmprect, 0, (oldvalue - curvalue) * curlist->entryspacing, tmprgn);
 	InvalRgn(tmprgn);
 	/* We'll need to redraw the headings line. */
@@ -705,6 +726,20 @@ set_list_sorting(List *list, enum sortkeys newkey, int mi)
 	}
 }
 
+void
+toggle_list_completed(List *list)
+{
+	list->completed_units = !list->completed_units;
+	reorganize_list(list);
+}
+
+void
+toggle_list_incomplete(List *list)
+{
+	list->incomplete_units = !list->incomplete_units;
+	reorganize_list(list);
+}
+
 void
 toggle_list_large_icons(List *list)
 {
@@ -713,6 +748,13 @@ toggle_list_large_icons(List *list)
 	force_update(list->window);
 }
 
+void
+toggle_list_sidecolors(List *list)
+{
+	list->sidecolors = !list->sidecolors;
+	force_update(list->window);
+}
+
 void
 update_unit_in_lists(Unit *unit)
 {
@@ -769,7 +811,6 @@ redraw_unit_list_entry(List *list, int n)
 	cliprect = listwin->portRect;
 	cliprect.right -= sbarwid;
 	ClipRect(&cliprect);
-	TextSize(9);
 	draw_unit_list_entry(list, n, TRUE);
 	SetClip(tmprgn);
 	DisposeRgn(tmprgn);
@@ -814,7 +855,7 @@ scroll_to_selected_unit_in_list(List *list)
 	}
 	unit = selected_unit_in_list(list);
 	if (unit != NULL && inside_area(unit->x, unit->y))
-	  scroll_best_map_to_unit(unit);
+	  scroll_best_map_to_unit(unit, TRUE);
 }
 
 void
