@@ -388,6 +388,7 @@ toggle_map_grid(Map *map)
 void
 toggle_map_topline(Map *map)
 {
+	int oldtoph = map->toph;
 	Rect tmprect;
 	RgnHandle tmprgn;
 	GrafPtr oldport;
@@ -397,6 +398,7 @@ toggle_map_topline(Map *map)
 	/* Erase other-map boxes in other windows. */
 	draw_related_maps(map);
 	map->toph = map->toplineh + map->topunith;
+#if 0
 	set_content_rect(map);
 	if (map->toplineh) {
  		GetPort(&oldport);
@@ -413,6 +415,21 @@ toggle_map_topline(Map *map)
 	}
 	/* Let update_gworld handle both cases. */
 	update_gworld(map);
+#endif
+
+	/* Update without scrolling if we are expanding and the buffer is too small. */ 
+	if (map->bufy - map->offsety < oldtoph - map->toph) {
+		set_content_rect(map);
+		update_gworld(map);
+		/* This is not done by update_gworld (unlike update_resized_gworld). */
+		if (map->toplineh > 0)
+		  draw_top_line(map);
+		if (map->topunith > 0)
+		  draw_unit_info(map);
+	} else {
+		/* Always scroll the map if we are shrinking, or if the buffer is big enough. */
+		update_resized_map(map);
+	}
 
 	/* Redraw other-map boxes in other windows. */
 	draw_related_maps(map);
