@@ -134,7 +134,13 @@ WindowPtr *winmenuwins;
 
 int numwindows;
 
-Map *worldmap = NULL;
+// --- WORLD MAP CRASH FIX ---------------------------
+
+extern Map *worldmap = NULL;	/* Needed in macmap.c. */
+
+//	  Map *worldmap = NULL;
+
+// ------------------------------------------------
 
 WindowPtr worldmapwin = nil;
 
@@ -798,7 +804,15 @@ no longer exists. For some reason, DisposeWindow fails to destroy the window rec
 worldmapwin == nil alone does not work here.  
 */
 					/* Also test for worldmap->id == 0 */
-					if (worldmapwin == nil || worldmap->id == 0) {
+
+// --- WORLD MAP CRASH FIX -------------------------------------------------
+
+					/* Destroy_map now explicitly sets worldmap = NULL! */
+					if (worldmapwin == nil || worldmap == NULL || worldmap->id == 0) {
+
+//					if (worldmapwin == nil || worldmap->id == 0) {
+
+// ---------------------------------------------------------------------
 
 						/* winwid should equal the default size of new windows */
 						int power, winwid, mainwidth;
