@@ -30,6 +30,13 @@ static pascal void scores_vscroll_fn(ControlHandle control, short code);
 static void calc_history_layout(void);
 static void set_history_scrollbar(void);
 
+// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
+
+extern char **current_colorscheme;			/* Color schemes by name (red,pink,blue) */
+									/* Defined in maccolors.c */
+									
+// ----------------------------------------------------------------------------
+
 /* Globals for the game window. */
 
 WindowPtr gamewin = nil;
@@ -1177,7 +1184,15 @@ side_rename_dialog(Side *side)
 				   so as to avoid fragmentation */				
 				reinit_all_emblem_images();
 
-				dissect_colorscheme(side_number(side), side->colorscheme);
+// --- 68K COLORSCHEME BUG FIX ---------------------------------------
+
+				current_colorscheme[side_number(side)] = side->colorscheme;
+				dissect_colorscheme(side_number(side));
+
+//				dissect_colorscheme(side_number(side), side->colorscheme);
+
+// ---------------------------------------------------------------
+
 				ui_update_state();
 				save_preferences();
 				
