@@ -3,9 +3,9 @@
 # This script extracts and compares the symbols defined in the manual
 # to those defined in the source code.
 
-grep DEF_ $1/../kernel/*.def | grep -v DEF_CMD | sed -e 's/^[^"]*"\([^ ]*\)".*$/\1/' | sort | uniq >src.syms
+grep DEF_ $1/../kernel/*.def | grep -v DEF_CMD | sed -e 's/^[^"]*"\([^ ]*\)".*$/\1/' | grep -v '^zz-' | sort | uniq >src.syms
 
-grep '@deffn' $1/../doc/refman.texi | grep '@code' | sed -e 's/^.*@code{\([^{ }]*\)}.*$/\1/' | sed -e 's/@@/@/' | sort | uniq >doc.syms
+grep '@deffn' $1/../doc/refman.texi | grep '@code' | sed -e 's/^.*@code{\([^{ }]*\)}.*$/\1/' | sed -e 's/@@/@/' | grep -v '^zz-' | sort | uniq >doc.syms
 
 echo "Differences between symbols in reference manual and in source code:"
 echo "('<' - in documentation,  '>' - in sources)"
