@@ -12,5 +12,5 @@ for gd in $srcdir/../lib/*.g ; do
 	echo "" >>$logname
 	echo ">>> Running skelconq on: " $gd "<<<" >>$logname
 	echo "" >>$logname
-	../kernel/skelconq -A -f $gd -L $srcdir/../lib <$srcdir/aitest.inp >>$logname
+	../kernel/skelconq -r ,ai@stdio -f $gd -L $srcdir/../lib <$srcdir/aitest.inp >>$logname
 done
