@@ -5,9 +5,9 @@
 srcdir=$1
 
 if [ -r ../kernel/skelconq ] ; then
-	echo "Skelconq found"
+	echo "skelconq found"
 else
-	echo "Error: Skelconq not found, exiting"
+	echo "Error: skelconq not found, exiting"
 	exit 1
 fi
 
