@@ -18,13 +18,16 @@ any later version.  See the file COPYING.  */
 #include <X11/Xresource.h>
 
 extern int numimages;
-
 extern ImageFamily **images;
-
 extern char *outdirname;
 
+int numremotes;
+int my_rid;
+
 char imdirname[1000];
 
+char *name_to_match;
+
 int showprogress = 0;
 
 void usage PARAMS ((void));
@@ -40,7 +43,8 @@ char *argv[];
 
     init_lisp();
 
-    if (argc == 1) usage();
+    if (argc == 1)
+      usage();
     for (i = 1; i < argc; ++i) {
 	arg = argv[i];
 	if (strcmp(arg, "-o") == 0) {
@@ -54,6 +58,17 @@ char *argv[];
 	    } else {
 		init_error("No output directory following -o");
 	    }
+	} else if (strcmp(arg, "-match") == 0) {
+	    if (i + 1 < argc) {
+		name_to_match = argv[i + 1];
+		/* Blast the arg because we'll be scanning the args again
+		   and we want to ignore it then. */
+		argv[i] = NULL;
+		argv[i + 1] = NULL;
+		++i;
+	    } else {
+		init_error("No name to match following -match");
+	    }
 	} else if (strcmp(arg, "-p") == 0) {
 	    showprogress = 1;
 	    argv[i] = NULL;
@@ -83,10 +98,13 @@ char *argv[];
 	ofp = NULL;
     }
     for (i = 0; i < numimages; ++i) {
+      if (name_to_match == NULL
+	  || strcmp(name_to_match, images[i]->name) == 0) {
 	printf("/* %s imf */\n", images[i]->name);
 	if (ofp != NULL)
 	  fprintf(ofp, "%s\n", images[i]->name);
 	write_x11_bitmaps(images[i], (ofp != NULL));
+      }
     }
     if (ofp != NULL) {
 	fclose(ofp);
@@ -98,6 +116,10 @@ void
 usage()
 {
     fprintf(stderr,
-	    "usage: imf2x [ files ... ] -o outdir [ files ... ] [ -p ]\n");
+	    "usage: imf2x [ files ... ] -o outdir [ files ... ] [ options ... ]\n");
+    fprintf(stderr,
+	    "             -match name\tonly convert imfs matching name\n");
+    fprintf(stderr,
+	    "             -p\t\t\tshow progress\n");
     exit(1);
 }
