@@ -20,6 +20,9 @@ extern ImageFamily **images;
 
 char tmpbuf[500];
 
+int numremotes;
+int my_rid;
+
 int
 main(argc, argv)
 int argc;
@@ -27,6 +30,8 @@ char *argv[];
 {
     int i;
 
+    init_lisp();
+
     for (i = 1; i < argc; ++i) {
 	if (!read_any_file(argv[i], NULL)) {
 	    fprintf(stderr, "Couldn't read \"%s\"\n", argv[i]);
