@@ -1,5 +1,5 @@
 /* Closeups for the X11 interface to Xconq.
-   Copyright (C) 1995-1997 Massimo Campostrini and Stanley T. Shebs.
+   Copyright (C) 1995-1998 Massimo Campostrini and Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -1523,13 +1523,12 @@ Side *side;
 	pic = XCreatePixmap(dpy, side->ui->rootwin,
 			    min_w_for_unit_image, min_h_for_unit_image,
 			    DefaultDepth(dpy, side->ui->screen));
-	XSetForeground(dpy, side->ui->gc, side->ui->bgcolor);
 	XFillRectangle(dpy, pic, side->ui->gc,
 		       0, 0, min_w_for_unit_image, min_h_for_unit_image);
 	XSetForeground(dpy, side->ui->gc, side->ui->fgcolor);
 	draw_unit_image(side, pic, 0, 0,
 			min_w_for_unit_image, min_h_for_unit_image,
-			u, -1, 1, 0, 0);
+			u, -1, -1, -1, 0);
 	side->ui->unitpics[4][u] = pic;
     }
     return side->ui->unitpics[4][u];
