@@ -9,6 +9,7 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kpublic.h"
 #include "xconq.h"
+extern void generic_do_doctrine PARAMS ((Side *side, char *str));
 extern void create_orders_window PARAMS ((Side *side, Map *map));
 
 char *cmdargstr = NULL;
@@ -552,9 +553,10 @@ do_copying(side, map)
 Side *side;
 Map *map;
 {
+    popup_help(side);
     /* Go directly to the copying help node. */
     side->ui->curhelpnode = copying_help_node;
-    popup_help(side);
+    update_help(side);
 }
 
 void
@@ -836,16 +838,7 @@ do_doctrine(side, map)
 Side *side;
 Map *map;
 {
-  /* (should make generic) */
-  if (cmdargstr) {
-    if (strcmp(cmdargstr, "set") == 0) {
-      notify(side, "Set what?");
-    } else if (strncmp(cmdargstr, "set ", 4) == 0) {
-      net_set_doctrine(side, cmdargstr + 4);
-    }
-    return;
-  }
-  notify_doctrine(side, cmdargstr);
+    generic_do_doctrine(side, cmdargstr);
 }
 
 void
@@ -1105,7 +1098,8 @@ Map *map;
 {
     /* Compose the help node for commands and make it be the first one. */
     if (side->ui->curhelpnode == NULL) {
-	commands_help_node = add_help_node("commands", describe_commands, 0, first_help_node);
+	commands_help_node =
+	  add_help_node("commands", describe_commands, 0, first_help_node);
 	side->ui->curhelpnode = first_help_node;
     }
     popup_help(side);
@@ -1406,7 +1400,8 @@ int cancel;
       return;
     if (grok_bool(side, map)) {
 	if (numsides > 2) {
-	    ask_side(side, map, "Who do you want to surrender to?", NULL,
+	    /* (should suggest resigning to a trusted side) */
+	    ask_side(side, map, "To whom do you wish to surrender?", NULL,
 		     aux_resign_2);
 	} else {
 	    net_resign_game(side, NULL);
@@ -1424,8 +1419,12 @@ int cancel;
 
     if (cancel)
       return;
-    grok_side(side, map, &side2);
-    net_resign_game(side, side2);
+    if (grok_side(side, map, &side2)) {
+	net_resign_game(side, side2);
+    } else {
+	/* Iterate until grok_side is happy. */
+	map->modalhandler = aux_resign_2;
+    }
 }
 
 /* Do the act of leaving the game. */
@@ -2056,9 +2055,10 @@ do_warranty(side, map)
 Side *side;
 Map *map;
 {
-	/* Go directly to the warranty help node. */
-	side->ui->curhelpnode = warranty_help_node;
     popup_help(side);
+    /* Go directly to the warranty help node. */
+    side->ui->curhelpnode = warranty_help_node;
+    update_help(side);
 }
 
 /* X-interface-specific commands. */
