@@ -208,6 +208,7 @@ typedef struct a_ui {
     short dflt_color_terr_images;
     short dflt_color_embl_images;
     short pref_solid_color_terrain;
+    short pref_color_unit_images;
     short default_latlong_interval;
     GC gc;			/* a tmp graphics context for this display */
     GC textgc;			/* foreground on background text */
@@ -272,6 +273,7 @@ typedef struct a_ui {
     ImageFamily **uimages;
     ImageFamily **timages;
     ImageFamily **eimages;
+    short *eimages_loaded;
     /* Terrain drawing machinery */
     enum whattouse usewhat[NUMPOWERS][MAXTTYPES];
     /* (should use as cache of image in image family - calc as needed) */
@@ -287,7 +289,6 @@ typedef struct a_ui {
     char unitchars[NUMPOWERS][MAXUTYPES];
     XFontStruct *unitfonts[NUMPOWERS][MAXUTYPES];
     XFontStruct *unitfont;	/* font for unit characters */
-    int unitw[NUMPOWERS][MAXUTYPES], unith[NUMPOWERS][MAXUTYPES];
     /* Emblem drawing machinery */
     Pixmap emblempics[MAXSIDES];
     Pixmap emblemmasks[MAXSIDES];
