@@ -207,12 +207,14 @@ no unit found matching filter criteria",
 "xconq.closeupList*Label.font:	-adobe-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*",
 "xconq.orders.title:				xconq standing orders",
 "xconq.orders*port.height:			250",
-"xconq.orders*port.width:			440",
+"xconq.orders*port.width:			449",
 "xconq.orders.form.?.font:	-adobe-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*",
 "xconq.orders*orders.Form.vertDistance:		-1",
 "xconq.orders*orders.Form.horizDistance:		-1",
 "xconq.orders*orders.Form.?.height:		20",
 "xconq.orders*orders.Form.toggle.width:		20",
+"xconq.orders*orders.Form.width:			438",
+"xconq.orders*orders.Form.height:		30",
 "xconq.utypeSelect*box.width:			156",
 "xconq.utypeSelect*box.Command.height:		20",
 "xconq.utypeSelect*box.cancel.font: -adobe-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*",
