@@ -1,5 +1,5 @@
 /* Designer tools for the X11 interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,6 +7,7 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+#include "kpublic.h"
 #include "xconq.h"
 
 #ifdef DESIGNERS
@@ -527,7 +528,7 @@ XtPointer call_data;
       return;
 
     sprintf(spbuf, "%d", nextfid);
-    feature = create_feature("feature", copy_string(spbuf));
+    feature = net_create_feature("feature", copy_string(spbuf));
     if (feature != NULL) {
 	side->ui->curfid = feature->id;
     }
@@ -577,7 +578,7 @@ XtPointer call_data;
       return;
 
     popdown_design(side);
-    become_nondesigner(side);
+    net_become_nondesigner(side);
     for_all_maps(side, map) {
 	map->curtool = looktool;
 	set_tool_cursor(side, map);
@@ -665,26 +666,28 @@ update_curutype(side)
 Side *side;
 {
     char buf[BUFSIZE];
+    Pixmap pic;
 
     sprintf(buf, "%s (%s)",
  	    u_type_name(side->ui->curutype),
  	    side_adjective(side_n(side->ui->curusidenumber)));
 
-#if 0  /* (should include side emblem also) */
+    /* (should have a unit pixmap generator for this...) */
+    pic = XCreatePixmap(side->ui->dpy, side->ui->rootwin,
+			min_w_for_unit_image, min_h_for_unit_image,
+			DefaultDepth(side->ui->dpy, side->ui->screen));
+    /* (should include side emblem also) */
     XSetFillStyle(side->ui->dpy, side->ui->unitgc, FillSolid);
     XSetForeground(side->ui->dpy, side->ui->unitgc, side->ui->bgcolor);
-    XFillRectangle(side->ui->dpy, side->ui->unitbuttonpic,
+    XFillRectangle(side->ui->dpy, pic,
 		   side->ui->unitgc, 0, 0, 32, 32);
-    draw_unit_image(side, side->ui->unitbuttonpic, 0, 0, 16, 16,
+    draw_unit_image(side, pic, 0, 0, 16, 16,
 		    side->ui->curutype, -1,
-		    side->ui->fgcolor, side->ui->bgcolor);
-#endif
+		    1, 0, 0);
 
     nargs = 0;
     XtSetArg(tmpargs[nargs], XtNlabel, buf);  nargs++;
-#if 0
-    XtSetArg(tmpargs[nargs], XtNleftBitmap, side->ui->unitbuttonpic);  nargs++;
-#endif
+    XtSetArg(tmpargs[nargs], XtNleftBitmap, pic);  nargs++;
     XtSetValues(side->ui->unit_label, tmpargs, nargs);
 }
 
@@ -825,7 +828,7 @@ int sx, sy;
 	    oldt = terrain_at(x, y);
 	    painttype = (side->ui->curttype == oldt ? side->ui->curbgttype :
 			 side->ui->curttype);
-	    paint_cell(side, x, y, side->ui->curbrushradius, painttype);
+	    net_paint_cell(side, x, y, side->ui->curbrushradius, painttype);
 	    break;
 	  case bordersubtype:
 	    if (!x_nearest_boundary(side, map, sx, sy, &bx, &by, &dir)) {
@@ -834,7 +837,7 @@ int sx, sy;
 	    }
 	    if (inside_area(bx, by)) {
 		painttype = side->ui->curttype;
-		paint_border(side, bx, by, dir, painttype, -1);
+		net_paint_border(side, bx, by, dir, painttype, -1);
 	    }
 	    break;
 	  case connectionsubtype:
@@ -844,7 +847,7 @@ int sx, sy;
 	    }
 	    if (inside_area(bx, by)) {
 		painttype = side->ui->curttype;
-		paint_connection(side, bx, by, dir, painttype, -1);
+		net_paint_connection(side, bx, by, dir, painttype, -1);
 	    }
 	    break;
 	  case coatingsubtype:
@@ -855,7 +858,7 @@ int sx, sy;
         }
 	break;
       case unitaddtool:
-	unit = designer_create_unit(side, side->ui->curutype,
+	unit = net_designer_create_unit(side, side->ui->curutype,
 					side->ui->cursidenumber, ax, ay);
 	if (unit != NULL) {
 		/* (should make it be current) */
@@ -868,12 +871,12 @@ int sx, sy;
 	    oldpeop = people_side_at(ax, ay);
 	    paintpeop =
 	      (side->ui->cursidenumber == oldpeop ? NOBODY : side->ui->cursidenumber);
-	    paint_people(side, ax, ay, side->ui->curbrushradius, paintpeop);
+	    net_paint_people(side, ax, ay, side->ui->curbrushradius, paintpeop);
 	break;
       case featurepainttool:
 	oldfid = raw_feature_at(ax, ay);
 	paintfid = (side->ui->curfid == oldfid ? 0 : side->ui->curfid);
-	paint_feature(side, ax, ay, side->ui->curbrushradius, paintfid);
+	net_paint_feature(side, ax, ay, side->ui->curbrushradius, paintfid);
 	break;
       default:
 	/* complain or ignore? */
