@@ -1101,6 +1101,7 @@ int x, y;
     }
 }
 
+/* (should be generic?) */
 int
 unit_actually_visible(side, unit)
 Side *side;
@@ -1110,6 +1111,8 @@ Unit *unit;
 
     if (unit->side == side)
       return TRUE;
+    if (side->see_all)
+      return TRUE;
     if (u_see_always(unit->type))
       return TRUE;
     /* (should be true if see chance >= 100) */
@@ -1711,18 +1714,13 @@ int sx, sy, sw, sh, u, s2, fg, bg, mod;
       return;
     /* Just draw a small box at 4x4 and below. */
     if (sw <= 4) {
-      XSetClipMask(dpy, gc, None);
-      XSetForeground(dpy, gc, side->ui->blackcolor);
-      XFillRectangle(dpy, win, gc, sx, sy, sw, sh);
-      return;
+	XSetClipMask(dpy, gc, None);
+	XSetForeground(dpy, gc, side->ui->blackcolor);
+	XFillRectangle(dpy, win, gc, sx, sy, sw, sh);
+	return;
     }
     uimg = best_image(side->ui->uimages[u], sw, sh);
-    if (uimg && sw == 8 && sh == 8 && uimg->w > 8 && uimg->h > 8) {
-      /* Don't even try to scale; just draw a box. */
-      XSetClipMask(dpy, gc, None);
-      XSetForeground(dpy, gc, side->ui->blackcolor);
-      XFillRectangle(dpy, win, gc, sx + 1, sy + 1, sw - 2, sh - 2);
-    } else if (uimg) {
+    if (uimg != NULL) {
 	/* Offset the image to draw in the middle of its area,
 	   whether larger or smaller than the given area. */
 	sx2 = sx + (sw - uimg->w) / 2;  sy2 = sy + (sh - uimg->h) / 2;
@@ -1784,33 +1782,28 @@ int sx, sy, sw, sh, u, s2, fg, bg, mod;
 		    XSetClipMask(dpy, gc, ximg->mono);
 		    XFillRectangle(dpy, win, gc, sx, sy, sw, sh);
 		}
-	    } else if (ximg->monochar) {
-		XSetForeground(dpy, gc, imagecolor);
-		buf[0] = ximg->monochar;
-		/*	    sy += uimg->font->max_bounds.ascent;  */
-		/* need to set font as the one in use here? */
-		XDrawString(dpy, win, gc, sx, sy, buf, 1);
 	    }
-	} else
-	  desperate = TRUE;
-    } else
-      desperate = TRUE;
+	} else {
+	    desperate = TRUE;
+	}
+    } else {
+	desperate = TRUE;
+    }
     if (desperate) {
-	/* a bad error? */
+	/* For some reason, we have no useful image; draw a blank box
+	   instead. */
 	XSetClipOrigin(dpy, gc, sx, sy);
-	/* Pretty desperate, or else we're at a scale that doesn't matter. */
-	/* should actually clip to cell boundaries, maybe even inside
-	   border terrain if necessary? */
+	/* (should actually clip to cell boundaries, maybe even inside
+	   border terrain if necessary?) */
 	XSetClipMask(dpy, gc, None);
 	/* Draw an edge, at least for larger mags. */
-	if (sw >= 4) {
+	if (sw >= 16) {
 	    XSetForeground(dpy, gc, maskcolor);
 	    XFillRectangle(dpy, win, gc, sx, sy, sw, sh);
 	}
-	/* Draw a filled box inside the edge. */
+	/* Draw a filled box. */
 	XSetForeground(dpy, gc, imagecolor);
 	XFillRectangle(dpy, win, gc, sx + 1, sy + 1, sw - 2, sh - 2);
-	/* Might be cool to draw unit type name or char */
     }
     if (mod != 0) {
 	XSetFillStyle(dpy, gc, FillStippled);
@@ -1823,7 +1816,8 @@ int sx, sy, sw, sh, u, s2, fg, bg, mod;
     /* Draw a side emblem if one was requested. */
     if (between(0, s2, numsides)) {
 	ename = (side_n(s2) ? side_n(s2)->emblemname : NULL);
-	if (emblem_position(uimg, ename, NULL /* eimg */, sw, sh, &ex, &ey, &ew, &eh)) {
+	if (emblem_position(uimg, ename, NULL /* eimg */, sw, sh,
+			    &ex, &ey, &ew, &eh)) {
 	    draw_side_emblem(side, win, sx + ex, sy + ey, ew, eh, s2, 0);
 	}
     }
@@ -1838,40 +1832,38 @@ Side *side;
 Window win;
 int ex, ey, ew, eh, s2, style;
 {
-    int ex2, ey2, ew2, eh2;
+    int ex2, ey2, ew2, eh2, offset;
     long imagecolor, maskcolor;
-    Image *img;
+    Image *eimg;
     X11Image *ximg;
     Display *dpy = side->ui->dpy;
     GC gc = side->ui->emblgc;
 
     /* Draw the emblem's mask, or else an enclosing box. */
-    img = best_image(side->ui->eimages[s2], ew, eh);
-    if (img != NULL) {
+    eimg = best_image(side->ui->eimages[s2], ew, eh);
+    if (eimg != NULL) {
 	/* Offset the image to draw in the middle of its area,
 	   whether larger or smaller than the given area. */
-	ex2 = ex + (ew - img->w) / 2;  ey2 = ey + (eh - img->h) / 2;
+	ex2 = ex + (ew - eimg->w) / 2;  ey2 = ey + (eh - eimg->h) / 2;
 	/* Only change the size of the rectangle being drawn if it's
 	   smaller than what was passed in. */
-	if (img->w < ew) {
+	if (eimg->w < ew) {
 	    ex = ex2;
-	    ew = img->w;
+	    ew = eimg->w;
 	}
-	if (img->h < eh) {
+	if (eimg->h < eh) {
 	    ey = ey2;
-	    eh = img->h;
+	    eh = eimg->h;
 	}
+	ximg = (X11Image *) eimg->hook;
+	/* Maybe draw with a gray shadow. */
 	if (style == 1) {
-	    ew2 = ew;  eh2 = eh;
-	    if (img->bboxw < ew2)
-	      ew2 = img->bboxw;
-	    if (img->bboxh < eh2)
-	      eh2 = img->bboxh;
+	    offset = (ew >= 16 ? 2 : 1);
 	    XSetForeground(dpy, gc, side->ui->graycolor);
-	    XSetClipMask(dpy, gc, None);
-	    XDrawRectangle(dpy, win, gc, ex - 1, ey - 1, ew2 + 2, eh2 + 2);
+	    XSetClipOrigin(dpy, gc, ex + offset, ey + offset);
+	    XSetClipMask(dpy, gc, (ximg ? ximg->mask : None));
+	    XFillRectangle(dpy, win, gc, ex + offset, ey + offset, ew, eh);
 	}
-	ximg = (X11Image *) img->hook;
 	/* Decide on the colors to use with the emblem. */
 	if (side->ui->numcolors[s2] > 0) {
 	    imagecolor = side->ui->colors[s2][0];
@@ -2077,7 +2069,7 @@ Map *map;
     win = map->sideswin;
     sx = 2 + 16 + 2;
     sy = (side_number(side2) - 1) * map->sidespacing;
-    draw_side_emblem(side, win, 4, sy + 4, 8, 8, side_number(side2), 1);
+    draw_side_emblem(side, win, 1, sy + 2, 16, 16, side_number(side2), 1);
     /* Build up and write the textual description of the side. */
     /* (should make generic?) */
     sidebuf[0] = '\0';
