@@ -838,7 +838,7 @@ Side *side;
     tmp_display = side->ui->dpy;
     tmp_root_window = side->ui->rootwin;
     for_all_unit_types(u) {
-	imf = get_unit_type_images(side, u, "c");
+	imf = get_unit_type_images(side, u, "s");
 	/* (should fabricate something as a substitute) */
 	if (DebugG)
 	  describe_imf(side, "unit type", u_type_name(u), imf);
@@ -861,7 +861,7 @@ Side *side;
 {
     int t, p;
     ImageFamily *imf;
-    Image *timg;
+    Image *img, *timg;
     X11Image *ximg;
 
     side->ui->timages =
@@ -889,7 +889,13 @@ Side *side;
 	    }
 	}
 	/* Look for a solid color. */
-	timg = best_image(side->ui->timages[t], 1, 1);
+	timg = NULL;
+	for_all_images(side->ui->timages[t], img) {
+	    if (img->w == 1 && img->h == 1) {
+		timg = img;
+		break;
+	    }
+	}
 	if (timg != NULL && timg->w == 1 && timg->h == 1) {
 	    if (timg->hook) {
 		ximg = (X11Image *) timg->hook;
