@@ -1661,15 +1661,18 @@ char **strp;
 }
 
 void
-ask_side(side, map, prompt, dflt, handler)
+ask_side(side, map, prompt, dfltside, handler)
 Side *side;
 Map *map;
 char *prompt;
-Side *dflt;
+Side *dfltside;
 void (*handler) PARAMS ((Side *side, Map *map, int cancel));
 {
+    char *dfltstr;
+
     sprintf(map->prompt, "%s", prompt);
-    sprintf(map->answer, "%s", side_desig(dflt));
+    dfltstr = (dfltside == NULL ? "nobody" : side_name(dfltside));
+    sprintf(map->answer, "%s", dfltstr);
     draw_prompt(side, map);
     map->modalhandler = handler;
 }
@@ -1680,8 +1683,50 @@ Side *side;
 Map *map;
 Side **side2p;
 {
+    char ch = map->inpch, *str;
+    int len;
+    Side *side3;
+
     *side2p = NULL;
-    return TRUE;
+    if (ch == '\r' || ch == '\n') {
+	if (empty_string(map->answer))
+	  return TRUE;
+	if (strcmp(map->answer, "nobody") == 0)
+	  return TRUE;
+	for_all_sides(side3) {
+	    if (!empty_string(side3->name)
+		&& strcmp(map->answer, side3->name) == 0) {
+		*side2p = side3;
+		clear_prompt(side, map);
+		return TRUE;
+	    }
+	    if (!empty_string(side3->noun)
+		&& strcmp(map->answer, side3->noun) == 0) {
+		*side2p = side3;
+		clear_prompt(side, map);
+		return TRUE;
+	    }
+	    if (!empty_string(side3->adjective)
+		&& strcmp(map->answer, side3->adjective) == 0) {
+		*side2p = side3;
+		clear_prompt(side, map);
+		return TRUE;
+	    }
+	}
+	beep(side);
+	return FALSE;
+    } else {
+	len = strlen(map->answer);
+	if (ch == BACKSPACE_CHAR || ch == DELETE_CHAR) {
+	    if (len > 0)
+	      --len;
+	} else {
+	    map->answer[len++] = ch;
+	}
+	map->answer[len] = '\0';
+	draw_prompt(side, map);
+	return FALSE;
+    }
 }
 
 /* Given a pixel in a map, describe what's there. */
