@@ -39,11 +39,15 @@ any later version.  See the file COPYING.  */
 
 #define MAXFAMS (MAXROWS*MAXCOLS)
 
-extern int numimages;
 int numfamilies;
-extern ImageFamily **images;
 extern char *outdirname;
 
+extern int tmp_valid;
+
+extern Display *tmp_display;
+
+extern Window tmp_root_window;
+
 /* the number of allowed magnifications NUMMAGN 
    and the magnification scales magnif can be changed safely */
 #define NUMMAGN 4
@@ -224,7 +228,6 @@ void reset_prep PARAMS ((int i));
 void set_cursor PARAMS ((FamilyStuff *fms, Cursor cursor));
 int delete_image PARAMS ((Widget w));
 int update_or_export_image PARAMS ((Widget w, int flag));
-Image *find_image PARAMS ((ImageFamily *imf, int w, int h));
 void destroy_family PARAMS ((int i));
 void delete_family PARAMS ((int i));
 int empty_family PARAMS ((int i)); 
@@ -261,7 +264,13 @@ char *argv[];
     dpy = XtDisplay(toplevel);
     rootwin = DefaultRootWindow(dpy);
     screen = DefaultScreen(dpy);
-    
+
+    imf_interp_hook = x11_interp_imf;
+    imf_load_hook = x11_load_imf;
+    tmp_display = dpy;
+    tmp_root_window = rootwin;
+    tmp_valid = TRUE;
+
     /* process non-X argument */
     for (i = 1; i < argc; ++i) {
 	arg = argv[i];
@@ -487,6 +496,7 @@ int first_time;
 				  XtNheight, needh,
 				  XtNwidth, needw+sw+1,
 				  NULL);
+
     } else {
 	/* not first_time */
 	if ((rows != oldrows && (oldrows < maxrows || rows < maxrows))
@@ -507,6 +517,11 @@ int first_time;
     oldrows = rows;
     oldcols = cols;
     
+    for (i = 0; i < numfamilies; i++) {
+      if (imf_interp_hook)
+	images[i] = (*imf_interp_hook)(images[i], NULL, TRUE);
+    }
+
     /* entries for each image family */
     for (i = 0; i < numfamilies; i++) {
 
@@ -515,9 +530,9 @@ int first_time;
 	XtSetArg(tmpargs[a], XtNwidth,  basew+8);  a++;
 	XtSetArg(tmpargs[a], XtNheight, baseh+4);  a++;
 	XtSetArg(tmpargs[a], XtNlabel, "");  a++;
-	x11_interp_imf(dpy, rootwin, images[i], TRUE);
 	img = best_image(images[i], basew, baseh);
-	if (!img)  continue;
+	if (!img)
+	  continue;
 	if (img->hook != NULL) {
 	    ximg = (X11Image *) img->hook;
 	    if (ximg->colr != None) {
@@ -645,7 +660,7 @@ int imagn;
     last = fms->label;
     up = None;
     img0 = 1;
-    for (img = images[i]->images; img != NULL; img = img->next) {
+    for_all_images(images[i], img) {
 	if (last)
 	  up = last;
 	last = NULL;
@@ -1292,8 +1307,9 @@ int h, w, s;
     Pixmap pix;
     int lo, ln, i, j, is, js;
     char *new, *n, *o, *so, mo = 0, mn = 0;
-    
+
     lo = computed_rowbytes(w, 1);
+    reverse_bit_endianness(data, lo * h);
     ln = computed_rowbytes(w, s);
     new = (char *) malloc(s * h * ln * sizeof(char));
     if (new == NULL)
@@ -1331,6 +1347,7 @@ int h, w, s;
     }
     pix = XCreateBitmapFromData(dpy, rootwin, new, s * w, s * h);
     free(new); 
+    reverse_bit_endianness(data, lo * h);
     return pix;      
 }
 
@@ -1523,15 +1540,19 @@ Widget w;
     Image *img;
     X11Image *ximg = NULL;
     
-    if (!w)  return 0;
+    if (!w)
+      return 0;
     i = shell_index(XtParent(XtParent(w)));
     fms = &family[i];
     ims = find_ims(fms, w);
-    if (!ims)  return 0;
-    img = find_image(images[i], ims->w, ims->h);
-    if (!img)  return 0;
+    if (!ims)
+      return 0;
+    img = find_img(images[i], ims->w, ims->h);
+    if (!img)
+      return 0;
     ximg = (X11Image *) img->hook;
-    if (!ximg)  return 0;
+    if (!ximg)
+      return 0;
     
     if (w == ims->mono_w) {
 	undisplay_family(i);
@@ -1544,7 +1565,6 @@ Widget w;
 	    }
 	}
 	ximg->mono = None;
-	ximg->monodata = NULL;
 	img->rawmonodata = NULL;
 	img->monodata = lispnil;
     } else if (w == ims->mask_w) {
@@ -1557,24 +1577,22 @@ Widget w;
 	    }
 	}
 	ximg->mask = None;
-	ximg->maskdata = NULL;
 	img->rawmaskdata = NULL;
 	img->maskdata = lispnil;
     } else if (w == ims->colr_w) {
 	undisplay_family(i);
 	ims->colr_p[0] = None;
-	for (j=1; j<NUMMAGN; j++) {
+	for (j = 1; j < NUMMAGN; j++) {
 	    if (ims->colr_p[j] != None) {
 		XFreePixmap(dpy, ims->colr_p[j]);
 		ims->colr_p[j] = None;
 	    }
 	}
 	ximg->colr = None;
-	ximg->colrdata = NULL;
 	img->rawcolrdata = NULL;
 	img->colrdata = lispnil;
     } else if (w == ims->comb_w) {
-	XBell(dpy,35);
+	XBell(dpy, 35);
 	return 0;
     }
     if (w == ims->mono_w || w == ims->mask_w) {
@@ -1585,7 +1603,6 @@ Widget w;
 	    }
 	}
     }
-    
     return 1;
 }
 
@@ -1609,7 +1626,7 @@ int flag;
     ims = find_ims(fms, w);
     if (!ims)
       return 0;
-    img = find_image(images[i], ims->w, ims->h);
+    img = find_img(images[i], ims->w, ims->h);
     if (!img)
       return 0;
 
@@ -1658,7 +1675,7 @@ int flag;
 	    }
 	}
     } else {
-	XBell(dpy,35);
+	XBell(dpy, 35);
 	return 0;
     }
 
@@ -1668,7 +1685,6 @@ int flag;
     if (ok && !flag) {
 	imag = fms->imagn;
 	undisplay_family(i);
-	x11_interp_imf(dpy, rootwin, images[i], TRUE);
 	img = best_image(images[i], basew, baseh);
 	if (img->hook != NULL) {
 	    X11Image *ximg = (X11Image *) img->hook;
@@ -1687,32 +1703,17 @@ int flag;
 	sprintf(buffer, "%s \"%s\"", flag ? "exported in" : "updated from",
 		shortbuf);
     } else {
-	XBell(dpy,35);
+	XBell(dpy, 35);
 	sprintf(buffer, "%s failed", flag ? "export" : "update");
     }
     XtVaSetValues(fms->label, XtNlabel, buffer, NULL);
     return ok;
 }
 
-Image *
-find_image(imf, w, h)
-ImageFamily *imf;
-int w, h;
-{
-    Image *img;
-
-    for (img = imf->images; img != NULL; img = img->next) {
-	if (w == img->w && h == img->h) {
-	    return img;
-	}
-    }
-    return NULL;
-}
-
 /* i>=0: delete family #i;  i<0: delete selected families */
 
 void 
-delete_family (i)
+delete_family(i)
 int i;
 {
     int j, n, hidden;
@@ -1720,11 +1721,11 @@ int i;
     
     hidden = 0;
     n = 0;
-    for (j=0; j<numfamilies; j++) {
-	if (i<0) {
+    for (j = 0; j < numfamilies; j++) {
+	if (i < 0) {
 	    XtVaGetValues(family[j].name, XtNstate, &state, NULL);
 	}
-	if ((i<0 && state) || (j==i)) {
+	if ((i < 0 && state) || (j == i)) {
 	    if (!hidden)  hide_image_families();
 	    hidden = 1;
 	    destroy_family(j);
@@ -1735,7 +1736,7 @@ int i;
 	}
     }
     
-    if (numfamilies>n) {
+    if (numfamilies > n) {
 	sprintf(buffer, "%d image families left", n);
 	XtVaSetValues(message, XtNlabel, buffer, NULL);
     }
@@ -1768,7 +1769,7 @@ int i;
     Image *img;
     X11Image *ximg;
 
-    for (img = images[i]->images; img != NULL; img = img->next) {
+    for_all_images(images[i], img) {
 	if (!img->hook)
 	  continue;
 	ximg = (X11Image *) img->hook;
