@@ -1,5 +1,5 @@
 /* Definitions used in X utility programs.
-   Copyright (C) 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1993-1995, 1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,10 +8,6 @@ any later version.  See the file COPYING.  */
 
 extern void close_displays PARAMS ((void));
 extern int write_entire_game_state PARAMS ((char *fname));
-extern void low_init_error PARAMS ((char *str));
-extern void low_init_warning PARAMS ((char *str));
-extern void low_run_error PARAMS ((char *str));
-extern void low_run_warning PARAMS ((char *str));
 extern int keyword_code PARAMS ((char *str));
 extern Image *read_xbm_file PARAMS ((char *filename, ImageFamily *imf,
 				    readimf_hook hook));
